#!/bin/sh

# This script is meant to be run on Linux, AIX, Darwin, and HP-UX.  It will
# create a file called /tmp/quest_hardware_information.out, which contains
# information about hardware and some general system information.
#
# The main purpose of this script is to obtain enough information to minimally
# support uncommon hardware.

# The descriptor number of normal output.  This will be opened to a file.
# Note: Anything contained in here that uses Output or PipedOutput will be sent
# to this descriptor.  All other output will be sent to stdout or stderr. 
OUTPUT_DESCRIPTOR=7



# Extended Regular expression of files to ignore when traversing directories.
IGNORE_REGEX='(^/proc/(\.|())[0-9]+)|(^/proc/vmnet)|(^/proc/self)|(^/proc/kcore)|(^/proc/ksyms)|(^/proc/kmsg)'

OpenOutput()
{
	CloseOutput || true
	if [ $# -gt 0 ]; then
		local_FILENAME=$1
	  exec 7>$local_FILENAME || return 1
	else
		exec 7>&1 || return 1
	fi
	return 0
}

CloseOutput()
{
  exec 7>&- || return 1
  return 0
}

# Output command-line items to a file descriptor.
Output()
{
	echo "$@" >&$OUTPUT_DESCRIPTOR
}

# Output data from a pipe (stdin) to a file descriptor.
PipedOutput()
{
  cat >&$OUTPUT_DESCRIPTOR 2>&1
}

FindAndCat()
{
	for MAIN_DIRECTORY in $@; do 
		for x in `find "$MAIN_DIRECTORY" -printf "\"%p\"\\n" 2>/dev/null`; do
			# Strip extra quotes from it, so that this is the only eval needed.
			FullPath=`eval echo $x`
			if echo "$FullPath" | egrep "${IGNORE_REGEX}" >/dev/null 2>&1; then
# This output has been removed to save space.
#				Output "Ingoring \"$FullPath\" due to pattern match"
				:
			elif [ -h "$FullPath" ] || [ -L "$FullPath" ]; then
				Output "Found symbolic link \"$FullPath\""
				Output "Running stat:"
				stat "$FullPath" | PipedOutput
			elif [ -d "$FullPath" ]; then
				Output "Found directory \"$FullPath\""
			elif [ -f "$FullPath" ] && [ -r "$FullPath" ]; then
				Output
				Output "Contents of \"$FullPath\":"
				cat "$FullPath" 2>&1 | PipedOutput
				Output
			else
				Output "Located file \"$FullPath\" of type `file -bs \"$FullPath\"`"
			fi
			Output "Running \"file\" on $FullPath:"
			file "$FullPath" | PipedOutput
		done
	done
}


UnameCheck()
{
	echo "Checking uname"
	Output "######################################################################"
	Output "Uname info:"
	Output "######################################################################"
	Output "Machine: `uname -m 2>&1`"
	Output "Processor: `uname -p 2>&1`"
	Output "NodeName: `uname -n 2>&1`"
	Output "Release: `uname -r 2>&1`"
	Output "Version: `uname -v 2>&1`"
	Output "Platform: `uname -i 2>&1`"
	Output "OS: `uname -o 2>&1`" 
	Output "Complete uname info: `uname -a 2>/dev/null`" 
}

LSBCheck()
{
	echo "Checking lsb"
	Output "######################################################################"
	Output "lsb info:"
	Output "######################################################################"
	lsb_release -a 2>&1 | PipedOutput || Output "No lsb located in path"
}

LocateExecutable()
{
	if [ $# -gt 0 ]; then
		for location in `echo $PATH | tr '[:]' '[ ]'` /{bin,sbin} /usr/{bin,sbin} /usr/local/{bin,sbin} ; do
			if [ -f "$location/$1" ] && [ -x "$location/$1" ]; then
				echo "$location/$1"
				return 0
			fi
		done
	fi
	echo false
}

RPMCheck()
{
	echo "Checking rpm"
	RPM=`LocateExecutable rpm`

	Output "######################################################################"
	Output "RPM Version:"
	Output "######################################################################"
	if [ $RPM = "false" ]; then
		Output "Failed to locate rpm"
		return 1
	fi
	$RPM --version 2>&1 | PipedOutput
	Output "######################################################################"
	Output "Release from rpm:"
	Output "######################################################################"
	$RPM -q --info `rpm -qa| grep -i release` 2>&1 | PipedOutput
	Output "######################################################################"
	Output "Query tags accepted by rpm:"
	Output "######################################################################"
	$RPM --querytags 2>&1 | PipedOutput
}

ReleaseFileCheck()
{
	echo "Checking release files from /etc"
	Output "######################################################################"
	Output "Release files from /etc"
	Output "######################################################################"
	for foo in `ls -1d /etc/* | grep -i "release"`; do
		Output "Contents of $foo:"
		Output "--------------------"
		cat $foo 2>&1 | PipedOutput
		Output "--------------------"
	done
}


ProcFSCheck()
{
	echo "Traversing /proc.  This will take a while."
	Output "######################################################################"
	Output "Traversing /proc:"
	Output "######################################################################"
	FindAndCat /proc
}

SysFSCheck()
{
	echo "Traversing /sys.  This may take a while."
	Output "######################################################################"
	Output "Traversing /sys:"
	Output "######################################################################"
	FindAndCat /sys
}


FullListing()
{
	if [ $# -gt 0 ]; then
		echo "Checking contents of $1"
		Output "######################################################################"
		Output "Contents of $1"
		Output "######################################################################"
		ls -laFR "$1" 2>&1 | PipedOutput	
	fi
}

ExecuteProgram()
{
  PROGRAM_NAME=$1
	PROGRAM_ARGS=$2
	shift 2
	PROGRAM_PATH=`LocateExecutable $PROGRAM_NAME`
	echo "Getting information for $PROGRAM_NAME with args \"$PROGRAM_ARGS\""
	if [ $PROGRAM_PATH != "false" ]; then
		Output "######################################################################"
		Output "$*: Executing $PROGRAM_PATH $PROGRAM_ARGS"
		Output "######################################################################"
		eval $PROGRAM_PATH $PROGRAM_ARGS 2>&1 | PipedOutput
	else
		Output "######################################################################"
		Output "$*: $PROGRAM_NAME not found"
		Output "######################################################################"
	fi
}

LSPCIOutput()
{
	LSPCI=`LocateExecutable lspci`
	echo "Checking pci devices"
	Output "######################################################################"
	Output "lspci output (numeric):"
	Output "######################################################################"
	$LSPCI -mnb 2>&1 | PipedOutput
	Output "######################################################################"
	Output "lspci output (Textual):"
	Output "######################################################################"
	$LSPCI -mb 2>&1 | PipedOutput
	Output "######################################################################"
	Output "lspci output (verbose):"
	Output "######################################################################"
	$LSPCI -v 2>&1 | PipedOutput
}


MountCheck()
{
	Output "######################################################################"
	Output "Mount files from /etc"
	Output "######################################################################"
	for foo in /etc/fstab /etc/mtab /etc/filesystems /etc/mnttab; do
		if [ -f "$foo" ]; then
			Output "Contents of $foo:"
			Output "--------------------"
			cat $foo 2>&1 | PipedOutput
			Output "--------------------"
		else
			Output "Could not find \"$foo\""
		fi
	done

	ExecuteProgram mount "" "File System Mounts"
	ExecuteProgram df "" "Free Disk Space"
	ExecuteProgram df "-B 1 -T -P" "Free Disk Space"
	ExecuteProgram df "-P -k" "Free Disk Space"
}

NetworkCheck()
{
	ExecuteProgram ifconfig "-a" "Network Info"
}

MemoryCheck()
{
	ExecuteProgram vmstat "" "vmstat output"
	ExecuteProgram vm_stat "" "vm_stat output"
	ExecuteProgram free "" "free output"
}


PerformLinuxChecks()
{
	# Some form of lsb should be installed... Check the version that way.
	LSBCheck

	# Check for release files under /etc
	ReleaseFileCheck

	# Traverse /proc
	ProcFSCheck

	# Do a recursive listing of /sys
	FullListing /sys

	# Traverse /sys
	SysFSCheck

	# lspci output
	LSPCIOutput
}

PerformSolarisChecks()
{
	PATH=$PATH:/usr/platform/`uname -m`/sbin
	export PATH
	ExecuteProgram swap "-l" "swap information"
	ExecuteProgram cfgadm "-lv -s delim='|'" "Config information"
	ExecuteProgram iostat "-E -n" "iostat information"
	ExecuteProgram sysdef "" "sysdef information"
	ExecuteProgram top "-n" "top information"
	ExecuteProgram sar "-A 3 3" "System activity reporter information"
	ExecuteProgram prtconf "-vp" "prtconf information"
	ExecuteProgram prtdiag "-v" "prtdiag information"
}

PerformHPUXChecks()
{
	ExecuteProgram ioscan "-n -k -F" "ioscan information"
}

PerformDarwinChecks()
{
	ExecuteProgram system_profiler "" "System Profiler Information"

# should we use pdisk??
}


PerformAIXChecks()
{
	ExecuteProgram oslevel "-r" "Operating system update level"
	ExecuteProgram lsdev "-C -F 'class | type | subclass | description | devid | prefix | uniquetype | DvDr | led | Configure | base | name | chgstatus | parent | connwhere | status | location |'" "lsdev output"

	ALL_DEVICES=`lsdev -C -F name`

	for device in $ALL_DEVICES; do
		Output "Gathering details for device $device"
		ExecuteProgram lsattr "-El $device" "Attributes for $device"
		ExecuteProgram lscfg "-v -l $device" "Config for $device"
		ExecuteProgram lspv "-l $device" "lspv data for $device"
		ExecuteProgram lslv "$device" "lslv data for $device"
		ExecuteProgram lsps "$device" "lsps data for $device"
	done
}

Main()
{
	OUTPUT_FILENAME=/tmp/quest_hardware_information.out
	echo "Checking your system.  Please wait."
	OpenOutput $OUTPUT_FILENAME

	Output "My hostname is `hostname`"

	# All OSes have uname.
	UnameCheck

	# Many OSes have rpm.
	RPMCheck

	# Check the current mount files.
	MountCheck

	# Check the current network status
	NetworkCheck

	# Check the memory
	MemoryCheck

	# Random programs that contain useful information...
	ExecuteProgram locale "" "locale output"
	ExecuteProgram uptime "" "uptime output"
	
	case `uname -s` in
		Linux) 
			PerformLinuxChecks 
			;;
		AIX)
			PerformAIXChecks
			;;
		Darwin)
			PerformDarwinChecks
			;;
		HP-UX)
			PerformHPUXChecks 
			;;
		SunOS)
			PerformSolarisChecks 
			;;
		*) 
			Output "######################################################################"
		  Output "Unknown or unexpected platform `uname -s`"
			Output "######################################################################"
			;;
	esac


	# Recursive directory listing of /proc and /dev
	FullListing /dev
	FullListing /proc

	Output "######################################################################"
	Output "######################################################################"
	Output "End of hardware information."
	Output "######################################################################"
	echo 
	echo "We are now done."
	echo "The file $OUTPUT_FILENAME is now ready for sending"
	echo "to Quest."
	echo 
	echo
	echo "Reply to your Quest support email and attach this file."
	echo "Alternatively, send this file to support@quest.com with"
	echo "\"hardware_information.sh\" in the subject line."
	echo
	echo "Feel free to inspect the contents before sending this file."
	CloseOutput
}


Main
