#!/bin/sh

# install_qmxcm_client.sh - Check to see what OS & version we're on and
# install/run the QMX client (if supported).
#
# Copyright (c) 2004,2005,2006,2007 Quest Software, Inc.
#

# Things to do:
# Low priority (highly annoying, wasteful, and/or hard to maintain):
#  * Maybe there should be a command-line option to specify some overriding
#    selection type so that the user is not presented with two "select file" menus.
#  * Merge the user/group creation/checking functions with those of UMI and
#    dump them in some usable portion of the UMI script library.
#  * Fix the indenting in this script.

# Set environment variables which could potentially result in incorrect libraries being loaded
# or incorrect system commands being executed.
VENDOR_PREFIX="/opt/quest"
QMX_PREFIX="/opt/quest/qmxcm"

PATH="/bin:/usr/bin:/usr/sbin"
UMI_PREFIX="/opt/quest/umi"
QMX_DEFAULT_LIB_PATH=${QMX_PREFIX}/lib:${UMI_PREFIX}/lib:/lib:/usr/lib
LD_LIBRARY_PATH=$QMX_DEFAULT_LIB_PATH
LD_LIBRARY_PATH_64=$QMX_DEFAULT_LIB_PATH
SHLIB_PATH=$QMX_DEFAULT_LIB_PATH
LIBPATH=$QMX_DEFAULT_LIB_PATH
DYLD_LIBRARY_PATH=$QMX_DEFAULT_LIB_PATH
DYLD_FALLBACK_LIBRARY_PATH=$QMX_DEFAULT_LIB_PATH

export PATH
export LD_LIBRARY_PATH
export SHLIB_PATH
export LIBPATH
export DYLD_LIBRARY_PATH
export DYLD_FALLBACK_LIBRARY_PATH

# Script will fail if any commands fail.
set -e
set -u

PRODUCT_NAME="QMXCM"
PRODUCT_SHORT_NAME="QMX for ConfigMgr"
REGISTERED_PRODUCT_NAME="qmxsms"
QMX_PACKAGE_NAME="qmxcm"
QMX_CLIENTTOOL="${QMX_PREFIX}/bin/clienttool"
STARTUP_DISABLED=0
INSTALLATION_ABORTED=0
DEBUG_ENABLED=0
ENABLE_DEBUGGING_NOW="set -x"
FILE_LIMIT_REQUIRED=256
# The qmxcmd user/group should match that used in the product registration.
QMXSMSD_GROUP="qmxsms"
QMXSMSD_USER="qmxsms"
COLLECTION_GROUP=owprovdr
COLLECTION_USER=${COLLECTION_GROUP}
LOG_GROUP=owcimomd
LOG_USER=${LOG_GROUP}
QMX_INIT_SCRIPT=${QMX_PREFIX}/sbin/qmxcmd_init
QMX_CONFIG_FILE=/etc${QMX_PREFIX}/qmxcm.conf
QMX_STATEDIR=/var${QMX_PREFIX}
NON_INSTALL_DIRECTORIES="bin diagnostics docs umi scripts"
VASTOOLARGS=""
RECONFIGURE_ONLY=0
REMOVE_SOURCE_DIR_ON_EXIT=0

# This is here to force all of the shell utilities to act properly.
LANG=POSIX
export LANG

# The script to use when installing on MacOS.
DMG_INSTALLER=dmg_installer.sh
DMG_INSTALLER_SHELL=/bin/sh
DMG_INSTALLER_OPTS=-i

FIXME_ENCOUNTERED=0
ERROR_ENCOUNTERED=0
WARNING_ENCOUNTERED=0
ALL_INSTALLER_ERRORS=
ALL_INSTALLER_WARNINGS=
ALL_INSTALLER_FIXMES=

# These are the versions determined from the install files.  This is *somewhat*
# easy to determine, since QMX has consistant naming of packages across the
# various platforms.  That is, we use names like
# quest-qmx-MAJOR.MINOR.MICRO{Alpha,Beta,Release,}.BUILD{-,_}*.
QMXSMS_INSTALL_FILE_VERSION=""

# UMI-specific variables
UMI_UNINSTALL_SCRIPT="${UMI_PREFIX}/bin/uninstall_umi.sh"
UMI_INSTALL_SCRIPT_NAME="install_umi.sh"
UMI_INSTALL_SCRIPT_PATH=""
UMI_INSTALL_OPTIONS=""
UMI_INIT_SCRIPT=${UMI_PREFIX}/sbin/quest-owcimomd_init
UMI_TOOLS_DIRECTORY=${UMI_PREFIX}/tools/bin
CIMOM_URL="--url localhost:owipc"

# The package directory.  This variable must ALWAYS be used within quotes.
BASE_INSTALLER_DIR=""
QMX_TEMP_DIRECTORY=
CACHED_PACKAGE_DIRECTORY=
EXECUTED_SCRIPT_NAME=""
UNINSTALL_SCRIPT_NAME="uninstall_client.sh"
QMX_UNINSTALL_SCRIPT_PATH="${QMX_PREFIX}/bin/${UNINSTALL_SCRIPT_NAME}"
QMX_INSTALL_SCRIPT_NAME="install_client.sh"
UNINSTALL_SCRIPT_PATH="${QMX_PREFIX}/bin/${UNINSTALL_SCRIPT_NAME}"
ALLOWED_COMPRESSION_EXTENSIONS=".gz .bz2"
VMX_INSTALL_SCRIPT_RUNNING=1
export VMX_INSTALL_SCRIPT_RUNNING
QMX_UPGRADE_IN_PROGRESS=${QMX_UPGRADE_IN_PROGRESS:-0}
export QMX_UPGRADE_IN_PROGRESS

# Options set by command line parameters
QUIET_OPT=0
UPGRADE_OPT=0
UNINSTALL_OPT=0
MPE_OPT=""
MPE_PORT_OPT=""
MPE_SSL_PORT_OPT=""
FORCE_OPT=0
SITE_CODE_OPT=""
TRUSTED_ROOT_KEY=""
INSTALL_TYPE=install
FATAL_ERROR=
UNSUPPORTED=0
DETACH_REQUIRED=0

# If the install detaches (due to parameters or environmental settings), save
# the install log in this file.
DETACHED_LOG_NAME=/tmp/qmxcm_install.log

# Variables used for upgrading.
INSTALLED_CLIENT_VERSION=
BACKUP_OW_CONFIG_FILE=
OLD_SITE_CODE=
OLD_MPE=
OLD_MPE_PORT=
OLD_MPE_SSL_PORT=
INSTALLER_RUNNING_FILE=/tmp/qmx_install_script_running

# Names the client has been packaged with at some point in time.
HISTORICAL_PACKAGE_NAMES="qmxcm qmxsms vintela-vmx vmx VintelaVMX"


# This determines if an existing client is installed and sets variables
# specific for the installed version.
DetermineInstalledSettings()
{
	CLIENT_IS_INSTALLED=0

	INSTALLED_CLIENT_PACKAGE=
	for local_package in "qmxcm" ${HISTORICAL_PACKAGE_NAMES}; do
		if QueryExistingPackage ${local_package} >&7 2>&1; then
			INSTALLED_CLIENT_PACKAGE=${local_package}
			break
		fi
	done

	INSTALLED_CLIENT_VERSION=
	for clienttool in \
		${QMX_CLIENTTOOL} \
		/opt/quest/qmxcm/bin/clienttool \
		/opt/quest/qmxsms/bin/clienttool \
		/opt/vintela/vmx/bin/vmxclienttool; do
		if [ -f ${clienttool} -o -h ${clienttool} ] && ${clienttool} --version >/dev/null 2>&1; then
			INSTALLED_CLIENT_VERSION=`${clienttool} --version`
		fi
	done

	INSTALLED_CLIENT_IS_BROKEN=0
	if [ x${INSTALLED_CLIENT_VERSION:+set} != xset ]; then
		if [ x${INSTALLED_CLIENT_PACKAGE:+set} != xset ]; then
			OutputVerbosely "Unable to determine installed client version"
			return
		fi

		INSTALLED_CLIENT_IS_BROKEN=1
	fi

	CLIENT_IS_INSTALLED=1

	case ${INSTALLED_CLIENT_VERSION}-${INSTALLED_CLIENT_PACKAGE} in
		2.0* | 2.1* | *-qmxcm)
			INSTALLED_PRODUCT_NAME="QMXCM"
			INSTALLED_PREFIX="/opt/quest/qmxcm"
			INSTALLED_INIT_SCRIPT="${INSTALLED_PREFIX}/sbin/qmxcmd_init"
			INSTALLED_UNINSTALL_SCRIPT="${INSTALLED_PREFIX}/bin/uninstall_client.sh"
			INSTALLED_USES_UMI=1
			INSTALLED_CONFIG_PATTERN=qmxcm
			INSTALLED_CONFIG_FILE=/etc${INSTALLED_PREFIX}/qmxcm.conf
			;;
		1.5* | 1.6* | *-qmxsms)
			INSTALLED_PRODUCT_NAME="QMXSMS"
			INSTALLED_PREFIX="/opt/quest/qmxsms"
			INSTALLED_INIT_SCRIPT="${INSTALLED_PREFIX}/sbin/qmxsmsd_init"
			INSTALLED_UNINSTALL_SCRIPT="${INSTALLED_PREFIX}/bin/uninstall_qmx_client.sh"
			INSTALLED_USES_UMI=1
			INSTALLED_CONFIG_PATTERN=qmxsms
			INSTALLED_CONFIG_FILE=/etc${INSTALLED_PREFIX}/qmxsms.conf
			;;
		1.* | *-vmx | *-VintelaVMX)
			INSTALLED_PRODUCT_NAME="VMX"
			INSTALLED_PREFIX="/opt/vintela/vmx"
			INSTALLED_INIT_SCRIPT="${INSTALLED_PREFIX}/sbin/vintela-owcimomd_init"
			INSTALLED_UNINSTALL_SCRIPT="${INSTALLED_PREFIX}/bin/uninstall_vmx_client.sh"
			INSTALLED_USES_UMI=0
			INSTALLED_CONFIG_PATTERN=vmx
			INSTALLED_CONFIG_FILE=/etc${INSTALLED_PREFIX}/vmx.conf
			;;
		*)
			INSTALLED_PRODUCT_NAME="${PRODUCT_NAME}"
			INSTALLED_PREFIX="${QMX_PREFIX}"
			INSTALLED_INIT_SCRIPT="${QMX_INIT_SCRIPT}"
			INSTALLED_UNINSTALL_SCRIPT="${QMX_UNINSTALL_SCRIPT_PATH}"
			INSTALLED_USES_UMI=1
			INSTALLED_CONFIG_PATTERN=qmxcm
			INSTALLED_CONFIG_FILE=${QMX_CONFIG_FILE}
			;;
	esac

	if [ -f ${INSTALLED_UNINSTALL_SCRIPT} ]; then
		:
	else
		INSTALLED_CLIENT_IS_BROKEN=1
	fi

	if [ ${INSTALLED_CLIENT_IS_BROKEN} = 1 ]; then
		OutputNormal "Installed client is broken.  It must be removed or reinstalled."
	fi
}


SUCCESSFULLY_FINISHED=0

###########################################################################
# Begin output functions...
###########################################################################

# Set up descriptors for verbose and debug output
SetupDescriptors()
{
	# Debug descriptor
	exec 7>&-
	if [ ${DEBUG_ENABLED:-0} = 1 ]; then
		exec 7>&2
	else
		exec 7>/dev/null
	fi


	# Verbose descriptor
	exec 6>&-
	if [ ${QUIET_OPT:-0} != 1 ]; then
		exec 6>&1
	else
		exec 6>/dev/null
	fi
}

# Define a bunch of trivial functions for various forms of output.
OutputDebug()     { echo "${@:-}" >&7; }
OutputVerbosely() { echo "${@:-}" >&6; }
OutputError()     { echo "${@:-}" >&2; }
OutputNormal()    { echo "${@:-}"; }
WARNING()
{
	WARNING_ENCOUNTERED=1
	OutputError "WARNING: " "$@"
	ALL_INSTALLER_WARNINGS="${ALL_INSTALLER_WARNINGS:-}${ALL_INSTALLER_WARNINGS:+\n}WARNING: $@"
}
ERROR()
{
	ERROR_ENCOUNTERED=1
	OutputError "ERROR: " "$@"
	ALL_INSTALLER_ERRORS="${ALL_INSTALLER_ERRORS:-}${ALL_INSTALLER_ERRORS:+\n}ERROR: $@"
}
FIXME()
{
	FIXME_ENCOUNTERED=1
	OutputError "FIXME: " "$@"
	ALL_INSTALLER_FIXMES="${ALL_INSTALLER_FIXMES:-}${ALL_INSTALLER_FIXMES:+\n}FIXME: $@"
}
# These functions have inverted logic.
HaveWarnings() { [ ${WARNING_ENCOUNTERED:-0} = 1 ] || return 1; }
HaveErrors()   { [ ${ERROR_ENCOUNTERED:-0} = 1 ] || return 1; }
HaveFixmes()   { [ ${FIXME_ENCOUNTERED:-0} = 1 ] || return 1; }

DisplayErrorState()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if HaveErrors; then
		echo "---------------------------------------------------------------------------"
		echo "                             Error Summary"
		echo "---------------------------------------------------------------------------"
		echo "${ALL_INSTALLER_ERRORS}" | sed 's/\\n/\n/g'
	fi >&2
	if HaveWarnings; then
		echo "---------------------------------------------------------------------------"
		echo "                            Warning Summary"
		echo "---------------------------------------------------------------------------"
		echo "${ALL_INSTALLER_WARNINGS}" | sed 's/\\n/\n/g'
	fi >&2
	if HaveFixmes; then
		echo "---------------------------------------------------------------------------"
		echo "                             FIXME Summary"
		echo "---------------------------------------------------------------------------"
		echo "${ALL_INSTALLER_FIXMES}" | sed 's/\\n/\n/g'
	fi >&2
	if HaveErrors || HaveWarnings || HaveFixmes; then
		echo "---------------------------------------------------------------------------"
	fi >&2
}

###########################################################################
# End output functions...
###########################################################################

###########################################################################
# Install related functions
###########################################################################
AbortInstallation()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ $# -gt 0 ]; then
		ERROR "(aborting)" "$@"
	fi
	INSTALLATION_ABORTED=1
	exit 1
}

AskUnsupportedQuestion()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	[ $FORCE_OPT = 1 ] && return 0
	[ $QUIET_OPT = 1 ] && AbortInstallation "Unsupported platform: Refusing to install quietly without being forced (-f)."

	PrintUnsupportedMessage
	while true; do
		OutputNormal ""
		OutputNormal "Do you want to $INSTALL_TYPE anyway?"
		read reply leftover
		case $reply in
			y* | Y*)
				return 0;;
			n* | N*)
				return 1;;
		esac
	done
}

CheckForSupportedPlatform()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3WS" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3ES" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3AS" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3DESKTOP" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4WS" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4ES" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4AS" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4DESKTOP" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3WS" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3ES" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3AS" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3DESKTOP" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4WS" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4ES" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4AS" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4DESKTOP" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "5SERVER" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "5CLIENT" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "5SERVER" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "5CLIENT" ] && return 0;

	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OS_MAJOR = "5" ] && [ ${OS_MINOR:-0} = "0" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OS_MAJOR = "5" ] && [ ${OS_MINOR:-0} = "0" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "2.1AS" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "2.1ES" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "2.1WS" ] && return 0;

	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "CentOS" ] && [ $OSVER = "4" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "CentOS" ] && [ $OSVER = "4" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "3" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "3" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "3" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "3" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "4" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "4" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "4" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "4" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "5" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "5" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "Fedora" ] && [ $OS_MAJOR = "10" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "Fedora" ] && [ $OS_MAJOR = "10" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "Fedora" ] && [ $OS_MAJOR = "11" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "Fedora" ] && [ $OS_MAJOR = "11" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "5" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "5" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "CentOS" ] && [ $OSVER = "5" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "CentOS" ] && [ $OSVER = "5" ] && return 0;

	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SuSE" ] && [ $OS_MAJOR = "8" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SuSE" ] && [ $OS_MAJOR = "9" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SuSE" ] && [ $OS_MAJOR = "10" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SuSE" ] && [ $OSVER = "11" ] && return 0;

	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SuSE" ] && [ $OS_MAJOR = "9" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SuSE" ] && [ $OS_MAJOR = "10" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SuSE" ] && [ $OSVER = "11" ] && return 0;

	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "VMWARE" ] && [ $OS_MAJOR = "2" ] && [ $OS_MINOR = "5" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "VMWARE" ] && [ $OSVER = "3" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "VMWARE" ] && [ $OS_MAJOR = "3" ] && [ $OS_MINOR = "0" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "VMWARE" ] && [ $OS_MAJOR = "3" ] && [ $OS_MINOR = "5" ] && return 0;
	[ "$OSVEND" = "IBM" ] && [ "$OSTYPE" = "AIX" ] && [ $OSVER = "5.1" ] && return 0;
	[ "$OSVEND" = "IBM" ] && [ "$OSTYPE" = "AIX" ] && [ $OSVER = "5.2" ] && return 0;
	[ "$OSVEND" = "IBM" ] && [ "$OSTYPE" = "AIX" ] && [ $OSVER = "5.3" ] && return 0;
	[ "$OSVEND" = "IBM" ] && [ "$OSTYPE" = "AIX" ] && [ $OSVER = "6.1" ] && return 0;
 	[ "$OSARCH" = "parisc" ] && [ "$OSVEND" = "HP" ] && [ "$OSTYPE" = "HP-UX" ] && [ $OSVER = "B.11.11" ] && return 0;
 	[ "$OSARCH" = "parisc" ] && [ "$OSVEND" = "HP" ] && [ "$OSTYPE" = "HP-UX" ] && [ $OSVER = "B.11.23" ] && return 0;
 	[ "$OSARCH" = "parisc" ] && [ "$OSVEND" = "HP" ] && [ "$OSTYPE" = "HP-UX" ] && [ $OSVER = "B.11.31" ] && return 0;
 	[ "$OSARCH" = "ia64" ] && [ "$OSVEND" = "HP" ] && [ "$OSTYPE" = "HP-UX" ] && [ $OSVER = "B.11.23" ] && return 0;
 	[ "$OSARCH" = "ia64" ] && [ "$OSVEND" = "HP" ] && [ "$OSTYPE" = "HP-UX" ] && [ $OSVER = "B.11.31" ] && return 0;
	[ "$OSVEND" = "Apple" ] && [ "$OSTYPE" = "DARWIN" ] && [ "$OSVARIANT" = "MacOS" ] && [ $OS_MAJOR = "10" ] && [ $OS_MINOR = "4" ] && return 0;
	[ "$OSVEND" = "Apple" ] && [ "$OSTYPE" = "DARWIN" ] && [ "$OSVARIANT" = "MacOS" ] && [ $OS_MAJOR = "10" ] && [ $OS_MINOR = "5" ] && return 0;
	[ "$OSVEND" = "Apple" ] && [ "$OSTYPE" = "DARWIN" ] && [ "$OSVARIANT" = "MacOS" ] && [ $OS_MAJOR = "10" ] && [ $OS_MINOR = "6" ] && return 0;

	# These are the kernel versions, not the OS version.
	[ "$OSVEND" = "Apple" ] && [ "$OSTYPE" = "DARWIN" ] && [ $OS_MAJOR = "7" ] && return 0;
	[ "$OSVEND" = "Sun" ] && [ "$OSTYPE" = "SUNOS" ] && [ $OSVER = "5.8" ] && [ $OSARCH = sparc ] && return 0;
	[ "$OSVEND" = "Sun" ] && [ "$OSTYPE" = "SUNOS" ] && [ $OSVER = "5.9" ] && return 0;
	[ "$OSVEND" = "Sun" ] && [ "$OSTYPE" = "SUNOS" ] && [ $OSVER = "5.10" ] && return 0;

	# unsupported platform
	if [ $QUIET_OPT = 1 ] && [ $FORCE_OPT = 0 ]; then
		return 1;
	elif [ $QUIET_OPT = 1 ] && [ $FORCE_OPT = 1 ]; then
		return 0;
	elif [ $QUIET_OPT = 0 ] && [ $FORCE_OPT = 1 ]; then
		PrintUnsupportedMessage
		return 0;
	fi

	return 1;
}

# Variables that will be set:
#
# OSTYPE = output of uname (converted to uppercase): LINUX
# OSVEND = vendor (RedHat, UnitedLinux, SuSE)
# OSVER = version of the Linux distro.
# OSARCH = arch identifier (eg. x86, powerpc, etc.)
# The values will be UNKNOWN if the script is unable to determine a proper value.
GetPlatformInformation()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	# Some shells require that a variable be set before unsetting.
	OSTYPE=""
	OSVEND=""
	OSVER=""
	OSARCH="unknown"
	OSVARIANT=""
	unset OSTYPE OSVEND OSVER

	OSTYPE=`uname | tr '[a-z]' '[A-Z]'`

	if uname -p >&7 2>&1; then
		# This seems to be supported on everything except HP-UX.
		OSARCH=`uname -p`
		# Some new versions of linux are sticking the
		# *complete* processor info in the -p output, like
		# this:
		# Intel(R) Pentium(R) M processor 1600MHz
		for processor in intel amd processor; do
			if echo "$OSARCH" | grep -i $processor >&7; then
				OSARCH=unknown
				break
			fi
		done
	fi

	if [ "x$OSARCH" = "xunknown" ]; then
		OSARCH=`uname -m`
	fi

	# If Red Hat...
	if [ -f /etc/redhat-release ]; then
		# Prevent false-positives from some Red Hat derivatives...
		if grep "Red Hat" /etc/redhat-release >&7; then
			OSVEND=RedHat
			if rpm -q redhat-release >&7 2>&1; then
				OSVER=`rpm -q --qf '%{VERSION}' redhat-release`
			elif rpm -q redhat-release-as >&7 2>&1; then
				# RedHat AS 2.1
				OSVER=`rpm -q --qf '%{VERSION}' redhat-release-as`
			elif rpm -q redhat-release-ws >&7 2>&1; then
				# RedHat WS 2.1
				OSVER=`rpm -q --qf '%{VERSION}' redhat-release-ws`
			elif rpm -q redhat-release-es >&7 2>&1; then
				# RedHat ES 2.1
				OSVER=`rpm -q --qf '%{VERSION}' redhat-release-es`
			elif rpm -q vmware-release >&7 2>&1; then
				OSVEND=VMWARE
				OSVER=`rpm -q --qf '%{VERSION}' vmware-release`

				# If the VMware-esx package is installed, we can extract more
				# detailed version information from it.
				if rpm -q --queryformat '%{VERSION}' VMware-esx >&7 2>&1; then
					OSVER=`rpm -q --qf '%{VERSION}' VMware-esx`
					OSVARIANT=ESX
				fi
			else
				# Add other RedHat versions here when needed.
				OSVER=UNKNOWN
			fi
		elif [ -f /etc/fedora-release ]; then
			if grep "Fedora" /etc/fedora-release >&7; then
			   OSVEND=Fedora
                        else
			   if grep "STS" /etc/fedora-release >&7; then
			     OSVEND=Fedora
                           fi
		        fi	
		        if rpm -q fedora-release >&7 2>&1; then
				OSVER=`rpm -q --qf '%{VERSION}' fedora-release`
                        else
                                OSVER=UNKNOWN
                        fi
		elif grep "CentOS" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=CentOS
			if rpm -q centos-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' centos-release`
			fi
		elif grep -i "Fermi" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=SLF
			if rpm -q Fermi-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' Fermi-release`
			fi
		elif grep -i "SLF" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=SLF
			if rpm -q Fermi-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' Fermi-release`
			fi
		elif grep -i "LTS" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=SLF
			if rpm -q Fermi-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' Fermi-release`
			fi
		elif grep -i "Scientific" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=SL
			if rpm -q sl-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' sl-release`
			fi
		else
			# A RedHat clone...
			OS_VENDOR_TEXT=`cat /etc/redhat-release  | head -1 | awk '{ print $1 }' | tr '[A-Z]' '[a-z]'`
			if rpm -q ${OS_VENDOR_TEXT}-release >&7 2>&1; then
				OSVEND=`rpm -q --qf '%{VENDOR}' ${OS_VENDOR_TEXT}-release | sed 's/ /_/g'`
				OSVER=`rpm -q --qf '%{VERSION}' ${OS_VENDOR_TEXT}-release`

				# Handle items like Fedora.
				if echo "${OSVEND}" | egrep 'Red(()|[ _])Hat' >&7 2>&1; then
					OSVEND="RedHat"
					OSVER="${OS_VENDOR_TEXT}${OSVER}"
				fi
			else
				OSVEND=UNKNOWN
				OSVER=UNKNOWN
			fi
		fi
	# If SuSE...
	elif [ -f /etc/SuSE-release ]; then
		OSVEND=SuSE
		OSVER=`grep VERSION /etc/SuSE-release | cut -d' ' -f3`
	elif [ -f /etc/gentoo-release ]; then
		OSVEND=Gentoo
		OSVER=`grep -i VERSION /etc/gentoo-release | sed -e 's/[^0-9.]//g'`
	# If UnitedLinux...
	elif [ -f /etc/UnitedLinux-release ]; then
		OSVEND=UnitedLinux
		OSVER=`grep VERSION /etc/UnitedLinux-release | cut -d' ' -f3`
	# VMware ESX (at least 2.5) does not provide a release RPM nor does it have
	# a release file in /etc.
	elif [ "x${OSTYPE}" = "xLINUX" ] && rpm -q --queryformat '%{VERSION}' VMware-esx >&7 2>&1; then
		# Some version of VMware ESX.
		OSVEND=VMWARE
		OSVER=`rpm -q --queryformat '%{VERSION}' VMware-esx`
		OSVARIANT=ESX
	elif [ -f /usr/bin/lsb_release ]; then
		OSVEND=`/usr/bin/lsb_release -i | sed 's/^[^:]*:[ 	]*//g'`
		OSVER=`/usr/bin/lsb_release -r | sed 's/^[^:]*:[ 	]*//g'`
	# AIX
	elif [ "x$OSTYPE" = "xAIX" ]; then
		OSVEND=IBM
		OSVER=`uname -v`.`uname -r`
	# HPUX
	elif [ "x$OSTYPE" = "xHP-UX" ]; then
		OSVEND=HP
		OSVER=`uname -r`
	# Darwin (OS-X)
	elif [ "x$OSTYPE" = "xDARWIN" ]; then
		OSVEND=Apple
		if sw_vers -productVersion >&7 2>&1; then
			OSVER=`sw_vers -productVersion 2>&7`
			OSVARIANT="MacOS"
		else
			OSVER=`uname -r`
		fi
	# Solaris
	elif [ "x$OSTYPE" = "xSUNOS" ]; then
		OSVEND=Sun
		OSVER=`uname -r`
	# If we've gone this far, we're not on a supported distro...
	else
		OSVEND=UNKNOWN
		OSVER=UNKNOWN
	fi
	# Make it uppercase.
	OSVER=`echo $OSVER | tr '[a-z]' '[A-Z]'`

	# Convert all of those pesky arch numbers to something that can be used more easily.
	case "$OSARCH" in
		i86pc | i386 | i486 | i586 | i686 | *[Aa]thlon*)
			OSARCH="x86"
			;;
		9000/*)
			OSARCH="parisc"
			;;
		ppc | [Pp]owerpc)
			OSARCH="ppc"
			;;
		*)
			;;
	esac

	# Get the MAJOR, MINOR, and MICRO version segments.
	OS_MAJOR=`echo $OSVER | awk -F. '{ print $1; }'`
	OS_MINOR=`echo $OSVER | awk -F. '{ print $2; }'`
	OS_MICRO=`echo $OSVER | awk -F. '{ print $3; }'`
}


CleanupPackageDir()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ "x${REMOVE_SOURCE_DIR_ON_EXIT:-0}" = "x1" ] ; then
		OutputNormal "Cleaning up temporary package directory: $BASE_INSTALLER_DIR"
		rm -rf "$BASE_INSTALLER_DIR"
		OutputNormal "Done."
	fi
}

CleanupTempDir()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ x${QMX_TEMP_DIRECTORY:+set} = xset ]; then
		OutputVerbosely "Cleaning up temp directory: ${QMX_TEMP_DIRECTORY}"
		rm -rf ${QMX_TEMP_DIRECTORY}
		unset QMX_TEMP_DIRECTORY
		OutputVerbosely "Done."
	fi
}

CreateTempDir()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ x${QMX_TEMP_DIRECTORY:+set} = xset ]; then
		if [ -d "${QMX_TEMP_DIRECTORY}" ]; then
			OutputVerbosely "Using temp directory: ${QMX_TEMP_DIRECTORY}"
			return 0
		else
			OutputVerbosely "Temp directory disappeared (will create anew): ${QMX_TEMP_DIRECTORY}"
			unset QMX_TEMP_DIRECTORY
		fi
	fi
	if [ x${QMX_TEMP_DIRECTORY:+set} != xset ]; then
		create_temp_dir_failure=0
		# On some shells, if this assignment is done within the if statement, it
		# doesn't actually set the global variable.
		QMX_TEMP_DIRECTORY="`MakeTemp -d qmxcm_temp_space`" || create_temp_dir_failure=$?
		if [ ${create_temp_dir_failure} = 0 ]; then
			if StartsWith "${QMX_TEMP_DIRECTORY}" "/"; then
				:
			else
				ERROR "Temp directory is not an absolute path!"
				return 1
			fi
		else
			create_temp_dir_failure=$?
			ERROR "Failed to create ${PRODUCT_NAME} temp directory."
			return ${create_temp_dir_failure}
		fi
		OutputVerbosely "Created temp directory ${QMX_TEMP_DIRECTORY}"
	fi
}

# This will set the variable MENU_CHOICE for the choice the user selected.
# $1 = pattern
# $2 (hidden) = Flag to control if the "more" option is available.
SelectDirectoryFromMenu()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	MENU_CHOICE=
	MENU_SELECTION_MADE=0
	MENU_FILE_PATTERN="$1"
	# Make sure that nobody else can cause problems while displaying the
	# menu (writing to temp files and such).
	TEMP_MASK=`umask`
	umask 077

	TEMP_FILE=/tmp/umi_install.choices

	SafeCreateFile "$TEMP_FILE"

	if ListDirectories $MENU_FILE_PATTERN > $TEMP_FILE; then

		SafeCreateFile "${TEMP_FILE}.2"

		# Skip a bunch of directories that can't contain valid install files.
		for skip_directory in ${NON_INSTALL_DIRECTORIES}; do
			grep -v "/${skip_directory}\$" "${TEMP_FILE}" > "${TEMP_FILE}.2" || true
			mv "${TEMP_FILE}.2" "${TEMP_FILE}"
		done
		rm -f "${TEMP_FILE}.2" 2>&7

		num_choices=`cat $TEMP_FILE | wc -l`
		num_choices=`expr $num_choices`

		# Display a menu and read the user's data.
		display_menu()
		{
			[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
			menu_choice_number=0
			echo "----------------------------------------------------------------"
			if [ "$UNSUPPORTED" != 1 ]; then
				echo "No package found for supported OS $OSVEND $OSTYPE $OSVER"
			else
				echo "Your platform $OSVEND $OSTYPE $OSVER is unsupported by ${PRODUCT_NAME}."
			fi
			echo "You may try packages from the following platforms:"
			echo
			index=1
			while [ $index -le $num_choices ]; do
				package=`cat $TEMP_FILE | head -n $index | tail -1`
				base_package=`basename $package`
				echo "	($index) $base_package"
				index=`expr $index + 1`
			done
			echo
			# An evil hack to allow the user to specify they want more choices.
			if [ ${1:-MORE} = MORE ]; then
				echo "	(-1) Show more choices"
			fi
			echo "	(0) Quit without installing"
			echo "----------------------------------------------------------------"
			echo "Please select an item from 0 to $num_choices:"

			read reply leftover
			if echo $reply | egrep '^[ \t]*-?[0-9]+[ \t]*$' >&7 2>&1; then
				reply=`echo $reply`

				# An evil hack to allow the user to specify they want more choices.
				if [ x${reply} = x-1 ]; then
					if [ ${1:-MORE} = MORE ]; then
						reply=255
					else
						reply=127
					fi
				fi
				menu_choice_number=$reply
			else
				menu_choice_number=127
			fi
			return $menu_choice_number
		}

		if [ $QUIET_OPT = 1 ]; then
			# Take the first thing on the list.
			choice=1
		else
			# Repeat the menu until a valid choice is made.
			choice=0
			display_menu ${2:-MORE} || choice=$?
			while [ $choice -eq 127 ] || [ $choice -gt $num_choices ]; do

				# An evil hack to allow the user to specify they want more choices.
				if [ ${2:-MORE} = MORE ]; then
					if [ ${menu_choice_number} = 255 ]; then
						SelectDirectoryFromMenu "`dirname \"$1\"`/*" NO-MORE || return $?
						return 0
					fi
				fi

				echo
				echo "Invalid selection: $reply"
				echo
				choice=0
				display_menu ${2:-MORE} || choice=$?
			done
		fi

		if [ $choice -gt 0 ]; then
			MENU_CHOICE=`cat $TEMP_FILE | head -n $choice | tail -1 | sed 's| |\\ |g'`
			MENU_SELECTION_MADE=1
		else
			FATAL_ERROR="User quit requested"
			umask $TEMP_MASK
			rm -f $TEMP_FILE
			return 1
		fi
	fi
	umask $TEMP_MASK
	rm -f $TEMP_FILE
}

LocateBaseInstallerDir()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	EXECUTED_SCRIPT_NAME=`basename "$0"`
	BASE_INSTALLER_DIR=`dirname "$0"`
	if echo "${BASE_INSTALLER_DIR}" | grep '^/' >&7 2>&1; then
		:
	else
		if [ "x${BASE_INSTALLER_DIR}" != "x." ]; then
				BASE_INSTALLER_DIR=`pwd`/"${BASE_INSTALLER_DIR}"
		else
				BASE_INSTALLER_DIR=`pwd`
		fi
	fi

	if [ -f "${BASE_INSTALLER_DIR}/${EXECUTED_SCRIPT_NAME}" ] ; then
		:
	else
		temp_path=`which "${EXECUTED_SCRIPT_NAME}"`
		BASE_INSTALLER_DIR=`dirname "${temp_path}"`
		unset temp_path
	fi

	BASE_INSTALLER_DIR=`echo "${BASE_INSTALLER_DIR}" | sed 's| |\\ |g'`
}

GetPackageDirectory()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if [ "x${CACHED_PACKAGE_DIRECTORY:+set}" = "xset" ]; then
		echo "${CACHED_PACKAGE_DIRECTORY}"
		return 0
	fi

	local_package_dir=`GetPlatformPackageDirectory`

	if [ "${local_package_dir:-UNKNOWN}" != "UNKNOWN" ]; then
		CACHED_PACKAGE_DIRECTORY="${BASE_INSTALLER_DIR}/${local_package_dir}"
		unset local_package_dir
		return 0
	fi
	unset local_package_dir
	OutputError "Unable to determine package directory for ${PRODUCT_NAME} installation"
	return 1
}

ObtainPackageDirectory()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	[ "$UNSUPPORTED" = "1" ] && AskUnsupportedQuestion

	# Get the package directory and set the CACHED_PACKAGE_DIRECTORY variable.
	GetPackageDirectory >&7 || true
	if [ -d "$CACHED_PACKAGE_DIRECTORY" ]; then
		:
	else
		os_prefix_text=`GetPlatformPrefix`
		SelectDirectoryFromMenu "${BASE_INSTALLER_DIR}/${os_prefix_text}-*${OSARCH}"
		if [ $MENU_SELECTION_MADE -eq 1 ]; then
			CACHED_PACKAGE_DIRECTORY="${MENU_CHOICE}"
		else
			ERROR "Failed to find directory for packages."
			return 1
		fi
	fi
}

GetPlatformPackageSuffix()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	local_package_suffix=
	case ${OSTYPE} in
		LINUX)
			local_package_suffix=rpm
			;;
		HP-UX)
			local_package_suffix=depot
			;;
		SUNOS)
			local_package_suffix=pkg
			;;
		DARWIN)
			local_package_suffix=dmg
			;;
		AIX)
			local_package_suffix=bff
			;;
		*)
			AbortInstallation "Unknown package suffix for platform ${OSTYPE}"
			;;
	esac
	echo "${local_package_suffix}"
	unset local_package_suffix
}

GetPlatformPrefix()
{
	if [ "${OSTYPE}" = "DARWIN" ]; then
		echo "osx"
	else
		echo ${OSTYPE} | tr '[A-Z]' '[a-z]'
	fi
}

GetPlatformPackageDirectory()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	platform_package_dir=UNKNOWN
	# For supported platforms, we'll determine the package directory.  For
	# unsupported, we'll just report UNKNOWN so that any calling functions will
	# be forced to further prompt the user (eg. bring up a list of possible
	# directories).
	if [ ${UNSUPPORTED:-0} = 0 ]; then
		if [ "${OSTYPE}" = "LINUX" ]; then
			case ${OSVEND}-${OSVER} in
				# Old, obsolete platforms.
				RedHat-2.1[EAW]S | RedHat-7.3 | RedHat-8 | RedHat-8.* | SuSE-8 | SuSE-8.* | VMWARE-2.5*)
					platform_package_dir=linux-rh2.1-x86
					;;
				# Anything else.
				*)
					platform_package_dir=linux-rh3-${OSARCH}
					;;
			esac
		elif [ "${OSTYPE}" = "AIX" ]; then
			# If we ever support itanium, this will need modification.
			platform_package_dir="aix-power"
		elif [ "${OSTYPE}" = "HP-UX" ]; then
			# If we ever support itanium, this will need modification.
			platform_package_dir="hpux-${OSARCH}"
		elif [ "${OSTYPE}" = "DARWIN" ]; then
			platform_package_dir="osx-${OSARCH}"
		elif [ "${OSTYPE}" = "SUNOS" ]; then
			platform_package_dir="sunos-${OSARCH}"
		fi
	fi
	echo "${platform_package_dir}"
	unset platform_package_dir
}

###########################################################################
# End install related functions
###########################################################################


###########################################################################
# Config file functions
###########################################################################
# This function is fundamentally backwards.  It returns if there is an error in
# the site code, set by the "SITE_CODE_OPT" variable.
SiteCodeHasError()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	case "$SITE_CODE_OPT" in
		???)
			TEMP_SITE_CODE=`echo $SITE_CODE_OPT | tr '[a-z]' '[A-Z]'`
			if [ "$TEMP_SITE_CODE" = "$SITE_CODE_OPT" ]; then
				# This appears to be a perfectly valid site code.
				return 1
			fi
			[ $QUIET_OPT = 1 ] || echo "SMS Site code must be uppercase."
		;;
		*)
			[ $QUIET_OPT = 1 ] || echo "SMS Site code must be 3 characters long"
	esac
	return 0
}

MPEAddressHasError()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if echo "$MPE_OPT" | grep "NOT SET" >&7; then
		return 0
	else
		return 1
	fi
}
###########################################################################
# End config file functions
###########################################################################

###########################################################################
# SMS related functions
###########################################################################
RunPolicyUpdate()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	[ $QUIET_OPT = 1 ] || echo "Updating policy..."
	return_value=0
	output_file=`MakeTemp /tmp/qmx_policy_update`
	if ${QMX_CLIENTTOOL} --run-policy-update >$output_file 2>&1 ; then
		# Success!
		[ $QUIET_OPT = 1 ] || echo "Policy update was successful."
	else
		return_value=1
		# This will display an error, and give some useful details about why.
		# Resolution failure
		if grep "UnknownHostException" $output_file >&7 2>&1; then
			echo "The management point was not accessible at \"$MPE_OPT\"."
			echo "This is due to hostname resolution failure.  Please correct your DNS."
		# Incorrect IP
		elif grep "Connection refused" $output_file >&7 2>&1; then
			echo "Unable to reach the management point."
			echo "This could be caused by IIS failure or a firewalled management point."
		# Machine not responding (unplugged or otherwise)
		elif grep "No route to host" $output_file >&7 2>&1; then
			echo "Unable to reach the management point."
			echo "The management point may be down or unplugged. A firewall could also cause this."
		# No compression fix installed
		elif grep "MP returned compressed data" $output_file >&7 2>&1; then
			echo "Cannot communicate with MP.  Compression is enabled."
			# This is the current MS KB URL, but the location will likely change:
			# http://support.microsoft.com/default.aspx?scid=KB;EN-US;894604
			# Just direct them to the MS website.
			echo "See the Microsoft Knowledge Base article number 894604 for details on disabling"
			echo "compression. This can be located by searching for KB894604 at www.microsoft.com"
		# No CIMOM.
		elif grep "No such file or directory" $output_file >&7 2>&1; then
			if ${UMI_INIT_SCRIPT} status ; then
				# Check to see if we are doing this recursively, and retry if we aren't.
				if [ "x${RECURSIVE_POLICY_EXECUTION:-0}" != "x1" ]; then
					echo "Waiting 30 seconds for the cimom to respond..."
					sleep 30
					RECURSIVE_POLICY_EXECUTION=1
					if RunPolicyUpdate; then
						return_value=0
					else
						return_value=$?
					fi
				else
					echo "Unable to contact the CIMOM.  A high system load or CIMOM configuration error"
					echo "may have caused this problem.  The daemon is currently running, but may need to"
					echo "be restarted using: ${QMX_INIT_SCRIPT} restart"
				fi
			else
				echo "The cimom is not running, or died after startup."
				echo "Check your /etc${UMI_PREFIX}/openwbem/openwbem.conf and restart using:"
				echo "${QMX_INIT_SCRIPT} start"
			fi
		# Improper configuration
		elif grep "Unable to get config item" $output_file; then
			echo "The configuration file ${QMX_CONFIG_FILE} is in need of repair."
		# Something we didn't think of...
		else
			echo "Unexpected policy update failure:"
			cat $output_file
		fi >&2 # Send all output to stderr.
	fi
	rm -f $output_file
	return $return_value
}

RunDiscovery()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	[ $QUIET_OPT = 1 ] || echo "Running heartbeat discovery..."
	return_value=0
	if ${QMX_CLIENTTOOL} --run-discovery ; then
		# Success!
		[ $QUIET_OPT = 1 ] || echo "Heartbeat discovery was successful."
	else
		return_value=1
	fi
	return $return_value
}

###########################################################################
# End SMS related functions
###########################################################################

###########################################################################
# Begin file functions
###########################################################################
# This is used as an ls replacement, since ls on OSX 10.3 doesn't return an
# error code.  It relies on the fact that the shell will expand any arguments
# before passing them to this function.  As such, we can use find on them, and
# find will only report an error if they don't exist.  This includes the
# failure of the shell to expand wildcards, so find /foo/barf* wouldn't be
# expanded, the find would get it intact, and return an error code.  When
# successful, find prints out the files
ListFiles()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if find "$@" -print; then
		return 0
	else
		return 1
	fi
}

ListDirectories()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	lf_return_code=1
	while [ $# -gt 0 ]; do
		if test -d "$1" ; then
			echo "$1"
			lf_return_code=0
		fi
		shift
	done
	return $lf_return_code
}

# Since no local variables are possible, this helper is needed
# $1 = count of non-removed directories (pass zero as the first arg).  This
# argument is used for local storage.
# $2..$n = directories to remove.
RemoveIfEmptyHelper()
{
	# This function purposely avoids using variables as much as possible
	# (without being horribly inefficient) because it is recursive and only
	# numbered parameters can be considered local.
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	while [ $# -gt 1 ]; do
		base="`basename \"$2\"`"
		if [ "x${base}" != "x." -a "x${base}" != "x.." ]; then

			# More placeholders can be added here if they are needed.
			#
			# Technically, the *.rpmsave files aren't placeholders, but they have
			# been left around in cases where someone removed an rpm after
			# modifying a config or other file.
			for placeholder in '\.keep' '.*\.rpmsave' '.*\.bak' '#.*' '.*~' '.profile'; do
				if expr "//${base}" : "//${placeholder}\$" >&7; then
					OutputDebug "Removing placeholder file: $2"
					rm -f "$2"
				fi
			done

			# The value of the variable "base" cannot be used any longer.
			if [ -f "$2" -o -h "$2" ]; then
				OutputDebug "Non-directory not removed: $2"
				return 1
			elif [ -d "$2" ]; then
				OutputDebug "\"$2\" looks like a directory."
				if RemoveIfEmptyHelper 0 "$2"/* "$2"/.*; then
					OutputDebug "Empty directory removed: $2"
					rmdir "$2"
				else
					OutputDebug "Directory not removed (not empty): $2"

					count=`expr $1 + 1` || true
					shift
					set -- ${count} "$@"
				fi
			else
				# Something that doesn't exist. It could be a failed wildcard
				# expansion, or we may have removed it (as a placeholder).  We are
				# safe to ignore it either way.
				:
			fi
		fi
		count=$1
		shift
		if [ $# -gt 1 ]; then
			shift
			set -- ${count} "$@"
		else
			set -- ${count}
		fi
	done
	return ${1:-0}
}

# Remove a directory if it is empty or only contains empty subdirectories.
# This must terminate at the first non-directory found because of the lack of
# local state in shell functions.
#
# $1..$n = directory to check (not suggested to supply more than one)
RemoveIfEmpty()
{
	RemoveIfEmptyHelper 0 "$@" || return $?
}

# $1 .. $n = Directories to recursively check and remove (if empty).
RemoveEmptyDirectories()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	while [ $# -gt 0 ]; do
		RemoveIfEmpty "$1" || true
		shift
	done
}

# Remove an empty directory tree or the symlink, should it exist.
RemoveTreeOrLink()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ -h "$1" ]; then
		# Delete the old link.
		rm -f "$1"
	elif [ -d "$1" ]; then
		# Attempt to remove the directory tree.
		RemoveIfEmpty "$1"
	elif [ -f "$1" ]; then
		# Error.  Not a directory or link.
		return 1
	else
		# Not there (or maybe a special file).  Ignore this.
		return 0
	fi
}
###########################################################################
# End file functions
###########################################################################


###########################################################################
# Packaging related functions.
###########################################################################
# Everything in here should be kept in sync with the same constructs in
# the install_umi.sh file.

# This function will query an installed package for the version number.
#
# This function can be used to determine if the package database is hosed.
# This should hopefully allow errors to be caught before the cimom is shut
# down.  Using it this way is mainly targeted at Redhat8 (which the RPM
# database gets locked quite frequently), everything else just does a simple
# query (the results don't matter, so long as it returns).
QueryExistingPackage()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	package_to_query=${QMX_PACKAGE_NAME}
	query_result=0
	if [ $# -gt 0 ]; then
		package_to_query="$1"
	fi

	case $OSTYPE in
		"LINUX")
			# note that rpmquery doesn't exist on some older distros (e.g. SUSE 8)
			rpm -q --queryformat '%{VERSION}' $package_to_query || query_result=$?
			;;
		"SUNOS")
			if /usr/bin/pkginfo -i $package_to_query >&7 2>&1; then
				/usr/bin/pkginfo -il $package_to_query | grep -i version | sed 's/^[ 	]*VERSION:[ 	]*//g' || true
			else
				query_result=$?
			fi
			;;
		"DARWIN")
			# 10.6 uses pkgutil db, but upgraded systems will still have files in /LibraryReceipts
			# Just look to see if /Library/Receipts/${package_to_query}.pkg exists.
			if ls -1 /Library/Receipts | grep -i "${package_to_query}\\.pkg" >&7 2>&1; then
				if pkg_version_key=`GetPlistKey /Library/Receipts/${package_to_query}.pkg/Contents/Info.plist CFBundleGetInfoString`; then
					# Delete everything prior to the last space.
					echo "${pkg_version_key}" | sed 's/.*[ ][ ]*//g'
				else
					temp_query_result=$?

					if [ ${IGNORE_MISSING_KEYS:-0} = 1 ]; then

						# There have been cases where the Info.plist files do not
						# contain any useful version information.  While this is
						# probably a bug in the package, we want to treat it as
						# installed.
						echo UNKNOWN
					else
						query_result=${temp_query_result}
					fi
				fi
			else
				# 10.6 exclusively uses the pkgutil db

				# The pkgutil db changes dots in package names to hyphens
				# i.e. quest-openssl-0.9.8 changes to quest-openssl-0-9-8
				munged_package_to_query=`echo ${package_to_query} | sed 's/\./-/g'`
				if pkgutil --pkgs=com.quest.${munged_package_to_query} >&7 2>&1 ; then
					pkg_version=`pkgutil --pkg-info com.quest.${munged_package_to_query} | grep version`
					# Delete everything prior to the last space
					echo "${pkg_version}" | sed 's/.*[ ][ ]*//g'
				else
					query_result=$?
				fi
			fi
			;;
		"HP-UX")
			if /usr/sbin/swlist ${package_to_query} >&7 2>&1; then
				/usr/sbin/swlist  -a revision ${package_to_query} | grep "#[ 	]*${package_to_query}" | sed 's/^.*[ 	]\([0-9].*\)/\1/g'
			else
				query_result=$?
			fi
			;;
		"AIX")
			if lslpp -hcq $package_to_query >&7 2>&1; then
				lslpp -hcq $package_to_query | cut -f3 -d':'
			else
				query_result=$?
			fi
			;;
		*)
			ERROR "Query method unknown on $OSTYPE"
			query_result=1
			;;
	esac

	return $query_result
}

InstallPackage()
{
	have_error=0
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	while [ $# -gt 0 ] && [ ${have_error} = 0 ]; do
		original_package_path="$1"
		package_to_install="${original_package_path}"
		shift

		if PackageIsCompressed "${package_to_install}"; then
			CreateTempDir
			if package_to_install="`DecompressPackage \"${original_package_path}\"`"; then
				:
			else
				ERROR "Failed to decompress package \"${original_package_path}\""
				return 1
			fi
		fi

		OutputVerbosely "Installing package $package_to_install ..."

		case $OSTYPE in
			"LINUX")
				# RPM apparently needs extra backslashes (beyond other installers).
				escaped_package_name="`echo \"${package_to_install}\" | sed 's| |\\\\ |g'`"

				# Explicitly set the umask, as rpm has been shown to create invalid
				# permissions (causing problems with the client's unprivileged
				# users) if installed with a more restrictive umask.
				ip_old_mask=`umask`
				umask 022
				rpm -Uvh "${escaped_package_name}" || have_error=1
				umask ${ip_old_mask}
				unset ip_old_mask
				;;
			"SUNOS")
				# Solaris 10 systems with "zones" require the '-G" switch to pkgadd
				# We support Solaris 10 systems in the global zone ONLY
				NOZONE=
				if [ ${OS_MINOR} -ge 10 ] ; then
					NOZONE="-G"
				fi

				admin_file=/tmp/vmx_pkgadd_admin_file
				SafeCreateFile "$admin_file"
				cat > $admin_file << EOI
mail=
instance=overwrite
partial=nocheck
runlevel=nocheck
idepend=nocheck
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=default
EOI
				/usr/sbin/pkgadd ${NOZONE} -a $admin_file -d "$package_to_install" all || have_error=1
				rm -f $admin_file
				;;
			"DARWIN")
				TARGET_DIRECTORY=/

				# It seems as though a fresh install of OSX doesn't include a
				# /Library/StartupItems directory.  Create one now and save some
				# trouble with failing postinstall scripts.
				if [ ! -d /Library/StartupItems ]; then
					mkdir -p /Library/StartupItems
				fi

				if FindDmgInstaller; then
					${DMG_INSTALLER_SHELL} "${DMG_INSTALL_SCRIPT_PATH}" ${DMG_INSTALLER_OPTS} "${package_to_install}" -t ${TARGET_DIRECTORY} || have_error=1
				else
					OutputError "ERROR: The ${DMG_INSTALLER} script could not be found."
					OutputError "       The installation cannot proceed."
					return 1
				fi
				;;
			"HP-UX")
				# The '-x enforce_dependencies=false' is here becuase there are strange dependency issues
				# on some HP-UX B.11.11 boxes where the "quest-umi-providers.core,l=/,r=1.1.0Alpha.0632" package
				# will not uninstall because it somehow depends on "qmxcm.core,l=/,r=1.5.0Alpha.0441"
				/usr/sbin/swinstall -x enforce_dependencies=false -s "$package_to_install" \* || have_error=1
				;;
			"AIX")
				# The -X option to installp causes the filesystems to be automatically resized.  Should we use it by default?
				/usr/sbin/installp -d "$package_to_install" -ac all || have_error=1
				;;
			*)
				ERROR "Install method unknown on $OSTYPE"
				have_error=1
				;;
		esac
		if [ "${package_to_install}" != "${original_package_path}" ]; then
			rm -r "${package_to_install}" || { have_error=1; ERROR "Failed to remove temporary (decompressed) package \"${package_to_install}\""; }
		fi
	done

	if [ ${have_error} = 1 ]; then
		ERROR "Package installation failed."
	fi
	return ${have_error}
}

InstallPackages()
{
	non_exist_action=$1
	shift
	for package_name in "$@"; do
		if QueryExistingPackage "${package_name}" >&7 2>&1; then
			# We don't handle upgrades in this function.
			ERROR "Already installed: ${package_name}"
		else
			if ListFiles "${CACHED_PACKAGE_DIRECTORY}"/${package_name}* >&7 2>&7; then
				FILE_TO_INSTALL=`ListFiles "${CACHED_PACKAGE_DIRECTORY}"/${package_name}* | tail -1 | sed 's| |\\ |g'`
				if InstallPackage "$FILE_TO_INSTALL"; then
					:
				else
					# Eep.  We're in a bunch of trouble here.  We can't roll back
					# the packages we've already installed, as they might be used by
					# something else.
					ERROR "Failed to install package ${FILE_TO_INSTALL}"
					return 1
				fi
			else
				if [ ${non_exist_action} = report ]; then
					ERROR "Cannot find package files for ${package_name}"
					return 1
				fi
			fi
		fi # package is not already installed
	done
}

DarwinRemoveFiles()
{
	while read filename; do
		filename="`printf '%s\n' \"${filename}\" | sed -e \"s,^${INSTALLER_HACK_TEMPDIR},/,g\" -e 's,//,/,g' `"
		# Protect against removing /var, /opt, /bin, /etc, and such.
		if echo "${filename}" | egrep '^/(var|opt|bin|etc|tmp)$|^(var|opt|bin|etc|tmp)$' >&7 2>&1 ; then
			OutputVerbosely "Skipping protected file/dir: ${filename}"
			continue
		fi
		if [ -f "${filename}" ] || [ -L "${filename}" ]; then
			OutputVerbosely "Removing ${filename}"
			[ $UNINSTALL_MAKE_CHANGES = 0 ] || rm -f "${filename}"
		elif [ -d "${filename}" ]; then
			uninstall_files_in_directory=`ls -1a "${filename}/" | egrep -v '^\.$|^\.\.$' | wc -l`
			if [ $uninstall_files_in_directory -ne 0 ]; then
				OutputNormal "Directory not empty: ${filename}"
			else
				OutputNormal "Removing empty directory: ${filename}"
				[ $UNINSTALL_MAKE_CHANGES = 0 ] || rmdir "${filename}"
			fi
		else
			OutputVerbosely "File was lost before remove: ${filename}"
		fi
	done < "$1"
}

DarwinPackageUninstaller()
{
	# This function needs some background, so I'll put it here:
	# OS-X has no uninstall command, so we'll have to delete anything we install.
	# It does, thankfully, put file lists in /Library/Receipts that have all of
	# the files that were installed by the installer program.
	#
	# Now for some background on the install path hack:
	# Due to the inability to use the OSX installer to install files into some
	# places on the system, the packages we make will install to a temporary
	# directory listed in a file Resources/INSTALLER_HACK_TEMPDIR, and then move
	# themselves to the proper location.  This may seem a little confusing,
	# since it is all hidden away by the 'vbuild' packaging system.  When
	# uninstalling, we need to remove the temp directory from any pathname that
	# contains it.
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	package_to_uninstall="$1"

	# Replace dots with hyphens for OS X 10.6 pkgutil
	munged_package_to_uninstall=`echo ${package_to_uninstall} | sed 's/\./-/g'`

	# This was here for debugging, but we could change it to be a 'pretend' option...
	UNINSTALL_MAKE_CHANGES=1
	INSTALLER_HACK_TEMPDIR=/
	# First... Get the package directory...
	OutputVerbosely "Checking receipts for ${package_to_uninstall}..."
	if ls -1 /Library/Receipts | grep -i "^${package_to_uninstall}\\.pkg" >&7 2>&1; then
		full_package_name=`ls -1 /Library/Receipts | grep -i "^${package_to_uninstall}\\.pkg"`
		uninstall_receipt_dir="/Library/Receipts/$full_package_name"
		OutputVerbosely "Using receipts from ${uninstall_receipt_dir}"
	else
		if pkgutil --pkgs=com.quest.${munged_package_to_uninstall} >&7 2>&1 ; then
			full_package_name=`pkgutil --pkgs=com.quest.${munged_package_to_uninstall}`
			uninstall_receipt_dir="NONE"
			OutputVerbosely "Using receipt database"
		else
			ERROR "Not installed: $package_to_uninstall"
			return 1
		fi
	fi
	# Second... Check to see if a preremove/postremove script was packaged...
	OutputVerbosely "Checking for {pre,post}remove scripts..."
	for postaction in preremove postremove; do
		if ListFiles "$uninstall_receipt_dir/Contents/Resources/${postaction}" >&7 2>&1; then
			eval "uninstall_have_${postaction}"=1
			OutputVerbosely "Located ${postaction} script..."
		else
			eval "uninstall_have_${postaction}"=0
		fi
	done
	if ListFiles "$uninstall_receipt_dir/Contents/Resources/INSTALLER_HACK_TEMPDIR" >&7 2>&1; then
		INSTALLER_HACK_TEMPDIR="`cat $uninstall_receipt_dir/Contents/Resources/INSTALLER_HACK_TEMPDIR`"
	fi

	# Call the preremove script (if it exists).
	if [ "x$uninstall_have_preremove" = "x1" ]; then
		OutputVerbosely "Executing preremove..."
		if $uninstall_receipt_dir/Contents/Resources/preremove ; then
			OutputVerbosely "Preremove script successful."
		else
			ERROR "Preremove script failed."
			return 1
		fi
	fi
	# Copy the postremove script (if it exists), so it won't dissapear...
	if [ "x$uninstall_have_postremove" = "x1" ]; then
		OutputVerbosely "Making copy of postremove..."
		uninstall_postremove_script="/tmp/postremove-${package_to_uninstall}"
		if SafeCreateFile $uninstall_postremove_script ; then
			cat $uninstall_receipt_dir/Contents/Resources/postremove > $uninstall_postremove_script
			chmod u+rwx $uninstall_postremove_script
		else
			return 1
		fi
	fi
	# Third, remove any files belonging to the package (from the bom file)
	# This jumps through some hoops to make sure any files with spaces
	# are properly removed.
	OutputVerbosely "Getting package file list..."
	if ListFiles "$uninstall_receipt_dir/Contents/Archive.bom" >&7 2>&1 ; then
		if uninstall_file_list=`MakeTemp /tmp/uninstall-$package_to_uninstall` ; then
			# Remove the files and symlinks first.
			# All of the packages seem to use ./ for the beginning.
			# Remove the . so only absolute paths are used.
			lsbom -s -f -d -l $uninstall_receipt_dir/Contents/Archive.bom | grep -v '^\.$' | sed 's|^./|/|g' | sort -r > $uninstall_file_list
			DarwinRemoveFiles $uninstall_file_list
			rm $uninstall_file_list
		fi
	else # Try getting the file list from the pkgutil db.
		if pkgutil --files com.quest.${munged_package_to_uninstall} >&7 2>&1 ; then
			if uninstall_file_list=`MakeTemp /tmp/uninstall-${package_to_uninstall}` ; then
				pkgutil --files com.quest.${munged_package_to_uninstall} | sort -r  > ${uninstall_file_list}
				DarwinRemoveFiles ${uninstall_file_list}
				rm ${uninstall_file_list}
			fi
		else  # No Archive.bom
			OutputVerbosely "Not removing files: Archive.bom not found."
		fi
	fi

	# Remove the receipts...
	OutputVerbosely "Removing receipts for ${package_to_uninstall}..."
	if [ $UNINSTALL_MAKE_CHANGES = 1 ] ; then
		if [ "x${uninstall_receipt_dir}" = "xNONE" ] ; then
			 pkgutil --forget com.quest.${munged_package_to_uninstall} >&7 2>&1 || true
		else
			uninstall_remove_uninstall_remove_verbose_flag=""
			[ $QUIET_OPT = 1 ] || uninstall_remove_verbose_flag="-v"
			rm -rf $uninstall_remove_verbose_flag "$uninstall_receipt_dir" | sed 's|^/|Removing /|g' || true
			unset uninstall_remove_verbose_flag
		fi
	fi

	# Now, we can run the postremove script...
	if [ "x$uninstall_have_postremove" = "x1" ]; then
		OutputVerbosely "Executing postremove..."
		if $uninstall_postremove_script ; then
			OutputVerbosely "Postremove script successful."
		else
			ERROR "Postremove script failed."
			return 1
		fi
		rm -f "${uninstall_postremove_script}"
	fi
	OutputVerbosely "Uninstall completed for ${package_to_uninstall}"
}

UninstallPackage()
{
	# This fairly simple function will uninstall whatever packages you list as
	# parameters.  It will abort (returning 1) after the first package fails to
	# uninstall (the rest won't even be attempted).
	while [ $# -gt 0 ]; do
		package_to_uninstall="$1"
		shift

		OutputNormal "Uninstalling $package_to_uninstall."
		case $OSTYPE in
			"LINUX")
				rpm -e "$package_to_uninstall" || return 1
			;;
			"SUNOS")
				admin_file=/tmp/vmx_pkgrm_admin_file
				SafeCreateFile "$admin_file"
				cat > $admin_file << EOI
mail=
instance=overwrite
partial=nocheck
runlevel=nocheck
idepend=nocheck
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=default
EOI
				have_error=0
				/usr/sbin/pkgrm -n -a $admin_file "$package_to_uninstall" || have_error=1
				rm -f $admin_file

				[ ${have_error} = 0 ] || return 1
			;;
			"DARWIN")
				# This is nice and clean now that all of the darwin uninstallation
				# script has been moved to its own function.
				DarwinPackageUninstaller "$package_to_uninstall" || return 1
			;;
			"HP-UX")
				# The '-x enforce_dependencies=false' is here because there are strange dependency issues
				# on some HP-UX B.11.11 boxes where the "qmxcm.core,l=/,r=1.5.0Beta1.0475" package
				# will not uninstall because the "quest-umi-providers.core,l=/,r=1.1.0Beta1.0657" depends on it
				/usr/sbin/swremove -x enforce_dependencies=false "$package_to_uninstall" || return 1
			;;
			"AIX")
				# AIX can handle both RPMs and bff (installp) packages.  Try the RPM first.
				if rpm -qi "${package_to_uninstall}" >&7 2>&1; then
					rpm -e "$package_to_uninstall" || return 1
				else
					/usr/sbin/installp -u "$package_to_uninstall" || return 1
				fi
			;;
			*)
				ERROR "Uninstall method unknown on $OSTYPE"
				return 1
			;;
		esac
	done
}

###########################################################################
# End packaging related functions
###########################################################################


###########################################################################
# String functions
###########################################################################

# Escape any dots or pluses, so they can be used properly in a regular expression.
EscapeForRegex()
{
	if [ $# -gt 0 ]; then
		echo "$@" | sed -e 's/\([+.]\)/\\\1/g'
	else
		sed -e 's/\([+.]\)/\\\1/g'
	fi
}

EndsWith()
{
	if echo "$1" | grep "`EscapeForRegex $2`\$" >&7; then
		return 0
	fi
	return 1
}

StartsWith()
{
	if echo "$1" | grep "^`EscapeForRegex $2`" >&7; then
		return 0
	fi
	return 1
}

GetPlistKey()
{
	# $1=file
	# $2=key name
	#
	# This function assumes that the key you are searching for is unique among
	# the plist file.  No real XML parsing is done, as this would be a true pain
	# in shell script.  NOTE: When multiple keys have values, the last one found
	# is the one that is returned.

	plist_key_retval=
	plist_key_error=1
	temp_output_file=`MakeTemp /tmp/dmg_installer.plist_key.$$`
	if grep -A 1 "<key>$2</key>" "$1" > ${temp_output_file} 2>&7; then
		key_data_line="`tail -1 ${temp_output_file} | sed 's/^[ 	]*//g'`"
		# Possible values (ones that we expect) are:
		#   <true/>
		#   <false/>
		#   <string>.*</string> or <string/>
		#   <integer>.*</integer>
		#   <real>.*</real>
		#   <date>.*</date>
		# Everything else here is considered invalid (difficult to handle and non-useful for us).
		#   <array>.*</array>
		#   <data>.*</data>
		#   <dict>.*</dict>
		GetPlistKeyInnerData()
		{
			echo "${key_data_line}" | sed "s?<$1>\(.*\)</$1>?\1?g"
		}
		case ${key_data_line} in
			'<true/>') plist_key_retval="true"; plist_key_error=0 ;;
			'<false/>') plist_key_retval="false"; plist_key_error=0 ;;
			'<string>'*'</string>') plist_key_retval="`GetPlistKeyInnerData string`"; plist_key_error=0 ;;
			'<integer>'*'</integer>') plist_key_retval="`GetPlistKeyInnerData integer`"; plist_key_error=0 ;;
			'<real>'*'</real>') plist_key_retval="`GetPlistKeyInnerData real`"; plist_key_error=0 ;;
			'<date>'*'</date>') plist_key_retval="`GetPlistKeyInnerData date`"; plist_key_error=0 ;;
			'<array>'* | '<dict>'* | '<data>'*) echo "ERROR: Array, dict, and data properties cannot be returned." >&2;;
			*) echo "Unexpected property line: \"${key_data_line}\"" ;;
		esac

		if [ ${plist_key_error} != 1 ]; then
			echo "${plist_key_retval}"
		fi
	else
		echo "Key not found." >&2
	fi
	rm -f ${temp_output_file}
	return ${plist_key_error}
}

###########################################################################
# End string functions
###########################################################################


###########################################################################
# Compression functions
###########################################################################
PackageIsCompressed()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	for allowed_compression_extension in ${ALLOWED_COMPRESSION_EXTENSIONS}; do
		if EndsWith "$1" ${allowed_compression_extension}; then
			return 0
		fi
	done
	return 1
}

DecompressPackage()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ $# -gt 1 ]; then
		ERROR "Cannot decompress more than one package at a time."
		return 1
	fi
	decompress_package_name="$1"
	decompress_package_suffix=
	decompressor=
	if EndsWith "${decompress_package_name}" .gz; then
		decompressor=gzip
		decompress_package_suffix=.gz
	elif EndsWith "${decompress_package_name}" .bz2; then
		decompressor=bzip2
		decompress_package_suffix=.bz2
	else
		ERROR "Package (${decompress_package_name}) is not compressed."
		return 1
	fi

	CreateTempDir >&7 || return $?

	decompress_output_file="${QMX_TEMP_DIRECTORY}/`basename \"${decompress_package_name}\" ${decompress_package_suffix}`"

	# Make sure the tools directory has set the proper variables.
	GetToolsDirectory >&7 || return $?
	if	"${CACHED_TOOLS_DIRECTORY}"/${decompressor} -d -c "${decompress_package_name}" > "${decompress_output_file}"; then
		echo "${decompress_output_file}"
		return 0
	else
		decompress_return_value=$?
		ERROR "Failed to decompress package \"${decompress_package_name}\" with ${decompressor}"
		return ${decompress_return_value}
	fi
}

###########################################################################
# End compression functions
###########################################################################

###########################################################################
# Version checking functions
###########################################################################

# Get rid of any leading characters that are non-numeric, any trailing spaces,
# and any trailing zeroes in a version string.  All numeric digits are
# zero-padded to a length of 5 for easy numeric comparison.  The padding also
# makes the length of each segment uniform (if less than 5 digits) so
# converting "1.2.01" and "1.2.1" would result in the same string.
CleanupVersionString()
{
	# If you modify this function, please update this comment.
	#
	# How this works...
	# 1. Trailing spaces (or tabs) are removed.
	# 2. Any leading non-numeric characters (could be spaces) are removed.
	# 3. Remove any trailing '.0's (with function implementations of sed)
	# 4. Dots are changed to *TWO* spaces (to create non-overlapping testable
	#   regions).  Non-overlapping regions are important.
	# 5-6. Spaces are added at the start and end of the string.
	# 7-10. Leading 0's are added.  Each step increases the length by 1.
	#   The test region includes a space at the start, and some non-number at the
	#   end.  This requires that the regions do not overlap or global
	#   substitution would not apply the 0 padding to some of the numbers (hence
	#   the double spaces).
	# 11-12. Leading and trailing spaces are removed.
	# 13. Double spaces are converted to dots.
	temp_cleaned_version_string=`
	echo "$1" | \
	sed \
		-e 's/[	 ]*$//' \
		-e 's/^[^0-9]*//' \
		-e 's/\(\.00*\)*$//' \
		-e 's/\./  /g' \
		-e 's/^/ /' \
		-e 's/$/ /' \
		-e 's/ \([0-9]\{1\}[^0-9]\)/ 0\1/g' \
		-e 's/ \([0-9]\{2\}[^0-9]\)/ 0\1/g' \
		-e 's/ \([0-9]\{3\}[^0-9]\)/ 0\1/g' \
		-e 's/ \([0-9]\{4\}[^0-9]\)/ 0\1/g' \
		-e 's/^ //' \
		-e 's/ $//' \
		-e 's/  /./g'
`
	# Since the solaris sed does not properly match \(.\)* or \(.\){<NUMBER>},
	# we need to remove any remaining trailing '\.00*'s (if they are present).
	temp_cleaned_version_string2=`echo "${temp_cleaned_version_string}" | sed 's/\.00*$//'`
	while [ "${temp_cleaned_version_string2}" != "${temp_cleaned_version_string}" ]; do
		temp_cleaned_version_string="${temp_cleaned_version_string2}"
		temp_cleaned_version_string2=`echo "${temp_cleaned_version_string}" | sed 's/\.00*$//'`
	done
	echo "${temp_cleaned_version_string}"
	unset temp_cleaned_version_string temp_cleaned_version_string2
}

# Compare two versions in the Major.Minor.Micro.Build format.
# Both versions must be passed as dotted numeric strings, although they can
# include letters like in openssl's numbering scheme.
CompareVersionStrings()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	local_version_1=`CleanupVersionString "$1"`
	local_version_2=`CleanupVersionString "$2"`

	greater_version=`{ echo ${local_version_1}; echo ${local_version_2}; } | grep -v '^[ 	]*$' | sort -rn | head -n 1`
	local_compare=0
	if [ "${local_version_1}" = "${local_version_2}" ]; then
		# Returns 0.
		:
	elif [ "${greater_version}" = "${local_version_1}" ]; then
		local_compare=1
	else
		local_compare=-1
	fi
	unset local_version_1 local_version_2
	echo $local_compare
	unset local_compare
	return 0
}

# Return 0 (true) if version from $1 is greater than from $2
VersionIsGreater()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	comparison=`CompareVersionStrings "$1" "$2"`
	if [ ${comparison} = 1 ]; then
		return 0
	fi
	return 1
}

VersionIsEqualOrGreater()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	comparison=`CompareVersionStrings "$1" "$2"`
	if [ ${comparison} = 1 ]; then
		return 0
	elif [ ${comparison} = 0 ]; then
		return 0
	fi
	return 1
}

ExtractVersionFromFileName()
{
	evfn_file="$1"

	# Apply some transforms to extract version numbers from a filename.  This
	# includes stripping the OS name from the filename where it could include a
	# version string that would otherwise be extracted by this set of
	# transforms.
	#
	# With a functional sed, this could probably be done with nothing but sed
	# expressions.  Unfortunately, we support too many systems that have a
	# busted sed.
	basename "${evfn_file}" |
	sed -e 's/[Ss][Uu][Nn][Oo][Ss][-_.]5.[0-9][0-9]*//g' \
		-e 's/[Ss][Oo][Ll][Aa][Rr][Ii][Ss][-_.]5.[0-9][0-9]*//g' \
		-e 's/[Aa][Ii][Xx].[45].[0-9]//g' \
		-e 's/[Hh][Pp]-?[Uu][Xx].*//g' \
		-e 's/\.pkg.*$//g' \
		-e 's/9000\(.*\)\.depot.*$/\1/g' \
		-e 's/\.depot.*$//g' \
		-e 's/\.bff.*$//g' \
		-e 's/\.dmg.*$//g' \
		-e 's/\.rpm.*$//g' \
		-e 's/^[A-Za-z][A-Za-z0-9]*\.*//g' \
		-e 's/^[-_]*//g' \
		-e 's/\([0-9][0-9A-Za-z]*\.[0-9][0-9A-Za-z]*\)[-_]\([0-9]\)/\1.\2/g' \
		-e 's/\.\([A-Za-z]\)/!\1/g' |
	tr '\-_!' '\n\n\n' |
	grep '^[0-9].*\.' |
	sort -rn |
	head -n 1
	return 0
}

# Assumes: 	ObtainPackageDirectory has been called.
# $1 = package name (including any relative path)
ExtractVersionFromPackageFile()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	ev_package_name=$1
	ev_package_version=UNKNOWN

	ev_package_path="`ls -1 \"${CACHED_PACKAGE_DIRECTORY}\"/${ev_package_name}* | sort -r | head -n1`"
	ev_package_file="`basename \"${ev_package_path}\"`"

	ExtractVersionFromFileName "${ev_package_file}" || return $?
}

###########################################################################
# End version checking functions
###########################################################################


###########################################################################
# Upgrade functions
###########################################################################

GetNewClientVersion()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ "x${QMXSMS_INSTALL_FILE_VERSION:+set}" != "xset" ]; then
		QMXSMS_INSTALL_FILE_VERSION=`ExtractVersionFromPackageFile ${QMX_PACKAGE_NAME}`
	fi
}


# Compare the file list in QMXSMS_INSTALL_FILE_VERSION to that reported by the clienttool
SafeToUpgradeVMX()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if [ x${INSTALLED_CLIENT_IS_BROKEN:-0} = x1 ]; then
		# Always say it is safe to upgrade a broken client.
		return 0
	fi

	GetNewClientVersion

	if [ "x${QMXSMS_INSTALL_FILE_VERSION:+set}" = "xset" ]; then
		if VersionIsGreater "${QMXSMS_INSTALL_FILE_VERSION}" "${INSTALLED_CLIENT_VERSION}"; then
			return 0
		fi
	else
		OutputError "Cannot determine the version from the new install files"
	fi
	return 1
}

UseOldSettingsIfNoneProvided()
{
	# Use old settings as obtained from GetOldSettings.  Any values specified on
	# the command line will override these.
	[ "x${MPE_OPT:+set}" = xset ] || MPE_OPT="${OLD_MPE:-<NOT SET>}"
	[ "x${SITE_CODE_OPT:+set}" = xset ] || SITE_CODE_OPT="${OLD_SITE_CODE:-<NOT SET>}"
	[ "x${MPE_PORT_OPT:+set}" = xset ] || MPE_PORT_OPT="${OLD_MPE_PORT:-80}"
	[ "x${MPE_SSL_PORT_OPT:+set}" = xset ] || MPE_SSL_PORT_OPT="${OLD_MPE_SSL_PORT:-443}"
}

SaveVariableValue()
{
	# $1 = variable name
	# $2 = value to reset
	eval saved_$1="\"\${$1}\""
	eval $1="'$2'"
}

RestoreVariableValue()
{
	# $1 = variable name
	eval $1="\"\${saved_$1}\""
	eval unset saved_$1 || true
}

Upgrade()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	# A failure to obtain old settings will cause the new install to fail.  Abort now.
	GetOldSettings ||	AbortInstallation "Unable to retrieve existing ${PRODUCT_NAME} configuration."

	UseOldSettingsIfNoneProvided

	# Verify the settings and abort now if they are determined to be
	# incorrect (only if quiet).  Note: aborting is ok.  Restarting a
	# misconfigured client will just lead to confusion.
	if [ $QUIET_OPT = 1 ] ; then
		if SiteCodeHasError; then
			AbortInstallation "Invalid site code \"$SITE_CODE_OPT\""
		elif MPEAddressHasError; then
			AbortInstallation "Invalid site management point address \"$MPE_OPT\""
		fi
	fi

	# Since some other process may be doing some operation on the cimom which
	# could prevent us from getting a lock to save the backup state, we stop
	# both the cimom and the polling daemon and start the cimom by itself.
	if ExistingDaemonRunning ; then
		StopExistingDaemon
	fi

	if ${UMI_INIT_SCRIPT} start; then
		BackupVMXState

		if VersionIsEqualOrGreater "${INSTALLED_CLIENT_VERSION}" "1.5.0"; then
			# UMI exists.  While it will do its best to upgrade while preserving the
			# existing product data, we are safer to save our own.
			PRODUCT_TOOL_DUMP_FILE=${VMX_STATE_BACKUP_DIRECTORY}/qmxsms.statedump
			${UMI_PREFIX}/bin/product_tool --dump-product-state ${PRODUCT_TOOL_DUMP_FILE} || WARNING "Failed to save product state."
		fi

		StopExistingDaemon || true
	else
		WARNING "Failed to start CIMOM to preserve state for upgrade."
	fi

	[ ${DEBUG_ENABLED:-0} = 0 ] || uninstall_options="${uninstall_options:-} -debug"

	# Remove the old version.
	if [ x${INSTALLED_CLIENT_IS_BROKEN:-0} = x1 ]; then
		DoMainUninstall ${uninstall_options:-} || AbortInstallation "Failed to uninstall non-functional installed version."
	else
		# Do not let the uninstall script inherit these...
		SaveVariableValue QMX_TEMP_DIRECTORY ""
		SaveVariableValue REMOVE_SOURCE_DIR_ON_EXIT 0

		uninstall_failed=0
		${INSTALLED_UNINSTALL_SCRIPT} ${uninstall_options:-} || uninstall_failed=1

		# Restore the settings for this script.
		RestoreVariableValue QMX_TEMP_DIRECTORY
		RestoreVariableValue REMOVE_SOURCE_DIR_ON_EXIT

		if [ ${uninstall_failed} = 1 ]; then
			ErrorRestartClient "Unable to uninstall ${PRODUCT_SHORT_NAME}"
		fi
	fi

	# Upgrade UMI (or install if it does not exist).
	# Note that any registered products in umi (such as QMXSMS) will have their
	# state backed up and restored as a part of the upgrade.
	InstallUMI

	# Install the new client, umi, etc.
	CreateUsers || AbortInstallation "User/Group creation failed."

	# Set a flag so that the upgrade will handle failures by restarting if
	# possible instead of aborting.
	QMX_UPGRADE_IN_PROGRESS=1
	InstallVMX || AbortInstallation "Installation of new version failed."

	if [ x${PRODUCT_TOOL_DUMP_FILE:+set} = xset ]; then
		# restore product state from the saved file.
		${UMI_PREFIX}/bin/product_tool --restore-product-state ${PRODUCT_TOOL_DUMP_FILE} || WARNING "Unable to restore product state"
	fi

	Configure || AbortInstallation "Configuration of new version failed."

	RestoreVMXState || OutputError "Unable to restore previous ${PRODUCT_SHORT_NAME} client state."

	return 0
}

###########################################################################
# End upgrade functions
###########################################################################

GetToolsBaseDir()
{
	local_umi_subdir=umi/bin
	if [ "${OSTYPE}" = "LINUX" ]; then
		# This is a generic directory for *all* versions of linux of a given arch type.
		echo "${local_umi_subdir}/linux-${OSARCH}"
	elif [ "${OSTYPE}" = "AIX" ]; then
		# If we ever support itanium, this will need modification.
		echo "${local_umi_subdir}/aix-power"
	elif [ "${OSTYPE}" = "HP-UX" ]; then
		# If we ever support itanium, this will need modification.
		echo "${local_umi_subdir}/hpux-${OSARCH}"
	elif [ "${OSTYPE}" = "DARWIN" ]; then
		# We may want to make these things universal, otherwise we'll need to include two copies.
		echo "${local_umi_subdir}/osx-${OSARCH}"
	elif [ "${OSTYPE}" = "SUNOS" ]; then
		echo "${local_umi_subdir}/sunos-${OSARCH}"
	fi
}

GetToolsDirectory()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if [ "${CACHED_TOOLS_DIRECTORY:+set}" = "set" ]; then
		echo "${CACHED_TOOLS_DIRECTORY}"
		return 0
	fi

	if [ -f "${UMI_TOOLS_DIRECTORY}/bzip2" ]; then
		# If the umi tools are installed, use them instead of the binaries in the
		# install directory.  UMI has special cases in its install script to
		# ensure it has executable tools, and we really don't need to duplicate
		# that because UMI installs the tools.
		CACHED_TOOLS_DIRECTORY="${UMI_TOOLS_DIRECTORY}"
		echo "${CACHED_TOOLS_DIRECTORY}"
		return 0
	fi

	local_tools_dir=`GetToolsBaseDir`

	if [ "${local_tools_dir:-UNKNOWN}" != "UNKNOWN" ]; then
		CACHED_TOOLS_DIRECTORY="${BASE_INSTALLER_DIR}/${local_tools_dir}"
	fi
	local_tools_dir="${local_tools_dir:+bin/}${local_tools_dir:-UNKNOWN}"
	echo "${BASE_INSTALLER_DIR}/${local_tools_dir}"
	unset local_tools_dir
}

VerifyRootUser()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ "${EUID:-EUID_NOT_SET}" = "0" ] || [ "${UID:-UID_NOT_SET}" = "0" ]; then
		:
	else
		if id | grep 'uid=0' >&7 2>&7; then
			:
		else
			AbortInstallation "This script must be executed as root."
		fi
	fi
	return 0
}

# Print an error message when the shell exits and clean up.
ShellExited()
{
	shell_return_value=$?

	# NOTE: This ShellExited function is not called if we are the parent process
	# and we've already detached.
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	CleanupPackageDir

	CleanupTempDir

	if [ "${INSTALL_SCRIPT_DETACHED:-0}" = 1 ]; then
		# Don't print anything if it detached.
		:
	elif [ "${INSTALLATION_ABORTED}" = "1" ]; then
		# Don't print anything, because an error was already printed.
		:
	elif [ "${SUCCESSFULLY_FINISHED}" != "1" ]; then
		if [ "x${FATAL_ERROR:-}" != "x" ]; then
			OutputError "${FATAL_ERROR}"
		fi
		OutputError "The ${PRODUCT_NAME} ${INSTALL_TYPE} was not successful."
		shell_return_value=1
	fi

	rm -f $INSTALLER_RUNNING_FILE

	DisplayErrorState

	if [ ${INSTALLATION_ABORTED} = 1 ]; then
		shell_return_value=1
	elif HaveErrors || HaveFixmes; then
		shell_return_value=1
		if [ "x${SUCCESSFULLY_FINISHED:-0}" = "x1" ]; then
			OutputError "The client should be functional but you should correct all warnings and errors shown above."
		fi
	fi

	trap "" EXIT
	exit ${shell_return_value}
}


Usage()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ $# -gt 0 ]; then
		ERROR "$@"
	fi

	if [ "x${EXECUTED_SCRIPT_NAME}" != "x${UNINSTALL_SCRIPT_NAME}" ]; then
		echo "Usage: $0 [-q] [-m <Management Point IP>] [-s <Site Code>] [-f]"
		echo "          [[-trk <hex_key>] | [-trkfile <filename>]]"
		echo "  -q : Perform an automated quiet install (-m and -s must also be"
		echo "       specified in this case)."
		echo "  -t : Display required source file expression and exit"
		echo "  -m : Specify the hostname or IP address of the Management Point"
		echo "       or Management Point Proxy this client will use."
		echo "       If the client will be using SSL in a Native Mode"
		echo "       environment then you MUST specify an FQDN."
		echo "  -s : Specify the SMS Site Code the client will belong to."
		echo "  -p : Specify the port on which the Management Point"
		echo "       or MP Proxy is listening."
		echo "  -ssl : Specify the SSL port on which the MP Proxy is listening."
		echo "  -f : Force install on an unsupported platform."
		echo "  -r : Remove (uninstall) qmxcm client."
		echo "  -u : Upgrade qmxcm client."
		echo "  -d : Detach the install script from the parent process."
		echo "  -trk : Specify the trusted root key as <arg> (a long hex string)"
		echo "  -trkfile : Get the trusted root key from the file <arg>"
		echo "  -nostartup : Prevent the client from starting or obtaining policy."
		echo "       This will result in a non-operational client installation."
		echo "       See the documentation for details."
	else
		echo "Usage: $0 [-q]"
		echo "  -q : Perform a quiet (less noisy) uninstall"
	fi
	AbortInstallation
}

PrintUnsupportedMessage()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	echo "The platform you are attempting to install on is currently unsupported."
	echo "Please contact support@quest.com to request support for additional"
	echo "platforms.  You may continue the installation (at your own risk), but"
	echo "we will not be able to provide support for it at this time."
	echo
	echo "For reference, the following information was determined:"
	variant_text=""
	if [ "x${OSVARIANT}" = "x" ]; then
		:
	else
		variant_text="   variant=${OSVARIANT}"
	fi
	echo "arch=${OSARCH}   ostype=${OSTYPE}   vendor=${OSVEND}${variant_text}   version=${OSVER}"
	UNSUPPORTED=1
}

ParseCommandLine()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	NeedValue()
	{
		if [ x${2:+set} != xset ]; then
			Usage "\"$1\" requires a value." >&2
		fi
	}

	while [ $# -gt 0 ]; do
		case $1 in
			"")
				# Empty args should just be eaten.
				;;
			"-q")
				UMI_INSTALL_OPTIONS="${UMI_INSTALL_OPTIONS:-} -q"
				QUIET_OPT=1;;
			"-m")
				NeedValue "$@"
				MPE_OPT=$2
				shift
				;;
			"-s")
				NeedValue "$@"
				SITE_CODE_OPT=$2
				shift
				;;
			"-trk")
				NeedValue "$@"
				TRUSTED_ROOT_KEY="$2"
				shift
				;;
			"-trkfile")
				NeedValue "$@"
				if [ -f "$2" ]; then
					TRUSTED_ROOT_KEY="`cat $2`"
				elif [ -f "${BASE_INSTALLER_DIR}/$2" ]; then
					TRUSTED_ROOT_KEY="`cat \"${BASE_INSTALLER_DIR}/$2\"`"
				else
					AbortInstallation "Trusted root key file not found: \"$2\""
				fi
				shift
				;;
			"-p")
				NeedValue "$@"
				MPE_PORT_OPT=$2
				shift
				;;
			"-ssl")
				NeedValue "$@"
				MPE_SSL_PORT_OPT=$2
				shift
				;;
			"-f")
				UMI_INSTALL_OPTIONS="${UMI_INSTALL_OPTIONS:-} -f"
				FORCE_OPT=1;;
			"-r")
				UNINSTALL_OPT=1
				INSTALL_TYPE=uninstall
				;;
 			"-u")
 				UPGRADE_OPT=1
 				INSTALL_TYPE=upgrade
 				;;
			"-setpath")  # This is for internal use only.  Do not use this option.
				UMI_INSTALL_OPTIONS="${UMI_INSTALL_OPTIONS:-} -setpath $2"
				PATH="$2"
				export PATH
				shift
				;;
			"-vastool")
				NeedValue "$@"
				VASTOOLARGS="$2"
				shift
				;;
			"-debug")
				# This option isn't meant to be used by everyone.  It's here to
				# allow some way to propagate the debugging flags from one function
				# to another (some shells discard them).
				UMI_INSTALL_OPTIONS="${UMI_INSTALL_OPTIONS:-} -debug"
				DEBUG_ENABLED=1

				${ENABLE_DEBUGGING_NOW}


				# Reconfigure the descriptors now that debug is enabled.
				SetupDescriptors

				;;
			"-nostartup")
				STARTUP_DISABLED=1
				;;
			"-t")
				DisplayPackageRegex
				SUCCESSFULLY_FINISHED=1
				exit
				;;
 			"-d")
				UMI_INSTALL_OPTIONS="${UMI_INSTALL_OPTIONS:-} -q"
 				DETACH_REQUIRED=1
 				;;

			# This is not an option that users should use.  It is used to tell the
			# install script that it needs to remove the package directory when
			# the detached install script exits.
			"--remove-source-directory")
				REMOVE_SOURCE_DIR_ON_EXIT=1
				;;
			*)
				Usage "Invalid option: $1" >&2
				;;
		esac
		shift
	done

	if [ $UNINSTALL_OPT = 1 ] && [ $UPGRADE_OPT = 1 ]; then
		AbortInstallation "Uninstall and upgrade cannot both be specified."
	fi

	# check validity of args
	if [ $UPGRADE_OPT != 1 ] && [ $QUIET_OPT = 1 ] && [ $UNINSTALL_OPT != 1 ]; then
		if [ -z "$MPE_OPT" ]; then
			echo "Management Point must be identified with the -m option when doing a"
			echo "quiet unattended (or detached) install"
			AbortInstallation
		fi
		if [ -z "$SITE_CODE_OPT" ]; then
			echo "SMS site code must be identified with the -s option when doing a quiet"
			echo "unattended (or detached) install"
			AbortInstallation
		elif SiteCodeHasError ; then
			AbortInstallation "The given site code \"$SITE_CODE_OPT\" is invalid."
		fi
	fi

	CheckTRK
}

# List extra files that need to be distributed.
ListDistFiles()
{
	ldf_local_package_dir=`GetPlatformPackageDirectory`
	echo "${QMX_INSTALL_SCRIPT_NAME}"
	if [ ${OSTYPE} = DARWIN ]; then
		echo "${ldf_local_package_dir}/${DMG_INSTALLER}"
	fi
	unset ldf_local_package_dir
}

DisplayPackageRegex()
{
	UMI_REGEX_ERR_STR="Unable to determine the required UMI files for this system: OS=${OSTYPE}, Vendor=${OSVEND}, Version=${OSVER}, Arch=${OSARCH}"
	VMX_REGEX_ERR_STR="Unable to determine the required files for this system: OS=${OSTYPE}, Vendor=${OSVEND}, Version=${OSVER}, Arch=${OSARCH}"

	local_package_dir="`GetPlatformPackageDirectory | EscapeForRegex`"
	local_package_suffix="`GetPlatformPackageSuffix`"
	local_combined_regex="(`GetToolsBaseDir`/.*)"

	local_compression_suffix="()"
	for compression_extension in ${ALLOWED_COMPRESSION_EXTENSIONS}; do
		# This results in a leading '()|', which is exactly what we want to match an empty string (eg. no compression).
		local_compression_suffix="${local_compression_suffix}|`echo ${compression_extension} | EscapeForRegex`"
	done

	if CheckForSupportedPlatform -q; then
		for dist_file in `ListDistFiles | EscapeForRegex`; do
			local_combined_regex="${local_combined_regex:-}${local_combined_regex:+|}(${dist_file})"
		done
		for package_name in ${QMX_PACKAGE_NAME}; do
			local_combined_regex="${local_combined_regex:-}${local_combined_regex:+|}(${local_package_dir}/${package_name}.*\\.${local_package_suffix}(${local_compression_suffix}))"
		done

		if FindUMIInstaller -q ; then
			if umi_expression=`${UMI_INSTALL_SCRIPT_PATH} -t`; then
				# strip the surrounding parens and **umi-install-script** markers from umi_expression
				umi_expression="`echo ${umi_expression} | sed -e 's/^\*\*umi\-install\-script\*\*\ (//' -e 's/)\ \*\*umi\-install\-script\*\*$//'`"
				combined_expression="((umi/(${umi_expression}))|${local_combined_regex})"
			else
				AbortInstallation "${UMI_REGEX_ERR_STR}"
			fi
		else
			AbortInstallation "${UMI_REGEX_ERR_STR}"
		fi
		echo "--qmx-install-script-- ${combined_expression} --qmx-install-script--"
	else
		echo "Unsupported Platform"
		AbortInstallation "${VMX_REGEX_ERR_STR}"
	fi
}

DisableVMXStartup()
{
	touch /etc/${QMX_PREFIX}/${PRODUCT_NAME}.nostartup
}

ExistingDaemonRunning()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if [ x${INSTALLED_INIT_SCRIPT:+set} = xset ]; then
		${INSTALLED_INIT_SCRIPT} status || return $?
	else
		${QMX_INIT_SCRIPT} status || return $?
	fi
}

StartExistingDaemon()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	${INSTALLED_INIT_SCRIPT} start || return $?
}

StopExistingDaemon()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ x${INSTALLED_INIT_SCRIPT:+set} = xset ]; then
		${INSTALLED_INIT_SCRIPT} stop || return $?
	else
		${QMX_INIT_SCRIPT} stop || return $?
	fi
}

IsQMXSMSDRunning()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	${QMX_INIT_SCRIPT} status || return $?
}

StartQMXSMSD()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	${QMX_INIT_SCRIPT} start || return $?
}

# find file install_umi.sh
FindUMIInstaller()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	fui_be_quiet=0
	fui_no_error=0
	if [ $# -gt 0 ]; then
		for x in "$@"; do
			if [ "x$x" = "x-q" ]; then
				fui_be_quiet=1
			elif [ "x$x" = x--no-error ]; then
				fui_no_error=1
			fi
		done
	fi

	# Check the default location of the umi install script before attempting a
	# find.  If things are symlinked for some reason, the find can fail.
	fui_default_umi_script="${BASE_INSTALLER_DIR}/umi/${UMI_INSTALL_SCRIPT_NAME}"
	if [ -f "${fui_default_umi_script}" -o -h "${fui_default_umi_script}" ]; then
		UMI_INSTALL_SCRIPT_PATH="${fui_default_umi_script}"
	else
		UMI_INSTALL_SCRIPT_PATH=`find "${BASE_INSTALLER_DIR}/" -name ${UMI_INSTALL_SCRIPT_NAME} -print 2>&7` || true
	fi

	if [ x${UMI_INSTALL_SCRIPT_PATH:+set} = xset ]; then
		if [ -f "${UMI_INSTALL_SCRIPT_PATH}" ] || [ -h "${UMI_INSTALL_SCRIPT_PATH}" ]; then
			[ ${fui_be_quiet} = 1 ] || OutputNormal "The UMI install script is located at ${UMI_INSTALL_SCRIPT_PATH}"
			return 0
		fi
	fi

	if [ ${fui_no_error} = 0 ]; then
		ERROR "Cannot find the UMI install script."
	fi
	return 1
}

# Find the dmg_installer.sh script from UMI.
FindDmgInstaller()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	fui_be_quiet=0
	if [ $# -gt 0 ]; then
		for x in "$@"; do
			if [ "x$x" = "x-q" ]; then
				fui_be_quiet=1
			fi
		done
	fi

	if DMG_INSTALL_SCRIPT_PATH=`find "${BASE_INSTALLER_DIR}/" -name ${DMG_INSTALLER} -print | grep osx-${OSARCH}`; then
		if [ -f "${DMG_INSTALL_SCRIPT_PATH}" ] || [ -h "${DMG_INSTALL_SCRIPT_PATH}" ]; then
			[ ${fui_be_quiet} = 1 ] || OutputNormal "The dmg_installer.sh script is located at ${DMG_INSTALL_SCRIPT_PATH}"
			return 0
		fi
	fi

	return 1
}

# Uninstall UMI
UninstallUMI()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	OutputNormal "Attempting to uninstall UMI..."
	# If the install is broken, do not trust any existing uninstall script
	# unless we do not have a known good copy, such as would come from the
	# install CD.
	if [ x${INSTALLED_CLIENT_IS_BROKEN:-0} = x1 ] && FindUMIInstaller -q --no-error; then
		uninstall_umi_path="${UMI_INSTALL_SCRIPT_PATH}"
	elif [ -f "${UMI_UNINSTALL_SCRIPT}" ]; then
		# Use the version that was installed.
		uninstall_umi_path="${UMI_UNINSTALL_SCRIPT}"
	else
		AbortInstallation "Failed to find umi installer and no umi uninstall script exists (broken system).  Cannot uninstall."
	fi

	"${uninstall_umi_path}" ${UMI_INSTALL_OPTIONS} -r
}

InstallUMI()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if FindUMIInstaller; then
		if VASJoinedToAD && VASHostIsInstalled; then
			if [ "x${VASTOOLARGS}" = "x" ] ; then
				# Only disable the vashost upgrade if we are currently joined with
				# vashost and no join command was provided to rejoin with the new
				# version of VAS.
				WARNING "Disabling upgrade from vashost because no new join command was supplied.  Some features will not function."
				UMI_INSTALL_OPTIONS="${UMI_INSTALL_OPTIONS} --disable-vashost-upgrade"
			else
				# With an old vashost installed and a new join command supplied, we
				# are safe to unjoin.
				OutputNormal "An old version of vas-host is joined to a domain.  Unjoining (will rejoin later)."
				UnjoinFromAD
			fi
		else
			OutputVerbosely "Not unjoining from AD.  Not joined with vashost or joined with a full version of VAS."
		fi
		"${UMI_INSTALL_SCRIPT_PATH}" ${UMI_INSTALL_OPTIONS} || AbortInstallation "Failed to install UMI.  Cannot continue."
		OutputVerbosely "UMI was successfully installed."

		# This displays an error and will cause the installation to report as
		# failed when main() exits.  The client should still be in a functional
		# state (if everything else is successful), but the VAS integration will
		# not work.
		JoinToAD || ERROR "Could not join to AD as requested.  A manual VAS join will be required for full VAS integration."
	else
		AbortInstallation "Cannot proceed without UMI."
	fi
}

# Reverse the operations done in CreateSymlinks roughly in an opposite order
# from which they were created.
RemoveSymlinks()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	RemoveVendorAndProductSymlink()
	{
		# $1 = vendor base (eg. vintela or quest)
		# $2 = product base (eg. vmx or qmxsms)
		[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

		ProtectedClean()
		{
			# $1 = directory
			# $2 = protected directory name
			[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

			if [ "x$1" != "x$2" ]; then
				RemoveTreeOrLink "$1" || true
			elif [ -h "$1" ]; then
				rm -f "$1"
			else
				OutputDebug "Not cleaning new product directory $1"
			fi
		}

		vendor_prefix="/opt/$1"
		compat_prefix="${vendor_prefix}/$2"
		for fs_prefix in "" "/etc" "/var"; do
			compat_vendor_prefix=${fs_prefix}${vendor_prefix}
			compat_product_prefix=${fs_prefix}${compat_prefix}

			new_product_prefix=${fs_prefix}${QMX_PREFIX}
			new_vendor_prefix=${fs_prefix}${VENDOR_PREFIX}

			# Remove any existing items...  Always remove the product link before
			# the vendor link.  New product directories are protected.
			ProtectedClean "${compat_product_prefix}" "${new_product_prefix}"
			ProtectedClean "${compat_vendor_prefix}" "${new_vendor_prefix}"
		done
	}

	# Init script symlinks... These depend on symlinks which will also be
	# removed below.
	RemoveTreeOrLink /opt/vintela/vmx/sbin/vintela-vmx
	RemoveTreeOrLink /opt/quest/qmxsms/sbin/qmxsmsd_init

	# Config file symlinks...
	RemoveTreeOrLink /etc/opt/vintela/vmx/vmx.conf
	RemoveTreeOrLink /etc/opt/quest/qmxsms/qmxsms.conf

	# Vendor and product symlinks for backwards compatibility
	RemoveVendorAndProductSymlink vintela vmx
	RemoveVendorAndProductSymlink quest qmxsms
}

# Create compatibility symlinks for all of the old product locations.
CreateSymlinks()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	Symlink()
	{
		# This is like an ln -sf, but will work on all of our platforms.
		# $1 = source file
		# $2 = path of symlink
		[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
		if [ -f "$2" -o -h "$2" ]; then
			rm -f "$2"
		fi
		ln -s "$1" "$2"
	}

	CreateVendorAndProductSymlink()
	{
		# $1 = vendor base (eg. vintela or quest)
		# $2 = product base (eg. vmx or qmxsms)
		[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

		vendor_prefix="/opt/$1"
		compat_prefix="${vendor_prefix}/$2"
		for fs_prefix in "" "/etc" "/var"; do
			# Assumption made:
			# `dirname ${new_vendor_prefix}` == `dirname ${compat_vendor_prefix}`
			# (/opt == /opt)

			compat_vendor_prefix=${fs_prefix}${vendor_prefix}
			compat_product_prefix=${fs_prefix}${compat_prefix}
			new_product_prefix=${fs_prefix}${QMX_PREFIX}
			new_vendor_prefix=${fs_prefix}${VENDOR_PREFIX}

			# Create the vendor symlink
			if [ ! -d "${compat_vendor_prefix}" ]; then
				Symlink "${new_vendor_prefix}" "${compat_vendor_prefix}"
			fi

			# Create the product symlink
			if [ ! -d "${compat_product_prefix}" ]; then
				Symlink "${new_product_prefix}" "${compat_product_prefix}"
			fi
		done
	}

	# Remove any existing symlinks and empty directories that would alter the
	# creation of the new symlinks.
	RemoveSymlinks

	# Vendor and product symlinks for backwards compatibility
	CreateVendorAndProductSymlink vintela vmx
	CreateVendorAndProductSymlink quest qmxsms

	# Init script symlinks...  These depend on symlinks created above.
	Symlink "${QMX_INIT_SCRIPT}" /opt/vintela/vmx/sbin/vintela-vmx
	Symlink "${QMX_INIT_SCRIPT}" /opt/quest/qmxsms/sbin/qmxsmsd_init

	# Config file symlinks...
	Symlink "${QMX_CONFIG_FILE}" /etc/opt/vintela/vmx/vmx.conf
	Symlink "${QMX_CONFIG_FILE}" /etc/opt/quest/qmxsms/qmxsms.conf
}

InstallVMX()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	ObtainPackageDirectory || return $?

	GetNewClientVersion

	InstallPackages report qmxcm
	# Create backwards-compatibility symlinks.
	CreateSymlinks
}

GetVASInstallPath()
{
	VAS3_PREFIX=/opt/quest
	VAS2_PREFIX=/opt/vintela/vas
	OLD_VAS_PREFIX=/opt/vas
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ -d ${VAS3_PREFIX}/bin ] && [ -f ${VAS3_PREFIX}/bin/vastool ]; then
		VASTOOL=${VAS3_PREFIX}/bin/vastool
	elif [ -d ${VAS2_PREFIX}/bin ] && [ -f ${VAS2_PREFIX}/bin/vastool ]; then
		VASTOOL=${VAS2_PREFIX}/bin/vastool
	elif [ -d ${OLD_VAS_PREFIX}/bin ] && [ -f ${OLD_VAS_PREFIX}/bin/vastool ]; then
		VASTOOL=${OLD_VAS_PREFIX}/bin/vastool
	else
		return 1
	fi
}

VASJoinedToAD()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	GetVASInstallPath || return 1
	${VASTOOL} info domain 2>&7 >&7 || return $?
}

RunVASTool()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	OutputNormal "Running vastool as: ${VASTOOL} $@"
	${VASTOOL} "$@" || return $?
}

UnjoinFromAD()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	[ "x$VASTOOLARGS" = "x" ] && return 0

	GetVASInstallPath || return 0
	VASJoinedToAD ||  return 0

	vastoolargs_user=`echo ".${VASTOOLARGS}" | sed -n 's/.*-u[ \t]*\([-_\.A-Za-z0-9]*\).*/\1/p'`

	if [ "x${vastoolargs_user:+set}" != xset ]; then
		WARNING "Cannot unjoin, join arguments do not contain user."
		return 0
	fi

	unjoin_command="-u ${vastoolargs_user} unjoin"
	if RunVASTool ${unjoin_command} 2>&1; then
		:
	else
		WARNING "Unjoin command failed: ${VASTOOL} ${unjoin_command}"
	fi
	return 0
}

VasNotSupportedOnThisPlatform()
{
	# MacOS 10.3 will never be supported by VAS.
	[ "$OSVARIANT" = "MacOS" ] && [ "$OS_MAJOR" = "10" ] && [ "$OS_MINOR" = "3" ] && return 0;
	# FIXME: MacOS 10.6 is not yet supported by VAS.
	[ "$OSVARIANT" = "MacOS" ] && [ "$OS_MAJOR" = "10" ] && [ "$OS_MINOR" = "6" ] && return 0;
	return 1;
}

JoinToAD()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	[ "x${VASTOOLARGS:+set}" != "xset" ] && return 0

	OutputVerbosely "A VAS join was requested with arguments \"${VASTOOLARGS}\""

	if VasNotSupportedOnThisPlatform; then
		WARNING "VAS is not supported on this platform. Not joininig to Active Directory."
		return 0
	fi

	if GetVASInstallPath; then
		:
	else
		ERROR "VAS installation not found.  Not executing requested join command."
		return 1
	fi

	# Replace any "vastool" with the correct path to the installed vastool.
	# NOTE: The push installer does not include "vastool" in the auto-generated
	# command line.  To make this work with both the auto-generated and
	# user-supplied versions, we will remove the vastool from the start of every
	# command and run every subcommand with ${VASTOOL}.  This will prevent the
	# user from executing non-vastool commands as a part of the join process.
	#
	# NOTE: Normally we wouldn't put a newline in this printf, but the broken
	# sed in HP-UX 11.11 requires it to prevent strange malfunctions.
 	corrected_vastool_args=`printf '%s\n' "${VASTOOLARGS}" \
			| tr '\t' ' ' \
 			| sed \
 				-e '/^$/d' \
 				-e "s~^[ ]*[/A-Za-z0-9_-]*vastool~~g" \
 				-e "s~;[ ]*[/A-Za-z0-9_-]*vastool~;~g"`

	joined_to_ad_succesfully=0
	have_vastool_error=0

	# IFS twiddling is usually a bad idea.  We do want to execute one vas
	# command at a time so we can report individual errors.
	OLD_IFS="${IFS}"
	IFS=';'
	for vastool_cmd in $corrected_vastool_args ; do
		IFS="${OLD_IFS}"
		case " ${vastool_cmd} " in
			*" join "*)
				# Ignore the return value of join because of inconsistent return
				# values.  The joined status will be detected by an "-u host/ kinit"
				# query after the join was attempted.
				RunVASTool ${vastool_cmd} || true
				if RunVASTool -u host/ kinit >&7 2>&7; then
					OutputNormal "AD join appears to be successful."
					joined_to_ad_succesfully=1
				else
					ERROR "Failed to join AD with command ${vastool_cmd}"
					have_vastool_error=1
					break
				fi
				;;
			*" timesync "* | *" timesync")
				if ${VASTOOL} -v 2>&7 | grep 'VAS 2\.6' >&7; then
					# Strip off any options to timesync.
					vastool_cmd=`printf '%s\n' "${vastool_cmd}" | sed -e 's/timesync.*$/timesync/g'`
				fi
				# Timesync may return an error.  We'll ignore that.
				RunVASTool ${vastool_cmd} || true
				;;
			*)
				if RunVASTool ${vastool_cmd}; then
					OutputVerbosely "Successfully executed VAS command: ${vastool_cmd}"
				else
					ERROR "VAS command failed: ${vastool_cmd}"
					have_vastool_error=1
					break
				fi
				;;
		esac
		IFS=";"
	done
	IFS="${OLD_IFS}"

	if [ x${have_vastool_error} = x0 ] && [ x${joined_to_ad_succesfully} != x1 ]; then
		# No errors, but no join.
		WARNING "Executed VAS commands but did not join domain."
	fi

	return ${have_vastool_error}
}

CleanupOldFiles()
{
	# Any files the product tool missed (in the case of a broken install)
	for product in ${REGISTERED_PRODUCT_NAME}; do
		rm -f /etc${UMI_PREFIX}/openwbem/openwbem.conf.d/${product}* 2>&7 || true
		rm -f /etc${UMI_PREFIX}/openwbem/privileges/oop*${product}* 2>&7 || true
		rm -f /etc${UMI_PREFIX}/openwbem/privileges/owcimomd/oop*${product}* 2>&7 || true
	done

	# Remove the client's unix domain sockets, should they still exist.
	rm -f /var${INSTALLED_PREFIX:-/opt/quest/qmxcm}/qmxcmd/listenersocket* || true
	rm -f /tmp/OW@LCL@APIIPC_72859_Xq47Bf_P9r761-5_J-7_Qquest* || true

	RemoveEmptyDirectories /etc${UMI_PREFIX} /var${UMI_PREFIX} ${UMI_PREFIX} || true

	# Go through and remove any empty directories within any of the previously used QMX prefixes.
	for oldpath in /opt/quest/qmxcm /opt/quest/qmxsms /opt/vintela/vmx; do
		# Any conf files or backups...
		rm -f /etc${oldpath}/openwbem/* 2>&7 || true
		rm -f /etc${oldpath}/* 2>&7 || true

		# Any log files...
		rm -rf /var${oldpath}/logs 2>&7 || true
		rm -rf /var${oldpath}/owlogs 2>&7 || true

		# Any remaining temp files from file collection
		rm -rf /var${oldpath}/fileColTmp 2>&7 || true

		RemoveEmptyDirectories /etc${oldpath} /var${oldpath} ${oldpath} || true
	done

	RemoveEmptyDirectories /etc/opt/quest /var/opt/quest /opt/quest || true
}

Uninstall()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	echo "Uninstalling ${INSTALLED_PRODUCT_NAME:-QMXCM}."

	if [ x${INSTALLED_CLIENT_PACKAGE:+set} = xset ] && QueryExistingPackage ${INSTALLED_CLIENT_PACKAGE} >&7 2>&1; then
		UninstallPackage ${INSTALLED_CLIENT_PACKAGE}
	elif QueryExistingPackage ${QMX_PACKAGE_NAME} >&7 2>&1; then
		UninstallPackage ${QMX_PACKAGE_NAME}
	fi

	RemoveSymlinks

	# When upgrading, we don't want to deregister or uninstall umi.
	if [ ${QMX_UPGRADE_IN_PROGRESS:-0} = 0 ] || [ x${INSTALLED_CLIENT_IS_BROKEN:-0} = x1 ]; then
		# A full uninstall...  We should deregister and clean up nicely.
		if [ ${INSTALLED_USES_UMI} = 1 ]; then
			if [ -f ${UMI_PREFIX}/bin/product_tool ]; then
				${UMI_PREFIX}/bin/product_tool --name ${REGISTERED_PRODUCT_NAME} --deregister || true
			fi
			UninstallUMI || WARNING "Did not uninstall UMI."
		else
			OutputNormal "The installed version did not use UMI.  Not attempting an UMI uninstall."
		fi

		CleanupOldFiles
	fi
}


CompileMof()
{
	cm_namespace="$1"
	cm_mof_file="$2"
	OutputVerbosely "Running mof compiler as: ${UMI_PREFIX}/bin/owmofc ${CIMOM_URL} -n ${cm_namespace} \"${cm_mof_file}\""
	${UMI_PREFIX}/bin/owmofc ${CIMOM_URL} -n ${cm_namespace} "${cm_mof_file}"
}


InjectInstallationMessage()
{
	current_date=`date -u`
	current_cimdate=`date -u +"%Y%m%d%H%M%S.000000-000"`

	qmxversion=${QMXSMS_INSTALL_FILE_VERSION}

	# First, we must create the message."
	if [ ${INSTALL_TYPE} = upgrade ] ; then
		sms_message_id="${PRODUCT_NAME} upgraded to ${qmxversion} on ${current_date}"
		sms_message_type="SMS_RemoteClient_Upgraded"
	else
		sms_message_id="${PRODUCT_NAME} version ${qmxversion} installed on ${current_date}"
		sms_message_type="SMS_RemoteClient_Installed"
	fi

	OutputNormal "Creating scheduled messages for ${INSTALL_TYPE}"

	if injection_file=`MakeTemp /tmp/qmx_install_injection.mof`; then
		cat > $injection_file <<EOF
#pragma namespace("vmx/localpolicy")

instance of CCM_Scheduler_ScheduledMessage
{
	ScheduledMessageID="{VMXScheduledMessage-$sms_message_id}";
	TargetEndpoint="direct:VMXScheduledMessage";
	TriggerMessage="<?xml version='1.0' ?><${sms_message_type}><ClientVersion>QMX for SMS/Configuration Manager ${qmxversion}</ClientVersion><DateTime type=\\"datetime\\">${current_cimdate}</DateTime></${sms_message_type}>";
	PolicyVersion="1.00";
	Triggers={"OneShot;MaxRandomDelay=5"};
};
EOF

		OutputNormal "Adding ${INSTALL_TYPE} notification message to the database."
		# Now that the message injection file has been created, compile it.
		install_message_results=0
		CompileMof vmx/localpolicy ${injection_file} || install_message_results=$?

		rm -f ${injection_file}
		unset injection_file
		unset sms_message_id
		unset sms_message_type
		return ${install_message_results}
	else
		AbortInstallation "Unable to create temp file for install notification"
	fi
}


CheckTRK()
{
	if [ "x${TRUSTED_ROOT_KEY:+set}" = xset ]; then
		if printf '%s' "${TRUSTED_ROOT_KEY}" | tr -c '[ABCDEFabcdef0123456789]' '[!*]' | grep '!' >&7; then
			AbortInstallation "Supplied TRK contains invalid (non-hex) characters."
		fi
		# This would be easier to read as ${#TRUSTED_ROOT_KEY}, but not all of
		# our platforms have shells that support it.
		supplied_size=`printf '%s' "${TRUSTED_ROOT_KEY}" | wc -c`
		# Trim off any leading or trailing spaces
		supplied_size=`echo ${supplied_size}`
		expected_size=296
		if [ ${supplied_size} != ${expected_size} ]; then
			AbortInstallation "Supplied TRK is an incorrect length.  Expected length=${expected_size}, Supplied length=${supplied_size}"
		fi
	fi
}

InjectTRK()
{
	if [ "x${TRUSTED_ROOT_KEY:+set}" = xset ]; then
		if injection_file=`MakeTemp /tmp/qmx_trk_injection.mof`; then
			cat > $injection_file <<EOF
#pragma namespace("vmx/ccm/LocationServices")

instance of TrustedRootKey
{
	TrustedRootKey="${TRUSTED_ROOT_KEY}";
};
EOF
			OutputNormal "Adding the trusted root key to the database."

			# Since the user may be making a horrible mistake in upgrading with a
			# different TRK while leaving the old certificates intact, we will
			# delete the existing certificates.  If no certificates were currently
			# installed, this will have no effect.  If any certificates are
			# deleted here, they will be obtained again when the client does some
			# operation that requires the MP certificates (policy update, etc.).
			${UMI_PREFIX}/bin/owexecwql -u localhost:owipc/vmx/ccm/LocationServices -w 'delete from SMS_MPInformation' >&7 2>&1 || true

			trk_injection_result=0
			CompileMof vmx "${injection_file}" || trk_injection_result=$?

			rm -f $injection_file
			unset injection_file
			return $trk_injection_result
		else
			AbortInstallation "Unable to create temp file for TRK insertion"
		fi
	fi
}

DoSedInPlace()
{
	sed_file_name="$1"
	shift
	rm -f "${sed_file_name}.bak"
	cp "${sed_file_name}" "${sed_file_name}.bak"
	sed "$@" "${sed_file_name}.bak" > "${sed_file_name}"
}

Configure()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ "$QUIET_OPT" != "1" ]; then
		echo "Configuring..."
		while [ -z "$SITE_CODE_OPT" ] || SiteCodeHasError; do
			echo "Enter the SMS Site Code: "
			read SITE_CODE_OPT leftover
		done
		while [ -z "$MPE_OPT" ] || MPEAddressHasError; do
			echo "Enter the IP address or hostname of the Management Point: "
			read MPE_OPT leftover
		done
	elif SiteCodeHasError; then
		AbortInstallation "Invalid site code \"$SITE_CODE_OPT\""
	elif MPEAddressHasError; then
		AbortInstallation "Invalid site management point address \"$MPE_OPT\""
	fi

	echo "Compiling MOF. This may take a while. Please be patient."

	log_name=/tmp/qmxmofinstaller.sh.log.$$

	SafeCreateFile $log_name

	pt_log_option=
	if [ x${DEBUG_ENABLED} = x1 ]; then
		SafeCreateFile ${log_name}.product_tool
		pt_log_option="--debug --log-file ${log_name}.product_tool"
	fi

	if ${QMX_PREFIX}/bin/product_tool_commands.sh ${pt_log_option} > $log_name 2>&1; then
		# Nothing to do here.  We were successful.
		echo "MOF Compilation was successful."
		rm -f ${log_name} ${log_name}.product_tool
	else
		# We failed, print an error and cat the mof installer log.
		echo "MOF Compilation failed.  Results:"
		cat $log_name
		AbortInstallation "MOF Compilation failed."
	fi

	# Reset config file values that may have been set before.
	if [ x${OLD_OPT_POLL_INTERVAL:+set} = xset ] ; then
		StoreConfigValue qmxcm.poll_interval "${OLD_OPT_POLL_INTERVAL}"
	fi
	if [ x${OLD_OPT_CONNECT_TIMEOUT:+set} = xset ] ; then
		StoreConfigValue qmxcm.connect_timeout "${OLD_OPT_CONNECT_TIMEOUT}"
	fi
	if [ x${OLD_OPT_PACKAGE_CACHE:+set} = xset ] ; then
		StoreConfigValue qmxcm.package_cache "${OLD_OPT_PACKAGE_CACHE}"
	fi
	if [ x${OLD_OPT_CUSTOM_INVENTORY_TIMEOUT:+set} = xset ] ; then
		StoreConfigValue qmxcm.custom_inventory.timeout "${OLD_OPT_CUSTOM_INVENTORY_TIMEOUT}"
	fi

	# Automatically alter some platform-specific values now that we know more
	# about the actual OS version on which we will be running.
	if [ "${OSVEND}" = "Apple" ] && [ x${OSVARIANT} = "xMacOS" ]; then
		if VersionIsGreater 10.4 ${OSVER}; then
			# This is a 10.3.x box.  We need to enable the package search option
			# because spotlight does not exist.
			StoreConfigValue qmxcm.osx.use_packages_search "true"
		fi
	fi

	# Under some conditions, the CIMOM could potentially be stopped.  It needs
	# to be running for the clienttool operations to work.
	${UMI_INIT_SCRIPT} start

	# Set the values which may have been supplied on the command line or read
	# from an existing config file.
	if ${QMX_CLIENTTOOL} --set-mp ${MPE_OPT} --set-port ${MPE_PORT_OPT:-80} --set-ssl-port ${MPE_SSL_PORT_OPT:-443} --set-site-code ${SITE_CODE_OPT} --reset-stored-site-info; then
		:
	else
		# The clienttool did not work.  Set a few options manually before
		# restarting/aborting.  This will leave the client in a better state if
		# there is a temporary failure in name resolution.
		StoreConfigValue qmxcm.mpe "${MPE_OPT}"
		StoreConfigValue qmxcm.mpe.port "${MPE_PORT_OPT:-80}"
		StoreConfigValue qmxcm.mpe.secureport "${MPE_SSL_PORT_OPT:-443}"
		StoreConfigValue qmxcm.site_code "${SITE_CODE_OPT}"

		# If we were tyring to upgrade, we should restart the client.
		if [ x${QMX_UPGRADE_IN_PROGRESS:-0} = x1 ]; then
			ErrorRestartClient "Failed to set site parameters"
		else
			AbortInstallation "Failed to set site parameters"
		fi
	fi
}

VASHostIsInstalled()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	VAS_HOST_PACKAGES_LIST_TO_QUERY_FOR="vashost vas-host"
	if [ "x$OSTYPE" = "xAIX" ] ; then
		VAS_HOST_PACKAGES_LIST_TO_QUERY_FOR=`echo "${VAS_HOST_PACKAGES_LIST_TO_QUERY_FOR} " | sed 's/ /\.AIX* /g'`
	fi

	for local_package in ${VAS_HOST_PACKAGES_LIST_TO_QUERY_FOR} ; do
		if QueryExistingPackage ${local_package} >&7 2>&1; then
			return 0
		fi
	done
	return 1
}

ClientIsInstalled()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ x${CLIENT_IS_INSTALLED:-0} = x1 ]; then
		return 0
	fi
	return 1
}

QueryAndSaveInstances()
{
	query_namespace=$1
	query_classname=$2
	query_mof_file="${VMX_STATE_BACKUP_DIRECTORY}/${query_classname}.mof"
	OutputVerbosely "Saving instances of ${query_namespace}:${query_classname}"
	if [ ${INSTALLED_USES_UMI} = 1 ]; then
		EXECWQL="${UMI_PREFIX}/bin/owexecwql -u localhost:owipc/${query_namespace} -w"
	else
		EXECWQL="${INSTALLED_PREFIX}/bin/owexecwql localhost:owipc ${query_namespace}"
	fi
	echo "#pragma namespace(\"${query_namespace}\")" > "${query_mof_file}"
	#Enumerate instances only of ${query_classname} class
	query_temp_file="${VMX_STATE_BACKUP_DIRECTORY}/${query_classname}.dump"
	if $EXECWQL "select * from ${query_classname}" > "${query_temp_file}"; then
		cat "${query_temp_file}" | grep -v "Got response: .* CIMInstance" | grep -v "CIMInstance .*:" | awk 'BEGIN { state=0 } /^INSTANCE OF '${query_classname}'$/ { state=1 } state!=0 { print $0 } /^};$/ { state=0 }'  >> "${query_mof_file}"
	else
		WARNING "Failed to save instances of ${query_namespace}:${query_classname}.  The existing repository may be corrupt."
		# The file won't have anything useful in it.  Any useful error messages
		# should have (hopefully) been sent to stderr of owexecwql.
		# If we don't delete it, we may cause mof compilation errors when restoring it.
		rm -f "${query_mof_file}"
	fi
	rm -f "${query_temp_file}"
}

BackupVMXState()
{
	VMX_STATE_BACKUP_DIRECTORY=`MakeTemp -d /tmp/qmx_state`
	QueryAndSaveInstances "vmx" "Vintela_VMXData"
	if VersionIsGreater "1.5.0" "${INSTALLED_CLIENT_VERSION}"; then
		# QMXSMS and VMX < 1.5.0 do not have a product tool to save and restore
		# state.  Back up the data before upgrading anything else.
		QueryAndSaveInstances "vmx/policy" "CCM_Scheduler_ScheduledMessageState"
		QueryAndSaveInstances "vmx/invagt" "C00000000_0000_0000_0000_000000000010"
	fi

	BackupNOIDMofs || HAVE_EXTHWINV_FILES=0
	BackupSINVScripts || HAVE_EXTSWINV_FILES=0
}

RestoreVMXState()
{
	OWMOFC="${UMI_PREFIX}/bin/owmofc -u localhost:owipc/vmx"
	if [ x${VMX_STATE_BACKUP_DIRECTORY:+set} = xset ]; then
		for file in ${VMX_STATE_BACKUP_DIRECTORY}/*.mof; do
			OutputVerbosely "Restoring state for `basename ${file}`"
			if $OWMOFC $file ; then
				:
			else
				echo "Failed to import mof: $file"
				MOF_IMPORT_FAILED=1
			fi
		done
	fi

	if [ "${HAVE_EXTHWINV_FILES:-0}" = 1 ]; then
		RestoreNOIDMofs || WARNING "Unable to restore MOF files for Extended Hardware Inventory."
	fi
	if [ "${HAVE_EXTSWINV_FILES:-0}" = 1 ]; then
		RestoreSINVScripts || WARNING "Unable to restore scripts files for Extended Software Inventory."
	fi

	if [ ${MOF_IMPORT_FAILED:-0} = "0" ]; then
		if [ x${VMX_STATE_BACKUP_DIRECTORY:+set} = xset ]; then
			rm -rf ${VMX_STATE_BACKUP_DIRECTORY}
		fi
	fi
}

# Backup of MOF files required for Extented HW inventory
BackupNOIDMofs()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	HAVE_EXTHWINV_FILES=1
	LOCAL_RETURN_VALUE=1
	if ListFiles /var${INSTALLED_PREFIX}/noidmofs/* >&7 2>&1; then
		OLD_MASK=`umask`
		umask 077
		if NOIDMOFS_BACKUP_DIRECTORY=`MakeTemp -d /tmp/vmx_noidmofs_files`; then
			echo "Backing up Extended Hardware Inventory MOF files."
			cp -pR /var${INSTALLED_PREFIX}/noidmofs/* $NOIDMOFS_BACKUP_DIRECTORY 2>&7
			LOCAL_RETURN_VALUE=0
		fi
		umask $OLD_MASK
	fi
	return $LOCAL_RETURN_VALUE
}

RestoreNOIDMofs()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	LOCAL_RETURN_VALUE=1
	if [ -z "$NOIDMOFS_BACKUP_DIRECTORY" ]; then
		:
	elif ListFiles $NOIDMOFS_BACKUP_DIRECTORY/* >&7 2>&7; then
		echo "Restoring Extended Hardware Inventory MOF files"
		mkdir -p /var/${QMX_PREFIX}/noidmofs/
		cp -Rp $NOIDMOFS_BACKUP_DIRECTORY/* /var/${QMX_PREFIX}/noidmofs/
		rm -rf $NOIDMOFS_BACKUP_DIRECTORY
		LOCAL_RETURN_VALUE=0
	fi
	return $LOCAL_RETURN_VALUE
}

# Backup of scripts required for Extented SW inventory
BackupSINVScripts()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	HAVE_EXTSWINV_FILES=1
	LOCAL_RETURN_VALUE=1
	if ListFiles /var${INSTALLED_PREFIX}/sinv_scripts/* >&7 2>&1; then
		OLD_MASK=`umask`
		umask 077
		if SINV_SCRIPTS_BACKUP_DIRECTORY=`MakeTemp -d /tmp/vmx_sinv_scripts_files`; then
			echo "Backing up Extended Software Inventory Scripts files."
			cp -pR /var${INSTALLED_PREFIX}/sinv_scripts/* $SINV_SCRIPTS_BACKUP_DIRECTORY 2>&7
			LOCAL_RETURN_VALUE=0
		fi
		umask $OLD_MASK
	fi
	return $LOCAL_RETURN_VALUE
}

RestoreSINVScripts()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	LOCAL_RETURN_VALUE=1
	if [ -z "$SINV_SCRIPTS_BACKUP_DIRECTORY" ]; then
		:
	elif ListFiles $SINV_SCRIPTS_BACKUP_DIRECTORY/* >&7 2>&7; then
		echo "Restoring Extended Software Inventory Scripts files"
		mkdir -p /var/${QMX_PREFIX}/sinv_scripts/
		cp -Rp $SINV_SCRIPTS_BACKUP_DIRECTORY/* /var/${QMX_PREFIX}/sinv_scripts/
		rm -rf $SINV_SCRIPTS_BACKUP_DIRECTORY
		LOCAL_RETURN_VALUE=0
	fi
	return $LOCAL_RETURN_VALUE
}

# Get the config file value matching $2 and store it in a variable with name $1
GetConfigValue()
{
	local_variable="$1"
	local_config_opt="$2"

	if grep "^[ 	]*${local_config_opt}[ 	]*=.*" "$INSTALLED_CONFIG_FILE" >&7 2>&7; then
		local_config_line=`grep "^[ 	]*${local_config_opt}[ 	]*=.*" "$INSTALLED_CONFIG_FILE" 2>&7 | tail -1`

		if [ x${local_config_line:+set} = xset ]; then
			local_config_value=`echo ".${local_config_line}" | cut -f2- -d'=' | sed 's/^[ 	]*//'`
			eval "${local_variable}=\"${local_config_value}\""
			return 0
		fi
	else
		OutputDebug "No ${local_config_opt} option set in config file"
	fi
	return 1
}

StoreConfigValue()
{
	local_config_opt="$1"
	local_config_value="$2"
	local_match_regex="^[;#]*[ 	]*${local_config_opt}[ 	]*=.*"
	local_match_replacement="${local_config_opt}=${local_config_value}"

	if grep "${local_match_regex}" "${INSTALLED_CONFIG_FILE:-/etc${INSTALLED_PREFIX:-/opt/quest/qmxcm}/qmxcm.conf}" >&7 2>&7; then
		DoSedInPlace ${QMX_CONFIG_FILE} -e "s/${local_match_regex}/${local_match_replacement}/g"
	else
		echo "#### Inserted by the install script." >> ${QMX_CONFIG_FILE}
		echo "${local_match_replacement}" >> ${QMX_CONFIG_FILE}
	fi
}

GetOldSettings()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if [ -f ${INSTALLED_CONFIG_FILE} ]; then
		:
	else
		OutputError "Config file for installed client not found: ${INSTALLED_CONFIG_FILE}"
		return 1
	fi

	# Options which may be reset with install script command-line parameters.
	GetConfigValue OLD_MPE "${INSTALLED_CONFIG_PATTERN}\.mpe" || true
	GetConfigValue OLD_SITE_CODE "${INSTALLED_CONFIG_PATTERN}\.site_code" || true
	GetConfigValue OLD_MPE_PORT "${INSTALLED_CONFIG_PATTERN}\.mpe\.port" || true
	GetConfigValue OLD_MPE_SSL_PORT "${INSTALLED_CONFIG_PATTERN}\.mpe\.secureport" || true

	# Options which are only set by editing the config file.
	GetConfigValue OLD_OPT_POLL_INTERVAL "${INSTALLED_CONFIG_PATTERN}\.poll_interval" || true
	GetConfigValue OLD_OPT_CONNECT_TIMEOUT "${INSTALLED_CONFIG_PATTERN}\.connect_timeout" || true
	GetConfigValue OLD_OPT_PACKAGE_CACHE "${INSTALLED_CONFIG_PATTERN}\.package_cache" || true
	GetConfigValue OLD_OPT_CUSTOM_INVENTORY_TIMEOUT "${INSTALLED_CONFIG_PATTERN}\.custom_inventory\.timeout" || true

	if [ x${OLD_SITE_CODE:+set} != xset -o x${OLD_MPE:+set} != xset ]; then
		return 1
	fi

	return 0
}

ErrorRestartClient()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	echo "$*.  Attempting to restart existing client." >&2

	if StartExistingDaemon; then
		:
	else
		AbortInstallation "Failed to restart existing client.  A complete ${PRODUCT_NAME} install is neccessary. Please reinstall."
	fi
}

Detach()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	# Query the packaging system.  This does basically nothing on every platform
	# except for RedHat linux, where a corrupt RPM database will cause this
	# query to hang, and cause a timeout for the calling process (if called from
	# the CIMOM), so that errors of this type can be detected early-on.
	echo "Attempting to query packaging system..."
	QueryExistingPackage >&7 2>&1 || true
	echo "Package query completed."

	SafeCreateFile $DETACHED_LOG_NAME

	# Strip some of the parameters (like the detach param).
	PARAMETERS=
	while [ $# -gt 0 ]; do
		case "$1" in
			-d)
				# Ignore the detach parameter.
			;;
			*)
				if [ "x$1" != "x" ]; then
					PARAMETERS="$PARAMETERS \"$1\""
				fi
			;;
		esac
		shift
	done

	if echo ". $PARAMETERS" | grep "\-u" >&7 2>&1 ; then
		:
	elif [ -z "${VINTELA_VMX_STARTED_PROCESS:-}" ] ; then
		:
	else
		echo "Refusing to install.  Upgrade option was not given."
		exit 1
	fi

	# Since the package cache directory cleans itself up immediately after the
	# software distribution process returns, this script and the proper files
	# will need to be moved elsewhere before detaching.
	#
	# This is now being done even in the case of manual installs, because an
	# upgrade from a 1.0 client to 1.1 won't have the proper environment
	# variable set, so there is no easy way to detect how this was started.
	#
	# Unfortunately, this will double the amount of space required on /var,
	# albeit temporarily.
	echo "Making a copy of the package directory, since the ${PRODUCT_NAME} package cache will be deleted."
	if [ x${INSTALLED_PREFIX:+set} = xset ]; then
		NEW_PACKAGE_DIR=`dirname "/var${INSTALLED_PREFIX}"`/qmx_upgrade_temp
	else
		NEW_PACKAGE_DIR=`dirname "/var${QMX_PREFIX}"`/qmx_detached_install
	fi
	mkdir -p "$NEW_PACKAGE_DIR"
	cp -Rp "${BASE_INSTALLER_DIR}"/* "$NEW_PACKAGE_DIR"
	BASE_INSTALLER_DIR="$NEW_PACKAGE_DIR"

	# Remove the QMX env variable so that it won't detach again (if it was in
	# fact the QMX client that ran this script).
	VINTELA_VMX_STARTED_PROCESS=0
	unset VINTELA_VMX_STARTED_PROCESS

	# Set a variable to the new detached copy will know that it's detached. It
	# pauses for 60 seconds.
	INSTALL_SCRIPT_DETACHED=1
	export INSTALL_SCRIPT_DETACHED

	# Close stdin, since it isn't going to be available.
	exec 0<&-

	EXECUTABLE_NAME="${EXECUTED_SCRIPT_NAME}"
	EXECUTABLE_PATH="${BASE_INSTALLER_DIR}/$EXECUTABLE_NAME"
	echo "Saving output in $DETACHED_LOG_NAME"
	[ -x "$EXECUTABLE_PATH" ] || chmod u+x "$EXECUTABLE_PATH" || AbortInstallation "Unable to make installer script executable."
	cd "${BASE_INSTALLER_DIR}"


	# Rerun the install script and tell it to remove the package directory.
	eval /bin/sh "$EXECUTABLE_PATH" "--remove-source-directory $PARAMETERS >$DETACHED_LOG_NAME 2>&1 &"

	trap "echo Detached." EXIT

	exit 0
}

###########################################################################
# Stuff to remove
###########################################################################

user_exists()
{
	user_to_check=$1
	if [ $OSTYPE != DARWIN ]; then
		if id ${user_to_check} >&7 2>&1; then
			OutputDebug "Yes, user ${user_to_check} exists."
			return 0
		fi
	elif dscl . -read "/users/${user_to_check}" >&7 2>&1; then
		OutputDebug "Yes, user ${user_to_check} exists."
		return 0
	fi
	OutputDebug "No, user ${user_to_check} does not exist."
	return 1
}

group_exists()
{
	group_to_check=$1
	if [ ${OSTYPE} != DARWIN ]; then
		if grep "^${group_to_check}:" /etc/group >&7 2>&1; then
			OutputDebug "Yes, group ${group_to_check} exists"
			return 0
		fi
	elif dscl .  -read /groups/${group_to_check} >&7 2>&1; then
		OutputDebug "Yes, group ${group_to_check} exists"
		return 0
	fi
	OutputDebug "No, group ${group_to_check} does not exist."
	return 1
}

mac_find_unused_id()
{
	id_type=$1 # Normally "users" or "groups"
	id_tag=$2  # Normally "UniqueID" or "PrimaryGroupID"
	id_start=$3 # Should be >= 100 for physical users, or <-2 for fake users.
	id_increment=$4 # Must be either "+" or "-"
	if [ "x${id_increment}x" != "x+x" ] && [ "x${id_increment}x" != "x-x" ]; then
		ERROR "mac_find_user_id(): The increment must be '+' or '-', not \"${id_increment}\""
		return 1
	fi

	id_current=${id_start}

	# dscl -search returns success *ALL THE TIME*.  This is messed up.  From
	# tests, it appears as though all supported versions of MacOS (10.3-10.5)
	# return a string like this:
	# $ dscl . -search '/users' uid -2
	# nobody          dsAttrTypeNative:uid = ("-2")
	#
	# MacOS 10.3 prints out other debug info that we don't care about.  We'll
	# just grep for the '='.  This would not be a good thing to do in
	# general... The search is not even be displayed in xml format when using
	# the -plist option (10.4+), so we are forced to do something less than
	# clean.
	while dscl . -search "/${id_type}" ${id_tag} ${id_current} 2>&7 | grep '=' >&7; do
		id_current=`expr ${id_current} ${id_increment} 1`
	done

	echo ${id_current}
	return 0
}

# Get the group ID for a named group.
mac_getgid()
{
	mggid_group_name=$1

	if dscl . -read "/groups/${mggid_group_name}" PrimaryGroupID >&7 2>&1; then
		dscl . -read "/groups/${mggid_group_name}" PrimaryGroupID \
			| sed -e 's/.*PrimaryGroupID:[ 	]//g' -e 's/[ 	]*$//g'
		return 0
	fi
	return 1
}

mac_groupadd()
{
	if [ $# -ne 1 ]; then
		echo "Usage: mac_groupadd <groupname>" >&2
		return 1
	fi
	mga_groupname="$1"
	mga_gid=

	if mga_gid=`mac_find_unused_id groups PrimaryGroupID -2 '-'`; then
		:
	else
		ERROR "mac_groupadd(): Cannot find empty group id for group ${mga_group}"
		return 1
	fi

	mga_failed=1

	# Primitive direct database manipulation.
	if dscl . -create "/groups/${mga_groupname}" PrimaryGroupID "${mga_gid}"; then
		mga_failed=0
	else
		ERROR "mac_groupadd(): Failed to create group ${mga_groupname}"
	fi
	return ${mga_failed}
}

mac_useradd()
{
	if [ $# -ne 5 ]; then
		ERROR "Usage: mac_useradd <username> <comment> <homedir> <shell> <groupname>"
		return 1
	fi
	mua_username="$1"
	mua_comment="$2"
	mua_homedir="$3"
	mua_shell="$4"
	mua_group="$5"
	mua_gid=
	mua_uid=

	if mua_gid=`mac_getgid "${mua_group}"`; then
		:
	else
		ERROR "mac_useradd: Cannot find group id for group ${mua_group}"
		return 1
	fi

	# Search down from -2 (nobody) to find an unused user id
	if mua_uid=`mac_find_unused_id users UniqueID -2 '-'`; then
		:
	else
		ERROR "mac_useradd: Cannot find an empty user id for ${mua_username}"
		return 1
	fi


	mua_failed=1

	# Primitive direct database manipulation.
	if dscl . -create "/users/${mua_username}"; then
		if	dscl . -create "/users/${mua_username}" realname "${mua_comment}" &&
			dscl . -create "/users/${mua_username}" home "${mua_homedir}" &&
			dscl . -create "/users/${mua_username}" shell "${mua_shell}" &&
			dscl . -create "/users/${mua_username}" PrimaryGroupID "${mua_gid}" &&
			dscl . -create "/users/${mua_username}" UniqueID "${mua_uid}" &&
			dscl . -create "/users/${mua_username}" passwd "*" ; then
			mua_failed=0
		else
			ERROR "mac_useradd: Failed to set properties for user ${mua_username}"
		fi
	fi
	return ${mua_failed}
}

# Some platforms need the name service cache to be flushed after users or
# groups are added.
FlushNameCache()
{
	case ${OSTYPE} in
		SUNOS)
			if [ -f /usr/sbin/svcadm ]; then
				/usr/sbin/svcadm restart svc:/system/name-service-cache || OutputVerbosely "Could not restart the name service cache"
			fi
			;;
		*)
			OutputDebug "Nothing to flush..."
			;;
	esac
}

CreateGroup()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	group_to_create=$1
	if group_exists ${group_to_create}; then
		: # already exists
	else
		if [ -x /usr/sbin/groupadd ]; then
			/usr/sbin/groupadd $group_to_create || return 1
		elif [ -x /usr/bin/mkgroup ]; then
			/usr/bin/mkgroup $group_to_create || return 1
		elif [ ${OSTYPE} = DARWIN ]; then
			mac_groupadd ${group_to_create} || return 1
		else
			# Just print an error.  It will abort below.
			ERROR "How do we add a group on this platform?"
		fi
	fi

	FlushNameCache

	# Make sure it worked
	if group_exists ${group_to_create} ; then
		return 0
	else
		AbortInstallation "Creating group $group_to_create failed."
	fi
}

CreateUser()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	user_to_create=$1
	home_dir=$2
	if user_exists ${user_to_create}; then
		: # already exists
	else
		case $OSTYPE in
			"LINUX")
				/usr/sbin/useradd -c "Quest $user_to_create user" -d ${home_dir} -g $user_to_create -M -s /sbin/nologin -r $user_to_create || return 1
				;;
			"SUNOS")
				/usr/sbin/useradd -c "Quest $user_to_create user" -d ${home_dir} -g $user_to_create -s /bin/false $user_to_create || return 1
				;;
			"DARWIN")
				mac_useradd $user_to_create "Quest $user_to_create user" ${home_dir} /sbin/nologin $user_to_create || return 1
				;;
			"HP-UX")
				/usr/sbin/useradd -d ${home_dir} -g $user_to_create -s /usr/bin/false $user_to_create || return 1
				;;
			"AIX")
				# Can't set a user's home directory unless it (or at least its parent) exists.
				mkdir -p ${home_dir}
				/usr/bin/mkuser pgrp=$user_to_create groups=$user_to_create home=${home_dir} shell=/bin/sh login=false rlogin=false $user_to_create || return 1
				;;
			*)
				ERROR "Create user method unknown on $OSTYPE"
				return 1
				;;
		esac
	fi

	# check it worked
	if user_exists ${user_to_create}; then
		return 0
	else
		AbortInstallation "Creating user $user_to_create failed."
	fi
}

RemoveUser()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	user_to_remove=$1

	if user_exists ${user_to_remove}; then
		case $OSTYPE in
			# Amazingly, these platforms are the same when it comes to removing users.
			"LINUX" | "SUNOS" | "HP-UX")
				/usr/sbin/userdel ${user_to_remove} || return 1
				;;
			"DARWIN")
				# Primitive direct database manipulation.
				dscl . -delete "/users/${user_to_remove}" || return 1
				;;
			"AIX")
				/usr/sbin/rmuser -p ${user_to_remove} || return 1
				;;
			*)
				echo "Remove user method unknown on $OSTYPE"
				return 1
				;;
		esac

		FlushNameCache

		# Make sure it worked
		if user_exists ${user_to_remove}; then
			echo "Removing user $user_to_remove failed." >&2
			return 1
		fi
	fi
	return 0
}

RemoveGroup()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	group_to_remove=$1

	if group_exists ${group_to_remove}; then
		if [ -x /usr/sbin/groupdel ]; then
			/usr/sbin/groupdel ${group_to_remove} || return 1
		elif [ -x /usr/sbin/rmgroup ]; then
			/usr/sbin/rmgroup ${group_to_remove} || return 1
		elif [ ${OSTYPE} = DARWIN ]; then
			dscl . -delete "/groups/${group_to_remove}" || return 1
		else
			# Just print an error.  It will abort below.
			echo "How do we remove a group on this platform?" >&2
		fi

		# check it worked
		if group_exists ${group_to_remove}; then
			echo "Failed to remove group ${group_to_remove}" >&2
			return 1
		fi
	fi
	return 0
}

CreateUsers()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	CreateGroup $QMXSMSD_GROUP
	CreateUser $QMXSMSD_USER ${QMX_STATEDIR}
}

RemoveUsers()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	RemoveUser $QMXSMSD_USER
	RemoveGroup $QMXSMSD_GROUP
}

GetTempDir()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	default_temp_directory=/tmp
	if [ ${OSTYPE} = AIX ]; then
		default_temp_directory=/var/tmp
	fi
	echo "${TMPDIR:-${default_temp_directory}}"
	unset default_temp_directory
}

# Make a temporary file while avoiding race conditions and potential symlink attacks.
# Unlike the mktemp command, this will not be guaranteed to create a unique
# name (although it usually will be, unless a name is forced); it only
# guarantees that if it returns a success code, the file (or directory) was
# created and will be empty (overwrites can and will happen).
#
#
# NOTE: The umask for the system is currently ignored (a side effect of
# securely creating the file).  Any file created will only be readable (or
# writeable) by UID 0.
MakeTemp()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	# Local variables...
	make_temp_use_given_name=0
	make_temp_directory=0
	make_temp_try_count=1
	make_temp_successful=0
	make_temp_dest_dir=`GetTempDir`
	make_temp_file_pattern=""
	make_temp_mask=`umask`
	make_temp_perms=""
	make_temp_flags=$-
	umask 077

	if [ x${RANDOM:+set} != xset ]; then
		# Older shells don't provide $RANDOM.
		RANDOM=
	fi

	# Do not bail on errors!
	set +e

	# Parse the given options.
	while [ $# -gt 0 ]; do
		case "$1" in
			-d)
				make_temp_directory=1
				;;
			-f)
				make_temp_use_given_name=1
				;;
			-r)
				make_temp_try_count=$2
				shift
				;;
			-p)
				make_temp_perms="$2"
				shift
				;;
			*)
				make_temp_file_pattern="$1"
				;;
		esac
		shift
	done


	make_temp_base=`basename "$make_temp_file_pattern"`
	make_temp_chosen_dir=`dirname "$make_temp_file_pattern"`

	if StartsWith "${make_temp_file_pattern}" "${make_temp_chosen_dir}"; then
		# No directory was specified, so use the temp dir.
		make_temp_chosen_dir="${TMPDIR:-/tmp}"
	fi

	# Retry as many times as requested until there is a succefful creation.
	while [ $make_temp_try_count -gt 0 ] && [ $make_temp_successful -eq 0 ]; do
		make_temp_try_count=`expr $make_temp_try_count - 1`
		make_temp_target_suffix="${RANDOM}.${RANDOM}.${RANDOM}.${make_temp_try_count}.$$"
		make_temp_temp_dir="${make_temp_dest_dir}/temp_dir.${make_temp_base}.${make_temp_target_suffix}"
		make_temp_created_file=""
		rm -rf $make_temp_temp_dir
		if mkdir $make_temp_temp_dir >&7 2>&1; then
			# Made a directory.
			if [ $make_temp_directory -eq 1 ]; then
				make_temp_created_file="${make_temp_dest_dir}/${make_temp_base}.${make_temp_target_suffix}"
				rm -rf "$make_temp_created_file"
				mkdir -p "`dirname \"$make_temp_created_file\"`"
				if mv -f "$make_temp_temp_dir" "$make_temp_created_file"; then
					:
				else
					ERROR "MakeTemp: Unable to make directory.  A running attack is possible."
					# Clean up some before aborting.
					rm -rf "$make_temp_temp_dir"
					break
				fi
			else
				make_temp_created_file="$make_temp_temp_dir/${make_temp_base}.${make_temp_target_suffix}"
				# Solaris doesn't support the "set -o noclobber" functionality..
				# This isn't as secure as the noclobber method, but broken
				# shells must be dealt with.
				if [ -f "$make_temp_created_file" ]; then
					# Clean up some before trying again.
					rm -rf "$make_temp_temp_dir"
					continue
				else
					# Make a zero-byte file.
					cat <<EOF > "$make_temp_created_file"
EOF
				fi
			fi

			# At this point, we are guaranteed to have a unique (and safe) file
			# and/or directory which has a umask set for the current users use.
			# That is, of course, unless an attack is running with the UID the
			# script is currently running with.

			# If file permissions were specified, set them.
			if [ "x$make_temp_perms" != "x" ]; then
				if chmod "$make_temp_perms" "$make_temp_created_file"; then
					:
				else
					ERROR "MakeTemp: Unable to set chosen permissions ($make_temp_perms) for temp file."
					# Clean up some before aborting the loop.
					rm -rf $make_temp_created_file
					rm -rf $make_temp_temp_dir
					break
				fi
			fi

			# Move the created file into the temp directory now...

			if [ $make_temp_use_given_name -eq 1 ]; then
				# Rename the file to be exactly what they specified.
				make_temp_destfile="${make_temp_chosen_dir}/${make_temp_base}"
				rm -rf "$make_temp_destfile"
				mkdir -p "`dirname \"$make_temp_destfile\"`"
				if mv -f "$make_temp_created_file" "$make_temp_destfile"; then
					make_temp_created_file="$make_temp_destfile"
				else
					ERROR "MakeTemp: Unable to create target: $make_temp_destfile"
					make_temp_created_file=""
				fi
				unset make_temp_destfile
			elif [ $make_temp_directory -eq 0 ] ; then
				make_temp_destfile="$make_temp_chosen_dir/`basename $make_temp_created_file`"
				rm -rf "$make_temp_destfile"
				mkdir -p "$make_temp_chosen_dir"
				if mv -f "$make_temp_created_file" "$make_temp_chosen_dir"; then
					make_temp_created_file="$make_temp_destfile"
				else
					ERROR "MakeTemp: Unable to create target: ${make_temp_chosen_dir}/`basename ${make_temp_created_file}`"
					make_temp_created_file=""
				fi
				unset make_temp_destfile
			else
				# An exact match was not requested, so we'll just use what we have.
				# Things are always safer this way.
				:
			fi

			if [ "x$make_temp_created_file" != "x" ] ; then
				make_temp_successful=1
				echo "$make_temp_created_file"
				rm -rf "$make_temp_temp_dir"
			else
				# Something above failed, so we'll just clean up, and retry if possible.
				rm -rf "$make_temp_temp_dir"
			fi

		else
			# We failed to create a directory.  This will try on the next loop.
			:
		fi
	done

	# Clean up.
	set +$-
	set -$make_temp_flags
	unset make_temp_flags
	umask $make_temp_mask
	unset make_temp_mask
	unset make_temp_base
	unset make_temp_temp_dir
	unset make_temp_use_given_name
	unset make_temp_directory
	unset make_temp_try_count
	unset make_temp_dest_dir
	unset make_temp_chosen_dir
	unset make_temp_created_file
	unset make_temp_file_pattern
	unset make_temp_perms
	unset make_temp_target_suffix

	if [ $make_temp_successful -eq 0 ]; then
		ERROR "MakeTemp: Failed to create desired file or directory."
		unset make_temp_successful
		return 1
	else
		unset make_temp_successful
		return 0
	fi
}

# Safely create a file with the name <arg1>.
SafeCreateFile()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if MakeTemp -f "$1" >&7; then
		:
	else
		ERROR "Unable to create \"$1\": A running local symlink (or other) attack is possible."
		return 1
	fi
}

###########################################################################
# End stuff to remove.
###########################################################################

# Inject installation messages, the TRK, and do a policy update (if startup is enabled).
InitializeClient()
{
	[ -d ${QMX_STATEDIR}/logs ] && chown -R ${QMXSMSD_USER}:${QMXSMSD_GROUP} ${QMX_STATEDIR}/logs
	# If startup is disabled, don't start it or attempt a policy update.
	if [ ${STARTUP_DISABLED} = 0 ]; then
		${UMI_INIT_SCRIPT} start
		InjectTRK
		# Skip the injection of the installation message if we are only reconfiguring.
		if [ ${RECONFIGURE_ONLY} = 0 ]; then
			InjectInstallationMessage
		fi
		RunPolicyUpdate || AbortInstallation "Policy update failed"
		RunDiscovery || AbortInstallation "Discovery failed"
		StartQMXSMSD || AbortInstallation "Failed to start the ${PRODUCT_NAME} client."
	else
		# We will start up the CIMOM to inject the TRK.  Nothing is allowed
		# to touch the repository directly.
		${UMI_INIT_SCRIPT} force-start
		InjectTRK
		InjectInstallationMessage
		${UMI_INIT_SCRIPT} force-stop

		# prevent the init script from starting..
		DisableVMXStartup
	fi
}

# Check the limit of file handles and change/warn the user that they are too low.
IsNumeric()
{
	if [ $# -lt 1 ]; then
		return 1
	fi
	for parameter in "$@"; do
		if [ "x$parameter" != "x" ]; then
			local_extracted_number=`echo "$parameter" | sed 's/[^0-9]//g'`
			if [ "x$local_extracted_number" != "x$parameter" ]; then
				return 1
			fi
		else
			return 1
		fi
	done
	return 0
}

CheckUlimit()
{
	ulimit_max_files=0
	if ulimit_max_files=`ulimit -n` ; then
		if [ ${ulimit_max_files} != "unlimited" ]; then
			if IsNumeric ${ulimit_max_files}; then
				if [ ${ulimit_max_files} -lt ${FILE_LIMIT_REQUIRED} ]; then
					OutputNormal "Limit on open files (${ulimit_max_files}) is too low.  Changing to ${FILE_LIMIT_REQUIRED}."
					if ulimit -n ${FILE_LIMIT_REQUIRED}; then
						if [ "`ulimit -n`" -lt ${FILE_LIMIT_REQUIRED} ]; then
							AbortInstallation "Failed to increase open file limit from ${ulimit_max_files} to ${FILE_LIMIT_REQUIRED}."
						fi
					else
						AbortInstallation "Failed to increase open file limit from ${ulimit_max_files} to ${FILE_LIMIT_REQUIRED}."
					fi
				fi
			else
				WARNING "Could not interpret the results of ulimit. The installation can potentially fail."
			fi
		fi
	else
		WARNING "Received an error from ulimit. The installation can potentially fail."
	fi
}

DoMainInstall()
{
	if ClientIsInstalled; then
		if SafeToUpgradeVMX; then
			# Instead of aborting, this will upgrade.
			OutputNormal "Existing client found.  Will automatically attempt an upgrade."
			DoMainUpgrade "$@" || return $?
			return
		elif [ x${QMXSMS_INSTALL_FILE_VERSION:+set} = xset ] \
			&& VersionIsGreater "${INSTALLED_CLIENT_VERSION}" "${QMXSMS_INSTALL_FILE_VERSION}"; then

			# An invalid attempt to downgrade.
			ERROR "Cannot automatically downgrade from ${INSTALLED_PRODUCT_NAME} ${INSTALLED_CLIENT_VERSION} to ${PRODUCT_NAME} ${QMXSMS_INSTALL_FILE_VERSION}"
			AbortInstallation "The newer version must be uninstalled before installing an old one."
		else
			# The client is installed but is not upgradable.  This should mean
			# that they are trying to run the install when it is already
			# installed.  We used to treat this as an error, but an install that
			# failed for some reason (eg. invalid MP specified, firewall, etc)
			# would lead to an unusable client.  Reconfiguring without any
			# additional installation may fix these problems.
			OutputNormal "Existing ${INSTALLED_PRODUCT_NAME} ${INSTALLED_CLIENT_VERSION} found.  Skipping to configuration with any newly-provided settings."
			RECONFIGURE_ONLY=1
			# Get the current settings and use them so the user will not be
			# prompted unless they were never properly set.
			GetOldSettings || true
			UseOldSettingsIfNoneProvided
		fi
	else
		# No client already installed.
		CreateUsers

		if [ $DETACH_REQUIRED = 1 ]; then
			Detach "$@"
		fi

		# Clean up any old sockets, backup files, empty directories, etc. that
		# may be present from old installs to prevent them from interfering with
		# the new install.
		CleanupOldFiles

		InstallUMI
		InstallVMX
	fi

	Configure
	InitializeClient

	OutputNormal "${PRODUCT_NAME} installation successful."
}

DoMainUninstall()
{
	OutputNormal "Checking to see if ${PRODUCT_NAME} is already installed."
	if [ x${INSTALLED_CLIENT_IS_BROKEN:-0} = x0 ] && ClientIsInstalled ; then
		# We don't want to handle uninstalling multiple versions.  We'll
		# forward the uninstall on to the existing uninstall script if
		# needed.
		if [ "${BASE_INSTALLER_DIR}/${EXECUTED_SCRIPT_NAME}" != "${INSTALLED_UNINSTALL_SCRIPT}" ]; then
			OutputNormal "The uninstall for this version should only be executed as ${INSTALLED_UNINSTALL_SCRIPT}."
			OutputNormal "Attempting to execute existing uninstall script."

			${INSTALLED_UNINSTALL_SCRIPT} "$@" || AbortInstallation "Unable to uninstall using the ${INSTALLED_PRODUCT_NAME} ${INSTALLED_CLIENT_VERSION} uninstall script."
			CleanupOldFiles
			return 0
		fi
	fi
	if [ $DETACH_REQUIRED = 1 ]; then
		Detach "$@"
	fi
	StopExistingDaemon || true

	Uninstall
	RemoveUsers
	OutputNormal "${PRODUCT_NAME} uninstallation successful."
}

DoMainUpgrade()
{
	OutputNormal "Checking to see if ${PRODUCT_NAME} is already installed."
	if ClientIsInstalled ; then
		INSTALL_TYPE=upgrade
	else
		OutputNormal "Existing install of ${PRODUCT_SHORT_NAME} was not found.  Attempting a full install instead."
		DoMainInstall "$@" || return $?
		return
	fi

	# Check to for the package directory.  This is done *before* detaching.
	ObtainPackageDirectory || return $?
	FindUMIInstaller || return $?

	SafeToUpgradeVMX || AbortInstallation "Cannot upgrade ${PRODUCT_SHORT_NAME}: Version (${QMXSMS_INSTALL_FILE_VERSION:-UNKNOWN}) must be newer than currently installed (${INSTALLED_PRODUCT_NAME} ${INSTALLED_CLIENT_VERSION})."
	OutputNormal "Upgrading from ${INSTALLED_PRODUCT_NAME} ${INSTALLED_CLIENT_VERSION} to ${PRODUCT_NAME} ${QMXSMS_INSTALL_FILE_VERSION}"

	if VersionIsEqualOrGreater "${INSTALLED_CLIENT_VERSION}" 2.0 && VersionIsGreater 2.1 "${INSTALLED_CLIENT_VERSION}"; then
		# Handle an issue in the 2.0 client that prevents proper distributed upgrades to 2.2+
		cp "${UMI_INSTALL_SCRIPT_PATH}" "${UMI_UNINSTALL_SCRIPT}"
		chmod 755 "${UMI_UNINSTALL_SCRIPT}"
	fi

	if [ $DETACH_REQUIRED = 1 ]; then
		Detach "$@"
	fi

	Upgrade
	"${UMI_INSTALL_SCRIPT_PATH}" --remove-compatibility-packages || WARNING "Failed to remove old UMI compability packages."
	InitializeClient
	OutputNormal "${PRODUCT_NAME} upgrade successful."
}

Main()
{
	trap ShellExited EXIT

	# Set the PATH and umask to some known values.
	PATH=/bin:/usr/bin:/sbin:/usr/sbin${PATH:+:}${PATH:-}
	export PATH
	umask 022 # Intentionally unrestrictive read access.

	# Set up some output descriptors...
	SetupDescriptors

	LocateBaseInstallerDir

	# Change the current directory, as the directory we were running under may
	# be removed in some lower part of this script.
	cd /

	GetPlatformInformation

	if [ "${EXECUTED_SCRIPT_NAME}" = "${UNINSTALL_SCRIPT_NAME}" ]; then
		ParseCommandLine "$@" -r
	else
		ParseCommandLine "$@"
	fi
	touch $INSTALLER_RUNNING_FILE


	if [ ${DEBUG_ENABLED:-0} = 1 ]; then
		${ENABLE_DEBUGGING_NOW}

		# If debugging was requested, run GetPlatformInformation again so that
		# the results can appear in the debug output as well.
		GetPlatformInformation
	fi

	CheckUlimit

	# Check to see if this install script was started from within QMX.
	if [ -z "${VINTELA_VMX_STARTED_PROCESS:-}" ]; then
		:
	else
		DETACH_REQUIRED=1
	fi

	# If we're in the detached script, we have to wait a while before beginning
	# the upgrade so that the client can complete the software distribution
	# process before we shut it down.
	if [ "x${INSTALL_SCRIPT_DETACHED:-0}" = "x1" ] ; then
		echo "Waiting: Allowing 60 seconds for the software distribution process to complete."
		sleep 60

		# Now that we've waited, clear the variable.  We don't want anything we
		# spawn (like the UMI install script) to get this.
		unset INSTALL_SCRIPT_DETACHED
	fi

	if [ $DETACH_REQUIRED = 1 ]; then
		# Add the quiet flag and reparse the command line (to locate errors early
		# on).
		set - "$@" -q
		ParseCommandLine "$@"
	fi

	if [ $UNINSTALL_OPT = 0 ]; then
		# Don't immediately abort if unsupported.  This will be prompted later.
		if CheckForSupportedPlatform; then
			:
		else
			UNSUPPORTED=1
		fi
	fi

	# This needs to be done before getting any existing VMX version, as the
	# client tool now requires root access even to obtain the version number.
	VerifyRootUser

	DetermineInstalledSettings

	case $INSTALL_TYPE in
		install)
			ObtainPackageDirectory || AbortInstallation "Cannot install without package files."
			DoMainInstall "$@"
			;;
		upgrade)
			ObtainPackageDirectory || AbortInstallation "Cannot upgrade without package files."
			DoMainUpgrade "$@"
			;;
		uninstall)
			DoMainUninstall "$@"
			;;
		*)
			AbortInstallation "Unknown install type $INSTALL_TYPE"
			;;
	esac

	SUCCESSFULLY_FINISHED=1
}

Main "" "${@:-}"
