#!/bin/sh

# This is a script to run a command in some detached way.  It will run in the
# background so the parent process can continue with its happy life.

set -e
set -u

# Change these as needed.  Always use a ${foo:-} type reference.  Be careful
# about setting the savedir as it will be deleted!
DETACH_SAVEDIR=${DETACH_SAVEDIR:-/tmp/detach_dir.$$}
DETACH_LOG=${DETACH_LOG:-/tmp/detach_log.$$}

FULL_PATH_TO_SELF=

# Locate the script.  Sets some environment variables:
# PATH_TO_SELF = The directory containing the script.
# SCRIPTNAME = The name of the script.
# FULL_PATH_TO_SELF = The full path to the script, including the script name.
locate_self()
{
	if [ x${PATH_TO_SELF:+set} = xset ]; then
		return
	fi

	PATH_TO_SELF=

	SCRIPTNAME=`basename "$0"`
	LS_SCRIPT_BASE_DIR=`dirname "$0"`
	LS_WHICH_PATH=`which "$0" 2>/dev/null || true`
	LS_WHICH_BASE_DIR=`dirname "${LS_WHICH_PATH}"`
	LS_PWD=`pwd`

	for absolutepath in \
		"${LS_PWD}/${LS_SCRIPT_BASE_DIR}" \
		"${LS_SCRIPT_BASE_DIR}" \
		"${LS_WHICH_BASE_DIR}" \
		"${LS_PWD}/${LS_WHICH_BASE_DIR}"; do
		if [ -f "${absolutepath}/${SCRIPTNAME}" ]; then
			PATH_TO_SELF="${absolutepath}"
			break;
		fi
	done

	unset LS_SCRIPT_BASE_DIR LS_WHICH_PATH LS_WHICH_BASE_DIR LS_PWD

	if [ x${PATH_TO_SELF:+set} != xset ]; then
		return 1
	fi

	FULL_PATH_TO_SELF="${PATH_TO_SELF}/${SCRIPTNAME}"

	return 0
}


abort()
{
	echo "Aborting: $@" >&2
	exit 1
}

savefiles()
{
	mkdir "${DETACH_SAVEDIR}" || abort "Failed to make save dir"

	# Protection in case the copy fails or the detach itself fails.
	trap deletefiles EXIT

	cp -Rp "${PATH_TO_SELF}" "${DETACH_SAVEDIR}/"
}

deletefiles()
{
	if [ x${I_AM_DETACHED:-0} = x1 ]; then
		echo "Removing savedir (${DETACH_SAVEDIR})" >> "${DETACH_LOG}"
		cd /
		rm -rf "${DETACH_SAVEDIR}"
	fi
}

detach()
{
	if [ x${I_AM_DETACHED:-0} = x0 ]; then
		savefiles

		I_AM_DETACHED=1
		export I_AM_DETACHED
		export DETACH_SAVEDIR DETACH_LOG

		nohup /bin/sh  "${DETACH_SAVEDIR}/${SCRIPTNAME}" "$@" >>"${DETACH_LOG}" 2>&1 &
		unset I_AM_DETACHED

		echo "Detached."

		exit 0
	fi

	# The script should delete files when it finishes.
	trap deletefiles EXIT
}

main()
{
	locate_self || abort "Cannot locate myself.  PWD=`pwd` \$0=$0"

	trap "abort Interrupted" INT

	# Find the script we are to run before we detach so if we do not
	# have a script to run we can fail the distribution.
	for ext in .sh .py .pl; do
		script_to_run=`find . -type f -name "*${ext}" | grep -v detach\.sh | head -1`
		if [ x${script_to_run:+set} = xset ] ; then
			break
		fi
	done

	if [ x${script_to_run:+set} != xset ]; then
		 abort "Could not find a python, perl or shell script to run."
	fi

	# This doesn't return for the parent -- it just goes away.
	detach "$@"

	# Do whatever.  Add a sleep if needed so the parent can complete whatever it
	# was doing.  Be sure you properly handle any errors from commands you run.
	# Notice -e and -u are set for this script.
	echo "I am detached.  PID=$$ \$@=$@" >> "${DETACH_LOG}"
	if [ -x "${DETACH_SAVEDIR}/${script_to_run}" ]; then
		shell=""
	else
		case "${script_to_run}" in
			*.sh) shell=/bin/sh ;;
			*.py) shell="/usr/bin/env python" ;;
			*.pl) shell="/usr/bin/env perl -e" ;;
		esac
	fi
	echo "Running ${shell} \"${DETACH_SAVEDIR}/${script_to_run}\" \"$@\"" >> "${DETACH_LOG}"
	nohup ${shell}  "${DETACH_SAVEDIR}/${script_to_run}" "$@" >>"${DETACH_LOG}" 2>&1 &

	# The copy of the script will disappear at the end of the script.
}

if [ $# -gt 0 ]; then
	main "$@"
else
	main ""
fi
