#!/bin/sh
################################################################################
# Copyright (c) 2008 Quest Software, Inc.  All Rights Reserved.
#
# bundle.sh
#
# Purpose:      Create a .tar.gz of normal system files and system information to 
#               aid in the resolving of issues.  This works with any version of
#               QMX greater than version 1.5.
#
# Author(s):    John Stewart (john.stewart@quest.com)
#
# Version:      1.0
#
#
################################################################################

################################################################################
# 
# The qmx_snapshot.sh script gathers the following information:
# 
# Basic:
# Hostname
# Full hostname
# Platform
# uname -a
# Date
# 
# QMX Basics if installed:
# Clenttool version
# Client ID
# Discovery output
# Policy update output
#                
# System commands, patch information: 
# ipcs
# df -k
# Solaris:
#     showrev -p
# HP-UX:
#     swlist -l product *,c=patch
#     ls /dev/*random*
# AIX:
#     oslevel -s
#     oslevel -r
#     instfix -i
# ls -ld / /etc /etc/opt /opt /var /var/opt /tmp
# ls -l /tmp
# uptime
# ps -aux
# ulimit -a
# 
# Extended QMX information:
# Depending upon options given, this will do the following:
# Hardware Inventory
# Software Inventory
# Software Metering
# File Collection
# List Available Software
# Software Distribution
# QMX Socket Dump
#
# If one of those options are given, the script will restart qmx
# in DEBUG3 mode and increase the log size until the command(s)
# finish.  Then qmx will return to its previous logging level and
# these log files will be collected:
# /var/opt/quest/qmxsms/owlogs/qmxlog.txt
# /var/opt/quest/qmxsms/owlogs/qmxlog.xml
# /var/opt/quest/qmxsms/owlogs/qmx_status.log
# /var/opt/quest/qmxsms/logs/qmxsmsd.log
# /var/opt/quest/umi/owcimomd.monitor.log
# /var/opt/quest/umi/logs/owcimomd.log
# 
# System files:
# hosts, only localhost and host lines, others are not gathered.
# /etc/resolv.conf
# /etc/syslog.conf
# 
# QMX files:
# /etc/opt/quest/qmxsms/qmxsms.conf
# /etc/opt/quest/umi/openwbem/openwbem.conf.d/qmxsms.allowed_users.conf
# /etc/opt/quest/umi/openwbem/openwbem.conf.d/qmxsms.conf
# /etc/opt/quest/umi/openwbem/openwbem.conf.d/umi.conf
#
# WQL Queries:
# /opt/quest/umi/bin/owexecwql -u localhost:owipc/vmx/policy -w "select * from CCM_Policy"
# The next query only runs when software metering is selected:
# /opt/quest/umi/bin/owexecwql -u localhost:owipc/vmx/provider -w "select * from CIM_IndicationFilter"
# 
# This set of information was chosen to best assist Support with any issue, and
# familiarize them with the system so they can give better support when asking
# for additional information. 
# 
# The resulting file will be in /tmp/ with a name of
# qmx_snapshot.<hostname>.<date>.tar.gz.
# 
# If you don't wish Support to have specific gathered information, either
# modify the script to not gather it, or open the archive after and remove
# the file(s) before sending.
# 
################################################################################
# Variables.
################################################################################
OPTION_HELP_SHORT="-h"
OPTION_HELP_LONG="--help"
OPTION_HARDWARE_INVENTORY="--run-hardware-inventory"
OPTION_SOFTWARE_INVENTORY="--run-software-inventory"
OPTION_SOFTWARE_AVAILABLE="--list-available-software"
OPTION_FILE_COLLECTION="--run-file-collection"
OPTION_SOFTWARE_METERING="--run-software-metering-usage-report"
OPTION_SOFTWARE_DISTRIBUTION="--run-software-distribution"
OPTION_DEBUG_CLASS="--debug-class"
OPTION_WQL_QUERIES="--wql"
OPTION_NO_RESTART="--no-restart"
OPTION_GRAB_CURRENT_LOGS="--grab-current-logs"
OPTION_SOCKET_DUMP="--socket-dump"
OPTION_NO_DISCOVERY_UPDATE="--no-discovery-update"

HARDWARE_INVENTORY=
SOFTWARE_INVENTORY=
SOFTWARE_AVAILABLE=
FILE_COLLECTION=
SOFTWARE_METERING=
SOFTWARE_DISTRIBUTION=
ADVERTISEMENT_ID=
PACKAGE_ID=
PROGRAM_ID=
DEBUG_CLASS=
CLASS_TO_DEBUG=
WQL_QUERIES=
RESTART=
GRAB_CURRENT_LOGS=
SOCKET_DUMP=
DISCOVERY_UPDATE=

DATE_MODIFIED=20090421

INCREASED_LOGGING="false"

################################################################################
# Functions. 
################################################################################
Usage()
{
    echo "------------------------------------------------"
    echo "Usage: qmx_snapshot.sh [options]"
    echo ""
    echo "This script collects debug information from one or more of the following clienttool options:
  ${OPTION_HELP_SHORT}, ${OPTION_HELP_LONG}                Show help about options.
  ${OPTION_HARDWARE_INVENTORY}  Run hardware inventory.
  ${OPTION_SOFTWARE_INVENTORY}  Run software inventory.
  ${OPTION_SOFTWARE_AVAILABLE} Print the list of software available for
                            installation.
  ${OPTION_FILE_COLLECTION} Run file collection with delta parameter.
  ${OPTION_SOFTWARE_METERING} Run software metering usage report
                            generation.
  ${OPTION_SOFTWARE_DISTRIBUTION} <Advertisement ID> <Package ID> <Program ID> Run
                            software distribution for the specified
                            Advertisement/Package/Program.
  ${OPTION_DEBUG_CLASS} <arg>       Get hardware inventory debug information for the
                            specified class.  To list all classes, specify
                            \"all\"
  ${OPTION_WQL_QUERIES}                     Run other wql queries in addition
                            to policies.
  ${OPTION_NO_RESTART}              This will not restart the client.  WARNING: This also
                            means that the options will run at the current
                            debug level instead of DEBUG3.
  ${OPTION_GRAB_CURRENT_LOGS}       When the snapshot script runs,
                            the old log files on the system are deleted
                            and the new log files are captured.
                            This option saves the current log files
                            before deleting them when restarting the
                            agent.
  ${OPTION_SOCKET_DUMP}             This will generate a socket dump for only
                            the QMX client to assist in examining data
                            coming from the management point.
  ${OPTION_NO_DISCOVERY_UPDATE}     This will cause discovery and update not to run."
    echo "------------------------------------------------"
}

CheckOptions()
{
    while [ $# != 0 ]
    do
        case $1 in
            $OPTION_HELP_SHORT)
                Usage
                exit 0
                ;;
            $OPTION_HELP_LONG)
                Usage
                exit 0
                ;;
            $OPTION_HARDWARE_INVENTORY)
				HARDWARE_INVENTORY="true"
                ;;
            $OPTION_SOFTWARE_INVENTORY)
				SOFTWARE_INVENTORY="true"
                ;;
            $OPTION_SOFTWARE_AVAILABLE)
				SOFTWARE_AVAILABLE="true"
                ;;
            $OPTION_FILE_COLLECTION)
				FILE_COLLECTION="true"
                ;;
            $OPTION_SOFTWARE_METERING)
				SOFTWARE_METERING="true"
                ;;
            $OPTION_SOFTWARE_DISTRIBUTION)
                SOFTWARE_DISTRIBUTION="true"
                shift

                if [ $# -ge 3 ]; then
                    ADVERTISEMENT_ID="${1}"
                    shift
                    PACKAGE_ID="${1}"
                    shift
                    PROGRAM_ID="${1}"
                    # Do not shift here; that is covered later
                else
                    echo "Invalid usage of option: ${OPTION_SOFTWARE_DISTRIBUTION}"
                    Usage
                    exit 1
                fi

                ;;
            $OPTION_DEBUG_CLASS)
                DEBUG_CLASS="true"
                shift

                if [ $# -ge 1 ]; then
                    CLASS_TO_DEBUG="${1}"
                    # Do not shift here; that is covered later
                else
                    echo "Invalid usage of option: ${OPTION_DEBUG_CLASS}"
                    Usage
                    exit 1
                fi
                ;;
            $OPTION_WQL_QUERIES)
                WQL_QUERIES="true"
                ;;
            $OPTION_NO_RESTART)
                RESTART="false"
                ;;
            $OPTION_GRAB_CURRENT_LOGS)
                GRAB_CURRENT_LOGS="true"
                ;;
            $OPTION_SOCKET_DUMP)
                SOCKET_DUMP="true"
                ;;
            $OPTION_NO_DISCOVERY_UPDATE)
                DISCOVERY_UPDATE="false"
                ;;
            *)
                echo "Invalid option: $1"
                Usage
                exit 1
                ;;
        esac
        shift
    done
}

Prep()
{
    echo "*** This script will be gathering System and QMX information ***"
    echo "qmx_snapshot last modified: ${DATE_MODIFIED}"

    # First thing, check for root.
    if [ "`id | sed 's/uid=\([0-9]*\).*/\1/'`" -ne 0 ] ; then
        echo "Must be run as root."
        exit
    fi
    
    # Set up the variables and prep the directories. 

    if [ -x "/opt/quest/qmxcm" ] ; then
        QMX_DIR="qmxcm"
    else
        QMX_DIR="qmxsms"
    fi

    DATE=`date "+%Y-%m-%d_%H-%M-%S"`
    HOSTNAME=`hostname | cut -d. -f 1`
    DOMAINNAME=`domainname`
    QMXBIN=/opt/quest/$QMX_DIR/bin
    UMIBIN=/opt/quest/umi/bin
    QMX_INIT_SCRIPT=/opt/quest/$QMX_DIR/sbin/${QMX_DIR}d_init
    QMX=${QMXBIN}/clienttool
    WQL="${UMIBIN}/owexecwql -u localhost:owipc/"
    UMI_QMX_CONF="/etc/opt/quest/umi/openwbem/openwbem.conf.d/qmxsms.conf"
    UMI_CONF="/etc/opt/quest/umi/openwbem/openwbem.conf.d/umi.conf"
    QMX_CONF="/etc/opt/quest/${QMX_DIR}/${QMX_DIR}.conf"
    if [ -x "${QMX}" ] ; then
        HAVE_QMX=1
        VERSION=`$QMX -v | cut -c1-3`
    else
        HAVE_QMX=0
    fi

    case ${VERSION} in
        "1.5" | "1.6" | "2.0")
            HAS_2_1_TOOLS="false"
            ;;
        *)
            HAS_2_1_TOOLS="true"
            ;;
    esac

    OUT_DIR=/tmp/${DATE}
    OUT_MAIN=${OUT_DIR}/main.txt
    OUT_DIR_CMDS=${OUT_DIR}/cmds_out
    OUT_DIR_FILES=${OUT_DIR}/files
    OLD_LOG_FILES=${OUT_DIR}/old_files
    OUT_BUNDLE=/tmp/qmx_snapshot.${HOSTNAME}.${DATE}
    ADJUSTED_LOGS="false"
    mkdir ${OUT_DIR}
    mkdir ${OUT_DIR_CMDS}
    mkdir ${OUT_DIR_FILES}

    if [ "${GRAB_CURRENT_LOGS}" = "true" ]; then
        mkdir ${OLD_LOG_FILES}
    fi

    if [ "${SOCKET_DUMP}" = "true" ]; then
        OUT_SOCKET_DUMP=${OUT_DIR}/socket
        mkdir ${OUT_SOCKET_DUMP}
        chown owprovdr ${OUT_SOCKET_DUMP}
    fi
}


DeterminePlatform()
{
    unamea=`uname -a`
    name=`echo $unamea | awk '{ print $1 }'`
    case $name in
        "Linux")
            if [ -f /etc/redhat-release ] ; then
                dist=`cat /etc/redhat-release | grep Enterprise`
                if [ $? -eq 0 ]; then
                    level=`cat /etc/redhat-release | sed 's/.*release\ //' | sed 's/\ .*//'`
                    platform="RedHat $level"
                else
                    platform="RedHat"
                fi
            elif [ -f /etc/SuSE-release ] ; then
                level=`cat /etc/SuSE-release | tr "\n" ' ' | sed 's/.*=\ //' | sed 's/\([0-9]*\).*/\1/'`
                platform="SuSE $level"
            else
                platform="Linux - `uname -r`"
            fi
        ;;
        "SunOS")
            level=`echo $unamea | awk '{ print $3 }' | sed 's/[0-9]*\.\([0-9]*\)/\1/'`
            platform="Solaris $level"
            ;;
        "AIX")
            level=`oslevel | sed 's/\([0-9]*\.[0-9]*\).*/\1/'`
            platform="AIX $level"
            ;;
        "HP-UX")
            level=`echo $unamea | awk '{print $3}' | sed 's/[a-zA-Z]*\.\(.*\)/\1/'`
            platform="HP-UX ${level}"
            ;;
        "Darwin")
            level=`echo $unamea | awk '{print $3}'`
            platform="Darwin ${level}"
            ;;
        *)
            platform="UNKNOWN"
            ;;
    esac
}

GetCommandOutput()
{
    # Runs command $1, puts the output into file ${OUT_DIR_CMDS}/$2
    LogMainBasic "Saving output of command <${1}>"    
    eval "$1" >>${OUT_DIR_CMDS}/$2 2>&1
}

LogMain()
{
    echo "Saving <${1}>"
    echo "${1}" >> ${OUT_MAIN}
}

LogMainBasic()
{
    echo "${1}"
    echo "${1}" >> ${OUT_MAIN}
}

GrabFile()
{
    # If it exists, grab file $1.
    if [ -f "${1}" ] ; then
        cp ${1} ${OUT_DIR_FILES}/ 2>/dev/null
        LogMainBasic "Storing file ${1}"
    fi
}

GrabDirectory()
{
    # If it exists, grab directory $1
    if [ -x "${1}" ]; then
        cp -r ${1} ${OUT_DIR_FILES}/ 2>/dev/null
        LogMainBasic "Storing directory ${1}"
    fi
}

GrabLog()
{
    cp ${1} ${2}/ 2>/dev/null
    LogMainBasic "Storing log file ${1}"
}

GrabLogs()
{
    GrabLog "/var/opt/quest/$QMX_DIR/owlogs/qmxlog.txt*" "${1}"
    GrabLog "/var/opt/quest/$QMX_DIR/owlogs/qmx_status.log*" "${1}"
    GrabLog "/var/opt/quest/$QMX_DIR/logs/${QMX_DIR}d.log*" "${1}"
    GrabLog "/var/opt/quest/umi/owcimomd.monitor.log*" "${1}"
    GrabLog "/var/opt/quest/umi/logs/owcimomd.log*" "${1}"

    cp ${1} ${OUT_DIR_FILES}/ 2>/dev/null
}


GetSystemBasics()
{
    LogMainBasic "*** Gathering the system basics ***"
    DeterminePlatform
    LogMain "Hostname:          ${HOSTNAME}"    
    LogMain "Full Hostname:     `hostname`"    
    LogMain "Platform:          ${platform}"
    LogMain "Uname -a:          ${unamea}"
    LogMain "Domainname:        ${DOMAINNAME}"
    LogMain "Date:              ${DATE}"
}

CheckIfInstalled()
{
    case $1 in
        "Linux")
            GetCommandOutput "rpm -qa | grep ${2}" "rpm.quest.out"
            ;;
        "SunOS")
            GetCommandOutput "pkginfo | grep ${2}" "pkginfo.quest.out"
            ;;
        "AIX")
            GetCommandOutput "lslpp -Lc | grep ${2}" "lslpp.quest.out"
            platform="AIX$level"
            ;;
        "HP-UX")
            GetCommandOutput "swlist | grep ${2}" "swlist.quest.out"
            platform="HP-UX${level}"
            ;;
        "Darwin")
            GetCommandOutput "ls -l /Library/Receipts/ | grep ${2}" "receipts.quest.out"
            ;;
    esac
}

GetQMXBasics()
{
    if [ "${HAVE_QMX}" -eq 0 ] ; then
        LogMain "QMX does not appear to be installed, skipping QMX Basics."
        return
    fi
    LogMainBasic "*** Gathering QMX basics ***"   
    LogMain "Clienttool version:   `${QMX} -v 2>&1 | tr '\n' ' '`"
    LogMain "Client ID: `${QMX} --get-client-id  2>&1 | tr '\n' ' ' | awk '{ print $4 }'`"
    unamea=`uname -a`
    name=`echo $unamea | awk '{ print $1 }'`
    CheckIfInstalled $name "quest"
    CheckIfInstalled $name "qmx"
    CheckIfInstalled $name "vintela"
    CheckIfInstalled $name "vmx"
}

GetSystemCommands()
{
    LogMainBasic "*** Gathering output of OS Specific commands ***"
    GetCommandOutput "ipcs -a" "ipcs.a.out"
    GetCommandOutput "df -k" "df.k.out"
    case ${platform} in
        Solaris*)
            GetCommandOutput "showrev -p" "showrev.p.out"
            ;;
        HP-UX*)
            GetCommandOutput "/usr/sbin/swlist -l product *,c=patch" "swlist.l.out"
            GetCommandOutput "/usr/sbin/swlist -l product *,c=patch | grep PHSS_35379" "swlist.l.PHSS_35379.out"
            GetCommandOutput "ls -l /dev/*random*" "ls.dev.random.out"
            GetCommandOutput "ls -la /tcb/files/auth" "tcb.ls.out"
            ;;
        AIX*)
            GetCommandOutput "oslevel -r" "oslevel.r.out"
            GetCommandOutput "oslevel -s" "oslevel.s.out"
            GetCommandOutput "instfix -i" "instfix.i.out"
            ;;
        *)
            ;;
    esac
    GetCommandOutput "ls -ld / /etc /etc/opt /opt /tmp /var /var/opt" "ls.ld.out"
    GetCommandOutput "ls -la /tmp/" "ls.tmp.out"
    GetCommandOutput "uptime" "uptime.out"
    GetCommandOutput "ulimit -a" "ulimit.a.out"

    case $platform in
        Darwin*)
            GetCommandOutput "ps eAux" "ps.eAux.out"
            ;;
        AIX*)
            GetCommandOutput "ps -efo user,group,pid,ppid,vsz,rssize,pmem,cpu,pcpu,start,cputime,wchan,args" "ps.efo.out"
            ;;
        HP-UX*)
            GetCommandOutput "UNIX95= /usr/bin/ps -eo user,group,state,pid,ppid,vsz,sz,stime,time,cpu,pcpu,flags,wchan,args" "ps.eo.out"
            ;;
        Solaris*)
            GetCommandOutput "ps -efo user,group,s,pid,ppid,vsz,rss,pmem,stime,time,pcpu,f,wchan,args" "ps.efo.out"
            ;;
        *)
            GetCommandOutput "ps aux" "ps.aux.out"
            ;;
    esac

    case ${name} in
        Linux*)
            GetCommandOutput "free -m" "free.m.out"
            ;;
        *)
            ;;
    esac
}

GetQMXCommandOutput()
{
    GetCommandOutput "${QMX} ${1}" "${2}" 
}

GetWQLCommandOutput()
{
    GetCommandOutput "${WQL}${1} -w \"select * from ${2}\"" "${3}"
}

ChangeDebugLevel()
{
    # $1 = Directory
    # $2 = Source and Destination filename
    cat $OUT_DIR_FILES/$2 | sed 's/level = \([A-Z1-3]*\)/level = DEBUG3/' \
        | sed 's/max_file_size = \([0-9]*\)/max_file_size = 1000000/' \
        | sed 's/umi.format = \([A-Za-z0-9 -\%]*\)/format = \[\(\%d\{\%d-\%b-\%Y \%H:\%M:\%S\ \%Z\}\) \%r \%t \%p \%c\] - \%m/' > $1/$2
    echo "" >> $1/$2
    echo "$3 = [(%d{%d-%b-%Y %H:%M:%S %Z}) %-6r %5.5t %-5p %c] - %m" \
        >> $1/$2
}

RestartQMX()
{
    if [ "${RESTART:-true}" = "true" ]; then
        GetCommandOutput "${QMX_INIT_SCRIPT} restart" "qmx.restart.out"
    fi
}

IncreaseLogging()
{
    if [ "$INCREASED_LOGGING" = "false" ] ; then

        # Because most of the information is in qmxlog.txt, don't increase
        # the debugging for owcimomd.log.  If it needs to be increased,
        # uncomment these two lines.

        # ChangeDebugLevel /etc/opt/quest/umi/openwbem/openwbem.conf.d \
        #    umi.conf "log.umi.format"
        ChangeDebugLevel /etc/opt/quest/umi/openwbem/openwbem.conf.d \
            qmxsms.conf "log.qmx.format"

        if [ "${GRAB_CURRENT_LOGS}" = "true" ]; then
            GrabLogs "${OLD_LOG_FILES}"
        fi
        RestartQMX

        rm -rf /var/opt/quest/$QMX_DIR/owlogs/qmxlog*
        rm -rf /var/opt/quest/$QMX_DIR/logs/*.log*
        rm -rf /var/opt/quest/umi/owcimomd.monitor.log*
        rm -rf /var/opt/quest/umi/logs/owcimomd.log*
        
        INCREASED_LOGGING="true"
    fi
}

DecreaseLogging()
{
    GrabLogs "${OUT_DIR_FILES}"

    if [ "$INCREASED_LOGGING" = "true" ] ; then
        cp $OUT_DIR_FILES/umi.conf $UMI_CONF
        cp $OUT_DIR_FILES/qmxsms.conf $UMI_QMX_CONF

        if [ "${SOCKET_DUMP:-false}" = "true" ]; then
            if [ "${QMX_DIR}" = "qmxsms" ]; then
                cp $OUT_DIR_FILES/qmxsms.qmxsms.conf ${QMX_CONF}
            else
                cp $OUT_DIR_FILES/qmxcm.conf ${QMX_CONF}
            fi
        fi

        RestartQMX
    fi
}

EnableSocketDump()
{
    echo "${QMX_DIR}.dump_socket_io = ${OUT_SOCKET_DUMP}" >> ${QMX_CONF} 2>/dev/null
}

GetQMXCommands()
{
    if [ "${HAVE_QMX}" -eq 0 ] ; then
        LogMain "QMX does not appear to be installed, skipping QMX Commands."
        return
    fi
    LogMainBasic "*** Gathering output of QMX commands ***"
    for file in ${QMX}; do
        if [ -f $file ] ; then
            GetCommandOutput "file $file" "file.clienttool.out"
            break
        fi
    done
    GetCommandOutput "ls -laR /etc/opt/quest" "ls.etc.out"
    GetCommandOutput "ls -laR /opt/quest/" "ls.opt.out"
    GetCommandOutput "ls -laR /var/opt/quest/" "ls.var.out"

    if [ "${SOCKET_DUMP:-false}" = "true" ]; then
        EnableSocketDump
    fi
    IncreaseLogging

    # For DISCOVERY_UPDATE, true is the default.  It's only false if they
    # supply a command line argument.
    if [ "${DISCOVERY_UPDATE:-true}" = "true" ]; then
        GetQMXCommandOutput "--run-discovery" "qmx.discovery.out"
        GetQMXCommandOutput "--run-policy-update" "qmx.policy.update.out"
    fi

    if [ "${HAS_2_1_TOOLS:-false}" = "true" ]; then
        GetQMXCommandOutput "--run-native-mode-readiness-check" "qmx.readiness.out"
        GetQMXCommandOutput "--run-native-mode-ssl-check" "qmx.ssl.out"
        GetQMXCommandOutput "--show-schedules 5" "qmx.schedules.out"
        GetQMXCommandOutput "--show-maintenance-windows 5" "qmx.windows.out"
        GetQMXCommandOutput "--show-failed-actions" "qmx.failed.actions.out"
    fi

    if [ "${HARDWARE_INVENTORY}" = "true" ]; then
        GetQMXCommandOutput "--run-hardware-inventory" "qmx.hardware.out"
        GrabDirectory "/var/opt/quest/${QMX_DIR}/noidmofs"
        GrabDirectory "/var/opt/quest/${QMX_DIR}/sinv_scripts"
    fi
    if [ "${SOFTWARE_INVENTORY}" = "true" ]; then
        GetQMXCommandOutput "--run-software-inventory" "qmx.software.out"
    fi
    if [ "${SOFTWARE_AVAILABLE}" = "true" ]; then
        GetQMXCommandOutput "--list-available-software" "qmx.list.software.out"
    fi
    if [ "${FILE_COLLECTION}" = "true" ]; then
        GetQMXCommandOutput "--run-file-collection" "qmx.file.collection.out"
    fi
    if [ "${SOFTWARE_METERING}" = "true" ]; then
        GetQMXCommandOutput "--run-software-metering-usage-report" "qmx.metering.out"
        GetWQLCommandOutput "vmx/provider" "CIM_IndicationFilter" "wql.indication.filters.out"
    fi
    if [ "${SOFTWARE_DISTRIBUTION}" = "true" ]; then
        GetQMXCommandOutput "${OPTION_SOFTWARE_DISTRIBUTION} ${ADVERTISEMENT_ID} ${PACKAGE_ID} \"${PROGRAM_ID}\"" "qmx.distribution.out"
    fi
    if [ "${DEBUG_CLASS}" = "true" ]; then
        GetQMXCommandOutput "${OPTION_DEBUG_CLASS} ${CLASS_TO_DEBUG}" "qmx.debug.class.out"
    fi
    DecreaseLogging
}

GetSystemFiles()
{
    LogMainBasic "*** Gathering system files ***"
    LogMainBasic "Storing only localhost and ${HOSTNAME} information from /etc/hosts"
    cat /etc/hosts | grep localhost >> ${OUT_DIR_FILES}/hosts 2>/dev/null
    cat /etc/hosts | grep ${HOSTNAME} >> ${OUT_DIR_FILES}/hosts 2>/dev/null
    if [ ! -z "$SERVERS" ] ; then
        for server in $SERVERS ; do
            cat /etc/hosts | grep $server >> ${OUT_DIR_FILES}/hosts 2>/dev/null
            cat /etc/inet/ipnodes 2>/dev/null | grep $server >> ${OUT_DIR_FILES}/ipnodes 2>/dev/null
        done
    fi    
    GrabFile /etc/resolv.conf
    GrabFile /etc/syslog.conf
}

GetQMXFiles()
{
    if [ "${HAVE_QMX}" -eq 0 ] ; then
        LogMain "QMX does not appear to be installed, skipping QMX files."
        return
    fi
    LogMainBasic "*** Gathering QMX files ***"
    if [ -f /etc/opt/quest/qmxcm/qmxcm.conf ]; then
        GrabFile /etc/opt/quest/qmxcm/qmxcm.conf
    else
        cp /etc/opt/quest/qmxsms/qmxsms.conf ${OUT_DIR_FILES}/qmxsms.qmxsms.conf 2>/dev/null
        LogMainBasic "Storing file /etc/opt/quest/qmxsms/qmxsms.conf"
    fi
    GrabFile /etc/opt/quest/umi/openwbem/openwbem.conf.d/umi.conf
    GrabFile /etc/opt/quest/umi/openwbem/openwbem.conf.d/qmxsms.conf
    GrabFile /etc/opt/quest/umi/openwbem/openwbem.conf.d/qmxsms.allowed_users.conf
}

GetWQLCommands()
{
    LogMainBasic "*** Gathering WQL Commands ***"
    GetWQLCommandOutput "vmx/policy" "CCM_Policy" "wql.policies.out"
    # GetCommandOutput "${WQL}vmx/policy -w \"select * from CCM_Policy\"" "wql.policies.out"
    if [ "${WQL_QUERIES}" = "true" ]; then
        GetCommandOutput "${WQL}vmx/provider -w \"select * from CIM_IndicationFilter\"" "wql.filters.out"
    fi
}

Finish()
{
    LogMainBasic "Script is finished, information is saved in the file ${OUT_BUNDLE}.tar.gz" 
    tar -cf ${OUT_BUNDLE}.tar -C /tmp ${DATE} 2>/dev/null
    gzip ${OUT_BUNDLE}.tar 2>/dev/null
    rm -r ${OUT_DIR} 2>/dev/null
}

################################################################################
# Main
################################################################################
CheckOptions "$@"
Prep
GetSystemBasics
GetQMXBasics
GetQMXFiles
GetSystemCommands
GetQMXCommands
GetWQLCommands
GetSystemFiles
Finish
