#!/bin/sh

# install_umi.sh - Check to see what OS & version we're on and
# install/run UMI (if supported).
#
# Copyright (c) 2004,2005 Quest Software, Inc.
#

# Set environment variables which could potentially result in incorrect
# libraries being loaded or incorrect system commands being executed. CR#42564
PATH="/bin:/usr/bin:/usr/sbin"
UMI_PREFIX=/opt/quest/umi
UMI_DEFAULT_LIB_PATH=${UMI_PREFIX}/lib:/lib:/usr/lib
LD_LIBRARY_PATH=$UMI_DEFAULT_LIB_PATH
LD_LIBRARY_PATH_64=$UMI_DEFAULT_LIB_PATH
SHLIB_PATH=$UMI_DEFAULT_LIB_PATH
LIBPATH=$UMI_DEFAULT_LIB_PATH
DYLD_LIBRARY_PATH=$UMI_DEFAULT_LIB_PATH
DYLD_FALLBACK_LIBRARY_PATH=$UMI_DEFAULT_LIB_PATH

export PATH
export LD_LIBRARY_PATH
export SHLIB_PATH
export LIBPATH
export DYLD_LIBRARY_PATH
export DYLD_FALLBACK_LIBRARY_PATH

# Script will fail if any commands fail.
set -e
set -u

INSTALLATION_ABORTED=0
EXECUTED_FROM_CRON=0
DEBUG_ENABLED=${DEBUG_ENABLED:-0}
ENABLE_DEBUGGING_NOW="set -x"
UPGRADE_OPT=0
STARTUP_DISABLED=0
CACHED_PACKAGE_DIRECTORY=
CACHED_TOOLS_DIRECTORY=
UMI_TEMP_DIRECTORY=
FIXME_ENCOUNTERED=0
ERROR_ENCOUNTERED=0
WARNING_ENCOUNTERED=0
ALL_INSTALLER_ERRORS=
ALL_INSTALLER_WARNINGS=
ALL_INSTALLER_FIXMES=
INSTALL_IGNORE_PATTERN='cksum$'
NON_INSTALL_DIRECTORIES="bin"
PRODUCT_NAME=UMI
OW_LIBRARY_VERSION=7
LIBRARY_BINARY_VERSION=2
UNINSTALL_OPTIONS=
DEBUG_FILE_NAME=/dev/null
#DEBUG_FILE_NAME=/tmp/install_debug.txt


# This is here to force all of the shell utilities to act properly.
LANG=POSIX
export LANG

# Set up descriptors for verbose and debug output
SetupDescriptors()
{
	# Debug descriptor
	exec 7>&-
	if [ ${DEBUG_ENABLED:-0} = 1 ]; then
		exec 7>&2
	else
		exec 7>>${DEBUG_FILE_NAME}
		echo "Reopened debug output on `date`" >&7
	fi

	# Verbose descriptor
	exec 6>&-
	if [ ${QUIET_OPT:-0} != 1 ]; then
		exec 6>&1
	else
		exec 6>>${DEBUG_FILE_NAME}
		echo "Reopened verbose output on `date`" >&6
	fi
}

# Define a bunch of trivial functions for various forms of output.
OutputDebug()     { echo "$@" >&7; }
OutputVerbosely() { echo "$@" >&6; }
OutputError()     { echo "$@" >&2; }
OutputNormal()    { echo "$@"; }
WARNING()
{
	WARNING_ENCOUNTERED=1
	OutputError "WARNING: " "$@"
	ALL_INSTALLER_WARNINGS="${ALL_INSTALLER_WARNINGS:-}${ALL_INSTALLER_WARNINGS:+\n}WARNING: $@"
}
ERROR()
{
	ERROR_ENCOUNTERED=1
	OutputError "ERROR: " "$@"
	ALL_INSTALLER_ERRORS="${ALL_INSTALLER_ERRORS:-}${ALL_INSTALLER_ERRORS:+\n}ERROR: $@"
}
FIXME()
{
	FIXME_ENCOUNTERED=1
	OutputError "FIXME: " "$@"
	ALL_INSTALLER_FIXMES="${ALL_INSTALLER_FIXMES:-}${ALL_INSTALLER_FIXMES:+\n}FIXME: $@"
}
# These functions have inverted logic.
HaveWarnings() { [ ${WARNING_ENCOUNTERED} = 1 ] || return 1; }
HaveErrors()   { [ ${ERROR_ENCOUNTERED} = 1 ] || return 1; }
HaveFixmes()   { [ ${FIXME_ENCOUNTERED} = 1 ] || return 1; }

DisplayErrorState()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if HaveErrors; then
		echo "---------------------------------------------------------------------------"
		echo "                             Error Summary"
		echo "---------------------------------------------------------------------------"
		echo "${ALL_INSTALLER_ERRORS}" | sed 's/\\n/\n/g'
	fi >&2
	if HaveWarnings; then
		echo "---------------------------------------------------------------------------"
		echo "                            Warning Summary"
		echo "---------------------------------------------------------------------------"
		echo "${ALL_INSTALLER_WARNINGS}" | sed 's/\\n/\n/g'
	fi >&2
	if HaveFixmes; then
		echo "---------------------------------------------------------------------------"
		echo "                             FIXME Summary"
		echo "---------------------------------------------------------------------------"
		echo "${ALL_INSTALLER_FIXMES}" | sed 's/\\n/\n/g'
	fi >&2
	if HaveErrors || HaveWarnings || HaveFixmes; then
		echo "---------------------------------------------------------------------------"
	fi >&2
}


AbortInstallation()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ $# -gt 0 ]; then
		ERROR "(aborting)" "$@"
	fi
	INSTALLATION_ABORTED=1
	exit 1
}

# Escape any dots or pluses, so they can be used properly in a regular expression.
EscapeForRegex()
{
	if [ $# -gt 0 ]; then
		echo "$@" | sed -e 's/\([+.]\)/\\\1/g'
	else
		sed -e 's/\([+.]\)/\\\1/g'
	fi
}

EndsWith()
{
	if echo "$1" | grep "`EscapeForRegex $2`\$" >&7; then
		return 0
	fi
	return 1
}

StartsWith()
{
	if echo "$1" | grep "^`EscapeForRegex $2`" >&7; then
		return 0
	fi
	return 1
}

VerifyRootUser()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ "${EUID:-EUID_NOT_SET}" = "0" ] || [ "${UID:-UID_NOT_SET}" = "0" ]; then
		:
	else
		if id | grep 'uid=0' >&7 2>&7; then
			:
		else
			AbortInstallation "This script must be executed as root."
		fi
	fi
	return 0
}

# This is used as an ls replacement, since ls on OSX doesn't return an error
# code.  It relies on the fact that the shell will expand any arguments before
# passing them to this function.  As such, we can use find on them, and find
# will only report an error if they don't exist.  This includes the failure of
# the shell to expand wildcards, so find /foo/barf* wouldn't be expanded, the
# find would get it intact, and return an error code.  When successful, find
# prints out the files
ListFiles()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if find "$@" -print; then
		return 0
	else
		return 1
	fi
}

ListDirectories()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	lf_return_code=1
	while [ $# -gt 0 ]; do
		if test -d "$1" ; then
			echo "$1"
			lf_return_code=0
		fi
		shift
	done
	return $lf_return_code
}

GetTempDir()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	default_temp_directory=/tmp
	if [ ${OSTYPE} = AIX ]; then
		default_temp_directory=/var/tmp
	fi
	echo "${TMPDIR:-${default_temp_directory}}"
	unset default_temp_directory
}

# Make a temporary file while avoiding race conditions and potential symlink attacks.
# Unlike the mktemp command, this will not be guaranteed to create a unique
# name (although it usually will be, unless a name is forced); it only
# guarantees that if it returns a success code, the file (or directory) was
# created and will be empty (overwrites can and will happen).
#
#
# NOTE: The umask for the system is currently ignored (a side effect of
# securely creating the file).  Any file created will only be readable (or
# writeable) by UID 0.
MakeTemp()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	# Local variables...
	make_temp_use_given_name=0
	make_temp_directory=0
	make_temp_try_count=1
	make_temp_successful=0
	make_temp_dest_dir=`GetTempDir`
	make_temp_file_pattern=""
	make_temp_mask=`umask`
	make_temp_perms=""
	make_temp_flags=$-
	umask 077

	if [ x${RANDOM:+set} != xset ]; then
		# Older shells don't provide $RANDOM.
		RANDOM=
	fi

	# Do not bail on errors!
	set +e

	# Parse the given options.
	while [ $# -gt 0 ]; do
		case "$1" in
			-d)
				make_temp_directory=1
				;;
			-f)
				make_temp_use_given_name=1
				;;
			-r)
				make_temp_try_count=$2
				shift
				;;
			-p)
				make_temp_perms="$2"
				shift
				;;
			*)
				make_temp_file_pattern="$1"
				;;
		esac
		shift
	done


	make_temp_base=`basename "$make_temp_file_pattern"`
	make_temp_chosen_dir=`dirname "$make_temp_file_pattern"`

	if StartsWith "${make_temp_file_pattern}" "${make_temp_chosen_dir}"; then
		# No directory was specified, so use the temp dir.
		make_temp_chosen_dir="${TMPDIR:-/tmp}"
	fi

	# Retry as many times as requested until there is a succefful creation.
	while [ $make_temp_try_count -gt 0 ] && [ $make_temp_successful -eq 0 ]; do
		make_temp_try_count=`expr $make_temp_try_count - 1`
		make_temp_target_suffix="${RANDOM}.${RANDOM}.${RANDOM}.${make_temp_try_count}.$$"
		make_temp_temp_dir="${make_temp_dest_dir}/temp_dir.${make_temp_base}.${make_temp_target_suffix}"
		make_temp_created_file=""
		rm -rf $make_temp_temp_dir
		if mkdir $make_temp_temp_dir >&7 2>&1; then
			# Made a directory.
			if [ $make_temp_directory -eq 1 ]; then
				make_temp_created_file="${make_temp_dest_dir}/${make_temp_base}.${make_temp_target_suffix}"
				rm -rf "$make_temp_created_file"
				mkdir -p "`dirname \"$make_temp_created_file\"`"
				if mv -f "$make_temp_temp_dir" "$make_temp_created_file"; then
					:
				else
					ERROR "MakeTemp: Unable to make directory.  A running attack is possible."
					# Clean up some before aborting.
					rm -rf "$make_temp_temp_dir"
					break
				fi
			else
				make_temp_created_file="$make_temp_temp_dir/${make_temp_base}.${make_temp_target_suffix}"
				# Solaris doesn't support the "set -o noclobber" functionality..
				# This isn't as secure as the noclobber method, but broken
				# shells must be dealt with.
				if [ -f "$make_temp_created_file" ]; then
					# Clean up some before trying again.
					rm -rf "$make_temp_temp_dir"
					continue
				else
					# Make a zero-byte file.
					cat <<EOF > "$make_temp_created_file"
EOF
				fi
			fi

			# At this point, we are guaranteed to have a unique (and safe) file
			# and/or directory which has a umask set for the current users use.
			# That is, of course, unless an attack is running with the UID the
			# script is currently running with.

			# If file permissions were specified, set them.
			if [ "x$make_temp_perms" != "x" ]; then
				if chmod "$make_temp_perms" "$make_temp_created_file"; then
					:
				else
					ERROR "MakeTemp: Unable to set chosen permissions ($make_temp_perms) for temp file."
					# Clean up some before aborting the loop.
					rm -rf $make_temp_created_file
					rm -rf $make_temp_temp_dir
					break
				fi
			fi

			# Move the created file into the temp directory now...

			if [ $make_temp_use_given_name -eq 1 ]; then
				# Rename the file to be exactly what they specified.
				make_temp_destfile="${make_temp_chosen_dir}/${make_temp_base}"
				rm -rf "$make_temp_destfile"
				mkdir -p "`dirname \"$make_temp_destfile\"`"
				if mv -f "$make_temp_created_file" "$make_temp_destfile"; then
					make_temp_created_file="$make_temp_destfile"
				else
					ERROR "MakeTemp: Unable to create target: $make_temp_destfile"
					make_temp_created_file=""
				fi
				unset make_temp_destfile
			elif [ $make_temp_directory -eq 0 ] ; then
				make_temp_destfile="$make_temp_chosen_dir/`basename $make_temp_created_file`"
				rm -rf "$make_temp_destfile"
				mkdir -p "$make_temp_chosen_dir"
				if mv -f "$make_temp_created_file" "$make_temp_chosen_dir"; then
					make_temp_created_file="$make_temp_destfile"
				else
					ERROR "MakeTemp: Unable to create target: ${make_temp_chosen_dir}/`basename ${make_temp_created_file}`"
					make_temp_created_file=""
				fi
				unset make_temp_destfile
			else
				# An exact match was not requested, so we'll just use what we have.
				# Things are always safer this way.
				:
			fi

			if [ "x$make_temp_created_file" != "x" ] ; then
				make_temp_successful=1
				echo "$make_temp_created_file"
				rm -rf "$make_temp_temp_dir"
			else
				# Something above failed, so we'll just clean up, and retry if possible.
				rm -rf "$make_temp_temp_dir"
			fi

		else
			# We failed to create a directory.  This will try on the next loop.
			:
		fi
	done

	# Clean up.
	set +$-
	set -$make_temp_flags
	unset make_temp_flags
	umask $make_temp_mask
	unset make_temp_mask
	unset make_temp_base
	unset make_temp_temp_dir
	unset make_temp_use_given_name
	unset make_temp_directory
	unset make_temp_try_count
	unset make_temp_dest_dir
	unset make_temp_chosen_dir
	unset make_temp_created_file
	unset make_temp_file_pattern
	unset make_temp_perms
	unset make_temp_target_suffix

	if [ $make_temp_successful -eq 0 ]; then
		ERROR "MakeTemp: Failed to create desired file or directory."
		unset make_temp_successful
		return 1
	else
		unset make_temp_successful
		return 0
	fi
}

# Safely create a file with the name <arg1>.
SafeCreateFile()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if MakeTemp -f "$1" >&7; then
		:
	else
		ERROR "Unable to create \"$1\": A running local symlink (or other) attack is possible."
		return 1
	fi
}

PKGDIR=`dirname "$0"`
SCRIPT_NAME=`basename "$0"`


if echo "$PKGDIR" | grep '^/' > /dev/null 2>&1; then
	:
else
	if [ "x$PKGDIR" != "x." ]; then
		PKGDIR=`pwd`/"$PKGDIR"
	else
		PKGDIR=`pwd`
	fi
fi

if [ -f "${PKGDIR}/${SCRIPT_NAME}" ] ; then
	:
else
	temp_path=`which "${SCRIPT_NAME}"`
	PKGDIR=`dirname "$temp_path"`
	unset temp_path
fi

PKGDIR=`echo "$PKGDIR" | sed 's| |\\ |g'`

# Change the current directory, as the directory we were running under may be
# removed in some lower part of this script.
cd /

QUIET_OPT=0
UNINSTALL_OPT=0
KEEP_USERS_OPT=0
KEEP_DEV_PACKAGES_OPT=0
FORCE_OPT=0
UMI_1_0_PREFIX=/opt/quest
INSTALL_TYPE=install
BACKUP_PRODUCTS_DUMP_FILE=
FATAL_ERROR=
UNSUPPORTED=0
DETACH_REQUIRED=0
DISABLE_VASHOST_UPGRADE=0
UNINSTALL_SCRIPT_NAME="uninstall_umi.sh"
INIT_SCRIPT="$UMI_PREFIX/sbin/quest-owcimomd_init"
PRNGD_INIT_SCRIPT="$UMI_PREFIX/sbin/quest-prngd_init"
EXISTING_UMI_VERSION=
IGNORE_MISSING_KEYS=0

# If the install detaches (due to parameters or environmental settings), save
# the install log in this file.
DETACHED_LOG_NAME=/tmp/umi_install.log

REPOSITORY_DIRECTORY=/var${UMI_PREFIX}/openwbem/repository

# Backup information for the upgrade process.
BACKUP_DIRECTORY=
BACKUP_OW_CONFIG_FILE=

TERMINATED_EARLY=1
# Print an error message when the shell exits.
ShellExited()
{
	shell_return_value=0
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	# This ShellExited function is not called if we are the parent process and
	# we've already detached.
	CleanupTempDir
	CleanupPackageDir

	if [ "${INSTALL_SCRIPT_DETACHED:-0}" = 1 ]; then
		# Don't print anything if it detached.
		:
	elif [ "$INSTALLATION_ABORTED" = "1" ]; then
		# Don't print anything, because an error was already printed.
		:
	elif [ "$TERMINATED_EARLY" != "0" ]; then
		if [ "x$FATAL_ERROR" != "x" ]; then
			OutputError "$FATAL_ERROR"
		fi
		OutputError "The UMI $INSTALL_TYPE was not successful."
		shell_return_value=1
	fi

	if [ ${INSTALLATION_ABORTED} = 1 ]; then
		shell_return_value=1
	else
		if HaveErrors || HaveFixmes; then
			shell_return_value=1
		fi
	fi

	DisplayErrorState

	# On some less common shells, the return value of the exit handler would
	# occasionally be ignored and the exit value of the script would be zero.  A
	# calling shell script would not see the return value and would continue
	# after an UMI install error.  This corrects that by explicitly exiting with
	# a known value.
   trap "" EXIT
   exit ${shell_return_value}
}


Usage()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ $# -gt 0 ]; then
		ERROR "$@"
	fi

	if [ "x$SCRIPT_NAME" != "x$UNINSTALL_SCRIPT_NAME" ]; then
		echo "Usage: $0 [-r] [-q] [-f] [-d]"
		echo "  -q : Perform an automated quiet install"
		echo "  -f : Force install on an unsupported platform."
		echo "  -r : Remove (uninstall) umi client."
		echo "  -d : Detach the install script from the parent process."
		echo "  -t : Display required source file expression and exit"
	else
		echo "Usage: $0 [-q]"
		echo "  -q : Perform a quiet uninstall"
	fi
	AbortInstallation
}

CreateTempDir()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ x${UMI_TEMP_DIRECTORY:+set} = xset ]; then
		if [ -d "${UMI_TEMP_DIRECTORY}" ]; then
			OutputVerbosely "Using temp directory: ${UMI_TEMP_DIRECTORY}"
			return 0
		else
			OutputVerbosely "Temp directory disappeared (will create anew): ${UMI_TEMP_DIRECTORY}"
			unset UMI_TEMP_DIRECTORY
		fi
	fi
	if [ x${UMI_TEMP_DIRECTORY:+set} != xset ]; then
		create_temp_dir_failure=0
		# On some shells, if this assignment is done within the if statement, it
		# doesn't actually set the global variable.
		UMI_TEMP_DIRECTORY="`MakeTemp -d umi_temp_space`" || create_temp_dir_failure=$?
		if [ ${create_temp_dir_failure} = 0 ]; then
			if StartsWith "${UMI_TEMP_DIRECTORY}" "/"; then
				:
			else
				ERROR "Temp directory is not an absolute path!"
				return 1
			fi
		else
			ERROR "Failed to create UMI temp directory."
			return ${create_temp_dir_failure}
		fi
		OutputVerbosely "Created temp directory ${UMI_TEMP_DIRECTORY}"
	fi
}

CleanupTempDir()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ x${UMI_TEMP_DIRECTORY:+set} = xset ]; then
		OutputVerbosely "Cleaning up temp directory: ${UMI_TEMP_DIRECTORY}"
		rm -rf ${UMI_TEMP_DIRECTORY}
		unset UMI_TEMP_DIRECTORY
		OutputVerbosely "Done."
	fi
}

CleanupPackageDir()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ "x${UMI_REMOVE_PACKAGE_DIR:-0}" = "x1" ] || [ "x${EXECUTED_FROM_CRON:-0}" = "x1" ]; then
		OutputVerbosely "Cleaning up temporary package directory: $PKGDIR"
		rm -rf "$PKGDIR"
		OutputVerbosely "Done."
	fi
}

PrintUnsupportedMessage()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	echo "The platform you are attempting to install on is currently unsupported."
	echo "Please contact sales@quest.com to request support for additional"
	echo "platforms.  You may continue the installation (at your own risk), but"
	echo "we will not be able to provide support for it at this time."
	echo
	echo "For reference, the following information was determined:"
	variant_text=""
	if [ "x${OSVARIANT}" = "x" ]; then
		:
	else
		variant_text="   variant=${OSVARIANT}"
	fi
	echo "arch=${OSARCH}   ostype=${OSTYPE}   vendor=${OSVEND}${variant_text}   version=${OSVER}"
	UNSUPPORTED=1
}

# Variables that will be set:
#
# OSTYPE = output of uname (converted to uppercase): LINUX
# OSVEND = vendor (RedHat, UnitedLinux, SuSE)
# OSVER = version of the Linux distro.
# The values will be UNKNOWN if the script is unable to determine.
GetPlatformInformation()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	# Some shells require that a variable be set before unsetting.
	OSTYPE=""
	OSVEND=""
	OSVER=""
	OSARCH=""
	OSVARIANT=""
	unset OSTYPE OSVEND OSVER OSARCH

	OSTYPE=`uname | tr '[a-z]' '[A-Z]'`

	if uname -p >&7 2>&1 ; then
		# This seems to be supported on everything except HP-UX.
		OSARCH=`uname -p`

		# Some new versions of linux are sticking the
		# *complete* processor info in the -p output.  Use -m.
		for processor in intel amd processor; do
			if echo "$OSARCH" | grep -i $processor >&7; then
				OSARCH=unknown
				break
			fi
		done
	else
		OSARCH=`uname -m`
	fi

	if [ "x$OSARCH" = "xunknown" ]; then
		OSARCH=`uname -m`
	fi

	# If Red Hat...
	if [ -f /etc/redhat-release ]; then
		# Prevent false-positives from some Red Hat derivatives...
		if grep "Red Hat" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=RedHat
			if rpm -q redhat-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' redhat-release`
			elif rpm -q redhat-release-as >&7 2>&1 ; then
				# RedHat AS 2.1
				OSVER=`rpm -q --qf '%{VERSION}' redhat-release-as`
			elif rpm -q redhat-release-ws >&7 2>&1 ; then
				# RedHat WS 2.1
				OSVER=`rpm -q --qf '%{VERSION}' redhat-release-ws`
			elif rpm -q redhat-release-es >&7 2>&1 ; then
				# RedHat ES 2.1
				OSVER=`rpm -q --qf '%{VERSION}' redhat-release-es`
			elif rpm -q vmware-release >&7 2>&1 ; then
				OSVEND=VMWARE
				OSVER=`rpm -q --qf '%{VERSION}' vmware-release`

				# If the VMware-esx package is installed, we can extract more
				# detailed version information from it.
				if rpm -q --queryformat '%{VERSION}' VMware-esx >&7 2>&1 ; then
					OSVER=`rpm -q --qf '%{VERSION}' VMware-esx`
					OSVARIANT=ESX
				fi
			else
				# Add other RedHat versions here when needed.
				OSVER=UNKNOWN
			fi
		elif [ -f /etc/fedora-release ]; then
                        if grep "Fedora" /etc/fedora-release >&7; then
                           OSVEND=Fedora
                        else
                           if grep "STS" /etc/fedora-release >&7; then
                             OSVEND=Fedora
                           fi
                        fi
                        if rpm -q fedora-release >&7 2>&1; then
                                OSVER=`rpm -q --qf '%{VERSION}' fedora-release`
                        else
                                OSVER=UNKNOWN
                        fi
		elif grep "CentOS" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=CentOS
			if rpm -q centos-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' centos-release`
			fi
		elif grep -i "slf" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=SLF
			if rpm -q Fermi-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' Fermi-release`
			fi
		elif grep -i "lts" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=SLF
			if rpm -q Fermi-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' Fermi-release`
			fi
		elif grep -i "Fermi" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=SLF
			if rpm -q Fermi-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' Fermi-release`
			fi
		elif grep "Scientific" /etc/redhat-release >&7 2>&1 ; then
			OSVEND=SL
			if rpm -q sl-release >&7 2>&1 ; then
				OSVER=`rpm -q --qf '%{VERSION}' sl-release`
			fi
		else
			# A RedHat clone...
			OS_VENDOR_TEXT=`cat /etc/redhat-release  | head -1 | awk '{ print $1 }' | tr '[A-Z]' '[a-z]'`
			if rpm -q ${OS_VENDOR_TEXT}-release >&7 2>&1 ; then
				OSVEND=`rpm -q --qf '%{VENDOR}' ${OS_VENDOR_TEXT}-release | sed 's/ /_/g'`
				OSVER=`rpm -q --qf '%{VERSION}' ${OS_VENDOR_TEXT}-release`

				# Handle items like Fedora.
				if echo "${OSVEND}" | egrep 'Red(()|[ _])Hat' >&7 2>&1 ; then
					OSVEND="RedHat"
					OSVER="${OS_VENDOR_TEXT}${OSVER}"
				fi
			else
				OSVEND=UNKNOWN
				OSVER=UNKNOWN
			fi
		fi
	# If SuSE...
	elif [ -f /etc/SuSE-release ]; then
		OSVEND=SuSE
		OSVER=`grep VERSION /etc/SuSE-release | cut -d' ' -f3`
	elif [ -f /etc/gentoo-release ]; then
		OSVEND=Gentoo
		OSVER=`grep -i VERSION /etc/gentoo-release | sed -e 's/[^0-9.]//g'`
	# If UnitedLinux...
	elif [ -f /etc/UnitedLinux-release ]; then
		OSVEND=UnitedLinux
		OSVER=`grep VERSION /etc/UnitedLinux-release | cut -d' ' -f3`
	# VMware ESX (at least 2.5) does not provide a release RPM nor does it have
	# a release file in /etc.
	elif [ "x${OSTYPE}" = "xLINUX" ] && rpm -q --queryformat '%{VERSION}' VMware-esx >&7 2>&1 ; then
		# Some version of VMware ESX.
		OSVEND=VMWARE
		OSVER=`rpm -q --queryformat '%{VERSION}' VMware-esx`
		OSVARIANT=ESX
	elif [ -f /usr/bin/lsb_release ]; then
		OSVEND=`/usr/bin/lsb_release -i | sed 's/^[^:]*:[ 	]*//g'`
		OSVER=`/usr/bin/lsb_release -r | sed 's/^[^:]*:[ 	]*//g'`
	# AIX
	elif [ "x$OSTYPE" = "xAIX" ]; then
		OSVEND=IBM
		OSVER=`uname -v`.`uname -r`
	# HPUX
	elif [ "x$OSTYPE" = "xHP-UX" ]; then
		OSVEND=HP
		OSVER=`uname -r`
	# Darwin (OS-X)
	elif [ "x$OSTYPE" = "xDARWIN" ]; then
		OSVEND=Apple
		if sw_vers -productVersion >&7 2>&1 ; then
			OSVER=`sw_vers -productVersion 2>&7`
			OSVARIANT="MacOS"
		else
			OSVER=`uname -r`
		fi
	# Solaris
	elif [ "x$OSTYPE" = "xSUNOS" ]; then
		OSVEND=Sun
		OSVER=`uname -r`
		OSSUBVER=`echo ${OSVER} | sed -e 's/.*\.\(.*\)/\1/'`

	# If we've gone this far, we're not on a supported distro...
	else
		OSVEND=UNKNOWN
		OSVER=UNKNOWN
	fi
	# Make it uppercase.
	OSVER=`echo $OSVER | tr '[a-z]' '[A-Z]'`

	# Convert all of those pesky arch numbers to something that can be used more easily.
	case "$OSARCH" in
		i386 | i486 | i586 | i686 | *[Aa]thlon*)
			OSARCH="x86"
			;;
		9000/*)
			OSARCH="parisc"
			;;
		ppc | [Pp]owerpc)
			OSARCH="ppc"
			;;
		*)
			;;
	esac

	# Get the MAJOR, MINOR, and MICRO version segments.
	OS_MAJOR=`echo $OSVER | awk -F. '{ print $1; }'`
	OS_MINOR=`echo $OSVER | awk -F. '{ print $2; }'`
	OS_MICRO=`echo $OSVER | awk -F. '{ print $3; }'`
}


AskUnsupportedQuestion()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	[ $FORCE_OPT = 1 ] && return 0
	[ $QUIET_OPT = 1 ] && AbortInstallation "Unsupported platform: Refusing to install quietly without being forced (-f)."

	PrintUnsupportedMessage
	ans=
	while [ x$ans = x ]; do
		echo
		echo "Do you want to $INSTALL_TYPE anyway?"
		read reply leftover
		case $reply in
			y* | Y*)
				return 0;;
			n* | N*)
				return 1;;
		esac
	done
}

CheckForSupportedPlatform()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3WS" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3ES" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3AS" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3DESKTOP" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4WS" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4ES" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4AS" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4DESKTOP" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3WS" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3ES" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3AS" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "3DESKTOP" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4WS" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4ES" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4AS" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "4DESKTOP" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "5SERVER" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "5CLIENT" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "5SERVER" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "5CLIENT" ] && return 0;

	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OS_MAJOR = "5" ] && [ ${OS_MINOR:-0} = "0" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "RedHat" ] && [ $OS_MAJOR = "5" ] && [ ${OS_MINOR:-0} = "0" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "2.1WS" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "2.1ES" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "RedHat" ] && [ $OSVER = "2.1AS" ] && return 0;

	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "CentOS" ] && [ $OSVER = "4" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "CentOS" ] && [ $OSVER = "4" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "CentOS" ] && [ $OSVER = "5" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "CentOS" ] && [ $OSVER = "5" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "3" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "3" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "3" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "3" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "4" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "4" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "4" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "4" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "5" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SLF" ] && [ $OS_MAJOR = "5" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "5" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SL" ] && [ $OS_MAJOR = "5" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "Fedora" ] && [ $OS_MAJOR = "10" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "Fedora" ] && [ $OS_MAJOR = "10" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "Fedora" ] && [ $OS_MAJOR = "11" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "Fedora" ] && [ $OS_MAJOR = "11" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SuSE" ] && [ $OS_MAJOR = "8" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SuSE" ] && [ $OS_MAJOR = "9" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SuSE" ] && [ $OS_MAJOR = "10" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "SuSE" ] && [ $OSVER = "11" ] && return 0;

	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SuSE" ] && [ $OS_MAJOR = "9" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SuSE" ] && [ $OS_MAJOR = "10" ] && return 0;
	[ "$OSARCH" = "x86_64" ] && [ "$OSVEND" = "SuSE" ] && [ $OSVER = "11" ] && return 0;

	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "VMWARE" ] && [ $OS_MAJOR = "2" ] && [ $OS_MINOR = "5" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "VMWARE" ] && [ $OSVER = "3" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "VMWARE" ] && [ $OS_MAJOR = "3" ] && [ $OS_MINOR = "0" ] && return 0;
	[ "$OSARCH" = "x86" ] && [ "$OSVEND" = "VMWARE" ] && [ $OS_MAJOR = "3" ] && [ $OS_MINOR = "5" ] && return 0;
	[ "$OSVEND" = "IBM" ] && [ "$OSTYPE" = "AIX" ] && [ $OSVER = "5.1" ] && return 0;
	[ "$OSVEND" = "IBM" ] && [ "$OSTYPE" = "AIX" ] && [ $OSVER = "5.2" ] && return 0;
	[ "$OSVEND" = "IBM" ] && [ "$OSTYPE" = "AIX" ] && [ $OSVER = "5.3" ] && return 0;
	[ "$OSVEND" = "IBM" ] && [ "$OSTYPE" = "AIX" ] && [ $OSVER = "6.1" ] && return 0;
	[ "$OSARCH" = "parisc" ] && [ "$OSVEND" = "HP" ] && [ "$OSTYPE" = "HP-UX" ] && [ $OSVER = "B.11.11" ] && return 0;
	[ "$OSARCH" = "parisc" ] && [ "$OSVEND" = "HP" ] && [ "$OSTYPE" = "HP-UX" ] && [ $OSVER = "B.11.23" ] && return 0;
	[ "$OSARCH" = "parisc" ] && [ "$OSVEND" = "HP" ] && [ "$OSTYPE" = "HP-UX" ] && [ $OSVER = "B.11.31" ] && return 0;
	[ "$OSARCH" = "ia64" ] && [ "$OSVEND" = "HP" ] && [ "$OSTYPE" = "HP-UX" ] && [ $OSVER = "B.11.23" ] && return 0;
	[ "$OSARCH" = "ia64" ] && [ "$OSVEND" = "HP" ] && [ "$OSTYPE" = "HP-UX" ] && [ $OSVER = "B.11.31" ] && return 0;
	[ "$OSVEND" = "Apple" ] && [ "$OSTYPE" = "DARWIN" ] && [ "$OSVARIANT" = "MacOS" ] && [ $OS_MAJOR = "10" ] && [ $OS_MINOR = "4" ] && return 0;
	[ "$OSVEND" = "Apple" ] && [ "$OSTYPE" = "DARWIN" ] && [ "$OSVARIANT" = "MacOS" ] && [ $OS_MAJOR = "10" ] && [ $OS_MINOR = "5" ] && return 0;
	[ "$OSVEND" = "Apple" ] && [ "$OSTYPE" = "DARWIN" ] && [ "$OSVARIANT" = "MacOS" ] && [ $OS_MAJOR = "10" ] && [ $OS_MINOR = "6" ] && return 0;

	# These are the kernel versions, not the OS version.
	[ "$OSVEND" = "Apple" ] && [ "$OSTYPE" = "DARWIN" ] && [ $OS_MAJOR = "7" ] && return 0;
	[ "$OSVEND" = "Sun" ] && [ "$OSTYPE" = "SUNOS" ] && [ $OSVER = "5.8" ] && [ $OSARCH = sparc ] && return 0;
	[ "$OSVEND" = "Sun" ] && [ "$OSTYPE" = "SUNOS" ] && [ $OSVER = "5.9" ] && return 0;
	[ "$OSVEND" = "Sun" ] && [ "$OSTYPE" = "SUNOS" ] && [ $OSVER = "5.10" ] && return 0;

	# unsupported platform
	if [ $QUIET_OPT = 1 ] && [ $FORCE_OPT = 0 ]; then
		return 1;
	elif [ $QUIET_OPT = 1 ] && [ $FORCE_OPT = 1 ]; then
		return 0;
	elif [ $QUIET_OPT = 0 ] && [ $FORCE_OPT = 1 ]; then
		PrintUnsupportedMessage
		return 0;
	fi

	return 1;
}

ParseCommandLine()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	while [ $# -gt 0 ]; do
		case "$1" in
			"")
				# Empty args should just be eaten.
				;;
			"-q" | "--quiet")
				QUIET_OPT=1;;
			"-f" | "--force")
				FORCE_OPT=1;;
			"-r" | "--uninstall")
				# If a -r and a --partial-uninstall were both specified, then we
				# don't want to have any later -r change to a full uninstall.
				if [ ${UNINSTALL_OPT:-0} = 0 ]; then
					UNINSTALL_OPT=1
					INSTALL_TYPE=uninstall
				fi
				;;
			"--partial-uninstall")
				UNINSTALL_OPT=1
				INSTALL_TYPE=partial_uninstall
				;;
			"--remove-compatibility-packages")
				# This option is for internal use only.
				# It tells the script to remove the packages that were obsoleted by
				# the install of this latest client version.  These packages will
				# be the libraries which external providers will depend.  The
				# compatibility packages will only be removed if there is ONE
				# registered product, since it will be the upgrade script for that
				# product which is requesting the compatibility removal after it
				# has upgraded itself.
				INSTALL_TYPE=remove_compatibility
				;;
			"-t" | "--display-regex")
				DisplayPackageRegex
				TERMINATED_EARLY=0
				exit 0
				;;
			"-cron")
				EXECUTED_FROM_CRON=1
				;;
			"-setpath")
				PATH="$2"
				export PATH
				shift
				;;
			"-debug" | "--debug")
				# This option isn't meant to be used by everyone.  It's here to
				# allow some way to propagate the debugging flags from one function
				# to another (some shells discard them).
				DEBUG_ENABLED=1
				UNINSTALL_OPTIONS="${UNINSTALL_OPTIONS:-} -debug"
				${ENABLE_DEBUGGING_NOW}
				;;
			"--check-security")
				TERMINATED_EARLY=0
				CheckSecurity
				exit 0
				;;
			"-nostartup")
				STARTUP_DISABLED=1
				;;
 			"-d" | "--detach")
 				DETACH_REQUIRED=1
 				;;
 			"--disable-vas-upgrade")
 				DISABLE_VAS_UPGRADE=1
 				;;
			"--keep-umi-users")
				# This is meant for internal use only.
				# Using this option will prevent the users from being removed when
				# uninstalling.
				KEEP_USERS_OPT=1
				;;
			"--keep-dev-packages")
				# This is meant for internal use only.
				# Using this option will prevent packages from being uninstalled
				# that are required for devopment.
				KEEP_DEV_PACKAGES_OPT=1
				;;
			"--developer-mode")
				# This is meant for internal use only.
				# Using this option will auto-enable options that are needed for
				# development.  For now, it is the same as using both
				# --keep-dev-packages and --keep-umi-users
				KEEP_USERS_OPT=1
				KEEP_DEV_PACKAGES_OPT=1
				;;
			*)
				Usage "Invalid option: $1" >&2
				;;
		esac
		shift
	done
}

Start()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ "x$INIT_SCRIPT" != "xUNKNOWN" ]; then
		$INIT_SCRIPT start || return 1
	else
		LOCAL_ERROR="I don't know how to start the cimom--No startup script found."
		if [ $# -lt 1 -o "$1" != "-q" ]; then
			ERROR "$LOCAL_ERROR"
		else
			FATAL_ERROR="$LOCAL_ERROR"
		fi
		return 1
	fi
	return 0
}

Stop()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ "x$INIT_SCRIPT" != "xUNKNOWN" ]; then
		[ -x "$INIT_SCRIPT" ] && { $INIT_SCRIPT stop || return 1; }
	else
		if [ $# -lt 1 -o "$1" != "-q" ]; then
			ERROR "I don't know how to stop the cimom--No startup script found."
		fi
		return 1
	fi
	return 0
}

StartPrngd()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ "x${PRNGD_INIT_SCRIPT}" != "xUNKNOWN" ]; then
		${PRNGD_INIT_SCRIPT} start || return 1
	else
		LOCAL_ERROR="quest-prngd init script not found."
		if [ $# -lt 1 -o "$1" != "-q" ]; then
			ERROR "$LOCAL_ERROR"
		else
			FATAL_ERROR="$LOCAL_ERROR"
		fi
		return 1
	fi
	return 0
}

StopPrngd()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ "x${PRNGD_INIT_SCRIPT}" != "xUNKNOWN" ]; then
		[ -x "${PRNGD_INIT_SCRIPT}" ] && { ${PRNGD_INIT_SCRIPT} stop || return 1; }
	else
		if [ $# -lt 1 -o "$1" != "-q" ]; then
			ERROR "quest-prngd init script not found."
		fi
		return 1
	fi
	return 0
}

# This will set the variable MENU_CHOICE for the choice the user selected.
# $1 = pattern
# $2 (hidden) = Flag to control if the "more" option is available.
SelectDirectoryFromMenu()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	MENU_CHOICE=
	MENU_SELECTION_MADE=0
	MENU_FILE_PATTERN="$1"
	# Make sure that nobody else can cause problems while displaying the
	# menu (writing to temp files and such).
	TEMP_MASK=`umask`
	umask 077

	TEMP_FILE=/tmp/umi_install.choices

	SafeCreateFile "$TEMP_FILE"

	if ListDirectories $MENU_FILE_PATTERN > $TEMP_FILE; then

		SafeCreateFile "${TEMP_FILE}.2"

		# Skip a bunch of directories that can't contain valid install files.
		for skip_directory in ${NON_INSTALL_DIRECTORIES}; do
			grep -v "/${skip_directory}\$" "${TEMP_FILE}" > "${TEMP_FILE}.2" || true
			mv "${TEMP_FILE}.2" "${TEMP_FILE}"
		done
		rm -f "${TEMP_FILE}.2" 2>&7

		num_choices=`cat $TEMP_FILE | wc -l`
		num_choices=`expr $num_choices`

		# Display a menu and read the user's data.
		display_menu()
		{
			[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
			menu_choice_number=0
			echo "----------------------------------------------------------------"
			if [ "$UNSUPPORTED" != 1 ]; then
				echo "No package found for supported OS $OSVEND $OSTYPE $OSVER"
			else
				echo "Your platform $OSVEND $OSTYPE $OSVER is unsupported by ${PRODUCT_NAME}."
			fi
			echo "You may try packages from the following platforms:"
			echo
			index=1
			while [ $index -le $num_choices ]; do
				package=`cat $TEMP_FILE | head -n $index | tail -1`
				base_package=`basename $package`
				echo "	($index) $base_package"
				index=`expr $index + 1`
			done
			echo
			# An evil hack to allow the user to specify they want more choices.
			if [ ${1:-MORE} = MORE ]; then
				echo "	(-1) Show more choices"
			fi
			echo "	(0) Quit without installing"
			echo "----------------------------------------------------------------"
			echo "Please select an item from 0 to $num_choices:"

			read reply leftover
			if echo $reply | egrep '^[ \t]*-?[0-9]+[ \t]*$' >&7 2>&1 ; then
				reply=`echo $reply`

				# An evil hack to allow the user to specify they want more choices.
				if [ x${reply} = x-1 ]; then
					if [ ${1:-MORE} = MORE ]; then
						reply=255
					else
						reply=127
					fi
				fi
				menu_choice_number=$reply
			else
				menu_choice_number=127
			fi
			return $menu_choice_number
		}

		if [ $QUIET_OPT = 1 ]; then
			# Take the first thing on the list.
			choice=1
		else
			# Repeat the menu until a valid choice is made.
			choice=0
			display_menu ${2:-MORE} || choice=$?
			while [ $choice -eq 127 ] || [ $choice -gt $num_choices ]; do

				# An evil hack to allow the user to specify they want more choices.
				if [ ${2:-MORE} = MORE ]; then
					if [ ${menu_choice_number} = 255 ]; then
						SelectDirectoryFromMenu "`dirname \"$1\"`/*" NO-MORE || return $?
						return 0
					fi
				fi

				echo
				echo "Invalid selection: $reply"
				echo
				choice=0
				display_menu ${2:-MORE} || choice=$?
			done
		fi

		if [ $choice -gt 0 ]; then
			MENU_CHOICE=`cat $TEMP_FILE | head -n $choice | tail -1 | sed 's| |\\ |g'`
			MENU_SELECTION_MADE=1
		else
			FATAL_ERROR="User quit requested"
			umask $TEMP_MASK
			rm -f $TEMP_FILE
			return 1
		fi
	fi
	umask $TEMP_MASK
	rm -f $TEMP_FILE
}

# Do not run this function unless you are piping the output to something.  It
# will not exit until the echo gets a pipe error or is terminated by some other
# signal.
AlwaysSayYes()
{
	while echo y ; do
		:
	done
}

GetPlistKey()
{
	# $1=file
	# $2=key name
	#
	# This function assumes that the key you are searching for is unique among
	# the plist file.  No real XML parsing is done, as this would be a true pain
	# in shell script.  NOTE: When multiple keys have values, the last one found
	# is the one that is returned.

	plist_key_retval=
	plist_key_error=1
	temp_output_file=`MakeTemp /tmp/dmg_installer.plist_key.$$`
	if grep -A 1 "<key>$2</key>" "$1" > ${temp_output_file} 2>&7; then
		key_data_line="`tail -1 ${temp_output_file} | sed 's/^[ 	]*//g'`"
		# Possible values (ones that we expect) are:
		#   <true/>
		#   <false/>
		#   <string>.*</string> or <string/>
		#   <integer>.*</integer>
		#   <real>.*</real>
		#   <date>.*</date>
		# Everything else here is considered invalid (difficult to handle, and non-useful for us).
		#   <array>.*</array>
		#   <data>.*</data>
		#   <dict>.*</dict>
		GetPlistKeyInnerData()
		{
			echo "${key_data_line}" | sed "s?<$1>\(.*\)</$1>?\1?g"
		}
		case ${key_data_line} in
			'<true/>') plist_key_retval="true"; plist_key_error=0 ;;
			'<false/>') plist_key_retval="false"; plist_key_error=0 ;;
			'<string>'*'</string>') plist_key_retval="`GetPlistKeyInnerData string`"; plist_key_error=0 ;;
			'<integer>'*'</integer>') plist_key_retval="`GetPlistKeyInnerData integer`"; plist_key_error=0 ;;
			'<real>'*'</real>') plist_key_retval="`GetPlistKeyInnerData real`"; plist_key_error=0 ;;
			'<date>'*'</date>') plist_key_retval="`GetPlistKeyInnerData date`"; plist_key_error=0 ;;
			'<array>'* | '<dict>'* | '<data>'*) echo "ERROR: Array, dict, and data properties cannot be returned." >&2;;
			*) echo "Unexpected property line: \"${key_data_line}\"" ;;
		esac

		if [ ${plist_key_error} != 1 ]; then
			echo "${plist_key_retval}"
		fi
	else
		echo "Key ($2) not found in file: $1." >&2
	fi
	rm -f ${temp_output_file}
	return ${plist_key_error}
}


# This function will query an installed package for the version number.
#
# This function can be used to determine if the package database is hosed.
# This should hopefully allow errors to be caught before the cimom is shut
# down.  Using it this way is mainly targeted at Redhat8 (which the RPM
# database gets locked quite frequently), everything else just does a simple
# query (the results don't matter, so long as it returns).
QueryExistingPackage()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	package_to_query=quest-umi
	query_result=0
	if [ $# -gt 0 ]; then
		package_to_query="$1"
	fi

	case $OSTYPE in
		"LINUX")
			# note that rpmquery doesn't exist on some older distros (e.g. SUSE 8)
			if rpm -q --queryformat '%{VERSION}.%{RELEASE}' $package_to_query >&7 2>&1 ; then
				# The sed used here removes most of the vendor-specific tags that
				# appear at the end of the release number.  Similar actions are
				# done when guessing versions from filenames elsewhere in this
				# script.
				rpm -q --queryformat '%{VERSION}.%{RELEASE}' $package_to_query | sed -e 's/\.[a-zA-Z].*$//'
			else
				query_result=$?
			fi
			;;
		"SUNOS")
			# Solaris packages have their dots converted into dashes.  Do that for the query now.
			package_to_query=`echo ${package_to_query} | sed 's/\./-/g'`
			if /usr/bin/pkginfo -i ${package_to_query} >&7 2>&1 ; then
				/usr/bin/pkginfo -il ${package_to_query} | grep -i version | sed 's/^[ 	]*VERSION:[ 	]*//g' || true
			else
				query_result=$?
			fi
			;;
		"DARWIN")
			# 10.6 uses pkgutil db, but upgraded systems will still have files in /LibraryReceipts
			# Just look to see if /Library/Receipts/${package_to_query}.pkg exists.
			if ls -1 /Library/Receipts | grep -i "${package_to_query}\\.pkg" >&7 2>&1 ; then
				if pkg_version_key=`GetPlistKey /Library/Receipts/${package_to_query}.pkg/Contents/Info.plist CFBundleGetInfoString`; then
					# Delete everything prior to the last space.
					echo "${pkg_version_key}" | sed 's/.*[ ][ ]*//g'
				else
					temp_query_result=$?

					if [ ${IGNORE_MISSING_KEYS:-0} = 1 ]; then

						# There have been cases where the Info.plist files do not
						# contain any useful version information.  While this is
						# probably a bug in the package, we want to treat it as
						# installed.
						echo UNKNOWN
					else
						query_result=${temp_query_result}
					fi
				fi
			else
				# 10.6 exclusively uses the pkgutil db

				# The pkgutil db changes dots in package names to hyphens
				# i.e. quest-openssl-0.9.8 changes to quest-openssl-0-9-8
				munged_package_to_query=`echo ${package_to_query} | sed 's/\./-/g'`
				if pkgutil --pkgs=com.quest.${munged_package_to_query} >&7 2>&1 ; then
					pkg_version=`pkgutil --pkg-info com.quest.${munged_package_to_query} | grep version`
					# Delete everything prior to the last space
					echo "${pkg_version}" | sed 's/.*[ ][ ]*//g'
				else
					query_result=$?
				fi
			fi
			;;
		"HP-UX")
			# HP-UX packages have their dots converted into underscores.
			# Unfortunately, this means that a subpackage cannot be queried.
			package_to_query=`echo ${package_to_query} | sed 's/\./_/g'`
			if /usr/sbin/swlist ${package_to_query} >&7 2>&1 ; then
				/usr/sbin/swlist  -a revision ${package_to_query} | grep "#[ 	]*${package_to_query}" | sed 's/^.*[ 	]\([0-9].*\)/\1/g'
			else
				query_result=$?
			fi
			;;
		"AIX")
			if lslpp -hcq $package_to_query >&7 2>&1 ; then
				lslpp -hcq $package_to_query | cut -f3 -d':'
			else
				# Try again with a wildcard, because some package names have suffixes like .AIX_5_1.rte
				if lslpp -hcq $package_to_query.\* >&7 2>&1 ; then
					lslpp -hcq $package_to_query.\* | cut -f3 -d':'
				else
					query_result=$?
				fi
			fi
			;;
		*)
			ERROR "Query method unknown on $OSTYPE"
			query_result=1
			;;
	esac

	return $query_result
}

PackageIsCompressed()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	for allowed_compression_extension in .gz .bz2; do
		if EndsWith "$1" ${allowed_compression_extension}; then
			return 0
		fi
	done
	return 1
}

DecompressPackage()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ $# -gt 1 ]; then
		ERROR "Cannot decompress more than one package at a time."
		return 1
	fi
	decompress_package_name="$1"
	decompress_package_suffix=
	decompressor=
	if EndsWith "${decompress_package_name}" .gz; then
		decompressor=gzip
		decompress_package_suffix=.gz
	elif EndsWith "${decompress_package_name}" .bz2; then
		decompressor=bzip2
		decompress_package_suffix=.bz2
	else
		ERROR "Package (${decompress_package_name}) is not compressed."
		return 1
	fi

	CreateTempDir >&7 || return $?

	decompress_output_file="${UMI_TEMP_DIRECTORY}/`basename \"${decompress_package_name}\" ${decompress_package_suffix}`"

	# Make sure the tools directory has set the proper variables.
	GetToolsDirectory >&7 || return $?
	if	"${CACHED_TOOLS_DIRECTORY}/${decompressor}" -d -c "${decompress_package_name}" > "${decompress_output_file}"; then
		echo "${decompress_output_file}"
		return 0
	else
		decompress_return_value=$?
		ERROR "Failed to decompress package \"${decompress_package_name}\" with ${decompressor}"
		return ${decompress_return_value}
	fi
}

InstallPackage()
{
	have_error=0
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	while [ $# -gt 0 ] && [ ${have_error} = 0 ]; do
		original_package_path="$1"
		package_to_install="${original_package_path}"
		shift

		if PackageIsCompressed "${package_to_install}"; then
			CreateTempDir
			if package_to_install="`DecompressPackage \"${original_package_path}\"`"; then
				:
			else
				ERROR "Failed to decompress package \"${original_package_path}\""
				return 1
			fi
		fi

		OutputVerbosely "Installing package $package_to_install ..."

		case $OSTYPE in
			"LINUX")
				# RPM apparently needs extra backslashes (beyond other installers).
				escaped_package_name="`echo \"$package_to_install\" | sed 's| |\\\\ |g'`"

				# Explicitly set the umask, as rpm has been shown to create invalid
				# permissions (causing problems with the client's unprivileged
				# users) if installed with a more restrictive umask.
				ip_old_mask=`umask`
				umask 022
				rpm -Uvh "${escaped_package_name}" || have_error=1
				umask ${ip_old_mask}
				unset ip_old_mask
				;;
			"SUNOS")
				# Solaris 10 systems with "zones" require the '-G" switch to pkgadd
				# We support Solaris 10 systems in the global zone ONLY
				NOZONE=
				if [ ${OS_MINOR} -ge 10 ] ; then
					NOZONE="-G"
				fi

				admin_file=/tmp/umi_pkgadd_admin_file
				SafeCreateFile "$admin_file"
				cat > $admin_file << EOI
mail=
instance=overwrite
partial=nocheck
runlevel=nocheck
idepend=nocheck
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=default
EOI
				/usr/sbin/pkgadd ${NOZONE} -a $admin_file -d "$package_to_install" all || have_error=1
				rm -f $admin_file
				;;
			"DARWIN")
				TARGET_DIRECTORY=/

				# It seems as though a fresh install of OSX doesn't include a
				# /Library/StartupItems directory.  Create one now and save some
				# trouble with failing postinstall scripts.
				if [ ! -d /Library/StartupItems ]; then
					mkdir -p /Library/StartupItems
				fi

				dmg_installer_options=
				if [ ${DEBUG_ENABLED:-0} = 1 ]; then
					dmg_installer_options=-v
				fi

				if [ -f "${PKGDIR}/osx-${OSARCH}/dmg_installer.sh" ]; then
					/bin/sh "${PKGDIR}/osx-${OSARCH}/dmg_installer.sh" ${dmg_installer_options} -i "${package_to_install}" -t ${TARGET_DIRECTORY} || have_error=1
				else
					OutputError "ERROR: The dmg_installer.sh script is missing in the osx-${OSARCH} directory."
					OutputError "       The installation cannot proceed."
					return 1
				fi
				;;
			"HP-UX")
				# Dependencies are disabled because they are frequently misinterpreted.
				/usr/sbin/swinstall -x enforce_dependencies=false -s "$package_to_install" \* || have_error=1
				;;
			"AIX")
				# The -X option to installp causes the filesystems to be automatically resized.  Should we use it by default?
				/usr/sbin/installp -d "$package_to_install" -ac all || have_error=1
				;;
			*)
				ERROR "Install method unknown on $OSTYPE"
				have_error=1
				;;
		esac
		if [ "${package_to_install}" != "${original_package_path}" ]; then
			rm -r "${package_to_install}" || { have_error=1; ERROR "Failed to remove temporary (decompressed) package \"${package_to_install}\""; }
		fi
	done

	if [ ${have_error} = 1 ]; then
		ERROR "Package installation failed."
	fi
	return ${have_error}
}

DarwinRemoveFiles()
{
	while read filename; do
		filename="`printf '%s\n' \"${filename}\" | sed -e \"s,^${INSTALLER_HACK_TEMPDIR},/,g\" -e 's,//,/,g' `"
		# Protect against removing /var, /opt, /bin, /etc, and such.
		if echo "${filename}" | egrep '^/(var|opt|bin|etc|tmp)$|^(var|opt|bin|etc|tmp)$' >&7 2>&1 ; then
			OutputVerbosely "Skipping protected file/dir: ${filename}"
			continue
		fi
		if [ -f "${filename}" ] || [ -L "${filename}" ]; then
			OutputVerbosely "Removing ${filename}"
			[ $UNINSTALL_MAKE_CHANGES = 0 ] || rm -f "${filename}"
		elif [ -d "${filename}" ]; then
			uninstall_files_in_directory=`ls -1a "${filename}/" | egrep -v '^\.$|^\.\.$' | wc -l`
			if [ $uninstall_files_in_directory -ne 0 ]; then
				OutputNormal "Directory not empty: ${filename}"
			else
				OutputNormal "Removing empty directory: ${filename}"
				[ $UNINSTALL_MAKE_CHANGES = 0 ] || rmdir "${filename}"
			fi
		else
			OutputVerbosely "File was lost before remove: ${filename}"
		fi
	done < "$1"
}

DarwinPackageUninstaller()
{
	# This function needs some background, so I'll put it here:
	# OS-X has no uninstall command, so we'll have to delete anything we install.
	# It does, thankfully, put file lists in /Library/Receipts that have all of
	# the files that were installed by the installer program.
	#
	# Now for some background on the install path hack:
	# Due to the inability to use the OSX installer to install files into some
	# places on the system, the packages we make will install to a temporary
	# directory listed in a file Resources/INSTALLER_HACK_TEMPDIR, and then move
	# themselves to the proper location.  This may seem a little confusing,
	# since it is all hidden away by the 'vbuild' packaging system.  When
	# uninstalling, we need to remove the temp directory from any pathname that
	# contains it.
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	package_to_uninstall="$1"

	# Replace dots with hyphens for OS X 10.6 pkgutil
	munged_package_to_uninstall=`echo ${package_to_uninstall} | sed 's/\./-/g'`

	# This was here for debugging, but we could change it to be a 'pretend' option...
	UNINSTALL_MAKE_CHANGES=1
	INSTALLER_HACK_TEMPDIR=/
	# First... Get the package directory...
	OutputVerbosely "Checking receipts for ${package_to_uninstall}..."
	if ls -1 /Library/Receipts | grep -i "^${package_to_uninstall}\\.pkg" >&7 2>&1 ; then
		full_package_name=`ls -1 /Library/Receipts | grep -i "^${package_to_uninstall}\\.pkg"`
		uninstall_receipt_dir="/Library/Receipts/$full_package_name"
		OutputVerbosely "Using receipts from ${uninstall_receipt_dir}"
	else
		if pkgutil --pkgs=com.quest.${munged_package_to_uninstall} >&7 2>&1 ; then
			full_package_name=`pkgutil --pkgs=com.quest.${munged_package_to_uninstall}`
			uninstall_receipt_dir="NONE"
			OutputVerbosely "Using receipt database"
		else
			ERROR "Not installed: $package_to_uninstall"
			return 1
		fi
	fi
	# Second... Check to see if a preremove/postremove script was packaged...
	OutputVerbosely "Checking for {pre,post}remove scripts..."
	for postaction in preremove postremove; do
		if ListFiles "$uninstall_receipt_dir/Contents/Resources/${postaction}" >&7 2>&1 ; then
			eval "uninstall_have_${postaction}"=1
			OutputVerbosely "Located ${postaction} script..."
		else
			eval "uninstall_have_${postaction}"=0
		fi
	done
	if ListFiles "$uninstall_receipt_dir/Contents/Resources/INSTALLER_HACK_TEMPDIR" >&7 2>&1 ; then
		INSTALLER_HACK_TEMPDIR="`cat $uninstall_receipt_dir/Contents/Resources/INSTALLER_HACK_TEMPDIR`"
	fi

	# Call the preremove script (if it exists).
	if [ "x$uninstall_have_preremove" = "x1" ]; then
		OutputVerbosely "Executing preremove..."
		if $uninstall_receipt_dir/Contents/Resources/preremove ; then
			OutputVerbosely "Preremove script successful."
		else
			ERROR "Preremove script failed."
			return 1
		fi
	fi
	# Copy the postremove script (if it exists), so it won't dissapear...
	if [ "x$uninstall_have_postremove" = "x1" ]; then
		OutputVerbosely "Making copy of postremove..."
		uninstall_postremove_script="/tmp/postremove-${package_to_uninstall}"
		if SafeCreateFile $uninstall_postremove_script ; then
			cat $uninstall_receipt_dir/Contents/Resources/postremove > $uninstall_postremove_script
			chmod u+rwx $uninstall_postremove_script
		else
			return 1
		fi
	fi
	# Third, remove any files belonging to the package (from the bom file)
	# This jumps through some hoops to make sure any files with spaces
	# are properly removed.
	OutputVerbosely "Getting package file list..."
	if ListFiles "$uninstall_receipt_dir/Contents/Archive.bom" >&7 2>&1 ; then
		if uninstall_file_list=`MakeTemp /tmp/uninstall-$package_to_uninstall` ; then
			# Remove the files and symlinks first.
			# All of the packages seem to use ./ for the beginning.
			# Remove the . so only absolute paths are used.
			lsbom -s -f -d -l $uninstall_receipt_dir/Contents/Archive.bom | grep -v '^\.$' | sed 's|^./|/|g' | sort -r > $uninstall_file_list
			DarwinRemoveFiles $uninstall_file_list
			rm $uninstall_file_list
		fi
	else # Try getting the file list from the pkgutil db.
		if pkgutil --files com.quest.${munged_package_to_uninstall} >&7 2>&1 ; then
			if uninstall_file_list=`MakeTemp /tmp/uninstall-${package_to_uninstall}` ; then
				pkgutil --files com.quest.${munged_package_to_uninstall} | sort -r  > ${uninstall_file_list}
				DarwinRemoveFiles ${uninstall_file_list}
				rm ${uninstall_file_list}
			fi
		else  # No Archive.bom
			OutputVerbosely "Not removing files: Archive.bom not found."
		fi
	fi

	# Remove the receipts...
	OutputVerbosely "Removing receipts for ${package_to_uninstall}..."
	if [ $UNINSTALL_MAKE_CHANGES = 1 ] ; then
		if [ "x${uninstall_receipt_dir}" = "xNONE" ] ; then
			 pkgutil --forget com.quest.${munged_package_to_uninstall} >&7 2>&1 || true
		else
			uninstall_remove_uninstall_remove_verbose_flag=""
			[ $QUIET_OPT = 1 ] || uninstall_remove_verbose_flag="-v"
			rm -rf $uninstall_remove_verbose_flag "$uninstall_receipt_dir" | sed 's|^/|Removing /|g' || true
			unset uninstall_remove_verbose_flag
		fi
	fi

	# Now, we can run the postremove script...
	if [ "x$uninstall_have_postremove" = "x1" ]; then
		OutputVerbosely "Executing postremove..."
		if $uninstall_postremove_script ; then
			OutputVerbosely "Postremove script successful."
		else
			ERROR "Postremove script failed."
			return 1
		fi
		rm -f "${uninstall_postremove_script}"
	fi
	OutputVerbosely "Uninstall completed for ${package_to_uninstall}"
}

UninstallPackage()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	# This fairly simple function will uninstall whatever packages you list as
	# parameters.  It will abort (returning 1) after the first package fails to
	# uninstall (the rest won't even be attempted).
	while [ $# -gt 0 ]; do
		package_to_uninstall="$1"
		shift

		OutputNormal "Uninstalling package $package_to_uninstall."
		case $OSTYPE in
			"LINUX")
				rpm -e "$package_to_uninstall" || return 1
			;;
			"SUNOS")
				# Solaris packages have their dots converted into dashes.  Do that for the query now.
				package_to_uninstall=`echo ${package_to_uninstall} | sed 's/\./-/g'`
				admin_file=/tmp/umi_pkgrm_admin_file
				SafeCreateFile "$admin_file"
				cat > $admin_file << EOI
mail=
instance=overwrite
partial=nocheck
runlevel=nocheck
idepend=nocheck
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=default
EOI

				have_error=0
				/usr/sbin/pkgrm -n -a $admin_file "$package_to_uninstall" || have_error=1
				rm -f $admin_file

				[ ${have_error} = 0 ] || return 1
			;;
			"DARWIN")
				# This is nice and clean now that all of the darwin uninstallation
				# script has been moved to its own function.
				DarwinPackageUninstaller "$package_to_uninstall" || return 1
			;;
			"HP-UX")
				# HP-UX packages have their dots converted into underscores.
				# Unfortunately, this means that a subpackage cannot be uninstalled
				# separately from the containing package.
				package_to_uninstall=`echo ${package_to_uninstall} | sed 's/\./_/g'`
				/usr/sbin/swremove -x enforce_dependencies=false "$package_to_uninstall" || return 1
			;;
			"AIX")
				# AIX can handle both RPMs and bff (installp) packages.  Try the RPM first.
				if rpm -qi "${package_to_uninstall}" >&7 2>&1 ; then
					rpm -e "$package_to_uninstall" || return 1
				else
					/usr/sbin/installp -u "$package_to_uninstall" || return 1
				fi
			;;
			*)
				ERROR "Uninstall method unknown on $OSTYPE"
				return 1
			;;
		esac
	done
}

InstallPackages()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	noerror=0
	if [ "x${1:-}" = x--noerror ]; then
		noerror=1
		shift
	fi

	for package_name in "$@"; do
		if ListFiles "${CACHED_PACKAGE_DIRECTORY}"/${package_name}* 2>&7 | grep -v "${INSTALL_IGNORE_PATTERN}" >&7; then
			FILE_TO_INSTALL=`ListFiles "${CACHED_PACKAGE_DIRECTORY}"/${package_name}* | grep -v "${INSTALL_IGNORE_PATTERN}" | tail -1 | sed 's| |\\ |g'`
			if InstallPackage "$FILE_TO_INSTALL"; then
				:
			else
				# Eep.  We're in a bunch of trouble here.  We can't roll back
				# the packages we've already installed, as they might be used by
				# something else (in the case of an upgrade).
				ERROR "Failed to install package ${FILE_TO_INSTALL}"
				return 1
			fi
		elif [ x${noerror:-0} != x1 ]; then
			ERROR "Cannot find package files for ${package_name}"
			return 1
		else
			OutputVerbosely "Package ${package_name} is not found and will be ignored."
		fi
	done
}

ObtainPackageDirectory()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	# Get the package directory and set the CACHED_PACKAGE_DIRECTORY variable.
	GetPackageDirectory >&7 2>&1  || true
	if [ -d "$CACHED_PACKAGE_DIRECTORY" ]; then
		:
	else
		os_prefix_text=`GetPlatformPrefix`
		SelectDirectoryFromMenu "$PKGDIR/${os_prefix_text}-*${OSARCH}"
		if [ $MENU_SELECTION_MADE -eq 1 ]; then
			CACHED_PACKAGE_DIRECTORY="${MENU_CHOICE}"
		else
			ERROR "Failed to find directory for packages."
			return 1
		fi
	fi
}

Install()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	ObtainPackageDirectory || return $?

	# Automatically upgrade, when possible.
	if UMIIsInstalled; then
		Upgrade
	else
		InstallOrUpgradePackages `ListRequiredPackages` || return $?

		# Don't bail if the VAS packages aren't there.  This will be little
		# chance of having a completely broken install if we continue.
		UpgradeVASHost || true

		# This should be removed when the files in the packages are verified to
		# be owned by the correct users on all of the platforms.
		SetRepositoryPermissions
	fi

	if [ "${OSTYPE}" = "LINUX" ]; then
		case ${OSVEND}-${OSVER} in
			RedHat-5* | SL-5[01234567] | SLF-5[01234567] | Fedora-11| Fedora-10 | CentOS-5) # 5SERVER, 5CLIENT, 5.*

				# If selinux is enabled, change owcimomd to be unconfined.  The
				# OpenWBEM CIMOM has its own security model for restricting
				# privileged commands/access, and operations required for proper
				# client functionality do not work properly with the initrc_t
				# restrictions.
				if selinuxenabled >&7 2>&1 ; then
					chcon -t unconfined_exec_t "${UMI_PREFIX}/sbin/owcimomd" || WARNING "Failed to set security context for ${UMI_PREFIX}/sbin/owcimomd."
				fi
			;;
		esac
	fi
	return 0
}

# A handy one-line sed script to print things out with the line order reversed
# (the chars within each line are not changed).
ReverseLines()
{
	sed '1!G;h;$!d'
}

GetNonUMIRegisteredProducts()
{
	if ${UMI_PREFIX}/bin/product_tool --list-registered-products >&7 2>&1 ; then
		${UMI_PREFIX}/bin/product_tool --list-registered-products | grep -v '^umi$' || true
	fi
}

HaveRegisteredProducts()
{
	registered_products=`GetNonUMIRegisteredProducts`
	if [ "x${registered_products}" != x ]; then
		# Remove any spaces...
		registered_products=`echo ${registered_products} | tr '\n' ' ' | sed -e 's/[ 	][ 	]*/ /g' -e 's/^[ ]*//g' -e 's/[ ]*$//g'`
		OutputNormal "These products are registered with UMI: ${registered_products}"
		return 0
	fi
	return 1
}

DoPartialUninstall()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	uninstall_package_list="`ListRequiredNonLibPackages | ReverseLines`"

	IGNORE_MISSING_KEYS=1
	for package_name in ${uninstall_package_list}; do
		if QueryExistingPackage "${package_name}" >&7; then
			if UninstallPackage "${package_name}"; then
				:
			else
				ERROR "Failed to uninstall package ${package_name}.  Cannot continue."
				return 1
			fi
		else
			WARNING "Cannot uninstall ${package_name}: Not installed"
		fi
	done
	IGNORE_MISSING_KEYS=0

	return 0
}

GetVASInstallPath()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	VAS3_PREFIX=/opt/quest
	VAS2_PREFIX=/opt/vintela/vas
	OLD_VAS_PREFIX=/opt/vas

	if [ -d ${VAS3_PREFIX}/bin ] && [ -f ${VAS3_PREFIX}/bin/vastool ]; then
		VAS_PREFIX=${VAS3_PREFIX}
		VASTOOL=${VAS3_PREFIX}/bin/vastool
	elif [ -d ${VAS2_PREFIX}/bin ] && [ -f ${VAS2_PREFIX}/bin/vastool ]; then
		VAS_PREFIX=${VAS2_PREFIX}
		VASTOOL=${VAS2_PREFIX}/bin/vastool
	elif [ -d ${OLD_VAS_PREFIX}/bin ] && [ -f ${OLD_VAS_PREFIX}/bin/vastool ]; then
		VAS_PREFIX=${OLD_VAS_PREFIX}
		VASTOOL=${OLD_VAS_PREFIX}/bin/vastool
	else
		return 1
	fi
}

UninstallPackages()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if [ "x${1:-}" = x--nowarn ]; then
		nowarn=1
		shift
	fi

	for package_name in "$@"; do
		if QueryExistingPackage "${package_name}" >&7; then
			if UninstallPackage "${package_name}"; then
				:
			else
				ERROR "Failed to uninstall package ${package_name}.  Cannot continue."
				return 1
			fi
		elif [ ${nowarn:-0} != 1 ]; then
			WARNING "Cannot uninstall ${package_name}: Not installed"
		else
			OutputVerbosely "Package ${package_name} is not installed"
		fi
	done
	return 0
}

Uninstall()
{

	# CHECKME! After the binary version of the various packages is inflated, the
	# uninstall will need to be able to uninstall *ALL* package versions
	# provided with any prior UMI install.
	# For this version, there are no prior UMI libs to worry about, but any
	# subsequent releases will need to handle those cases.
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	IGNORE_MISSING_KEYS=1

	OutputNormal "Uninstalling UMI."

	UninstallPackages --nowarn `ListOldUMIPackages`
	UninstallPackages `ListRequiredPackages | ReverseLines`

	# Find out if a VAS package is installed.
	installed_vas_package=
	for vas_package in ${VASGP_PACKAGES} ${VAS_CLIENT_PACKAGE_LIST}; do
		if QueryExistingPackage ${vas_package} >&7 2>&1 ; then
			installed_vas_package="${installed_vas_package} ${vas_package}"
		fi
	done

	# Uninstall VAS if it is present and not in use.
	if [ x${installed_vas_package:+set} = xset ] && GetVASInstallPath; then
		if VASIsJoined; then
			# vastool says it is joined...  Report this and do not uninstall.
			WARNING "Not removing ${installed_vas_package}: Joined to domain ${JOINED_DOMAIN}"
		else
			OutputNormal "Uninstalling unused ${installed_vas_package}"
			if [ $OSTYPE = DARWIN ] && [ -d ${VAS_PREFIX}/bin/Uninstall.app ]; then
				# Run the vas uninstaller
				${VAS_PREFIX}/bin/Uninstall.app/Contents/MacOS/Uninstall --console --force ${installed_vas_package}
				rm -rf ${VAS_PREFIX}/bin/Uninstall.app
			else
				UninstallPackages ${installed_vas_package}
			fi

			# The old vas packages seem to leave some junk around.  Clean them if
			# it won't interfere with other items.  There are other quest products
			# that share the VAS3_PREFIX, so it isn't safe to remove.
			if [ x${VAS_PREFIX:+set} = xset ] && [ x${VAS_PREFIX} != x${VAS3_PREFIX} ]; then
				rm -rf /etc/${VAS_PREFIX}
				rm -rf /var/${VAS_PREFIX}
				rm -rf ${VAS_PREFIX}
			fi
		fi
	fi

	IGNORE_MISSING_KEYS=0

	# Remove any databases that the package uninstall misses.
	rm -rf /var${UMI_PREFIX}
	return 0
}

UMIIsInstalled()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	for providers_package in quest-umi-prov quest-umi-providers; do
		if QueryExistingPackage "${providers_package}" >&7; then
			EXISTING_UMI_VERSION=`QueryExistingPackage "${providers_package}"`
			return 0
		fi
	done
	return 1
}

PrngdIsInstalled()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if QueryExistingPackage "quest-prngd" >&7; then
		return 0
	fi
	return 1
}

GetUMIVersion()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ -f /etc${UMI_PREFIX}/umiversion ]; then
		cat /etc${UMI_PREFIX}/umiversion
	elif [ -f /etc${UMI_1_0_PREFIX}/umiversion ]; then
		cat /etc${UMI_1_0_PREFIX}/umiversion
	else
		echo "unknown"
	fi
}

GetToolsBaseDir()
{
	if [ "${OSTYPE}" = "LINUX" ]; then
		# This is a generic directory for *all* versions of linux of a given arch type.
		echo "linux-${OSARCH}"
	elif [ "${OSTYPE}" = "AIX" ]; then
		# If we ever support itanium, this will need modification.
		echo "aix-power"
	elif [ "${OSTYPE}" = "HP-UX" ]; then
		echo "hpux-${OSARCH}"
	elif [ "${OSTYPE}" = "DARWIN" ]; then
		# We may want to make these things universal, otherwise we'll need to include two copies.
		echo "osx-${OSARCH}"
	elif [ "${OSTYPE}" = "SUNOS" ]; then
		echo "sunos-${OSARCH}"
	fi
}

GetToolsDirectory()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if [ "${CACHED_TOOLS_DIRECTORY:+set}" = "set" ]; then
		echo ${CACHED_TOOLS_DIRECTORY}
		return 0
	fi

	local_tools_dir=`GetToolsBaseDir`

	if [ "${local_tools_dir:-UNKNOWN}" != "UNKNOWN" ]; then
		CACHED_TOOLS_DIRECTORY="${PKGDIR}/bin/${local_tools_dir}"
	fi
	local_tools_dir="${local_tools_dir:+bin/}${local_tools_dir:-UNKNOWN}"
	echo "${PKGDIR}/${local_tools_dir}"
	unset local_tools_dir
}

GetPlatformPrefix()
{
	if [ "${OSTYPE}" = "DARWIN" ]; then
		echo "osx"
	else
		echo ${OSTYPE} | tr '[A-Z]' '[a-z]'
	fi
}

GetPlatformPackageDirectory()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	platform_package_dir=UNKNOWN
	# For supported platforms, we'll determine the package directory.  For
	# unsupported, we'll just report UNKNOWN so that any calling functions will
	# be forced to further prompt the user (eg. bring up a list of possible
	# directories).
	if [ ${UNSUPPORTED:-0} = 0 ]; then
		if [ "${OSTYPE}" = "LINUX" ]; then
			case ${OSVEND}-${OSVER} in
				# Old, obsolete platforms.
				RedHat-2.1[EAW]S | RedHat-7.3 | RedHat-8 | RedHat-8.* | SuSE-8 | SuSE-8.* | VMWARE-2.5*)
					platform_package_dir=linux-rh2.1-x86
					;;
				# Anything else.
				*)
					platform_package_dir=linux-rh3-${OSARCH}
					;;
			esac
		elif [ "${OSTYPE}" = "AIX" ]; then
			# If we ever support itanium, this will need modification.
			platform_package_dir="aix-power"
		elif [ "${OSTYPE}" = "HP-UX" ]; then
			platform_package_dir="hpux-${OSARCH}"
		elif [ "${OSTYPE}" = "DARWIN" ]; then
			platform_package_dir="osx-${OSARCH}"
		elif [ "${OSTYPE}" = "SUNOS" ]; then
			platform_package_dir="sunos-${OSARCH}"
		fi
	fi
	echo "${platform_package_dir}"
	unset platform_package_dir
}

GetPlatformPackageSuffix()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	local_package_suffix=
	case ${OSTYPE} in
		LINUX)
			local_package_suffix=rpm
			;;
		HP-UX)
			local_package_suffix=depot
			;;
		SUNOS)
			local_package_suffix=pkg
			;;
		DARWIN)
			local_package_suffix=dmg
			;;
		AIX)
			local_package_suffix=bff
			;;
		*)
			AbortInstallation "Unknown package suffix for platform ${OSTYPE}"
			;;
	esac
	echo "${local_package_suffix}"
	unset local_package_suffix
}

GetPackageDirectory()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if [ "x${CACHED_PACKAGE_DIRECTORY:+set}" = "xset" ]; then
		echo ${CACHED_PACKAGE_DIRECTORY}
		return 0
	fi

	local_package_dir=`GetPlatformPackageDirectory`

	if [ "${local_package_dir:-UNKNOWN}" != "UNKNOWN" ]; then
		CACHED_PACKAGE_DIRECTORY="${PKGDIR}/${local_package_dir}"
		unset local_package_dir
		return 0
	fi
	unset local_package_dir
	OutputError "Unable to determine package directory"
	return 1
}

# Library packages will not be uninstalled when upgrading, and will be
# installed even if a newer umi version happens to be installed (to support
# providers from an older umi).
ListRequiredLibraryPackages()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	gcc_support_lib_required_version=0
	temp_os_pattern=${OSTYPE}-${OSVEND}-${OSVER}-${OSARCH}

	# If a directory has been chosen already, use it instead of the actual OS
	# type and version when determining the required versions of libgcc.  This
	# isn't needed in a general case, and should not even be needed here when
	# support for the old and broken versions of linux is removed.
	if [ x${CACHED_PACKAGE_DIRECTORY:+set} = xset ]; then
		case `basename "${CACHED_PACKAGE_DIRECTORY}"` in
			linux-rh2.1*) temp_os_pattern=LINUX-RedHat-2.1ES- ;;
			*)	;;
		esac
	fi
	case ${temp_os_pattern} in
		# Old, obsolete platforms (redhat)
		LINUX-RedHat-2.1[EAW]S-* | LINUX-RedHat-7.3-* | LINUX-RedHat-8-* | LINUX-RedHat-8.*-*)
			gcc_support_lib_required_version=5
			;;
		# Old, obsolete platforms (SuSE)
		LINUX-SuSE-8-* | LINUX-SuSE-8.*-*)
			gcc_support_lib_required_version=5
			;;
		LINUX-VMWARE-2.5*)
			gcc_support_lib_required_version=5
			;;
		# A more modern linux (at least we hope so).
		LINUX-*)
			gcc_support_lib_required_version=6
			;;
		AIX-*)
			gcc_support_lib_required_version=5
			;;
		SUNOS-*)
			gcc_support_lib_required_version=5
			;;
		# Any other platform where we don't ship the gcc support libs (such
		# as HP-UX or MacOS)
		*)
			gcc_support_lib_required_version=0
			;;
	esac
	unset temp_os_pattern

	if [ $gcc_support_lib_required_version != 0 ]; then
		echo "quest-gcc-support-${gcc_support_lib_required_version}"
	fi
	echo "quest-zlib-1"
	echo "quest-openssl-0.9.8"
	echo "quest-openwbem-libs${OW_LIBRARY_VERSION}"
	echo "quest-umi-libs${LIBRARY_BINARY_VERSION}"
}

ListOldUMIPackages()
{
	# All of the old UMI library packages
	lop_binary_version=1
	while [ ${lop_binary_version} -lt ${LIBRARY_BINARY_VERSION} ]; do
		echo "quest-umi-libs${lop_binary_version}"
		lop_binary_version=`expr ${lop_binary_version} + 1`
	done

	# All of the old OW library packages
	lop_ow_binary_version=5
	while [ ${lop_ow_binary_version} -lt ${OW_LIBRARY_VERSION} ]; do
		echo "quest-openwbem-libs${lop_ow_binary_version}"
		lop_ow_binary_version=`expr ${lop_ow_binary_version} + 1`
	done
	echo "quest-perl"
	echo "quest-modules-perl"
}

ListDeveloperPackages()
{
	echo "quest-umi-shellutils"
	if [ "${OSTYPE}" = "AIX" ] || [ "${OSTYPE}" = "HP-UX" ]; then
		if [ ! -c /dev/random ] && [ ! -c /dev/urandom ]; then
			echo "quest-prngd"
		fi
	fi
}

# Nonlib packages will be removed and replaced with the newer versions when upgrading.
ListRequiredNonLibPackages()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if [ ${UNINSTALL_OPT} = 1 ] && [ ${KEEP_DEV_PACKAGES_OPT} = 1 ]; then
		# Don't include the dev packages here...
		WARNING "Not listing developer packages for removal."
	else
		ListDeveloperPackages
	fi
	echo "quest-openwbem-tools"
	echo "quest-openwbem-cimomd"
	echo "quest-openwbem-schema"
	echo "quest-umi-prov"
	echo "quest-umi-repo"
	# For dev, there is quest-umi-devel
}

ListRequiredPackages()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	ListRequiredLibraryPackages
	ListRequiredNonLibPackages
}

# List the vas package, as required by the push installer.  This is NOT always
# going to be the same as what is installed (MacOS for instance, where they are
# doing non-standard packaging to make people feel warm and fuzzy).
ListVASPackage()
{
	if [ $OSTYPE != DARWIN ]; then
		for vas_package in ${VAS_CLIENT_PACKAGE_LIST} ${VASGP_PACKAGES}; do
			echo ${vas_package}
		done
	else
		# If VAS ever supports a site-licensed package on MacOS, and we support
		# installing it at the same time as QMXSMS, this will need to be
		# modified.
		echo "VAS"
	fi
}

# List extra files that need to be distributed.
ListDistFiles()
{
	ldf_local_package_dir=`GetPlatformPackageDirectory`
	echo "install_umi.sh"
	if [ ${OSTYPE} = DARWIN ]; then
		echo "${ldf_local_package_dir}/dmg_installer.sh"
	fi
	unset ldf_local_package_dir
}

DisplayPackageRegex()
{
	local_package_dir="`GetPlatformPackageDirectory | EscapeForRegex`"
	local_package_suffix="`GetPlatformPackageSuffix`"
	local_combined_regex="(bin/`GetToolsBaseDir`/.*)"

	if CheckForSupportedPlatform -q; then
		# Files required for installation
		for dist_file in `ListDistFiles | EscapeForRegex`; do
			local_combined_regex="${local_combined_regex:-}${local_combined_regex:+|}(${dist_file})"
		done
		# Packages
		for package_name in `ListRequiredPackages | EscapeForRegex` `ListVASPackage | EscapeForRegex`; do
			local_combined_regex="${local_combined_regex:-}${local_combined_regex:+|}(${local_package_dir}/${package_name}.*\\.${local_package_suffix}(\.bz2|\.gz|()))"
		done
		echo "**umi-install-script** (${local_combined_regex}) **umi-install-script**"
	else
		echo "Unsupported Platform"
		AbortInstallation "Unable to determine required files for this system: OS=${OSTYPE}, Vendor=${OSVEND}, Version=${OSVER}, Arch=${OSARCH}"
	fi
}

VAS_CLIENT_SITE_PACKAGE="vasclnts vas-client-site"
# This is purposely sorted so the site-licensed vas occurs before the
# single-licensed (or unlicensed) vas.
VAS_CLIENT_PACKAGE_LIST="vasclnts vasclnt"
VASGP_PACKAGES="vasgps vasgp"
VAS_2_6_CLIENT_PACKAGE_LIST="vasclient vas-client vas-client-eval vas-client-site"
VAS_HOST_CLIENT_PACKAGE_LIST="vashost vas-host"
VAS_PREFIX=
VASTOOL=

# This is only for OSX, which needs to be started from cron, if it was executed
# through software distribution.
CreateCronEntry()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	results=1
	if temp_file=`mktemp /tmp/root_cronXXXXXX`; then

		if crontab -l -u root >&7 2>&1 ; then
			OutputVerbosely "Adding a cron entry to start QuestUMI"
			crontab -l -u root 2>&7 | sed '/# BEGIN_UMI_INSTALLED_THIS/,/# END_UMI_INSTALLED_THIS/d' > $temp_file
		else
			OutputVerbosely "Creating new crontab with a cron entry to start QuestUMI"
		fi

		# Create a crontab entry...
		echo '# BEGIN_UMI_INSTALLED_THIS' >> $temp_file
		echo 'CRON_IS_RUNNING_UMI=1' >> $temp_file
		echo "*/1 * * * *  " "$@" >> $temp_file
		echo '# END_UMI_INSTALLED_THIS' >> $temp_file

		results=0
		crontab -u root $temp_file || results=$?

		rm -f $temp_file
		if [ $results -eq 0 ]; then
			OutputVerbosely "Crontab entry added"
		fi
	fi
	return $results
}

# This is only for OSX, which needs to be started from cron, if it was executed
# through software distribution.
RemoveCronEntry()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	results=1
	# Remove any existing crontab entry.
	if temp_file=`mktemp /tmp/root_cronXXXXXX`; then
		if crontab -l -u root >&7 2>&1 ; then
			if crontab -l -u root 2>&7 | grep 'BEGIN_UMI_INSTALLED_THIS' >&7 2>&1 ; then
				OutputVerbosely "Removing QuestUMI crontab entry."
				crontab -l -u root 2>&7 | sed '/# BEGIN_UMI_INSTALLED_THIS/,/# END_UMI_INSTALLED_THIS/d' > $temp_file

				results=0
				crontab -u root $temp_file || results=$?
			fi
		else
			OutputVerbosely "Cannot remove crontab entry: No crontab installed"
		fi
		rm -f $temp_file
		if [ $results -eq 0 ]; then
			OutputVerbosely "Crontab entry removed"
		fi
	fi
	return $results
}

Detach()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	# Query the packaging system.  This does basically nothing on every platform
	# except for RedHat linux, where a corrupt RPM database will cause this
	# query to hang, and cause a timeout for the calling process (if called from
	# the CIMOM), so that errors of this type can be detected early-on.
	OutputNormal "Attempting to query packaging system..."
	QueryExistingPackage >&7 2>&1  || true
	OutputNormal "Package query completed."

	SafeCreateFile $DETACHED_LOG_NAME

	# Strip some of the parameters (like the detach param).
	PARAMETERS=
	while [ $# -gt 0 ]; do
		case "$1" in
			-d)
				# Ignore the detach parameter.
			;;
			*)
				PARAMETERS="$PARAMETERS \"$1\""
			;;
		esac
		shift
	done

	# Since the package cache directory cleans itself up immediately after the
	# software distribution process returns, this script and the proper files
	# will need to be moved elsewhere before detaching.
	#
	# This is now being done even in the case of manual installs, because an
	# upgrade from a 1.0 client to 1.1 won't have the proper environment
	# variable set, so there is no easy way to detect how this was started.
	#
	# Unfortunately, this will double the amount of space required on /var,
	# albeit temporarily.
	OutputNormal "Making a copy of the package directory, since the UMI package cache will be deleted."
	UMI_REMOVE_PACKAGE_DIR=1
	export UMI_REMOVE_PACKAGE_DIR
	NEW_PACKAGE_DIR=`dirname "/var${UMI_PREFIX}"`/umi_upgrade_temp
	mkdir -p "$NEW_PACKAGE_DIR"
	cp -r "$PKGDIR"/* "$NEW_PACKAGE_DIR"
	PKGDIR="$NEW_PACKAGE_DIR"

	# Remove the UMI env variable so that it won't detach again (if it was in
	# fact the UMI client that ran this script).
	QUEST_UMI_STARTED_PROCESS=0
	unset QUEST_UMI_STARTED_PROCESS
	VINTELA_VMX_STARTED_PROCESS=0
	unset VINTELA_VMX_STARTED_PROCESS

	# Close stdin, since it isn't going to be available.
	exec 0<&-

	EXECUTABLE_NAME="$SCRIPT_NAME"
	EXECUTABLE_PATH="$PKGDIR/$EXECUTABLE_NAME"
	OutputNormal "Saving output in $DETACHED_LOG_NAME"
	[ -x "$EXECUTABLE_PATH" ] || chmod u+x "$EXECUTABLE_PATH" || AbortInstallation "Unable to make installer script executable."
	cd "$PKGDIR"

	OSX_HACK_REQUIRED=0
	if [ "x$OSTYPE" = "xDARWIN" ] && [ "x$UPGRADE_OPT" = "x1" ]; then
		OSX_HACK_REQUIRED=1
	elif [ "x$OSTYPE" = "xDARWIN" ] && [ "x$DETACH_OPT" = "x1" ]; then
		OSX_HACK_REQUIRED=1
	fi

	if [ $OSX_HACK_REQUIRED -eq 1 ]; then
		# BEGIN OSX HACK
		# This is a OSX specific hack to insert a temporary crontab entry and wait for cron to fire it off.
		# It determines if the install script was started by looking for the resulting log.
		rm -f "$DETACHED_LOG_NAME"
		CreateCronEntry "/bin/sh \"$EXECUTABLE_PATH\" $PARAMETERS -cron -setpath \"$PATH\" >$DETACHED_LOG_NAME 2>&1"
		OutputNormal "The cron entry should restart the install script within 1 minute."
		seconds_to_wait=70
		installer_started=0
		dot_string="..."
		while [ $seconds_to_wait -ge 0 ] && [ $installer_started -eq 0 ]; do
			seconds_to_wait=`expr $seconds_to_wait - 5`
			echo -e -n "\rWaiting$dot_string"
			dot_string="${dot_string}."
			sleep 5
			if [ -f "$DETACHED_LOG_NAME" ]; then
				installer_started=1
			fi
		done

		echo ""
		if [ -f "$DETACHED_LOG_NAME" ]; then
			installer_started=1
		fi
		if [ $installer_started -eq 0 ]; then
			RemoveCronEntry
			ERROR "Installer did not start."
			return 1
		fi
		# END OSX HACK
	else
		# Set a variable to the new detached copy will know that it's detached. It pauses for 60 seconds.
		INSTALL_SCRIPT_DETACHED=1
		export INSTALL_SCRIPT_DETACHED

		eval /bin/sh "$EXECUTABLE_PATH" $PARAMETERS >$DETACHED_LOG_NAME 2>&1 &

		INSTALL_SCRIPT_DETACHED=0
	fi

	unset UMI_REMOVE_PACKAGE_DIR

	trap "echo Detached." EXIT

	exit 0
}

DetachIfNeeded()
{
	if [ $DETACH_REQUIRED = 1 ]; then
		Detach "$@"
	fi
}

user_exists()
{
	user_to_check=$1
	if [ $OSTYPE != DARWIN ]; then
		if id ${user_to_check} >&7 2>&1; then
			OutputDebug "Yes, user ${user_to_check} exists."
			return 0
		fi
	elif dscl . -read "/users/${user_to_check}" >&7 2>&1; then
		OutputDebug "Yes, user ${user_to_check} exists."
		return 0
	fi
	OutputDebug "No, user ${user_to_check} does not exist."
	return 1
}

group_exists()
{
	group_to_check=$1
	if [ ${OSTYPE} != DARWIN ]; then
		if grep "^${group_to_check}:" /etc/group >&7 2>&1 ; then
			OutputDebug "Yes, group ${group_to_check} exists"
			return 0
		fi
	elif dscl .  -read /groups/${group_to_check} >&7 2>&1 ; then
		OutputDebug "Yes, group ${group_to_check} exists"
		return 0
	fi
	OutputDebug "No, group ${group_to_check} does not exist."
	return 1
}

mac_find_unused_id()
{
	id_type=$1 # Normally "users" or "groups"
	id_tag=$2  # Normally "UniqueID" or "PrimaryGroupID"
	id_start=$3 # Should be >= 100 for physical users, or <-2 for fake users.
	id_increment=$4 # Must be either "+" or "-"
	if [ "x${id_increment}x" != "x+x" ] && [ "x${id_increment}x" != "x-x" ]; then
		ERROR "mac_find_user_id(): The increment must be '+' or '-', not \"${id_increment}\""
		return 1
	fi

	id_current=${id_start}

	# dscl -search returns success *ALL THE TIME*.  This is messed up.  From
	# tests, it appears as though all supported versions of MacOS (10.3-10.5)
	# return a string like this:
	# $ dscl . -search '/users' UniqueID -2
	# nobody          dsAttrTypeNative:UniqueID = ("-2")
	#
	# MacOS 10.3 prints out other debug info that we don't care about.  We'll
	# just grep for the '='.  This would not be a good thing to do in
	# general... The search is not even be displayed in xml format when using
	# the -plist option (10.4+), so we are forced to do something less than
	# clean.
	while dscl . -search "/${id_type}" ${id_tag} ${id_current} 2>&7 | grep '=' >&7; do
		id_current=`expr ${id_current} ${id_increment} 1`
	done

	echo ${id_current}
	return 0
}

# Get the group ID for a named group.
mac_getgid()
{
	mggid_group_name=$1

	if dscl . -read "/groups/${mggid_group_name}" PrimaryGroupID >&7 2>&1 ; then
		dscl . -read "/groups/${mggid_group_name}" PrimaryGroupID \
			| sed -e 's/.*PrimaryGroupID:[ 	]//g' -e 's/[ 	]*$//g'
		return 0
	fi
	return 1
}

mac_groupadd()
{
	if [ $# -ne 1 ]; then
		echo "Usage: mac_groupadd <groupname>" >&2
		return 1
	fi
	mga_groupname="$1"
	mga_gid=

	if mga_gid=`mac_find_unused_id groups PrimaryGroupID -2 '-'`; then
		:
	else
		ERROR "mac_groupadd(): Cannot find empty group id for group ${mga_group}"
		return 1
	fi

	mga_failed=1

	# Primitive direct database manipulation.
	if dscl . -create "/groups/${mga_groupname}" PrimaryGroupID "${mga_gid}"; then
		mga_failed=0
	else
		ERROR "mac_groupadd(): Failed to create group ${mga_groupname}"
	fi
	return ${mga_failed}
}

mac_useradd()
{
	if [ $# -ne 5 ]; then
		ERROR "Usage: mac_useradd <username> <comment> <homedir> <shell> <groupname>"
		return 1
	fi
	mua_username="$1"
	mua_comment="$2"
	mua_homedir="$3"
	mua_shell="$4"
	mua_group="$5"
	mua_gid=
	mua_uid=

	if mua_gid=`mac_getgid "${mua_group}"`; then
		:
	else
		ERROR "mac_useradd: Cannot find group id for group ${mua_group}"
		return 1
	fi

	# Search down from -2 (nobody) to find an unused user id
	if mua_uid=`mac_find_unused_id users UniqueID -2 '-'`; then
		:
	else
		ERROR "mac_useradd: Cannot find an empty user id for ${mua_username}"
		return 1
	fi


	mua_failed=1

	# Primitive direct database manipulation.
	if dscl . -create "/users/${mua_username}"; then
		if	dscl . -create "/users/${mua_username}" realname "${mua_comment}" &&
			dscl . -create "/users/${mua_username}" home "${mua_homedir}" &&
			dscl . -create "/users/${mua_username}" shell "${mua_shell}" &&
			dscl . -create "/users/${mua_username}" PrimaryGroupID "${mua_gid}" &&
			dscl . -create "/users/${mua_username}" UniqueID "${mua_uid}" &&
			dscl . -create "/users/${mua_username}" passwd "*" ; then
			mua_failed=0
		else
			ERROR "mac_useradd: Failed to set properties for user ${mua_username}"
		fi
	fi
	return ${mua_failed}
}

# Some platforms need the name service cache to be flushed after users or
# groups are added.
FlushNameCache()
{
	case ${OSTYPE} in
		SUNOS)
			if [ -f /usr/sbin/svcadm ]; then
				/usr/sbin/svcadm restart svc:/system/name-service-cache || OutputVerbosely "Could not restart the name service cache"
			fi
			;;
		*)
			OutputDebug "Nothing to flush..."
			;;
	esac
}

CreateGroup()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	group_to_create=$1
	if group_exists ${group_to_create}; then
		: # already exists
	else
		if [ -x /usr/sbin/groupadd ]; then
			/usr/sbin/groupadd $group_to_create || return 1
		elif [ -x /usr/bin/mkgroup ]; then
			/usr/bin/mkgroup $group_to_create || return 1
		elif [ ${OSTYPE} = DARWIN ]; then
			mac_groupadd ${group_to_create} || return 1
		else
			# Just print an error.  It will abort below.
			ERROR "How do we add a group on this platform?"
		fi
	fi

	FlushNameCache

	# Make sure it worked
	if group_exists ${group_to_create} ; then
		return 0
	else
		AbortInstallation "Creating group $group_to_create failed."
	fi
}

CreateUser()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	user_to_create=$1
	if user_exists ${user_to_create}; then
		: # already exists
	else
		case $OSTYPE in
			"LINUX")
				/usr/sbin/useradd -c "Quest UMI $user_to_create user" -d /var${UMI_PREFIX}/openwbem -g $user_to_create -M -s /sbin/nologin -r $user_to_create || return 1
				;;
			"SUNOS")
				/usr/sbin/useradd -c "Quest UMI $user_to_create user" -d /var${UMI_PREFIX}/openwbem -g $user_to_create -s /bin/false $user_to_create || return 1
				;;
			"DARWIN")
				mac_useradd $user_to_create "Quest UMI $user_to_create user" /var${UMI_PREFIX}/openwbem /sbin/nologin $user_to_create || return 1
				;;
			"HP-UX")
				/usr/sbin/useradd -d /var${UMI_PREFIX}/openwbem -g $user_to_create -s /usr/bin/false $user_to_create || return 1
				;;
			"AIX")
				# Can't set a user's home directory unless it (or at least its parent) exists.
				mkdir -p /var${UMI_PREFIX}/openwbem
				/usr/bin/mkuser pgrp=$user_to_create groups=$user_to_create home=/var${UMI_PREFIX}/openwbem shell=/bin/sh login=false rlogin=false $user_to_create || return 1
				;;
			*)
				ERROR "Create user method unknown on $OSTYPE"
				return 1
				;;
		esac
	fi

	FlushNameCache

	# Make sure it worked
	if user_exists ${user_to_create}; then
		return 0
	else
		AbortInstallation "Creating user $user_to_create failed."
	fi
}

RemoveUser()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	user_to_remove=$1

	if user_exists ${user_to_remove}; then
		case $OSTYPE in
			# Amazingly, these platforms are the same when it comes to removing users.
			"LINUX" | "SUNOS" | "HP-UX")
				/usr/sbin/userdel ${user_to_remove} || return 1
				;;
			"DARWIN")
				# Primitive direct database manipulation.
				dscl . -delete "/users/${user_to_remove}" || return 1
				;;
			"AIX")
				/usr/sbin/rmuser -p ${user_to_remove} || return 1
				;;
			*)
				ERROR "Remove user method unknown on $OSTYPE"
				return 1
				;;
		esac

		# check it worked
		if user_exists ${user_to_remove}; then
			ERROR "Removing user $user_to_remove failed."
			return 1
		fi
	fi
	return 0
}

RemoveGroup()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	group_to_remove=$1

	if group_exists ${group_to_remove}; then
		if [ -x /usr/sbin/groupdel ]; then
			/usr/sbin/groupdel ${group_to_remove} || return 1
		elif [ -x /usr/sbin/rmgroup ]; then
			/usr/sbin/rmgroup ${group_to_remove} || return 1
		elif [ ${OSTYPE} = DARWIN ]; then
			dscl . -delete "/groups/${group_to_remove}" || return 1
		else
			# Just print an error.  It will abort below.
			ERROR "How do we remove a group on this platform?"
		fi

		# check it worked
		if group_exists ${group_to_remove}; then
			ERROR "Failed to remove group ${group_to_remove}"
			return 1
		fi
	fi
	return 0
}

CreateUsers()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	CreateGroup "owcimomd"
	CreateUser "owcimomd"
	CreateGroup "owprovdr"
	CreateUser "owprovdr"
}

RemoveUsers()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	RemoveUser "owcimomd"
	RemoveGroup "owcimomd"
	RemoveUser "owprovdr"
	RemoveGroup "owprovdr"
}

SetRepositoryPermissions()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	chown -R owcimomd:owcimomd $REPOSITORY_DIRECTORY
	chmod 700 $REPOSITORY_DIRECTORY
	chmod 600 $REPOSITORY_DIRECTORY/*
}

PrepareTools()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	GetToolsDirectory >&7
	if [ "${CACHED_TOOLS_DIRECTORY}" = UNKNOWN ]; then
		ERROR "Failed to determine tools directory."
		return 1
	fi
	if [ ! -d "${CACHED_TOOLS_DIRECTORY}" ] ; then
		ERROR "Tools directory does not exist: ${CACHED_TOOLS_DIRECTORY}"
		return 1
	fi

	prepare_tools_make_copy=0

	# Check for bzip2
	if [ -f "${CACHED_TOOLS_DIRECTORY}"/bzip2 ] || [ -h "${CACHED_TOOLS_DIRECTORY}"/bzip2 ]; then
		:
	else
		ERROR "No bzip2 found in ${CACHED_TOOLS_DIRECTORY}"
		return 1
	fi

	# Check for execute permissions and functionality of bzip2.
	if [ -x "${CACHED_TOOLS_DIRECTORY}"/bzip2 ] && "${CACHED_TOOLS_DIRECTORY}"/bzip2 --help >&7 2>&1 ; then
		:
	else
		prepare_tools_make_copy=1
	fi

	# See if we need to detach.  If so we'll need a copy.
	if [ ${DETACH_REQUIRED:-0} = 1 ]; then
		prepare_tools_make_copy=1
	fi

	if [ ${prepare_tools_make_copy} = 1 ]; then
		# We aren't executable or we could be deleted.  We'll need to make a copy somewhere.
		CreateTempDir || return $?

		OutputVerbosely "Making a copy of the tools."

		mkdir ${UMI_TEMP_DIRECTORY}/bin
		cp "${CACHED_TOOLS_DIRECTORY}"/* ${UMI_TEMP_DIRECTORY}/bin
		chmod +x ${UMI_TEMP_DIRECTORY}/bin/*

		# Check again for executable files...
		if ${UMI_TEMP_DIRECTORY}/bin/bzip2 --help >&7 2>&1 ; then
			CACHED_TOOLS_DIRECTORY=${UMI_TEMP_DIRECTORY}/bin
			OutputVerbosely "The tools are now functional in ${CACHED_TOOLS_DIRECTORY}"
		else
			ERROR "The tools remain non-executable."
			return 1
		fi
	fi
}

###########################################################################
###########################################################################
# Start of security checking functions
###########################################################################
###########################################################################

# Get a list of platform-specific users that can be considered secure, for the
# purposes of the UMI client.
GetSecureUsers()
{
	if [ ${OSTYPE} = "AIX" ]; then
		echo "root bin"
	elif [ ${OSTYPE} = "HP-UX" ]; then
		echo "root bin"
	else
		echo "root"
	fi
}

# Get a list of platform-specific groups that can be considered secure, for the
# purposes of the UMI client.
GetSecureGroups()
{
	if [ ${OSTYPE} = "SUNOS" ]; then
		echo "root sys"
	elif [ ${OSTYPE} = "DARWIN" ]; then
		echo "root admin"
	else
		echo "root"
	fi
}

IsDirectory()
{
	test -d "$1"
}

IsLink()
{
	test -h "$1"
}

IsFileOrDirectory()
{
	test -f "$1" -o -d "$1"
}

IsRegular()
{
	test -h "$1" -o -f "$1" -o -d "$1"
}

# Prints on stdout the type of file.
GetFileType()
{
	# $1 = file path
	if test -h "$1"; then
		echo "Symbolic link"
	elif test -f "$1"; then
		echo "File"
	elif test -d "$1"; then
		echo "Directory"
	else
		echo "(Unknown file type)"
	fi
}

GetOwner()
{
	# $1 = file path
	ls -ld "$1" | awk '{ print $3; }'
}

GetGroup()
{
	# $1 = file path
	ls -ld "$1" | awk '{ print $4; }'
}

GetFilePerms()
{
	# $1 = file path
	ls -ld "$1" | awk '{ print $1; }'
}

WorldCanWrite()
{
	# $1 = file path
	if IsFileOrDirectory "$1" && GetFilePerms "$1" | grep "^........w." >/dev/null; then
		return 0
	fi
	return 1
}

GroupCanWrite()
{
	# $1 = file path
	if IsFileOrDirectory "$1" && GetFilePerms "$1" | grep "^.....w...." >/dev/null; then
		return 0
	fi
	return 1
}

IsSecureUser()
{
	# $1 = user name
	if echo " `GetSecureUsers` owcimomd " | grep " $1 " >/dev/null 2>&1; then
		return 0
	fi
	return 1
}

IsSecureGroup()
{
	# $1 = group name
	if echo " `GetSecureGroups` owcimomd " | grep " $1 " >/dev/null 2>&1; then
		return 0
	fi
	return 1
}

IsRelativePath()
{
	# $1 = path
	if StartsWith "$1" "/"; then
		return 1
	fi

	return 0
}

# Check the supplied pathname for security, display errors for all security
# problems found with the file.  This checks user ownership, world writability,
# and group ownership (if group writable).
#
# Returns 0 if secure, 1 if insecure.
FileIsSecure()
{
	# $1 = file path

	# If it is a link, and it points to a directory, use the directory instead.
	if IsLink "$1"; then
		if IsDirectory "$1/."; then
			set -- "$1/."
		else
			# Links are supposed to be 777 and the owners don't matter.
			return 0
		fi
	fi

	filetype=`GetFileType "$1"`

	fis_retval=0

	if WorldCanWrite "$1"; then
		ERROR "${filetype} is world writable: $1"
		fis_retval=1
	fi

	if GroupCanWrite "$1"; then
		filegroup=`GetGroup "$1"`
		if IsSecureGroup "${filegroup}"; then
			:
		else
			ERROR "${filetype} belongs to insecure group (${filegroup}): $1"
			fis_retval=1
		fi
	fi

	fileowner=`GetOwner "$1"`
	if IsSecureUser "${fileowner}"; then
		:
	else
		ERROR "${filetype} belongs to insecure user (${fileowner}): $1"
		fis_retval=1
	fi

	if IsDirectory "$1"; then
		if IsRelativePath "$1"; then
			ERROR "Path to directory is relative: $1"
			fis_retval=1
		fi
	fi

	return ${fis_retval}
}

# Check a path and all of its ancestors for security.  If any are found to be
# insecure, this will return 1.
#
# Note: The functions called by this will display error messages for any of the
# files found to be insecure.
PathIsSecure()
{
	# $1 = path
	if IsRelativePath "$1"; then
		return 1
	fi

	secure_retval=0
	pathname="$1"

	while [ "${pathname}" != "/" ]; do
		if IsRegular "${pathname}"; then
			if FileIsSecure "${pathname}"; then
				:
			else
				secure_retval=1
			fi
		else
			# Not there, or not regular.
			:
		fi
		pathname=`dirname "${pathname}"`
	done

	# Check the root directory itself.
	FileIsSecure "/" || secure_retval=1

	return ${secure_retval}
}

CheckSecurePaths()
{
	# $1..$n = path names
	csp_retval=0
	while [ $# -gt 0 ]; do
		PathIsSecure "$1" || csp_retval=1
		shift
	done
	return ${csp_retval}
}

CheckTempDirectory()
{
	# no parameters
	tempdir=/tmp
	if IsLink ${tempdir}; then
		tempdir=/tmp/.
	fi

	if IsRegular "${tempdir}"; then
		tempfileperms=`GetFilePerms ${tempdir}`
		requiredperms="drwxrwxrwt"
		if [ "${tempfileperms}" != "${requiredperms}"  ]; then
#Seems like on Fedora 11 you get a . at the end of this
		    if [ "${tempfileperms}" != "${requiredperms}."  ]; then
			 ERROR "The temp directory /tmp has incorrect permissions."
			 ERROR "It should be ${requiredperms} but is ${tempfileperms}"
			 return 1
                    fi
		fi
	else
		ERROR "The temp directory /tmp is missing."
		return 1
	fi
}

CheckSecurity()
{
	cs_all_secure=1

	if CheckSecurePaths \
		/etc${UMI_PREFIX}/openwbem/openwbem.conf.d \
		${UMI_PREFIX}/libexec/openwbem \
		${REPOSITORY_DIRECTORY} \
		${UMI_PREFIX}/sbin
	then
		:
	else
		cs_all_secure=0

		ERROR "Valid secure users are: `GetSecureUsers`"
		ERROR "Valid secure groups are: `GetSecureGroups`"
	fi

	CheckTempDirectory || cs_all_secure=0

	if [ ${cs_all_secure} = 1 ]; then
		OutputVerbosely "File permissions all appear to be correct."
	else
		AbortInstallation "One or more files/directories had bad permissions or ownership"
	fi
}


###########################################################################
###########################################################################
# Start of upgrade functions
###########################################################################
###########################################################################

IsNumeric()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	for parameter in "$@"; do
		if [ "x$parameter" != "x" ]; then
			local_extracted_number=`echo "$parameter" | sed 's/[^0-9]//g'`
			if [ "x$local_extracted_number" != "x$parameter" ]; then
				return 1
			fi
		else
			return 1
		fi
	done
	return 0
}

# Get rid of any leading characters that are non-numeric, any trailing spaces,
# and any trailing zeroes in a version string.  All numeric digits are
# zero-padded to a length of 5 for easy numeric comparison.  The padding also
# makes the length of each segment uniform (if less than 5 digits) so
# converting "1.2.01" and "1.2.1" would result in the same string.
CleanupVersionString()
{
	# If you modify this function, please update this comment.
	#
	# How this works...
	# 1. Trailing spaces (or tabs) are removed.
	# 2. Any leading non-numeric characters (could be spaces) are removed.
	# 3. Remove any trailing '.0's (with function implementations of sed)
	# 4. Dots are changed to *TWO* spaces (to create non-overlapping testable
	#   regions).  Non-overlapping regions are important.
	# 5-6. Spaces are added at the start and end of the string.
	# 7-10. Leading 0's are added.  Each step increases the length by 1.
	#   The test region includes a space at the start, and some non-number at the
	#   end.  This requires that the regions do not overlap or global
	#   substitution would not apply the 0 padding to some of the numbers (hence
	#   the double spaces).
	# 11-12. Leading and trailing spaces are removed.
	# 13. Double spaces are converted to dots.
	temp_cleaned_version_string=`
	echo "$1" | \
	sed \
		-e 's/[	 ]*$//' \
		-e 's/^[^0-9]*//' \
		-e 's/\(\.00*\)*$//' \
		-e 's/\./  /g' \
		-e 's/^/ /' \
		-e 's/$/ /' \
		-e 's/ \([0-9]\{1\}[^0-9]\)/ 0\1/g' \
		-e 's/ \([0-9]\{2\}[^0-9]\)/ 0\1/g' \
		-e 's/ \([0-9]\{3\}[^0-9]\)/ 0\1/g' \
		-e 's/ \([0-9]\{4\}[^0-9]\)/ 0\1/g' \
		-e 's/^ //' \
		-e 's/ $//' \
		-e 's/  /./g'
`
	# Since the solaris sed does not properly match \(.\)* or \(.\){<NUMBER>},
	# we need to remove any remaining trailing '\.00*'s (if they are present).
	temp_cleaned_version_string2=`echo "${temp_cleaned_version_string}" | sed 's/\.00*$//'`
	while [ "${temp_cleaned_version_string2}" != "${temp_cleaned_version_string}" ]; do
		temp_cleaned_version_string="${temp_cleaned_version_string2}"
		temp_cleaned_version_string2=`echo "${temp_cleaned_version_string}" | sed 's/\.00*$//'`
	done
	echo "${temp_cleaned_version_string}"
	unset temp_cleaned_version_string temp_cleaned_version_string2
}

# Compare two versions in the Major.Minor.Micro.Build format.
# Both versions must be passed as dotted numeric strings, although they can
# include letters like in openssl's numbering scheme.
CompareVersionStrings()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	 local_version_1=`CleanupVersionString "$1"`
	 local_version_2=`CleanupVersionString "$2"`

	 greater_version=`{ echo ${local_version_1}; echo ${local_version_2}; } | sort -rn | head -n 1`
	 local_compare=0
	 if [ "${local_version_1}" = "${local_version_2}" ]; then
		 # Returns 0.
		 :
	 elif [ "${greater_version}" = "${local_version_1}" ]; then
		 local_compare=1
	 else
		 local_compare=-1
	 fi
	 unset local_version_1 local_version_2
	 echo $local_compare
	 unset local_compare
	 return 0
}

# Return 0 (true) if version from $1 is greater than from $2
VersionIsGreater()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	comparison=`CompareVersionStrings "$1" "$2"`
	if [ ${comparison} = 1 ]; then
		return 0
	fi
	return 1
}

VersionIsGreaterOrEqual()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	comparison=`CompareVersionStrings "$1" "$2"`
	if [ ${comparison} = 1 ]; then
		return 0
	elif [ ${comparison} = 0 ]; then
		return 0
	fi
	return 1
}

VersionIsEqual()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	comparison=`CompareVersionStrings "$1" "$2"`
	if [ ${comparison} = 0 ]; then
		return 0
	fi
	return 1
}

ExtractVersionFromFileName()
{
	evfn_file="$1"

	# Apply some transforms to extract version numbers from a filename.  This
	# includes stripping the OS name from the filename where it could include a
	# version string that would otherwise be extracted by this set of
	# transforms.
	#
	# With a functional sed, this could probably be done with nothing but sed
	# expressions.  Unfortunately, we support too many systems that have a
	# busted sed.
	basename "${evfn_file}" |
	sed -e 's/[Ss][Uu][Nn][Oo][Ss][-_.]5.[0-9][0-9]*//g' \
		-e 's/[Ss][Oo][Ll][Aa][Rr][Ii][Ss][-_.]5.[0-9][0-9]*//g' \
		-e 's/[Aa][Ii][Xx].[45].[0-9]//g' \
		-e 's/[Hh][Pp]-?[Uu][Xx].*//g' \
		-e 's/\.pkg.*$//g' \
		-e 's/9000\(.*\)\.depot.*$/\1/g' \
		-e 's/\.depot.*$//g' \
		-e 's/\.bff.*$//g' \
		-e 's/\.dmg.*$//g' \
		-e 's/\.rpm.*$//g' \
		-e 's/^[A-Za-z][A-Za-z0-9]*\.*//g' \
		-e 's/^[-_]*//g' \
		-e 's/\([0-9][0-9A-Za-z]*\.[0-9][0-9A-Za-z]*\)[-_]\([0-9]\)/\1.\2/g' \
		-e 's/\.\([A-Za-z]\)/!\1/g' |
	tr '\-_!' '\n\n\n' |
	grep '^[0-9].*\.' |
	sort -rn |
	head -n 1
	return 0
}

# Assumes: 	ObtainPackageDirectory has been called.
# $1 = package name (including any relative path)
ExtractVersionFromPackageFile()
{
	ev_package_name=$1
	ev_package_version=UNKNOWN

	ev_package_path="`ls -1 \"${CACHED_PACKAGE_DIRECTORY}\"/${ev_package_name}* | sort -rn | head -n1`"
	ev_package_file="`basename \"${ev_package_path}\"`"

	ExtractVersionFromFileName "${ev_package_file}" || return $?
}


# Returns true if the given package can be upgraded to the version in the
# user's chosen package directory (CACHED_PACKAGE_DIRECTORY).
CanUpgradePackage()
{
	upgrade_package_name="$1"
	if installed_package_version=`QueryExistingPackage "${upgrade_package_name}" 2>&7`; then
		source_package_version=`ExtractVersionFromPackageFile "${upgrade_package_name}"`
	   if VersionIsGreater ${source_package_version} ${installed_package_version}; then
			OutputNormal "New version (${source_package_version}) of ${upgrade_package_name} should be installed over version ${installed_package_version}"
			return 0
		else
			# We should not install packages with lesser or equal version numbers.
			return 1
		fi
	else
		OutputNormal "Not installed: ${upgrade_package_name}"
		return 0
	fi
}

InstallOrUpgradePackages()
{
	# This function makes the assumption that every time we break binary
	# compatibility we will increment the libs version number, and won't need to
	# worry about previous versions of UMI libraries that will continue to be
	# used by registered products.

	# Go through the packages and install anything that isn't installed, or was
	# uninstalled by a partial uninstall.  If a new version is already
	# installed, it will be skipped..
	for iop_package_name in "$@"; do
		iop_new_version=`ExtractVersionFromPackageFile "${iop_package_name}"`
		if QueryExistingPackage "${iop_package_name}" >&7 2>&1 ; then
			iop_old_version=`QueryExistingPackage "${iop_package_name}" 2>&7`
			if VersionIsGreater "${iop_new_version}" "${iop_old_version}" ; then
				OutputNormal "Upgrading ${iop_package_name} from version ${iop_old_version} to ${iop_new_version}"
				InstallPackages "${iop_package_name}" || return $?
			else
				# Either a newer version is really installed, or it was determined
				# above (upgrading via uninstall/reinstall) that it cannot be
				# upgraded.  Don't install an older version.
				OutputNormal "A newer version (${iop_old_version}) of ${iop_package_name} is already installed.  Not installing version ${iop_new_version}."
			fi
		else
			InstallPackages "${iop_package_name}" || return $?
		fi
	done
}

UpgradeUMINonLibs()
{
	InstallOrUpgradePackages `ListRequiredNonLibPackages` || return $?
}

UpgradeUMILibs()
{
	InstallOrUpgradePackages `ListRequiredLibraryPackages` || return $?
}

###########################################################################
# Begin VAS functions...
###########################################################################

InstallVASOnMacOS()
{

	# If vas ever supports a site-licensed package on MacOS, and we support
	# installing it at the same time as QMXSMS, this will need to be modified.
	package_name="VAS*.dmg"
	if ListFiles "${CACHED_PACKAGE_DIRECTORY}"/${package_name} >&7 2>&7; then
		packageToInstall=`ListFiles "${CACHED_PACKAGE_DIRECTORY}"/${package_name} | tail -1 | sed 's| |\\ |g'`
		osxVASClientDmgSubDir="VAS.mpkg/Contents/Packages/vasclnt.pkg"

		if [ -f "${PKGDIR}/osx-${OSARCH}/dmg_installer.sh" ]; then
			/bin/sh "${PKGDIR}/osx-${OSARCH}/dmg_installer.sh" -i "${packageToInstall}" -p ${osxVASClientDmgSubDir} || return $?
			/bin/sh "${PKGDIR}/osx-${OSARCH}/dmg_installer.sh" -i "${packageToInstall}" -c "ditto Uninstall.app /opt/quest/bin/Uninstall.app"
		else
			OutputError "ERROR: The dmg_installer.sh script is missing in the osx-${OSARCH} directory."
			OutputError "       The installation cannot proceed."
			return 1
		fi
	fi
}

VASNotSupportedOnThisPlatform()
{
	# MacOS 10.3 will never be supported by VAS.
	[ "$OSVARIANT" = "MacOS" ] && [ "$OS_MAJOR" = "10" ] && [ "$OS_MINOR" = "3" ] && return 0;
	# FIXME: MacOS 10.6 is not yet supported by VAS.
	[ "$OSVARIANT" = "MacOS" ] && [ "$OS_MAJOR" = "10" ] && [ "$OS_MINOR" = "6" ] && return 0;
	return 1;
}

VASIsSiteLicensed()
{
	for vaspkg in ${VAS_CLIENT_SITE_PACKAGE}; do
		if QueryExistingPackage ${vaspkg} >&7 2>&1; then
			OutputDebug "VAS Site Package (${vaspkg}) found. Do NOTHING."
			return 0 # Site pkg present
		fi
	done
	return 1
}

VASIsInstalled()
{
	# Find the installed vas package.
	INSTALLED_VAS_PACKAGE=
	INSTALLED_VAS_VERSION=
	for vaspkg in ${VAS_CLIENT_PACKAGE_LIST} ${VAS_2_6_CLIENT_PACKAGE_LIST} ${VAS_HOST_CLIENT_PACKAGE_LIST}; do
		if QueryExistingPackage "${vaspkg}" >&7 2>&1; then
			INSTALLED_VAS_PACKAGE=${vaspkg}
			INSTALLED_VAS_VERSION=`QueryExistingPackage "${vaspkg}" 2>&7`
			break;
		fi
	done

	if [ x${INSTALLED_VAS_PACKAGE:+set} = xset ]; then
		return 0
	fi
	return 1
}

# Check to see if VAS has a valid license
VASHasLicenses()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	if ${VASTOOL} license -i 2>&7 | grep "^Status:.*Valid License$" >&7; then
		# Since 3.5.0, VAS always includes a license (VAS_default).  To properly
		# detect real licensed installations, we need to check the number of unix
		# users.
		license_count=`${VASTOOL} license -q 2>&7 |
	 		grep -i 'licensed.*users' |
			sed -e 's/^.*:[ 	]*//g' -e 's/[ 	]*$//g'`

		if [ x${license_count:+set} = xset ] && [ ${license_count} -gt 1 ]; then
			return 0 # Have
		fi
	fi
	return 1 # Have not
}

# Check to see if VAS is joined
VASIsJoined()
{
	if ${VASTOOL} info domain >/dev/null 2>&7; then
		JOINED_DOMAIN=`${VASTOOL} info domain 2>&7`
		return 0 # Joined
	else
		return 1 # Not joined
	fi
}

FindVASPackageFiles()
{
	# Find the name of the vas package that should be installed.
	NEW_VAS_PACKAGE=
	NEW_VAS_VERSION=

	# Since VAS has so many packages with things specific to each particular OS
	# version, we need to add a package suffix to disambiguate the vas packages.
	vas_suffix=
	case ${OSTYPE}-${OSARCH} in
		AIX-*)
			if VersionIsGreaterOrEqual ${OSVER} 5.3; then
				vas_suffix=".AIX_5_3"
			else
				vas_suffix=".AIX_5_1"
			fi
			;;
		HP-UX-parisc)
			if [ "${OSVER}" = "B.11.11" ]; then
				vas_suffix="_9000"
			else
				vas_suffix="_hppa"
			fi
			;;
		HP-UX-ia64)
			vas_suffix="_ia64"
			;;
		SUNOS-sparc)
			vas_suffix="_SunOS_5.8_sparc"
			;;
		SUNOS-x86)
			vas_suffix="_SunOS_5.8_i386"
			;;
		SUNOS-x86_64)
			vas_suffix="_SunOS_5.10_i386"
			;;
	esac

	# The special case of "VAS" is here because the VAS team does not use the
	# same names for MacOS disk image files as they use for the actual packages.
	for vaspkg in "VAS" ${VAS_CLIENT_PACKAGE_LIST}; do
		if ListFiles "${CACHED_PACKAGE_DIRECTORY}"/${vaspkg}${vas_suffix}* >&7 2>&1 ; then
			NEW_VAS_PACKAGE=${vaspkg}${vas_suffix}
			NEW_VAS_VERSION=`ExtractVersionFromPackageFile "${vaspkg}" 2>&7`
			break;
		fi
	done

	if [ x${NEW_VAS_PACKAGE:+set} = xset ]; then
		return 0
	fi
	return 1
}

ShouldUpgradeVAS()
{
	# Check override flags
	if [ "x${DISABLE_VAS_UPGRADE:-0}" = "x1" ]; then
		OutputVerbosely "VAS upgrade was disabled externally."
		return 1
	fi

	if VASNotSupportedOnThisPlatform; then
		OutputVerbosely "Not upgrading VAS. Not supported on this platform: arch=${OSARCH}   ostype=${OSTYPE}   vendor=${OSVEND}${OSVARIANT:+   variant=}${OSVARIANT:-}   version=${OSVER}"
		return 1
	fi

	# This sets INSTALLED_VAS_PACKAGE and INSTALLED_VAS_VERSION
	if VASIsInstalled; then
		GetVASInstallPath
	else
		# Not installed... Safe to upgrade (a clean install).
		return 0
	fi

	# Do not upgrade a site-licensed vas no matter what the current join status
	# happens to be.
	if VASIsSiteLicensed; then
		OutputVerbosely "Not upgrading site-licensed vas."
		return 1
	fi

	# Compare versions to see if we can upgrade.
	if VersionIsGreaterOrEqual ${INSTALLED_VAS_VERSION} ${NEW_VAS_VERSION}; then
		OutputVerbosely "VAS package files for ${NEW_VAS_PACKAGE} ${NEW_VAS_VERSION} are older than the installed version (${INSTALLED_VAS_VERSION}).  Cannot upgrade."
		return 1
	fi

	if VASIsJoined; then
		OutputDebug "VAS is present and joined"
		if VASHasLicenses; then
			WARNING "VAS is joined and has licenses.  Refusing to automatically upgrade from ${INSTALLED_VAS_VERSION} to ${NEW_VAS_VERSION}."
			return 1
		fi

		# We can't upgrade a joined pre-3.0 vas client
		if VersionIsGreater 3.0 ${INSTALLED_VAS_VERSION}; then
			WARNING "Installed and joined VAS is too old to upgrade automatically."
			return 1
		fi

		OutputVerbosely "VAS is joined and can safely be upgraded in-place (no licenses found)."
		UPGRADE_VAS_IN_PLACE=1
	else
		OutputVerbosely "VAS is not joined.  Upgrade should be done normally"
		UPGRADE_VAS_IN_PLACE=0
	fi
	return 0
}

UpgradeVASHost()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	ObtainPackageDirectory || return $?

	# This sets NEW_VAS_PACKAGE and NEW_VAS_VERSION
	if FindVASPackageFiles; then
		:
	else
		# No package files.  Cannot upgrade.
		OutputVerbosely "Cannot locate VAS packages.  Not installing or upgrading VAS."
		return 1
	fi

	ShouldUpgradeVAS || return 0

	if [ x${UPGRADE_VAS_IN_PLACE:-0} = x0 ]; then
		# Uninstall any existing vas package.  Do not warn about missing vashost packages.
		UninstallPackages --nowarn ${INSTALLED_VAS_PACKAGE} ${VAS_HOST_CLIENT_PACKAGE_LIST}
	fi

	if [ $OSTYPE = DARWIN ]; then
		InstallVASOnMacOS || return $?
	else
		# If vasgp is present alongside the vas package, assume they want to use
		# it and install it.  The --noerror option prevents errors for files that
		# are missing (such as the vasgp packages).
		InstallPackages --noerror ${NEW_VAS_PACKAGE} ${VASGP_PACKAGES}
	fi

	if VASIsInstalled; then
		:
	else
		WARNING "VAS could not be installed."
	fi
}

###########################################################################
# End VAS functions...
###########################################################################

SaveRegisteredProductsState()
{
	BACKUP_PRODUCTS_DUMP_FILE=`MakeTemp registered_products_dump` || return $?
	OutputVerbosely "Saving data for registered products."
	# First dump all the state
	${UMI_PREFIX}/bin/product_tool --dump-product-state ${BACKUP_PRODUCTS_DUMP_FILE}
	# Next deregister all the products to ensure everything is fully cleaned
	for registered_product in `GetNonUMIRegisteredProducts` ""; do
		# If there are no registered products, this loop would be an error
		# (nothing to loop over).  The empty string was added to avoid that, so
		# we'll need to skip it here.
		if [ "x${registered_product}" = x ]; then
			continue
		fi

		OutputVerbosely "Deregistering ${registered_product}"
		if ${UMI_PREFIX}/bin/product_tool --deregister --name ${registered_product}; then
			:
		else
			WARNING "Failed to deregister ${registered_product}. Re-registration after upgrade may fail."
		fi
	done
}

RestoreRegisteredProductsState()
{
	if [ x${BACKUP_PRODUCTS_DUMP_FILE:+set} = xset ]; then
		if ${UMI_PREFIX}/bin/product_tool --restore-product-state ${BACKUP_PRODUCTS_DUMP_FILE}; then
			# We're done with the dump.  Remove it.
			rm -f ${BACKUP_PRODUCTS_DUMP_FILE}
		else
			ERROR "Failed to restore registered product state"
			return 1
		fi
	fi
}

Upgrade()
{
	UMI_VERSION_OLD=`GetUMIVersion`
	UMI_VERSION_NEW=`ExtractVersionFromPackageFile quest-umi-prov`

	if VersionIsGreaterOrEqual ${UMI_VERSION_NEW} ${UMI_VERSION_OLD} ; then
		if VersionIsEqual ${UMI_VERSION_NEW} ${UMI_VERSION_OLD} ; then
			OutputNormal "Reinstalling UMI ${UMI_VERSION_NEW}"
		else
			OutputNormal "Upgrading from UMI ${UMI_VERSION_OLD} to ${UMI_VERSION_NEW}"
		fi

		if HaveRegisteredProducts; then
			SaveRegisteredProductsState

			# Run the existing version's uninstall script to do a "partial uninstall"
			${UMI_PREFIX}/bin/${UNINSTALL_SCRIPT_NAME} --partial-uninstall ${UNINSTALL_OPTIONS}
		elif echo "${UMI_VERSION_OLD}" | grep '^1\.0' >&7; then
			# We do a full uninstall here of UMI 1.0x to ensure that everything is cleaned up.
			if [ -f ${UMI_1_0_PREFIX}/bin/${UNINSTALL_SCRIPT_NAME} ]; then
				${UMI_1_0_PREFIX}/bin/${UNINSTALL_SCRIPT_NAME} ${UNINSTALL_OPTIONS}
			fi
		else
			# We do a full uninstall here to ensure that everything is cleaned up properly.
			if [ -f ${UMI_PREFIX}/bin/${UNINSTALL_SCRIPT_NAME} ]; then
				${UMI_PREFIX}/bin/${UNINSTALL_SCRIPT_NAME} ${UNINSTALL_OPTIONS}
			fi
		fi

		# The users and groups could have been deleted when uninstalling the old
		# version.  If they exist, this won't do anything.
		CreateUsers || return $?

		# Install all required packages.
		InstallOrUpgradePackages `ListRequiredPackages` || return $?

		# This should be removed when the files in the packages are verified to
		# be owned by the correct users on all of the platforms.
		SetRepositoryPermissions

		# Don't bail if the VAS packages aren't there.  This will be little
		# chance of having a completely broken install if we continue.
		UpgradeVASHost || true

		RestoreRegisteredProductsState

	elif VersionIsGreater ${UMI_VERSION_OLD} ${UMI_VERSION_NEW}; then
		# A newer version of UMI is installed, so we'll just install the new
		# (older) version of the UMI library packages.
		OutputNormal "Installing (for compatibility) UMI libraries for UMI ${UMI_VERSION_NEW}"
		UpgradeUMILibs
	fi
}

DisableUMIStartup()
{
	touch /etc/${UMI_PREFIX}/UMI.nostartup
}

###########################################################################
###########################################################################
# End of upgrade stuff.
###########################################################################
###########################################################################

VerifyPrereqs()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	# Check for missing prerequisites that we cannot distribute.
	install_missing_support_packages=0
	if [ ${OSTYPE} = LINUX ]; then
		case ${OSVEND}-${OSVER}-${OSARCH} in
			RedHat-4[EAW]S-x86 | SL-4[012456789]-x86 | SLF-4[12456789]-x86 | CentOS-4-x86)
				# We only want to check for special rh4 dependencies on x86.  These
				# may be issues on later versions of rh*S as well.  This should not
				# cause much (if any) breakage even if missing.
				if QueryExistingPackage compat-db-4.1.25 >&7; then
					:
				else
					# Supply a warning about potential breakage of items we do not use directly.
					WARNING "compat-db-4.1.25 was not found."
				fi
				;;
			*)
				;;
		esac
	fi
}

StopEverything()
{
	Stop -q || true

	if PrngdIsInstalled ; then
		StopPrngd -q || true
	fi
}

DoUMIInstall()
{
	PrepareTools

	[ "$UNSUPPORTED" = "1" ] && AskUnsupportedQuestion

	# Check that all required packages (that we can't distribute) are installed.
	VerifyPrereqs

	# Do a pretest to make sure things are secure.
	CheckSecurity

	DetachIfNeeded "$@"

	StopEverything

	CreateUsers || return $?

	# Install (this will attempt to upgrade if it is already installed)
	Install || return $?

	if PrngdIsInstalled ; then
		StartPrngd || WARNING "Failed to start the quest-prngd daemon."
	fi
}

DoUMIUninstall()
{
	# Check for registered products before even stopping the CIMOM.
	if HaveRegisteredProducts; then
		AbortInstallation "Registered products are still in use.  Cannot uninstall UMI."
	fi

	DetachIfNeeded "$@"

	StopEverything

	Uninstall

	# Remove users
	if [ ${KEEP_USERS_OPT} = 0 ]; then
		RemoveUsers
	else
		WARNING "Not removing UMI users."
	fi
}


Main()
{
	trap ShellExited EXIT

	# Set up some output descriptors...
	SetupDescriptors

	GetPlatformInformation

	if [ ${DEVELOPER_MODE:-0} = 1 ]; then
		# Allow the developer mode option to be automagically added if
		# DEVELOPER_MODE is set.  This is here so any product trying to
		# install/uninstall umi won't need to worry about passing along any
		# --developer-mode options.
		set - "$@" --developer-mode
	fi

	if [ "$SCRIPT_NAME" = "$UNINSTALL_SCRIPT_NAME" ]; then
		ParseCommandLine "$@" -r
	else
		ParseCommandLine "$@"
	fi

	# Reconfigure the descriptors now that the command line is parsed.
	# This may enable the debug descriptor.
	SetupDescriptors

	if [ ${DEBUG_ENABLED:-0} = 1 ]; then
		${ENABLE_DEBUGGING_NOW}

		# If debugging was requested, run GetPlatformInformation again so that
		# the results can appear in the debug output as well.
		GetPlatformInformation
	fi

	if [ "x$EXECUTED_FROM_CRON" = "x1" ]; then
		RemoveCronEntry
	fi

	# Check to see if this install script was started from within UMI.
	if [ -n "${QUEST_UMI_STARTED_PROCESS:-}" ] || [ -n "${VINTELA_VMX_STARTED_PROCESS:-}" ]; then
		DETACH_REQUIRED=1
	fi

	# If we're in the detached script, we have to wait a while before beginning the
	# upgrade so that the client can complete the software distribution process before
	# we shut it down.
	if [ "x${INSTALL_SCRIPT_DETACHED:-}" = "x1" ] || [ "x${EXECUTED_FROM_CRON:-}" = "x1" ]; then
		OutputNormal "Waiting: Allowing 60 seconds for the software distribution process (if any) to complete."
		sleep 60
	fi

	if [ $DETACH_REQUIRED = 1 ]; then
		# Add the quiet flag and reparse the command line (to locate errors early
		# on).
		set - "$@" -q
		ParseCommandLine "$@"
	fi

	if [ $UNINSTALL_OPT = 0 ]; then
		# Don't immediately abort if unsupported.
		if CheckForSupportedPlatform; then
			:
		else
			UNSUPPORTED=1
		fi
	fi

	# All operations below here require root.
	VerifyRootUser

	case $INSTALL_TYPE in
		install)

			# Failure on upgrade should not cause an uninstall.
			if UMIIsInstalled; then
				OutputNormal "Detected existing umi install.  Packages will be upgraded."
				handle_failure_as_upgrade=1
			fi

			# Install UMI.  Any failures in a normal installation will cause the
			# client to be uninstalled completely.
			if DoUMIInstall "$@"; then
				# Start the CIMOM if required.  Failures to start will cause the
				# client to be uninstalled.
				if [ ${STARTUP_DISABLED} = 1 ]; then
					DisableUMIStartup
				else
					if Start; then
						:
					else
						ERROR "CIMOM startup failed."
						OutputError "CIMOM log contains: "
						tail -30 /var${UMI_PREFIX}/logs/owcimomd.log >&2
						ERROR "Installation is non-functional.  Rolling back any installed packages."
						DoUMIUninstall "$@" || AbortInstallation "Failed to roll back unsuccessful UMI instalation."
					fi
				fi

				HaveErrors || OutputNormal "Installation successful."
			elif [ ${handle_failure_as_upgrade:-0} = 1 ]; then
				if [ ${STARTUP_DISABLED} = 1 ]; then
					AbortInstallation "Failed to install UMI upgrade.  A complete reinstall may be necessary."
				else
					# Since we were upgrading, we'll try to restart the existing
					# client.  This may fail if we ran out of disk space during
					# installation or something else was half-installed when the
					# error occurred.
					ERROR "Failed to install an upgrade.  Attempting to restart existing CIMOM."
					Start	|| AbortInstallation "Failed to start the existing UMI client after a failed upgrade.  A complete reinstall may be necessary."
				fi
			else
				ERROR "UMI Installation failed.  Rolling back installation."
				# Failures on uninstall shouldn't happen here, but we need
				# to report it if it does.
				DoUMIUninstall "$@" || AbortInstallation "Failed to roll back unsuccessful UMI instalation."
			fi
		;;
		uninstall)
			DoUMIUninstall "$@"
			OutputNormal "UMI Uninstallation successful."
		;;
		partial_uninstall)
			OutputNormal "Checking to see if UMI is already installed."
			if UMIIsInstalled ; then
				:
			else
				AbortInstallation "Cannot uninstall UMI.  It does not appear to be installed."
			fi
			DoPartialUninstall
		;;
		remove_compatibility)
			client_count=`GetNonUMIRegisteredProducts | wc -l`
			# Strip any whitespace.
			client_count=`echo ${client_count}`
			if [ "x${client_count}" = x1 ] || [ "x${client_count}" = x0 ]; then
				# One client installed.  It is that client that is
				# requesting the compatibility packages be removed.
				UninstallPackages --nowarn `ListOldUMIPackages`
			else
				WARNING "Not removing compatibility libraries.  More than one product is registered."
			fi
		;;
		*)
			AbortInstallation "Unknown install type $INSTALL_TYPE"
		;;
	esac

	TERMINATED_EARLY=0
}

Main "" "${@:-}"
