#!/bin/sh

################################################################################
# Copyright (c) 2009 Quest Software, Inc.  All Rights Reserved.
#
# deleted_check.sh:     Maintains users and groups vasd cache, and RFC 2307
#                       netgroup vasypd cache consistency.
# 
# version:              1.8
#
# authors:              Seth Ellsworth
#
# This sample script is provided "as-is". It relies on internal knowledge of
# vasd and vasypd's internal cache schema, which is undocumented and subject to
# change between releases.
#
################################################################################

# TODO: Add single search path functionality. This means this queries the entire
#       domain, so could be slow for non-UPM machines with a search-base and
#       a lot of users.
# TODO: Add multi search path functionality.

# WARNING: This removes any users/groups not in the current search path. 
#          This means groups from PAC, or cross domain ( if not in search
#          path ) will disappear each time this runs. 

################################################################################
# Update: 
#         2008_12_09: Added duplicate group clearing functionity.
#
#         2008_10_30: Better message when in UPM mode and no local non-upm 
#                     users or groups. Better detect non-RFC2307 NIS maps 
#                     in earlier VAS versions.
#
#         2008_10_22: LOG/LOGE now use $@ instead of $1, some messages were
#                     getting cut of when sent to logger. Better logging when
#                     removing NIS maps.
#
#         2008_09_30: When processing deleted generic maps, a dangling reference
#                     was left in the nis2map table. This has been fixed.
#
#         2008_09_02: Removed most use of CheckThreshold because using list -f
#                     instead of direct delete removes any posssilby accidental
#                     conplete cache wipe.
#
#         2008_08_04: Better wrapping of ECHO variables, with the new
#                     abstraction layer unquoted echo calls were causing issues.
#
#         2008_08_02: Ensure DB read before AD read. ( preventing possible race
#                     condition )
#
#         2008_07_22: Cleanup duplicate entries during non-upm processing.
#                     Break up dual use of COMM. 
#
#         2008_07_13: Move to list -f {user|group} for udpating users/groups, 
#                     and use the USN server for NIS maps. 
#
#         2008_07_09: Re-write a LOT of the script. Should be much easier to
#                     follow now. Better handling of possible errors.
#
#         2008_06_29: Cleanup all tested files. umask set to 077. Handle full 
#                     file systems slightly different due to vastool search 
#                     returning 1 for both no entries found and errors writing 
#                     output due to file system full or read-only errors. 
#
#         2008_05_18: Add a new variable, LOG_PRIORITY, that specifies the 
#                     syslog level to log at for any failure that results in 
#                     immediate exit. 
#
#         2008_05_09: When using load-from-gc for groups it could remove all 
#                     groups. Fixed.
#
#         2008_03_28: If it fails, still log. 
#
#         2008_01_13: 3.2 changed netgroups to a vasd process, handle the new 
#                     location.
#
#         2007_10_03: Now take some settings from vas.conf. Still works from the
#                     the script at well. Section: [deleted_check], options:
#                     DO_NISMAPS = 0, DO_LOGGING = 1, DO_CONSISTENCY_CHECK = 1
#
#                     Lot more error checking. Both on failed AD searches and
#                     DB access.
#       
#         2007_09_14: Added new option, DO_NISMAPS. If set to 1, check nisMap
#                     objects for deleted maps and deleted objects.
#
#         2007_08_02: Adding support for the load-*-from-GC options.
#
#         Adding new functionality, cache consistency. If there are items
#         found in AD that are not in the cache, an update for those items
#         will be triggered. See the new variable DO_CONSISTENCY_CHECK
################################################################################

################################################################################
# User defined variables.
#  
# These are for sanity checks on the numbers being removed from the cache. 
# If AD or the query or the machine get into a bad state, and the script
# thinks it should be removing a large number of users, it will wait.
#
# Though all three caches will track this individually, they all use the same 
# settings. 
#
# THRESHOLD:    What percentedge of the users/groups/netgroups can be removed 
#               from the cache before the script refuses to comply until 
#               NUM_TIMES is reached.
#
# NUM_TIMES:    Number of times the script ignores an update removal of 
#               > THRESHOLD percentage of users from a cache. 
################################################################################
VASCONF=/etc/opt/quest/vas/vas.conf
if [ -z "$DEBUG" ] ; then
    # set -x in each of the functions if not passed in
    DEBUG=0
else
    DEBUG=1
fi
if [ "${DEBUG}" -eq 1 ] ; then set -x; fi

# Some setting can now be set from vas.conf, DO_LOGGING, DO_CONSISTENCY_CHECK and DO_NISMAPS.
# Under the deleted_check section, set to 1 or 0. 

GREP ()
{
    grep "$@"
    RVAL=$?
    if [ $RVAL -ge 2 ] ; then
        LOGE "grep failed with $RVAL"
        CleanUp
        exit 1
    fi
    return $RVAL
}

TR ()
{
    tr "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "tr failed with $RVAL"
        CleanUp
        exit 1
    fi
}

WC ()
{
    wc "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "wc failed with $RVAL"
        CleanUp
        exit 1
    fi
}

MV ()
{
    mv "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "mv failed with $RVAL"
        CleanUp
        exit 1
    fi
}

RM ()
{
    rm "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "rm failed with $RVAL"
        CleanUp
        exit 1
    fi
}

AWK ()
{
    awk "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "awk failed with $RVAL"
        CleanUp
        exit 1
    fi
}

SED ()
{
    sed "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "sed failed with $RVAL"
        CleanUp
        exit 1
    fi
}

CAT () 
{
    cat "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "cat failed with $RVAL"
        CleanUp
        exit 1
    fi
}

CUT () 
{
    cut "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "cut failed with $RVAL"
        CleanUp
        exit 1
    fi
}

COMM () 
{
    comm "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "comm failed with $RVAL"
        CleanUp
        exit 1
    fi
}

SORT () 
{
    sort "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "sort failed with $RVAL"
        CleanUp
        exit 1
    fi
}

UNIQ () 
{
    uniq "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "uniq failed with $RVAL"
        CleanUp
        exit 1
    fi
}

ECHO () 
{
    echo "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "echo failed with $RVAL"
        CleanUp
        exit 1
    fi
}

VASSEARCH ()
{
    TEST_FILE $VASTMP3
    $VAS search "$@" 2>$VASTMP3
    RVAL=$?
    if [ $RVAL -gt 1 ] ; then
        LOGE "vastool search <$@> failed with $RVAL"
        CleanUp
        exit 1
    elif [ $RVAL -eq 1 ] ; then
        CAT $VASTMP3 | GREP "VAS_ERR_NOT_FOUND" >/dev/null
        if [ $? -ne 0 ] ; then
            LOGE "vastool search <$@> failed with $RVAL"
            CleanUp
            exit 1
        fi
    fi    
}

CheckExec ()
{
    if [ $1 -ge 1 ] ; then
        LOGE "Previous exec failed with $1"
        CleanUp
        exit 1
    fi
}

LOG ()
{
#1 Message to log
#2 section to log from, leave blank if desired.
    $DO_LOGGING || return
    logger -t del_check "${LOGGER_ADD}: $@"
}

LOGE ()
{
#1 Message to log
#2 section to log from, leave blank if desired.
    logger -p $LOG_ERR -t del_check "${LOGGER_ADD}: $@"
}

# 1 is section.
# 2 is attribute.
# Result will be ECHO'ed, if nothing is echo'ed, then it wasn't there.
GET_CONF_SETTING ()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
# Make sure we got variables.
    if [ -z "$1" -o -z "$2" ] ; then
        return
    fi

# Make sure we have a vas.conf
    if [ ! -f "$VASCONF" ] ; then
        ECHO "Unable to find $VASCONF, exiting."
        exit 1
    fi

# See if the given section exists 
    GREP "^\[$1\]" $VASCONF >/dev/null
    if [ "$?" -ne 0 ] ; then
        return
    fi        

# Now we can really start processing it. 
# While loop, processing each line, keeping track of when we are in 
# the right section, and removing comments each step

# First, clean up the line.
    IN_SECTION=0
    while read line
    do
        LOCLINE="`ECHO \"$line\"| SED -e 's/#.*//' -e 's/^[ \t]*//'`" ; CheckExec $?    
        if [ -z "$LOCLINE" ] ; then continue ; fi

        if [ "$IN_SECTION" = "1" ] ; then
            ECHO "$LOCLINE" | GREP "^\\[" >/dev/null
            if [ "$?" = "0" ] ; then
                IN_SECTION=0
                continue
            fi    
# In the seciton, lets look for the setting.
            ECHO "$LOCLINE" | GREP "$2[ ]*=" >/dev/null
            if [ "$?" = "0" ] ; then
# Found it, this is our response.
                RESULT="`ECHO \"$LOCLINE\" | SED \"s/^$2[ ]*=[ ]*\\(.*\\)[ ]*/\\1/\"`" ;  CheckExec $?    
                if [ ! -z "$RESULT" ] ; then 
                    ECHO "$RESULT"
                fi
                return
            fi    
        else
# Look for the section
            ECHO "$LOCLINE" | GREP "^\\[$1\\]" >/dev/null
            if [ "$?" = "0" ] ; then
                IN_SECTION=1
                continue
            fi
        fi
    done < $VASCONF
}

# Percentage. Default is 25, meaning if the cache has 100 users, and AD says 
# there are now 75 or less users, then it is not done unless it happens 
# NUM_TIMES in a row.
THRESHOLD=25

# How many consecutive times a given cache must be asked to have > THRESHOLD %
# users/groups/netgroups removed. This is times of the script being called.
# If it is called once a day, on the fourth day it will actually do the deletion.
# This should give admins time to verify the validness of the removal. 
# Either way this will be logged to syslog using logger.
# This is tracked seperatly for each of users, groups, and netgroups. 
NUM_TIMES=3

# This will be put in the syslog message created at the default && logger level.
# This is to allow for easy searching in syslog output. 
LOGGER_ADD="***del_check output***"

# Set to 1 to track individual entry deletions. 
# Users will show UPN, groups samAccountName, and nisNetGroups CN.
DELETE_DEBUG=1

# Process nisObjects in AD for missing entries.
V="`GET_CONF_SETTING deleted_check DO_NISMAPS`" ; CheckExec $?
if [ -z "$V" ] ; then
# Default value
    DO_NISMAPS=1
else
    if [ "$V" = "1" ] ; then
        DO_NISMAPS=1
    else
        DO_NISMAPS=0
    fi
fi

# Keep netgroups up to date.
DO_NETGROUPS=1

# New option. If this is set ( default ), when processing the differences
# between AD and the local cache, items found in AD but not the local cache
# will be updated in an attempt to place in the local cache. 
# Due to the chance in non-UPM and with a search-base that users outside 
# the search-base will try to be forced in, this is disabled for non-UPM for
# now. Once the first two TODO's are done, that will be changed. 
# Also, if in workstation mode, this will be skipped, otherwise everything
# would be pulled down, breaking WS mode. 
V="`GET_CONF_SETTING deleted_check DO_CONSISTENCY_CHECK`" ; CheckExec $?    

if [ -z "$V" ] ; then
# Default value
    DO_CONSISTENCY_CHECK=1
else
    if [ "$V" = "1" ] ; then
        DO_CONSISTENCY_CHECK=1
    else
        DO_CONSISTENCY_CHECK=0
    fi
fi

V="`GET_CONF_SETTING deleted_check DO_LOGGING`" ; CheckExec $?    
if [ -z "$V" ] ; then
# Default value
    DO_LOGGING=true
else
    if [ "$V" = "1" ] ; then
        DO_LOGGING=true
    else
        DO_LOGGING=false
    fi
fi


# What level to log failures that stop execution.
LOG_ERR="daemon.err"

# New option. This is for Workstation mode. If set, all users will be cleared
# from the cache unless they have a running process ( logged in, or left 
# something running, in which case they will be kept.
WS_REMOVE_LOGGED_OUT_USERS=0

# New option. This is for UPM mode. If there are no regular AD users/groups in 
# the UPM paths, then skip that processing.
UPM_DO_REGULAR_USERS=1
UPM_DO_REGULAR_GROUPS=1

################################################################################
# Locally used variables.
################################################################################
# If the removal is deemed ok (lots of users were ACTUALLY removed) 
# then put a number > THRESHOLD into the appropreate file, and next 
# time the deletions will happen.
UCOUNT_FILE=/var/opt/quest/vas/vasd/.ucount
UPCOUNT_FILE=/var/opt/quest/vas/vasd/.upcount
GCOUNT_FILE=/var/opt/quest/vas/vasd/.gcount
GPCOUNT_FILE=/var/opt/quest/vas/vasd/.gpcount
NCOUNT_FILE=/var/opt/quest/vas/vasd/.ncount
TRACKERFILE=/var/opt/quest/vas/vasd/.deleted_check_last_run

UPPER=ABCDEFGHIJKLMNOPQRSTUVWXYZ
LOWER=abcdefghijklmnopqrstuvwxyz

VAS=/opt/quest/bin/vastool
SQL3=/opt/quest/libexec/vas/sqlite3
UL=/tmp/tmp.users.local.$$
UA=/tmp/tmp.users.ad.$$
UD=/tmp/tmp.users.delete.$$
UPL=/tmp/tmp.usersp.local.$$
UPA=/tmp/tmp.usersp.ad.$$
UPD=/tmp/tmp.usersp.delete.$$
GL=/tmp/tmp.groups.local.$$
GA=/tmp/tmp.groups.ad.$$
GD=/tmp/tmp.groups.delete.$$
GPL=/tmp/tmp.groupsp.local.$$
GPA=/tmp/tmp.groupsp.ad.$$
GPD=/tmp/tmp.groupsp.delete.$$
NL=/tmp/tmp.netgroups.local.$$
NA=/tmp/tmp.netgroups.ad.$$
ND=/tmp/tmp.netgroups.delete.$$
CLEANUP_FILES=
VAS_DB=/var/opt/quest/vas/vasd/vas_ident.vdb
VAS_YP_DB=/var/opt/quest/vas/vasypd/nismaps/rfc2307_nismaps.vdb
VAS_MISC_DB=/var/opt/quest/vas/vasd/vas_misc.vdb
NISLOC=
VASTMP=/tmp/tmp.$$
VASTMP2=/tmp/tmp2.$$
VASTMP3=/tmp/tmp3.$$
SEEALSO=
RFC2307=false
NISSERVER=

VAS_NS_DB=/var/opt/quest/vas/vasd/vas_ns.vdb

if [ ! -f "$VAS_NS_DB" ] ; then
    VAS_NS_DB="$VAS_YP_DB"
fi

# This keeps track of what vastool search returns when it can't find any results for the given query. 
VASSEARCHERROR=1

# Tests that the given file can be created, if not exit. 
TEST_FILE ()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    F="$1"
    if [ -f "$F" ] ; then 
        rm -f "$F"
        if [ -f "$F" ] ; then 
            ECHO "Unable to clear file $F, exiting."
            LOGE "$0: Unable to clear file $F, exiting."
            CleanUp
            exit 1
        fi
    fi    

    touch "$F"
    if [ ! -f "$F" ] ; then 
        ECHO "Unable to create file $F, exiting."
        LOGE "$0: Unable to create file $F, exiting."
        CleanUp
        exit 1
    fi

    ECHO "test a lenght of space. END" > "$F"
    GREP END "$F" >/dev/null
    if [ $? -ne 0 ] ; then
        ECHO "Unable to create file $F with data, exiting."
        LOGE "$0: Unable to create file $F with data, exiting."
        rm -f "$F"
        CleanUp
        exit 1
    fi

    ECHO "$CLEANUP_FILES" | GREP "$F" >/dev/null
    if [ $? -ne 0 ] ; then
        CLEANUP_FILES="$CLEANUP_FILES $F"
    fi        
}

SQLLOOP ()
{   
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
# $1 is the DB, $2 is the command. 
    DB="$1"
    COMMAND="$2"
    $SQL3 $DB "$COMMAND"
    if [ $? -ne 0 -a -f "$1" ] ; then
        D=1
        while [ $D -lt 5 ] ; do
            $SQL3 $DB "$COMMAND"
            RVAL=$?    
            if [ $RVAL -eq 0 ] ; then
                break
            fi
            D=`expr $D + 1` ;  CheckExec $?    
            sleep 1
        done
        if [ "$RVAL" -ne "0" ] ; then
            ECHO "VASDB query <$COMMAND> on DB <$DB> failed after 5 tries with error $RVAL, exiting." >&2
            LOGE "$0: VASDB query <$COMMAND> on DB <$DB> failed after 5 tries with error $RVAL, exiting."
            CleanUp
            exit 1
        fi
    fi
}

SanityCheck()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi

    # Log starting
    LOG "Starting"    

    # If called by vasd the umask could be 0 ( normal for daemons ), set to 077
    umask 077    
        
    # Test for root access. 
    ID="`id | SED 's/uid=\([0-9]*\).*/\1/'`" ; CheckExec $?
    if [ ! "$ID" = "0" ] ; then
        ECHO "Must be root to run."
        LOGE "NOT_ROOT: del_check not run as root, exiting."
        CleanUp
        exit 1
    fi

    # Test VAS and AD with a kinit of the computer object.
    # Also set up a credential, so we dont' have to request each time.
    ${VAS} kinit host/
    if [ "$?" -ne 0 ]
    then
        ECHO "Unable to verify VAS connectivity to AD, exiting."
        LOGE "BAD_KINIT: Unable to verify VAS connectivity to AD, exiting."
        CleanUp
        exit 1
    fi
    for f in $UL $UA $UD $UPL $UPA $UPD $GL $GA $GD $GPL $GPA $GPD $NL $NA $ND $VASTMP $VASTMP2 $VASTMP3; do
        TEST_FILE $f
    done
}

UPMCheck()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    # Determine if in UPM mode. 
    UPMPATH="`SQLLOOP ${VAS_MISC_DB} \"Select value from misc where key = 'upmSearchBase'\"`" ;  CheckExec $?
    if [ -z "${UPMPATH}" ] ; then 
        UPMMODE=0
    else
        UPMMODE=1
# This one is fun. Can't split it withoput using a file, because in some shells the \n's will disappear.
# So lets use a file. 
        VASSEARCH -s base -q -b "${UPMPATH}" '(objectClass=*)' seeAlso > ${VASTMP2}
        SEEALSO="`TR '\n' ':' < $VASTMP2`:" ; CheckExec $?
    fi
}

WSCheck()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    # Determine if in Workstation mode.
    SQLLOOP ${VAS_MISC_DB} "Select value from misc where key = 'workstationMode'" | GREP -i true >/dev/null 2>&1
    if [ "$?" -eq 0 ] ; then 
        # Don't do consistency check in ws mode, otherwise it will load EVERYTHING into the cache.
        DO_CONSISTENCY_CHECK=0
        WSMODE=1
    else
        WSMODE=0
    fi
}

UnlinkedUserCheck()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    # Determine if allowing unlinked users.
    SQLLOOP ${VAS_MISC_DB} "Select value from misc where key = 'allowUnlinkedUPP'" | GREP -i true >/dev/null 2>&1
    if [ "$?" -eq 0 ] ; then
        ALLOWUUPP=1
    else
        ALLOWUUPP=0
    fi
}

UnlinkedGroupCheck()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    # Determine if allowing unlinked groups.
    SQLLOOP ${VAS_MISC_DB} "Select value from misc where key = 'allowUnlinkedGPP'" | GREP -i true >/dev/null 2>&1
    if [ "$?" -eq 0 ] ; then
        ALLOWUGPP=1
    else
        ALLOWUGPP=0
    fi
}

UIDCheck()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    # Determine if in Workstation mode.
    SQLLOOP ${VAS_MISC_DB} "Select value from misc where key = 'upmUsernameUseCN'" | GREP -i true >/dev/null 2>&1
    if [ "$?" -eq 0 ] ; then
        UIDMODE=0
    else
        UIDMODE=1
    fi
}

UserGCCheck()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    # Determine if in load-users-from-gc mode.
    SQLLOOP ${VAS_MISC_DB} "Select value from misc where key = 'loadUsersFromGC'" | GREP -i true >/dev/null 2>&1
    if [ "$?" -eq 0 ] ; then
        USERGCMODE=1
    else
        USERGCMODE=0
    fi
}

GroupGCCheck()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    # Determine if in load-users-from-gc mode.
    SQLLOOP ${VAS_MISC_DB} "Select value from misc where key = 'loadGroupsFromGC'" | GREP -i true >/dev/null 2>&1
    if [ "$?" -eq 0 ] ; then
        GROUPGCMODE=1
    else
        GROUPGCMODE=0
    fi
}

GroupNameCheck()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    # Determine if in load-users-from-gc mode.
    SQLLOOP ${VAS_MISC_DB} "Select value from misc where key = 'groupNameAttribute'" | GREP -i cn >/dev/null 2>&1
    if [ "$?" -eq 0 ] ; then
        GROUPCNMODE=1
    else
        GROUPCNMODE=0
    fi
}

RFC2307Check ()
{
    V="`GET_CONF_SETTING vasypd update-process`" ; CheckExec $?
    if [ -z "$V" ] ; then
# Default value
        VASVERSION="`$VAS -v | grep -i version | head -1 | sed 's/vastool: VAS Version \([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)/\1 \2 \3 \4/'`"
        VASVERMAJOR="`echo $VASVERSION | awk '{print $1}'`"
        VASVERMINOR="`echo $VASVERSION | awk '{print $2}'`"
        if [ "$VASVERMAJOR" = "3" -a -n "$VASVERMINOR" -a $VASVERMINOR -lt 3 ] ; then
            RFC2307=false
        else
            RFC2307=true
        fi
    else
        ECHO "$V" | GREP "_2307" > /dev/null
        if [ $? -eq 0 ] ; then
            RFC2307=true
        fi
    fi
    if [ -f ${VAS_YP_DB} ] ; then
        NISLOC="`SQLLOOP ${VAS_YP_DB} \"select value from vasypinfo where key='searchbase';\"`" ; CheckExec $?

        if [ -z "${NISLOC}" ] ; then
    # Use the computer object location.
            NISLOC="`${VAS} info id | GREP LDAP | SED 's/LDAP://' | SED 's/ *CN=[^,]*,//'`" ; CheckExec $?
        fi
        NISSERVER="`SQLLOOP ${VAS_YP_DB} \"select value from vasypinfo where key='usnServer';\"`" ; CheckExec $?
        if [ -z "${NISSERVER}" ] ; then
            DO_NISMAPS=0
        fi        
    fi
}

SQLTIME ()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
# $1 is the DB, $2 is the command. 
    DBTMP=/tmp/tmp.dbtemp.$$
    DB="$1"
    if [ ! -f "$DB" ] ; then
        return 1
    fi
    TEST_FILE $DBTMP
    COMMAND="$2"
    ECHO ".timeout 10000" > $DBTMP
    ECHO "$COMMAND" >> $DBTMP
    ECHO | $SQL3 -init $DBTMP "$DB"
    RVAL=$?
    if [ "$RVAL" -ne "0" ] ; then
        ECHO "VASDB query <$COMMAND> on DB <$DB> failed with error $RVAL, exiting."
        LOGE "VASDB query <$COMMAND> on DB <$DB> failed with error $RVAL, exiting."
        CleanUp
        exit 1
    fi
    RM -f $DBTMP
    return $RVAL
}

CheckThreshold()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    # 1 is the local users file.
    # 2 is the file of users (local) to be removed.
    # 3 is the file to log to.
    # 4 is the name of the test for logger.
    # If the threshold is not reached the log file is removed. 
    # 
    touch "${3}"
    if [ ! -r "${1}" ] || [ ! -r "${2}" ] || [ ! -w "${3}" ] || [ -z "${4}" ] ; then
       # Hmm... No files or bad paramaters.... That is a bad thing. 
        LOG "BAD_PARAM: del_check called with bad paramaters. CheckThreshold is exiting. "
        return 1
    fi
        
    LC=`WC -l ${1}| AWK '{print $1}'` ; CheckExec $?
    DC=`WC -l ${2} | AWK '{print $1}'` ; CheckExec $?
    if [ "${DC}" -le 0 ] ; then
        LOG "CHECK: testing ${4}, none to remove from the local cache(${LC} currently)."
        RM -f "${3}"
        return 0
    else        
        if [ "${DC}" -eq "${LC}" -a "${DC}" -gt 0 ] ; then
            LOG "CHECK: testing ${4}. Local cache has ${LC} entries, asked to remove ${DC}. Will not remove all entries"
            return 1    
        else    
            LOG "CHECK: testing ${4}. Local cache has ${LC} entries, asked to remove ${DC}. "
        fi    
    fi
    DC2=`expr ${DC} \* 100` ; CheckExec $?
    PER=`expr ${DC2} / ${LC} + 1` ; CheckExec $?

    if [ "${PER}" -gt "${THRESHOLD}" ] ; then
        Z=`CAT ${3}` ; CheckExec $?
        if [ -z "${Z}" ] ; then
            # First time. 
            ECHO "1" > "${3}"
            LOG "IGNORED: del_check called 1 times, not past ${NUM_TIMES} yet, so ignoring this time."
        else
            TIMES=`CAT ${3}` ; CheckExec $?
            if [ "$TIMES" -ge "${NUM_TIMES}" ] ; then
                # NUM_TIMES exceeded, so do it anyways. 
                LOG "DOING_ANYWAY: del_check called `expr ${TIMES} + 1` times, greater then ${NUM_TIMES} so proceeding with the removal."
                RM -f "${3}"
                return 0
            else
                # Not there yet, so ignore this time. 
                R=`expr ${TIMES} + 1` ; CheckExec $?
                ECHO "${R}" > "${3}"
                LOG "IGNORED: del_check called ${R} times, not past ${NUM_TIMES} yet, so ignoring this time."
            fi    
        fi    
        return 1
    else
        RM -f "${3}"
    fi
    LOG "CHECK: testing ${4}, will remove ${DC} from the local cache."

    return 0
}

ClearDuplicateUPMs()
{
    DUPS="`SQLLOOP ${VAS_DB} \"SELECT ${DBATTR} FROM user_posix GROUP BY ${DBATTR} HAVING COUNT(*) > 1;\"`" ; CheckExec $?
    for name in $DUPS ; do
        SQLLOOP ${VAS_DB} "DELETE FROM user_ad WHERE userkey IN ( SELECT userkey FROM user_posix WHERE ${DBATTR}='${name}');"
        LOG "CLEARED: Found duplicate entry for user ${name}, clearing and re-loading."
        ${VAS} list -f user ${name} >/dev/null 2>&1
    done
}

ClearDuplicateUPMGroups()
{
    DUPS="`SQLLOOP ${VAS_DB} \"SELECT cn FROM group_posix GROUP BY cn HAVING COUNT(*) > 1;\"`" ; CheckExec $?
    for name in $DUPS ; do
        SQLLOOP ${VAS_DB} "DELETE FROM group_ad WHERE groupkey IN ( SELECT groupkey FROM group_posix WHERE cn='${name}');"
        LOG "CLEARED: Found duplicate entry for group ${name}, clearing and re-loading."
        ${VAS} list -f group ${name} >/dev/null 2>&1
    done
}

ClearDuplicateUPNs()
{
    DUPS="`SQLLOOP ${VAS_DB} \"SELECT userPrincipalName FROM user_ad GROUP BY userPrincipalName HAVING COUNT(*) > 1;\"`" ; CheckExec $?
    for name in $DUPS ; do
        SQLLOOP ${VAS_DB} "DELETE FROM user_ad WHERE userPrincipalName='${name}';"
        LOG "CLEARED: Found duplicate entry for user ${name}, clearing and re-loading."
        ${VAS} list -f user ${name} >/dev/null 2>&1
    done
}

ClearDuplicateGroups()
{
    if [ $GROUPCNMODE -eq 1 ] ; then
        GANAME=cn
    else
        GANAME=samAccountName
    fi        
    DUPS="`SQLLOOP ${VAS_DB} \"SELECT ${GANAME} FROM group_ad GROUP BY ${GANAME} HAVING COUNT(*) > 1;\"`" ; CheckExec $?
    for name in $DUPS ; do
        SQLLOOP ${VAS_DB} "DELETE FROM group_ad WHERE ${GANAME}='${name}';"
        LOG "CLEARED: Found duplicate entry for group ${name}, clearing and re-loading."
        ${VAS} list -f group ${name} >/dev/null 2>&1
    done
}

SQLITE ()
{
    $SQL3 "$@"
    RVAL=$?
    if [ $RVAL -ge 1 ] ; then
        LOGE "sql call <$@> failed with $RVAL"
        CleanUp
        exit 1
    fi
}

ProcessUsers()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
        
    if [ ${UPMMODE} -eq 1 ] ; then
        if [ $UIDMODE -eq 1 ] ; then
            DBATTR="uidName"
            ADATTR="uid"
        else
            DBATTR="CN"
            ADATTR="CN"
            # Don't ask, no idea why I wrote one upper cased :)
        fi

        # A pre-emptive clear of duplicate entries. 
        ClearDuplicateUPMs

        if [ $ALLOWUUPP -eq 1 ] ; then
            UPSEARCH="(&(objectCategory=Vintela-unixUserPersonality)(uidnumber=*)(gidnumber=*)(loginshell=*)(|(unixHomeDirectory=*)(homeDirectory=*)))"
        else
            UPSEARCH="(&(objectCategory=Vintela-unixUserPersonality)(vintela-sidBL=*))"
        fi

        # Grab User Personalities from local.
        SQLLOOP ${VAS_DB} "SELECT $DBATTR FROM user_posix where $DBATTR is not null and length(${DBATTR}) > 0" | SORT > ${UPL}
    
        if [ $UPM_DO_REGULAR_USERS -eq 1 ] ; then
            # Grab Users without a personality.
            SQLLOOP ${VAS_DB}  "SELECT user_ad.userPrincipalName FROM user_ad join user_posix on user_ad.userkey = user_posix.userkey where user_posix.cn is null or length(user_posix.cn) = 0;" | SORT > ${UL}
        fi

        # Grab User Personalities from the UPM base.
        VASSEARCH -s one -q -b "${UPMPATH}" "${UPSEARCH}" $ADATTR 2>/dev/null > $UPA
        if [ $UPM_DO_REGULAR_USERS -eq 1 ] ; then
            # Grab users from AD.
            VASSEARCH -s one -q -b "${UPMPATH}" -s "one" "(&(uidnumber=*)(objectCategory=Person)(gidNumber=*)(loginshell=*)(unixHomeDirectory=*))" userPrincipalName 2>/dev/null > ${UA}
        fi

        # Grab Users and User Personalities from seeAlso containers.
        RESORT=0    
        SEEALSO_LOC="${SEEALSO}"
        while [ ! -z "${SEEALSO_LOC}" ] ; do
            SAPATH="`ECHO \"${SEEALSO_LOC}\" | SED 's/\([^:]*\):.*/\1/'`" ; CheckExec $?
            SEEALSO_LOC="`ECHO \"${SEEALSO_LOC}\" | SED 's/[^:]*:\(.*\)/\1/'`" ; CheckExec $?
            if [ -z "${SAPATH}" ] ; then
                SEEALSO_LOC=''
            else
                RESORT=1
                VASSEARCH -s one -q -b "${SAPATH}" "${UPSEARCH}" $ADATTR 2>/dev/null >> ${UPA}
                if [ $UPM_DO_REGULAR_USERS -eq 1 ] ; then
                    VASSEARCH -s one -q -b "${SAPATH}" -s "one" "(&(uidnumber=*)(objectCategory=Person)(gidNumber=*)(loginshell=*)(unixHomeDirectory=*))" userPrincipalName 2>/dev/null >> ${UA}
                fi
            fi
        done 
        if [ "${RESORT}" -eq 1 ] ; then
            SORT ${UPA} > ${VASTMP}
            UNIQ ${VASTMP} > ${UPA}
            if [ $UPM_DO_REGULAR_USERS -eq 1 ] ; then
                SORT ${UA} > ${VASTMP}
                UNIQ ${VASTMP} > ${UA}
            fi
        fi

        COUNT="`WC -l ${UPA} | AWK '{print $1}'`" ; CheckExec $?
#        if [ "$COUNT" = "0" ] ; then 
#            if [  $UPM_DO_REGULAR_USERS -eq 1 ] ; then
#                COUNT="`WC -l ${UA} | AWK '{print $1}'`" ; CheckExec $?
#                if [ "$COUNT" = "0" ] ; then 
#                    ECHO "Not processing users, there appear to be no AD accounts, which shouldn't be true. If true, please manually flush."
#                    return
#                fi
#            else
#                ECHO "Not processing users, there appear to be no AD accounts, which shouldn't be true. If true, please manually flush."
#                return
#            fi
#        fi
        
        # Compare and pick those UPs that are deleted. 
        COMM -23 ${UPL} ${UPA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${UPD}
#CheckThreshold ${UPL} ${UPD} ${UPCOUNT_FILE} "unix personalities"
#        if [ $? -ne 1 ] ; then
            while read i ; do
                # Since in 2K in-site replication can be slow, and we aren't guarenteed to have
                # the same server for search and vasd, use list -f so vasd removes it if its
                # really gone on the server vasd is talking to.
                $VAS list -f user $i >/dev/null 2>&1
                if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
                    LOG "DELETED: called list -f user on name: ${i}."
                fi
            done < $UPD
#        fi
        
        if [ $UPM_DO_REGULAR_USERS -eq 1 ] ; then
            LCOUNT=`WC -l ${UL} 2>/dev/null  | AWK '{print $1}'` ; CheckExec $?
            if [ -z "$LCOUNT" ] || [ "$LCOUNT" -eq 0 ] ; then
                #ECHO "No local Users, so nothing more to delete."
                LOG "CHECK: no locally cached non-upm users so skipping."
            else
                # Compare and remove the deleted Users.
                COMM -23 ${UL} ${UA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${UD}
#                CheckThreshold ${UL} ${UD} ${UCOUNT_FILE} "unix users"
#                if [ $? -ne 1 ] ; then
                    while read i ; do
                        # Since in 2K in-site replication can be slow, and we aren't guarenteed to have
                        # the same server for search and vasd, use list -f so vasd removes it if its
                        # really gone on the server vasd is talking to.
                        # And of course, since the name is a UPN format, it doesn't matter what naming scheme
                        # is in use, this should trigger the update ( but might not display it )
                        $VAS list -f user $i >/dev/null 2>&1
                        if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
                            LOG "DELETED: called list -f user with userPrincipalName: ${i}."
                        fi
                    done < $UD
#                fi
            fi
        fi    

        if [ "$DO_CONSISTENCY_CHECK" -eq 1 ] ; then
            # Now check for new/missed UP's. No threshold needed, as we are adding. 
            COMM -13 ${UPL} ${UPA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${UPD}
            while read i ; do
                LOG "ADDED: forcing update on un-cached user personality with ${ADATTR}: ${i}."
                ${VAS} list -f user $i
            done < $UPD
        fi    
        
    else
        ClearDuplicateUPNs

        # Grab Users from the local DB.
        SQLLOOP ${VAS_DB} "SELECT userPrincipalName FROM user_ad" | SED 's/^CN=//' | SORT > ${UL}    
        if [ "$USERGCMODE" = "1" ] ; then
            # Grab users from AD, GC. 
            VASSEARCH -b "" -U "GC://" -q "(&(uidnumber=*)(objectCategory=Person)(gidNumber=*)(loginshell=*)(unixHomeDirectory=*))" userPrincipalName > ${VASTMP2}
            SORT ${VASTMP2} > ${UA}    
        else
            # Grab users from AD. 
            VASSEARCH -q "(&(uidnumber=*)(objectCategory=Person)(gidNumber=*)(loginshell=*)(unixHomeDirectory=*))" userPrincipalName > ${VASTMP2}
            SORT ${VASTMP2} > ${UA}    
        fi        
        
        # Compair the two lists, anything unique to the local cache will be removed. 
        COMM -23 ${UL} ${UA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${UD}
#        CheckThreshold ${UL} ${UD} ${UCOUNT_FILE} "unix users"
#        if [ $? -ne 1 ] ; then
            while read i ; do
                $VAS list -f user $i >/dev/null 2>&1
                if [ "${DELETE_DEBUG}" -eq 1 ] ; then
                    LOG "DELETED: called list -f user with userPrincipalName: ${i}."
                fi
            done < $UD
#        fi

    fi
    
    if [ "$DO_CONSISTENCY_CHECK" -eq 1 ] ; then
        # Now check for new/missed Users. No threshold needed, as we are adding. 
        COMM -13 ${UL} ${UA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${UD}
        while read i ; do
            LOG "ADDED: forcing update on un-cached user with UPN: ${i}."
            ${VAS} list -f user $i 2>/dev/null
        done < $UD
    fi

    # Test for WS_REMOVE_LOGGED_OUT_USERS
    if [ "$WSMODE" = "1" -a "$WS_REMOVE_LOGGED_OUT_USERS" = "1" ] ; then
        # Test for the right version of ps to use 
        ps -eo user >/dev/null 2>&1
        if [ $? -eq 0 ] ; then
            ps -eo user | SORT | UNIQ | GREP -v USER | GREP -v UID > $VASTMP2
        else
            ps -ef | AWK '{print $1}' | SORT | UNIQ | GREP -v USER | GREP -v UID > $VASTMP2
        fi

        # Trim off the @domain to do simple name matching. 
        MV ${UL} ${VASTMP}
        AWK -F@ '{print $1}' < $VASTMP > ${UL}
        
        # Now check for users that are in the cache, but are not running anything. 
        COMM -13 ${VASTMP2} ${UL} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${UD}
        while read i ; do
            LOG "REMOVED: removing user from cache with with name: ${i}."
            SQLLOOP ${VAS_DB} "DELETE from user_ad WHERE userPrincipalName like '${i}@%'"
        done < $UD
        
        if [ ${UPMMODE} -eq 1 ] ; then
            # Remove UPM's as well.
            COMM -23 ${UPL} ${VASTMP2} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${UPD}
            CheckThreshold ${UPL} ${UPD} ${UPCOUNT_FILE} "unix personalities"
            if [ $? -ne 1 ] ; then
                while read i ; do
                    # Get the userkey. 
                    USERKEY=`SQLLOOP ${VAS_DB} "SELECT userkey FROM user_posix WHERE $DBATTR = '${i}'"` ; CheckExec $?
                    # Delete on userkey    
                    SQLLOOP ${VAS_DB} "DELETE FROM user_ad WHERE userkey = '${USERKEY}'"
                    if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
                        LOG "DELETED: deleted user personality from the cache with ${ADATTR}: ${i}."
                    fi
                done < $UPD
            fi
        fi
    fi
}

ProcessGroups()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    if [ $GROUPCNMODE -eq 1 ] ; then
        GANAME=cn
    else
        GANAME=samAccountName
    fi        
    # Check groups.
    if [ ${UPMMODE} -eq 1 ]
    then
        
        ClearDuplicateUPMGroups

        if [ $ALLOWUGPP -eq 1 ] ; then
            UPSEARCH="(&(objectCategory=Vintela-unixGroupPersonality)(gidnumber=*))"
        else
            UPSEARCH="(&(objectCategory=Vintela-unixGroupPersonality)(vintela-sidBL=*))"
        fi

        
        # Grab Group Personalities from local.
        SQLLOOP ${VAS_DB} "SELECT cn FROM group_posix where cn is not null and length(cn) > 0" | SORT > ${GPL}
       
        if [ $UPM_DO_REGULAR_GROUPS -eq 1 ] ; then
            # Grab Groups without a personality. (Unlinked groups are handled above.)
            # The inforecType != 6 is to skip groups we already know are outside the search base, like from PAC processing.
            SQLLOOP ${VAS_DB}  "SELECT ga.$GANAME FROM group_ad as ga join group_posix as gp on ga.groupkey = gp.groupkey where ga.inforecType != 6 AND ( gp.cn is null or length(gp.cn) = 0 ) AND gp.gidNumber is not NULL;" | AWK -F@ '{print $1}' | SORT > ${GL}
        fi
        
        # Grab Group Personalities from the UPM base. 
        VASSEARCH -s one -q -b "${UPMPATH}" "${UPSEARCH}" CN 2>/dev/null > ${GPA}
        
        if [ $UPM_DO_REGULAR_GROUPS -eq 1 ] ; then
            # Grab Groups from AD.
            VASSEARCH -s one -q -b "${UPMPATH}" "(&(objectCategory=Group)(gidNumber=*))" $GANAME 2>/dev/null > ${GA}
        fi

        # Grab Groups and Group Personalities from seeAlso containers.
        RESORT=0    
        SEEALSO_LOC="${SEEALSO}"
        while [ ! -z "${SEEALSO_LOC}" ] ; do
            SAPATH="`ECHO \"${SEEALSO_LOC}\" | SED 's/\([^:]*\):.*/\1/'`" ; CheckExec $?
            SEEALSO_LOC="`ECHO \"${SEEALSO_LOC}\" | SED 's/[^:]*:\(.*\)/\1/'`" ; CheckExec $?
            if [ -z "${SAPATH}" ] ; then
                SEEALSO_LOC=''
            else    
                RESORT=1
                VASSEARCH -s one -q -b "${SAPATH}" "${UPSEARCH}" CN 2>/dev/null >> ${GPA}
                if [ $UPM_DO_REGULAR_GROUPS -eq 1 ] ; then
                    VASSEARCH -s one -q -b "${SAPATH}" "(&(objectCategory=Group)(gidNumber=*))" $GANAME 2>/dev/null >> ${GA}
                fi
            fi
        done 
        if [ "${RESORT}" -eq 1 ] ; then
            SORT ${GPA} > ${VASTMP}
            UNIQ ${VASTMP} > ${GPA}
            if [ $UPM_DO_REGULAR_GROUPS -eq 1 ] ; then
                SORT ${GA} > ${VASTMP}
                UNIQ ${VASTMP} > ${GA}
            fi
        fi
        
        COUNT="`WC -l ${GPA} | AWK '{print $1}'`" ; CheckExec $?
#        if [ "$COUNT" = "0" ] ; then 
#            if [ $UPM_DO_REGULAR_GROUPS -eq 1 ] ; then
#                COUNT="`WC -l ${GA} | AWK '{print $1}'`" ; CheckExec $?
#                if [ "$COUNT" = "0" ] ; then 
#                    ECHO "Not processing groups, there appear to be no AD groups, which shouldn't be true. If true, please manually flush."
#                    return
#                fi
#            else
#                ECHO "Not processing groups, there appear to be no AD groups, which shouldn't be true. If true, please manually flush."
#                return
#            fi
#        fi
        
        # Compare and pick those GPs that are deleted. 
        COMM -23 ${GPL} ${GPA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${GPD}
#        CheckThreshold ${GPL} ${GPD} ${GPCOUNT_FILE} "unix group personalities"
#        if [ $? -ne 1 ] ; then
            while read i ; do
                $VAS list -f group $i >/dev/null 2>&1
                if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
                    LOG "DELETED: called list -f group with : ${i}."
                fi
            done < $GPD
#        fi
        
        if [ $UPM_DO_REGULAR_GROUPS -eq 1 ] ; then
            LCOUNT=`WC -l ${GL} 2>/dev/null  | AWK '{print $1}'` ; CheckExec $?
            if [ -z "$LCOUNT" ] || [ "$LCOUNT" -eq 0 ]
            then
                LOG "CHECK: no locally cached non-upm groups so skipping."
            else
                # Compare and remove the deleted Groups.
                COMM -23 ${GL} ${GA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${GD}
#                CheckThreshold ${GL} ${GD} ${GCOUNT_FILE} "unix groups"
#                if [ $? -ne 1 ] ; then
                    while read i ; do
                        $VAS list -f group $i >/dev/null 2>&1
                        if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
                            LOG "DELETED: called list -f group on: ${i}."
                        fi
                    done < $GD
#                fi
            fi    
        fi    

        if [ "$DO_CONSISTENCY_CHECK" -eq 1 -o "$WSMODE" -eq 1 ] ; then
            # Now check for new/missed UP's. No threshold needed, as we are adding. 
            COMM -13 ${GPL} ${GPA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${GPD}
            while read i ; do
                LOG "ADDED: forcing update on un-cached group personality with CN: ${i}."
                ${VAS} list -f group $i
            done < $GPD
        fi    
        
    else
        ClearDuplicateGroups

        # Grab groups from the local DB.
        SQLLOOP ${VAS_DB} "select ga.$GANAME from group_ad as ga join group_posix as gp on ga.groupkey=gp.groupkey where gp.gidNumber is not NULL"| AWK -F@ '{print $1}' | SORT > ${GL}    
                
        if [ "$GROUPGCMODE" = "1" ] ; then
            # Grab users from AD, GC. 
            VASSEARCH -b "" -U "GC://" -q "(&(objectCategory=Group)(gidnumber=*))" $GANAME > ${VASTMP2}
            SORT "${VASTMP2}" > ${GA}    
        else
            # Grab users from AD. 
            VASSEARCH -q "(&(objectCategory=Group)(gidnumber=*))" $GANAME > ${VASTMP2}
            SORT "${VASTMP2}" > ${GA}
        fi

        # Compair the two lists, anything unique to the local cache will be removed. 
        COMM -23 ${GL} ${GA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${GD}
#        CheckThreshold ${GL} ${GD} ${GCOUNT_FILE} "unix groups"
#        if [ $? -ne 1 ] ; then
            # Look out for groups like 'Domain Users', as the for loop like to split those into 'Domain' and 'Users'.
            MV ${GD} ${VASTMP}
            SED 's/\([^ ]\) \([^ ]\)/\1~\2/' < ${VASTMP} > ${GD} 

            while read name ; do
                groupname="`ECHO \"$name\" | TR '~' ' '`" ; CheckExec $?
                $VAS list -f group $groupname >/dev/null 2>&1
#                SQLLOOP ${VAS_DB} "DELETE from group_ad WHERE samAccountName like '${groupname}@%'"
                if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
#                    LOG "DELETED: deleted group from the cache with samAccountName: ${groupname}."
                     LOG "DELETED: called list -f group on group with $GANAME: ${groupname}."
                fi
            done < $GD
#        fi
    fi
}

ProcessNetGroups()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    if [ "${DO_NETGROUPS}" -ne 1 ] ; then return 0; fi
    if [ ! -f "${VAS_NS_DB}" -o ! -f "${VAS_YP_DB}" ] ; then  return 0; fi

    if ${RFC2307} ; then
        # Text for local netgroups.
        if [ "$VAS_NS_DB" = "$VAS_YP_DB" ] ; then
            NGNUM=`SQLLOOP ${VAS_NS_DB} "SELECT COUNT(*) from vas2307_xlate where sqlmap='ngentity';"` ; CheckExec $?
        else
            TCOUNT=`SQLLOOP ${VAS_NS_DB} ".tables" | WC -l` ; CheckExec $?
            if [ "$TCOUNT" = "0" ] ; then    
                LOG "CHECK: no local nisNetGroup tables, so skipping."
                return 0
            else    
                NGNUM=`SQLLOOP ${VAS_NS_DB} "SELECT COUNT(*) from ngentity;"` ; CheckExec $?
            fi        
        fi

        if [ "${NGNUM}" -le 0 ] ; then
            LOG "CHECK: no local nisNetGroup tables, so skipping."
            return 0
        fi
            
        # Grab local netgroups.
        SQLLOOP ${VAS_NS_DB} "SELECT distinct netgroup_id from ngentity" 2>/dev/null | SORT > ${NL}
        # Don't need to check return code here. If it fails, then the ACOUNT will be 0, and we dont' delete the 
        # cache in that case.
        # But what if it failes because the disk is nearly full, so we get partial info? We do need to check this.
        VASSEARCH -U DC://$NISSERVER -s sub -q -b "${NISLOC}" "(objectCategory=nisNetGroup)" CN 2>/dev/null >$VASTMP2
        SORT $VASTMP2 > ${NA}
        LCOUNT=`WC -l ${NL} 2>/dev/null  | AWK '{print $1}'` ; CheckExec $?
        if [ -z "$LCOUNT" ] || [ "$LCOUNT" -eq 0 ]
        then
#            ECHO "No local nisNetGroups so nothing to delete."
            LOG "CHECK: no locally cached nisNetGroups so skipping."
            return 0
        fi
        ACOUNT=`WC -l ${NA} | AWK '{print $1}'` ; CheckExec $?
        if [ -z "$ACOUNT" ] || [ "$ACOUNT" -eq 0 ]
        then
            ECHO "No AD nisNetGroups, clearing the netgroup local cache."
            LOG "DELETED: For some reason the whole nisNetGroups local cache should be deleted. Not implimented."
            # Since I'm not sure yet this is perfect, don't clear when it is possible we jsut didn't see them.
            #SQLLOOP ${VAS_NS_DB} "DELETE FROM ng_entity"
            #SQLLOOP ${VAS_NS_DB} "DELETE FROM ng_ctrl"
        else
            COMM -23 ${NL} ${NA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${ND}
#            CheckThreshold ${NL} ${ND} ${NCOUNT_FILE} "unix nisNetGroups"
#            if [ $? -ne 1 ] ; then
                while read i ; do
                    SQLLOOP ${VAS_NS_DB} "DELETE from ngentity WHERE netgroup_id = '${i}'"
                    SQLLOOP ${VAS_NS_DB} "DELETE from netgroup WHERE netgroup_id = '${i}'"
                    if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
                        LOG "DELETED: deleted nisNetGroup from the cache with id: ${i}."
                    fi
                done < $ND
#            fi
            
            if [ "$DO_CONSISTENCY_CHECK" -eq 1 ] ; then
                # Now check for new/missed UP's. No threshold needed, as we are adding. 
                COMM -13 ${NL} ${NA} | SED 's/^[^A-Za-z0-9]*//' | GREP -v "^[ \t]*$" > ${ND}
                LCOUNT=`WC -l ${ND} | AWK '{print $1}'` ; CheckExec $?
                if [ ! -z "$LCOUNT" ] && [ "$LCOUNT" -ne 0 ] ; then
                    LOG "found extra nisNetGroups, ignoring."
# This needs to be re-addressed. In fact, what if its jsut because vasypd hasn't updated yet? 
# logger -t del_check "${LOGGER_ADD}: ADDED: forcing full NIS update, found extra nisNetGroups."
#SQLLOOP ${VAS_NX_DB} "INSERT OR REPLACE INTO vasypinfo VALUES( 'badLoad', 'true' );"
                fi
            fi  
        fi     
    fi
}

ProcessNetMaps()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi

    if [ "${DO_NISMAPS}" -eq 0 ] ; then return 0; fi
    if [ ! -f "${VAS_YP_DB}" ] ; then return 0; fi
        
    if ${RFC2307} ; then
        LOCAL1=/tmp/_local1_$$
        LOCAL2=/tmp/_local2_$$
        MAPSAD=/tmp/_maps_ad_$$
        MAPSLOCAL=/tmp/_maps_local_$$
        DBCMDS=/tmp/_DBCMDS_$$
        AD=/tmp/_ad_$$
        for f in $LOCAL2 $AD $LOCAL1 $DBCMDS $MAPSAD $MAPSLOCAL; do TEST_FILE $f; done

        SQLTIME ${VAS_YP_DB} "select nismap from vas2307_xlate where sqlmap not in ( 'ngentity', 'hosts', 'rpc', 'services', 'networks', 'protocols' );" | TR $UPPER $LOWER | SORT > $MAPSLOCAL

        VASSEARCH -U DC://$NISSERVER -b "$NISLOC" -s one -q "(objectCategory=NisMap)" cn 2>/dev/null > $LOCAL1
        TR $UPPER $LOWER < $LOCAL1 | SORT > $MAPSAD    

        COUNT1=`WC -l $MAPSAD | AWK '{print $1}'` ; CheckExec $?
        COUNT2=`WC -l $MAPSLOCAL | AWK '{print $1}'` ; CheckExec $?
        if [ $COUNT1 -le 0 -o $COUNT2 -le 0 ] ; then
            ECHO "No generic NIS maps found in AD, skipping."
            return 1
        fi

# First check for removed maps.
        DELMAPS=`COMM -23 $MAPSLOCAL $MAPSAD | TR '\n' ' '` ; CheckExec $?
        if [ ! -z "$DELMAPS" ] ; then
            LOG "removing AD missing maps <$DELMAPS>"
            ECHO ".timeout 10000" > $DBCMDS
            ECHO "BEGIN TRANSACTION;" >> $DBCMDS
            for map in $DELMAPS ; do
                MAPNAME=`SQLTIME ${VAS_YP_DB} "select sqlmap from vas2307_xlate where nisMap='$map';"` ; CheckExec $?
                if [ ! -z "$MAPNAME" ] ; then
                    ECHO "drop table $MAPNAME;" >> $DBCMDS
                    ECHO "delete from vas2307_xlate where sqlmap='$MAPNAME';" >> $DBCMDS
                    ECHO "delete from nis2map where tblname='$MAPNAME';" >> $DBCMDS
                fi
            done
            ECHO "COMMIT TRANSACTION;" >> $DBCMDS
            if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
                LOG "DELETED: deleted $MAPNAME (not in AD)"
            fi
            ECHO | SQLITE -init $DBCMDS ${VAS_YP_DB} | SORT > $LOCAL2
        fi
        RM -f $LOCAL1 $LOCAL2

# Gather local maps and entries ( no need for transaction, just timeout )
        MAPS="`cat $MAPSAD | TR '\n' ' '`" ; CheckExec $?
        ECHO ".timeout 10000" > $DBCMDS
        SQLTIME ${VAS_YP_DB} "select 'select ''' || nismap || ' '' || key from ' || sqlmap || ';' from vas2307_xlate where sqlmap not in ( 'ngentity', 'hosts', 'rpc', 'services', 'networks', 'protocols' );" >> $DBCMDS
        ECHO | SQLITE -init $DBCMDS $DB | SORT > $LOCAL2
          
# Gather AD maps.
        VASSEARCH -U DC://$NISSERVER -b "$NISLOC" -q "(objectCategory=NisObject)" nismapname cn 2>/dev/null > $LOCAL1 
        AWK '{last=line; line=$0}; NR%2 == 0 { print last, line}' < $LOCAL1 | SORT > $AD
        COUNT=`WC -l $AD | AWK '{print $1}'` ; CheckExec $?
        if [ $COUNT -le 0 ] ; then
            ECHO "No generic NIS maps found in AD, skipping."
            return 1
        fi

# Compare local to AD, and delete from local if its missing from AD
        COMM -23 $LOCAL2 $AD | SORT > $LOCAL1
        COUNT=`WC -l $LOCAL1  | AWK '{print $1}'` ; CheckExec $?
        if [ $COUNT -gt 0 ] ; then
            ECHO ".timeout 10000" > $DBCMDS
            ECHO "BEGIN TRANSACTION;" >> $DBCMDS
            while read map key ; do
                MAPNAME="`SQLLOOP ${VAS_YP_DB} \"select sqlmap from vas2307_xlate where nisMap='$map';\"`" ; CheckExec $?
                # If the query failes then this is blank anyway ( failure message goes to stderr )
                if [ ! -z "$MAPNAME" ] ; then
                    ECHO "delete from $MAPNAME where key='$key';" >> $DBCMDS
                fi
            done < $LOCAL1
            ECHO "COMMIT TRANSACTION;" >> $DBCMDS
            if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
                LOG "DELETED: deleted entry <$key> from $MAPNAME"
            fi
#           cat $DBCMDS
            ECHO | SQLITE -init $DBCMDS ${VAS_YP_DB} | SORT > $LOCAL2
        fi

        RM -f $LOCAL2 $AD $LOCAL1 $DBCMDS $MAPSAD $MAPSLOCAL
    fi
}

# This has to be seperate, as hosts are different then the rest. 
ProcessHostMaps()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi

    if [ "${DO_NISMAPS}" -eq 0 ] ; then return 0; fi
    if [ ! -f "${VAS_YP_DB}" ] ; then return 0; fi
        
    if ${RFC2307} ; then
        LOCAL1=/tmp/_local1_$$
        LOCAL2=/tmp/_local2_$$
        MAPSAD=/tmp/_maps_ad_$$
        MAPSLOCAL=/tmp/_maps_local_$$
        DBCMDS=/tmp/_DBCMDS_$$
        AD=/tmp/_ad_$$
        for f in $LOCAL2 $AD $LOCAL1 $DBCMDS $MAPSAD $MAPSLOCAL; do TEST_FILE $f; done

# Gather local entries 
        SQLTIME ${VAS_YP_DB} "select distinct alias from host_alias" | SORT >> $LOCAL2

# Gather AD entries for hosts.
# I wish I knew what Luke was thingking with RFC 2307. The key 
# is always IP, so why are the AD objects named by both alias
# and canon name? Makes figuring this all out more complex.
# Its MS, not Luke, since MS's LDAP doesn't support multi-valued CN/RDN.
        VASSEARCH -U DC://$NISSERVER -b "$NISLOC" -q "(&(objectCategory=device)(ipHostNumber=*))" cn 2>/dev/null > $LOCAL1
        SORT < $LOCAL1 > $AD    
        COUNT=`WC -l $AD | AWK '{print $1}'` ; CheckExec $?
        if [ $COUNT -le 0 ] ; then
            ECHO "No host map entries found in AD, skipping."
            return 1
        fi
# Don't have to check here, its only missing AD entries that we care about. 
        
# Thanks to the host speed increases, its now too complex to remove entries from here. The hosts table needs to be
# completly re-computed, since if an alias is removed, multiple hosts table entries might need to be modified,
# thanks to the wonderful RFC2307 format. So next best thing, next time vasypd does an incremental update, 
# just this one table will be re-loaded from AD. 
        COUNT="`COMM -23 $LOCAL2 $AD | WC -l | AWK '{print $1}'`" ; CheckExec $?
        if [ "$COUNT" -gt 0 ] ; then
            SQLTIME ${VAS_YP_DB} "insert or replace into vasypinfo ( 'key', 'value' ) values( 'hosts', 'true' );" >/dev/null 2>&1
            if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
                LOG "DELETED: hosts map different, setting 'bad' so next vasypd update re-processes it"
            fi
        fi
        
        RM -f $LOCAL2 $AD $LOCAL1 $DBCMDS $MAPSAD $MAPSLOCAL
    fi
}

FILTER1=""
FILTER2=""
MAP1=""
MAP2=""
KEY1=""
KEY2=""
GetMapVariables()
{
    old_ifs="${IFS}"
    IFS="~"
    set -- $1
    IFS="${old_ifs}"
    unset old_ifs

    FILTER1="${1:-}"
    FILTER2="${2:-}"
    MAP1="${3:-}"
    MAP2="${4:-}"
    KEY1="${5:-}"
    KEY2="${6:-}"
}


# Handle networks, protocols, rpc, and services
# Since they are all the same format in AD, loop for each one. 
# Store in the form <filter1>~<filter2>~<table1>~<table2>
ProcessSpecificMaps()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi

    if [ "${DO_NISMAPS}" -eq 0 ] ; then return 0; fi
    if [ ! -f "${VAS_YP_DB}" ] ; then return 0; fi
        
    if ${RFC2307} ; then
        LOCAL1=/tmp/_local1_$$
        LOCAL2=/tmp/_local2_$$
        MAPSAD=/tmp/_maps_ad_$$
        MAPSLOCAL=/tmp/_maps_local_$$
        DBCMDS=/tmp/_DBCMDS_$$
        AD=/tmp/_ad_$$
        for f in $LOCAL2 $AD $LOCAL1 $DBCMDS $MAPSAD $MAPSLOCAL; do TEST_FILE $f; done

        for maplist in "ipNetwork~ipNetworkNumber~networks~network_alias~canonical~alias" \
                       "ipProtocol~ipProtocolNumber~protocols~protocol_alias~canonical~alias" \
                       "oncRpc~oncRpcNumber~rpc~rpc_alias~canonical~alias" \
                       "ipService~ipServicePort~services~service_alias~canonical~alias" ; do
            GetMapVariables "${maplist}"
            if [ -z "$FILTER1" -o -z "$FILTER2" -o -z "$MAP1" -o -z "$MAP2" -o -z "$KEY1" -o -z "$KEY2" ] ; then
                ECHO "Unable to parse $maplist, skipping"
                continue
            fi

# Gather local entries 
            SQLTIME ${VAS_YP_DB} "select distinct alias from $MAP2;" | SORT > $LOCAL2

# Gather AD maps.
            VASSEARCH -U DC://$NISSERVER -b "$NISLOC" -q "(&(objectCategory=$FILTER1)($FILTER2=*))" cn 2>/dev/null > $LOCAL1
            SORT < $LOCAL1 > $AD    
            COUNT=`WC -l $AD | AWK '{print $1}'` ; CheckExec $?
            if [ $COUNT -le 0 ] ; then
                ECHO "No $MAP1 entries found in AD, skipping."
                continue
            fi

# Compare local to AD, and delete from local if its missing from AD
            COMM -23 $LOCAL2 $AD > $LOCAL1
            COUNT=`WC -l $LOCAL1  | AWK '{print $1}'` ; CheckExec $?
            if [ $COUNT -gt 0 ] ; then
                ECHO ".timeout 10000" > $DBCMDS
                ECHO "BEGIN TRANSACTION;" >> $DBCMDS
                while read key ; do
# We can delete from both, for aliases it does nothing against the MAP1 entry, 
# But it would be slower to check and have a switch deciding to add the string
                    ECHO "delete from $MAP1 where $KEY1='$key';" >> $DBCMDS
                    ECHO "delete from $MAP2 where $KEY2='$key';" >> $DBCMDS
                done < $LOCAL1
                ECHO "COMMIT TRANSACTION;" >> $DBCMDS
                if [ "${DELETE_DEBUG}" -eq 1 ] ; then 
                    LOG "DELETED: deleted entry <$key> from ${MAP1}/${MAP2}"
                fi
                ECHO | SQLITE -init $DBCMDS ${VAS_YP_DB} | SORT > $LOCAL2
            fi
        RM -f $LOCAL2 $AD $LOCAL1 $DBCMDS $MAPSAD $MAPSLOCAL
        done
    fi
}

CleanUp()
{
    if [ "${DEBUG}" -eq 1 ] ; then set -x; fi
    RM -f $UL $UA $UD $UPL $UPA $UPD $GL $GA $GD $GPL $GPA $GPD $NL $NA $ND $VASTMP $VASTMP2 $VASTMP3 $CLEANUP_FILES
    # Use a unique credential cache
    KRB5CCNAME=/tmp/krb5cc_deleted_check_$$
    export KRB5CCNAME
    ${VAS} kdestroy >/dev/null   
}

Checks()
{
    SanityCheck
    UPMCheck
    WSCheck
    UIDCheck
    UnlinkedUserCheck
    UnlinkedGroupCheck
    UserGCCheck
    GroupGCCheck
    GroupNameCheck
    RFC2307Check
}

################################################################################
#
# Main
#
################################################################################

CleanUp
Checks
ProcessUsers
ProcessGroups
ProcessNetGroups
ProcessNetMaps
ProcessHostMaps
ProcessSpecificMaps
CleanUp
touch $TRACKERFILE
exit 0
