#!/bin/sh
################################################################################
# Copyright (c) 2009 Quest Software, Inc.  All Rights Reserved.
#
# vas_check_host.sh
#
# Purpose:      Test the VAS join against AD and local configuration for various
#               issues. Assuming the machine at one time worked correctly.
#
# Author(s):    Seth Ellsworth (seth.ellsworth@quest.com)
#
# Version:      3.5.1.9
#
################################################################################

################################################################################
#
# The vas_check_host.sh script tests the following situations:
# 
#    001 Nothing serious found ( can still have warnings ).
#
# * vas.conf/vastool/keytab
#    101 51 vastool exists.
#    102 52 vastool runs ( no segfault/library issues )
#    103 54 VAS version ( < 3.0.3.17 or so and fail to run, as we won't have vastool auth )
#    104 vas.conf exists
#    105 3 vas.conf is valid ( ERROR: Could not allocate VAS context, err = 2 )
#    106 4 Check for missing default_realm setting.
#    107 5 Keytab file exists
#    108 6 keytab is valid ( not an empty/corrupted file ( krb5_kt_start_seq_get ) )
#
# * Local DB
#    201 7 misc cache exists.
#    202 8 ident cache exists.
#    203 9 misc cache is not locked.
#    204 10 ident cache is not locked.
#    205 45 Misc cache is a DB file.
#    206 46 Misc cache is not corrupt.
#    207 47 Ident cache is a DB file.
#    208 48 Ident cache is not corrupt.
#    209 11 All needed tables exist. ( No missing tables, careful here, given version differences ).
#    210 12 No tables are corrupt. 
#    211 13 machine has found a site, or has an override site set.
#    212 14 Misc table has defaultRealm 
#    213 15 Misc table has computerfqdn
#    214 16 Misc table has computername
#    215 17 Misc table has forestRoot
#
# * kinit results 
#    301 18 Machine can talk to AD ( KDC_UNREACH )
#    302 19 Object exists in AD. ( client unknown )
#    303 20 Machine timesync. ( TIME_SKEW ) 
#    304 21 The host.keytab knows the right password. ( preauth failed ).
#    304 22 The host/ account is usable ( CREDENTIALS REVOKED )
#    306 23 DES is set, but no entries ( KRB5KDC_ERR_PREAUTH_REQUIRED )
#    307 24 ( Misc kinit error, not one of the above known ones ).
# 
# * Computer account checks.
#    401 53 Computer can read AD object. 
#    401 25 Computer account has valid UPN.
#    403 26 Computer account userAccountControl has VAS_UF_PASSWD_NOTREQD
#    404 27 Computer account userAccountControl has VAS_UF_WORKSTATION_TRUST_ACCOUNT
#    405 28 Computer account userAccountControl doesn't have ADS_UF_USE_DES_KEY_ONLY
#    406 29 Computer account userAccountControl doesn't have ADS_UF_DONT_REQUIRE_PREAUTH
#
# * vastool auth/spn.
#    501 30 Computer account has valid SPNs.
#    502 31 host.keytab containes matching entries for each SPN in AD.
#    503 32 SPNs in AD match each entry in host.keytab.
#    504 33 auth -S host/$FQDN returns KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN, duplicate.
#    505 34 auth -S host/$FQDN fails.
#
# * Final local stuff.
#    601 35 File/directory permissions. Not enough.
#    602 36 File/directory permissions. Too much.
#    603 37 /etc/nsswitch.conf contains vas3. ( check for map entries ). 
#    604 38 Relevent PAM configuraiton file contains at least one pam_vas3 line.
#    605 39 AIX: IRS files have VAS configured.
#    606 44 System file permissions. Not enough.
#    607 53 System file permissions. Too much.
#
# * And whatever else.
#    701 50 Valid license 1 ( expired )
#    702 40 Valid license 2 ( no license )
#    703 41 Valid license 3 ( no directory )
#    704 42 vasd running
#    705 43 vasd responding
# 
################################################################################

if [ -z "$DEBUG" ] ; then
    DEBUG=false
else
    DEBUG=true
fi

$DEBUG && set -x

# Shhh... look away. This is a testing nomenclature, when called
# with the right option in addition to normal printout, print out
# an arbitraty but unique number, so a testing script can make sure
# the right response was given for a particular situation.
if [ "x$1" = "xNUMBER" ] ; then
    NUM=true
else
    NUM=false
fi
                
VAS=/opt/quest/bin/vastool
SQL3=/opt/quest/libexec/vas/sqlite3
IDENTDB=/var/opt/quest/vas/vasd/vas_ident.vdb
MISCDB=/var/opt/quest/vas/vasd/vas_misc.vdb
VASCONF=/etc/opt/quest/vas/vas.conf
KEYTAB=/etc/opt/quest/vas/host.keytab

get_vasconf_setting ()
{
# $1 section
# $2 setting
# Returns: value set, nothing if nothing is set

    $DEBUG && set -x

    SECTION="$1"
    SETTING="$2"

    if [ -z "$1" -o -z "$2" ] ; then
        return
    fi        

    SETTING="`cat $VASCONF | sed -e '/^[ \t]*#/d' -e 's/#.*//' -e 's/^[ \t]*//' < $VASCONF | grep \"^$2 \" | head -1`"
    RESULT="`echo $SETTING | sed 's/.* = \(.*\)[ \t]*/\1/'`"
    if [ ! -z "$RESULT" ] ; then echo "$RESULT" ; fi
}

CRITICAL ()
{
    $DEBUG && set -x
    printf "\nERROR: A critical error was found, the script will now exit.\n"
    printf "If the information and suggestions in the previous output fail to lead to\n"
    printf "resolving the issue, please contact Quest Support. When contacting Support,\n"
    printf "please include a snapshot dump, script output, and steps taken with results.\n\n"
    exit 1
}

# Test for root access.
if [ "`id | sed 's/uid=\([0-9]*\).*/\1/'`" -ne 0 ] ; then
    printf "Must be root to run, exiting.\n"
    exit 1
fi

#    101 vastool exists.
if [ ! -f $VAS ] ; then
    $NUM && printf "NUMBER:101\n"
    printf "\nISSUE: The machine is not fully installed.\n\n"
    printf "REASON: $VAS does not exist.\n\n"
    printf "FIX: Re-install the VAS client package.\n\n"
    CRITICAL
fi

#    102 vastool runs ( no segfault/library issues )
OUT="`$VAS -v 2>&1`"
echo "$OUT" | grep -i copyright >/dev/null
if [ $? -ne 0 ] ; then
    $NUM && printf "NUMBER:102\n"
    printf "\nISSUE: The machine is not fully installed.\n\n"
    printf "REASON: Running vastool to obtain version failed, output: <$OUT>.\n\n"
    printf "FIX: Try re-installing the VAS Client package.\n\n"
    CRITICAL
fi

#    103 VAS version ( < 3.0.3.17 or so and fail to run, as we won't have vastool auth )
VASVERSION="`$VAS -v | grep -i version | head -1 | sed 's/vastool: VAS Version \([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)/\1 \2 \3 \4/'`"
VASVERMAJOR="`echo $VASVERSION | awk '{print $1}'`"
VASVERMINOR="`echo $VASVERSION | awk '{print $2}'`"
VASVERREVISION="`echo $VASVERSION | awk '{print $3}'`"
VASVERBUILD="`echo $VASVERSION | awk '{print $4}'`"
if [ "$VASVERMINOR" = "0" -a -n "$VASVERREVISION" -a $VASVERREVISION -lt 3 ] ; then
    $NUM && printf "NUMBER:103\n"
    printf "\nISSUE: The machine is using an older VAS version.\n\n"
    printf "REASON: VAS 3.0.2 and earlier should not be used.\n\n"
    printf "FIX: Upgrade to VAS 3.0.3 at least, preferred the latest version.\n"
    printf "     Earlier VAS build are missing functionality needed to allow this script\n"
    printf "     to continue.\n\n"
    CRITICAL
fi

#    104 vas.conf exists
if [ ! -f $VASCONF ] ; then
    $NUM && printf "NUMBER:104\n"
    printf "\nISSUE: The machine is not fully joined.\n\n"
    printf "REASON: $VASCONF does not exist, no information available for\n        talking to AD.\n\n"
    printf "FIX: Try running:\n\n$VAS configure realm <domain the machine should be joined to>\n\n"
    printf "     If this machine was previously joined, then any previous\n" 
    printf "     vas.conf settings will need to be restored.\n\n"
    CRITICAL
fi

#    105 vas.conf is valid ( ERROR: Could not allocate VAS context, err = 2 )
$VAS ktutil list | grep krb5_init_context >/dev/null
if [ $? -eq 0 ] ; then
    $NUM && printf "NUMBER:105\n"
    printf "\nISSUE: The machine is not fully joined.\n\n"
    printf "REASON: $VASCONF is an invalid file or misconfigured.\n\n"
    printf "FIX: Try deleting $VASCONF, and running:\n\n$VAS configure realm <domain the machine should be joined to>\n\n"
    printf "     Any previous vas.conf settings will also need to be restored.\n\n"
    printf "     OR\n\n     Manually modify $VASCONF and fix the\n     misconfiguration.\n\n"
    CRITICAL
fi

#    106 Check for missing default_realm setting.
REALM="`get_vasconf_setting libdefaults default_realm`"
if [ -z "$REALM" ] ; then
    $NUM && printf "NUMBER:106\n"
    printf "\nISSUE: The machine is not fully joined.\n\n"
    printf "REASON: Missing required:\n\n[libdefaults]\n default_realm = <DOMAIN>\n\n        setting in $VASCONF\n\n"
    printf "FIX: Try running:\n\n$VAS configure realm <domain the machine should be joined to>\n\n"
    printf "     If this machine was previously joined, then any previous\n" 
    printf "     vas.conf settings will also need to be restored.\n\n"
    CRITICAL
fi

#    107 Keytab file exists
if [ ! -f $KEYTAB ] ; then
    $NUM && printf "NUMBER:107\n"
    printf "\nISSUE: The machine is not fully joined.\n\n"
    printf "REASON: $KEYTAB does not exist.\n        Unable to use the host/ object.\n\n"
    printf "FIX: Join AD. If this machine was previously joined, and everything else is in\n     place, try running:\n\n"
    printf "$VAS -u <admin or joining account> passwd -rk $KEYTAB host/\n\n"
    CRITICAL
fi

#    108 keytab is valid ( not an empty/corrupted file ( krb5_kt_start_seq_get ) )
$VAS ktutil list | grep krb5_kt_start_seq_get >/dev/null
if [ $? -eq 0 ] ; then
    $NUM && printf "NUMBER:108\n"
    printf "\nISSUE: The machine is not fully joined.\n\n"
    printf "REASON: $KEYTAB is invalid\n\n"
    printf "FIX: Delete $KEYTAB, and try running:\n\n$VAS -u <admin or joining account> passwd -rk $KEYTAB host/\n\n"
    CRITICAL
fi

# * Local DB
#    201 misc cache exists.
if [ ! -f $MISCDB ] ; then
    $NUM && printf "NUMBER:201\n"
    printf "\nISSUE: The machine is not fully joined.\n\n"
    printf "REASON: $MISCDB does not exist, without this cache\n        VAS is missing vital information.\n\n"
    printf "FIX: The machine will need to be re-joined to AD. Some VAS commands will\n     re-generate the file, but it will be missing needed information until\n     re-joined.\n\n"
    CRITICAL
fi

#    202 ident cache exists.
if [ ! -f $IDENTDB ] ; then
    $NUM && printf "NUMBER:202\n"
    printf "\nISSUE: The machine is not fully joined.\n\n"
    printf "REASON: $IDENTDB does not exist, without this cache\n        VAS is missing vital information.\n\n"
    printf "FIX: The machine will need to be re-joined to AD. Some VAS commands will\n     re-generate the file, but it will be missing needed information until\n     re-joined.\n\n"
    CRITICAL
fi

#    203 misc cache is not locked.
OUT="`echo | $SQL3 $MISCDB <<_EOF 2>&1
.timeout 10000
.tables
.q
_EOF`"
echo "$OUT" | grep "database is locked" >/dev/null
if [ $? -eq 0 ] ; then
    $NUM && printf "NUMBER:203\n"
    printf "\nISSUE: VAS is unavailable.\n\n"
    printf "REASON: The $MISCDB file is locked, and cannot be\n        accessed by other applications.\n\n"
    printf "FIX: Need to determine the process locking the DB. Usually, some application\n"
    printf "     holds a read lock, then vasd blocks on a write lock. Stopping vasd might\n"
    printf "     let users log in, but as soon as its started again it could lock again.\n"
    printf "     Start with 'fuser /var/opt/quest/vas/vasd/*.vdb', and use\n"
    printf "     'ps -ef | grep \"<pid>\" to track down processes that have file handles.\n"
    printf "     Stop/kill processes one at a time until the lock is released.\n"
    printf "     ( A file handle does not mean a read lock ). It is usually caused by\n"
    printf "     improper getXXent usage. Sunfreeware top and HP newgrp have been known in\n"
    printf "     the past to trigger this. One solution is to set the vas.conf [nss_vas]\n"
    printf "     option getent-use-memory-cache to true, and restart the offending process.\n\n"
    CRITICAL
fi

#    204 ident cache is not locked.
OUT="`echo | $SQL3 $IDENTDB <<_EOF 2>&1
.timeout 10000
.tables
.q
_EOF`"
echo "$OUT" | grep "database is locked" >/dev/null
if [ $? -eq 0 ] ; then
    $NUM && printf "NUMBER:204\n"
    printf "\nISSUE: VAS is unavailable.\n\n"
    printf "REASON: The $IDENTDB file is locked, and cannot be\n        accessed by other applications.\n\n"
    printf "FIX: Need to determine the process locking the DB. Usually, some application\n"
    printf "     holds a read lock, then vasd blocks on a write lock. Stopping vasd might\n"
    printf "     let users log in, but as soon as its started again it could lock again.\n"
    printf "     Start with 'fuser /var/opt/quest/vas/vasd/*.vdb', and use\n"
    printf "     'ps -ef | grep \"<pid>\" to track down processes that have file handles.\n"
    printf "     Stop/kill processes one at a time until the lock is released.\n"
    printf "     ( A file handle does not mean a read lock ). It is usually caused by\n"
    printf "     improper getXXent usage. Sunfreeware top and HP newgrp have been known in\n"
    printf "     the past to trigger this. One solution is to set the vas.conf [nss_vas]\n"
    printf "     option getent-use-memory-cache to true, and restart the offending process.\n\n"
    CRITICAL
fi

#    205 Misc cache is a DB file.
OUT="`echo | $SQL3 $MISCDB <<_EOF 2>&1
.timeout 1000
.tables
.q
_EOF`"
echo "$OUT" | grep "is not a database" >/dev/null
if [ $? -eq 0 ] ; then
    $NUM && printf "NUMBER:205\n"
    printf "\nISSUE: VAS is unavailable.\n\n"
    printf "REASON: The $MISCDB file is not a valid database file.\n\n"
    printf "FIX: Run:\n\n"
    printf "$VAS flush statedir\n\n"
    printf "     And then re-join AD. The affected files need to be removed and the machine\n"
    printf "     re-joined. If the cause is unknown, investigating what is in the\n"
    printf "     $MISCDB file might lead to an answer.\n\n"
    CRITICAL
fi

#    206 Misc cache is not corrupt.
OUT="`echo | $SQL3 $MISCDB <<_EOF 2>&1
.timeout 1000
.tables
.q
_EOF`"
echo "$OUT" | grep "malformed" >/dev/null
if [ $?  -eq 0 ] ; then
    $NUM && printf "NUMBER:206\n"
    printf "\nISSUE: The $MISCDB cache is corrupt.\n"
    printf "\nFIX: Run:\n\n$VAS flush statedir\n\n     And then re-join AD. The affected files need to be removed and the machine\n     re-joined. If the cause needs to be found, please copy off the\n     /var/opt/quest/vas/vasd/vas*.vdb files first and send to Support, in some\n     circumstances the corrupting application can be identified.\n"
    printf "\nINFO: There are three known causes of corrupted DBs:\n\n"
    printf "      1) nss_vas in older versions maintained file handles to the cache files\n         inbetween NSS calls, it was possible for an application to mistakenly\n         write to VASs file handles instead of its own. VMWare ESX 2.5.X is one\n         application known to do this ( fixed in ESX 3.+ ). Later VAS versions\n         ( 3.1.2.60+/3.2.0.125+/3.3+ ) are not supseptable to this.\n\n"
    printf "      2) Backup-software. The /var/opt/quest/vas/vasd/ directory should be\n         excluded from any back-ups. If a machine is restored, it should be\n         re-joined to AD.\n\n"
    printf "      3) Hardware failure leading to incorrect information written to disk.\n"
    CRITICAL
fi

#    207 Ident cache is a DB file.
OUT="`echo | $SQL3 $IDENTDB <<_EOF 2>&1
.timeout 1000
.tables
.q
_EOF`"
echo $OUT | grep "is not a database" >/dev/null
if [ $? -eq 0 ] ; then
    $NUM && printf "NUMBER:207\n"
    printf "\nISSUE: VAS is unavailable.\n\n"
    printf "REASON: The $IDENTDB file is not a valid database file.\n\n"
    printf "FIX: Run:\n\n"
    printf "$VAS flush statedir\n\n"
    printf "     And then re-join AD. The affected files need to be removed and the machine\n"
    printf "     re-joined. If the cause is unknown, investigating what is in the\n"
    printf "     $MISCDB file might lead to an answer.\n\n"
    CRITICAL
fi

#    208 Ident cache is not corrupt.
OUT="`echo | $SQL3 $IDENTDB <<_EOF 2>&1
.timeout 1000
.tables
.q
_EOF`"
echo $OUT | grep "malformed" >/dev/null
if [ $?  -eq 0 ] ; then
    $NUM && printf "NUMBER:208\n"
    printf "\nISSUE: The $IDENTDB cache is corrupt.\n"
    printf "\nFIX: Run:\n\n$VAS flush statedir\n\n     And then re-join AD. The affected files need to be removed and the machine\n     re-joined. If the cause needs to be found, please copy off the\n     /var/opt/quest/vas/vasd/vas*.vdb files first and send to Support, in some\n     circumstances the corrupting application can be identified.\n"
    printf "\nINFO: There are three known causes of corrupted DBs:\n\n"
    printf "      1) nss_vas in older versions maintained file handles to the cache files\n         inbetween NSS calls, it was possible for an application to mistakenly\n         write to VASs file handles instead of its own. VMWare ESX 2.5.X is one\n         application known to do this ( fixed in ESX 3.+ ). Later VAS versions\n         ( 3.1.2.60+/3.2.0.125+/3.3+ ) are not supseptable to this.\n\n"
    printf "      2) Backup-software. The /var/opt/quest/vas/vasd/ directory should be\n         excluded from any back-ups. If a machine is restored, it should be\n         re-joined to AD.\n\n"
    printf "      3) Hardware failure leading to incorrect information written to disk.\n"
    CRITICAL
fi

#    209 All needed tables exist. ( No missing tables, careful here, given version differences ).
MISCTABLES="misc srvinfo usn_cache table_ctrl"
IDENTTABLES="user_ad user_posix user_ovrd user_ovrd_bygroup group_ad group_posix group_ovrd group_member domain_sids nudn table_ctrl"
MISSINGTABLE=0
CURRENTTABLES="`$SQL3 $MISCDB \".tables\"`"
for table in $MISCTABLES ; do
    echo $CURRENTTABLES | grep $table >/dev/null
    if [ $? -ne 0 ] ; then 
        printf "Unable to locate table $table in $MISCDB\n"
        MISSINGTABLE=1
    fi
done    
    
CURRENTTABLES="`$SQL3 $IDENTDB \".tables\"`"
for table in $IDENTTABLES ; do
    echo $CURRENTTABLES | grep $table >/dev/null
    if [ $? -ne 0 ] ; then 
        printf "Unable to locate table $table in $IDENTDB\n"
        MISSINGTABLE=1
    fi
done    
    
if [ "$MISSINGTABLE" = "1" ] ; then
    $NUM && printf "NUMBER:209\n"
    printf "\nISSUE: The machine is not fully joined.\n\n"
    printf "REASON: The local DB cache is missing one or more needed tables.\n\n"
    printf "FIX: Try running a vastool flush. If that does not resolve the issue, run:\n\n"
    printf "$VAS flush statedir\n\n"
    printf "     And then re-join AD.\n\n"
    printf "INFO: vastool flush statedir deletes all local cache files, so the machine needs\n"
    printf "      to be re-joined to re-obtain all the caches.\n\n"
    CRITICAL
fi

#    210 No tables are corrupt. 
CORRUPTTABLE=0
CURRENTTABLES="`$SQL3 $MISCDB \".tables\"`"
for table in $CURRENTTABLES ; do
    $SQL3 $MISCDB "select count(*) from $table;" 2>&1 | grep malformed >/dev/null
    if [ $? -eq 0 ] ; then 
        printf "Table $table in $MISCDB is corrupt.\n"
        CORRUPTTABLE=1
    fi
done    
    
CURRENTTABLES="`$SQL3 $IDENTDB \".tables\"`"
for table in $CURRENTTABLES ; do
    $SQL3 $IDENTDB "select count(*) from $table;" 2>&1 | grep malformed >/dev/null
    if [ $? -eq 0 ] ; then 
        printf "Table $table in $IDENTDB is corrupt.\n"
        CORRUPTTABLE=1
    fi
done    
    
if [ "$CORRUPTTABLE" = "1" ] ; then
    $NUM && printf "NUMBER:210\n"
    printf "\nISSUE: The internal cache is corrupt.\n"
    printf "\nREASON: The one or more local DB cache tables are corrupt.\n"
    printf "\nFIX: Run:\n\n$VAS flush statedir\n\n     And then re-join AD. The affected files need to be removed and the machine\n     re-joined. If the cause needs to be found, please copy off the\n     /var/opt/quest/vas/vasd/vas*.vdb files first and send to Support, in some\n     circumstances the corrupting application can be identified.\n"
    printf "\nINFO: There are three known causes of corrupted DBs:\n\n"
    printf "      1) nss_vas in older versions maintained file handles to the cache files\n         inbetween NSS calls, it was possible for an application to mistakenly\n         write to VASs file handles instead of its own. VMWare ESX 2.5.X is one\n         application known to do this ( fixed in ESX 3.+ ). Later VAS versions\n         ( 3.1.2.60+/3.2.0.125+/3.3+ ) are not supseptable to this.\n\n"
    printf "      2) Backup-software. The /var/opt/quest/vas/vasd/ directory should be\n         excluded from any back-ups. If a machine is restored, it should be\n         re-joined to AD.\n\n"
    printf "      3) Hardware failure leading to incorrect information written to disk.\n"
    CRITICAL
fi

FORESTROOT="`$SQL3 $MISCDB \"select value from misc where key='forestRoot'\"`"
COMPUTERFQDN="`$SQL3 $MISCDB \"select value from misc where key='computerFQDN'\"`"
COMPUTERNAME="`$SQL3 $MISCDB \"select value from misc where key='computerName'\"`"
LOCALSITE="`$SQL3 $MISCDB \"select value from misc where key='localSite'\"`"
DEFAULTREALM="`$SQL3 $MISCDB \"select value from misc where key='defaultRealm'\"`"

#    211 machine has found a site, or has an override site set.
if [ -z "$LOCALSITE" ] ; then
    $NUM && printf "NUMBER:211\n"
    printf "\nWARNING: The computer object is not in a site.\n\n"
    printf "REASON: AD Configuration does not include a site for this machines sub-net or\n        port 389 UDP is blocked ( CLDAP ).\n\n"
    printf "INFO: Sites allow a given machine to know what AD machines it should contact for\n      information, without site information VAS will contact randomly picked\n      DCs. If this machine was joined with specific DCs or has\n      vas.conf [realms] properly set, this can be ignored.\n\n"
fi

#    212 Misc table has defaultRealm 
if [ -z "$DEFAULTREALM" ] ; then
    $NUM && printf "NUMBER:212\n"
    printf "\nISSUE: The machine is not fully joined.\n\n"
    printf "REASON: Missing defaultRealm|<DOMAIN> setting in the misc DB.\n\n"
    printf "FIX: Try running:\n\n$VAS configure realm <domain the machine should be joined to>\n\n"
    printf "     If this machine was previously joined, then any previous\n" 
    printf "     vas.conf settings will also need to be restored.\n\n     OR\n\n     Re-join the machine to AD.\n\n"
    CRITICAL
fi


#    213 Misc table has computerfqdn
if [ -z "$COMPUTERFQDN" ] ; then
    $NUM && printf "NUMBER:213\n"
    printf "\nWARNING: The cache does not have an FQDN set.\n\n"
    printf "REASON: The misc cache is missing the computerFQDN setting for this machine.\n\n"
    printf "INFO: When doing authentications VAS uses the computerFQDN to calculate the\n      ServicePrincipalName to use. If not present it guesses using the hostname\n      and /etc/hosts entries, and this can in some situations be wrong. A \n      re-join should fix this.\n\n"
fi

#    214 Misc table has computername
if [ -z "$COMPUTERNAME" ] ; then
    $NUM && printf "NUMBER:214\n"
    printf "\nWARNING: The cache does not have a Computer Name set.\n\n"
    printf "REASON: The misc cache is missing the computerName setting for this machine.\n\n"
    printf "INFO: When doing authentications VAS uses the computerName to calculate the\n      sAMAccountName to use. If not present it guesses using the hostname\n      and /etc/hosts entries, and this can in some situations be wrong. A \n      re-join should fix this.\n\n"
fi

#    215 Misc table has forestRoot
if [ -z "$FORESTROOT" ] ; then
    $NUM && printf "NUMBER:215\n"
    printf "\nWARNING: The cache does not have a Forest Root set.\n\n"
    printf "REASON: The misc cache is missing the forestRoot setting for this machine.\n\n"
    printf "INFO: The forest root is used to help locate Global Catalogs, if this is not set\n      then VAS has to resort to CLDAP pings for each AD GC call, possibly\n      slowing down response time.\n\n"
fi


# * kinit results 
# First, kdestroy just in case, so anything left in the credentials cache isn't 
# interfearing with the testing, which occasionally might fix things, but not
# sure hor to check/report that, so will jsut deal with it if it happens. 
$VAS kdestroy >/dev/null 2>&1
RESULTS="`$VAS -u host/ kinit 2>&1`"
if [ $? -ne 0 ] ; then
#   310 Machine can talk to AD ( KDC_UNREACH )
    echo "$RESULTS" | grep 1765328228 >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:301\n"
        printf "\nISSUE: The machine is unable to talk to AD.\n\n"
        printf "REASON: VAS cannot find a Domain Controller (DC) that it can communicate\n"
        printf "        with.\n\n"
        printf "FIX: Check /etc/resolve.conf and /etc/nsswitch.conf ( /etc/netsvc.conf on AIX )\n"
        printf "     for proper configuration. Run the following:\n\n"
        printf "nslookup _ldap._tcp.<domain>\nnslookup _ldap._tcp.dc._msdcs.<domain>\n"
        printf "nslookup <DC FQDN>\n\n"
        printf "INFO: VAS uses DNS calls to first resolve a domain name to a list of servers,\n"
        printf "      picks a server, and second uses the machine's resolver to turn the server\n"
        printf "      name into a IP address. The above calls are mimicing how VAS resolves a\n"
        printf "      domain name to a list of servers. The first resolving is skipped if\n"
        printf "      specific servers are specified during join time, which are stored as\n\n"
        printf "[realms]\n"
        printf " <DOMAIN>={\n"
        printf "  <method> = <DC FQDN>:<Port> [...]\n"
        printf "  [...]\n"
        printf " }\n\n"
        printf "      VAS can also do the second resolving using /etc/hosts entries, if\n"
        printf "      /etc/nsswitch.conf is set to do so, and the entries are in /etc/hosts.\n\n"
        printf "      VAS must be able to resolve both domain to server(s), and\n"
        printf "      server FQDN to IP to be able to work.\n\n"
        printf "NOTE: Some versions of VAS require both '<FQDN>' and '<FQDN>.'\n"
        printf "      entries. An example /etc/hosts entry:\n\n"
        printf "192.168.0.50	dc-1.example.com dc-1.example.com.\n\n"
        printf "      The /etc/hosts entry requires the '.', as during some requests VAS adds\n"
        printf "      a '.' to the given FQDN to suppress DNS from trying <name>.<search>\n"
        printf "      combinations.\n\n"
        printf "NOTE: The above behavior is fixed in VAS 3.3.1+, VAS 3.2.0.130+, and VAS 3.1.2.66+.\n\n"
        CRITICAL
    fi    

#   302 Object exists in AD. ( client unknown )
    echo "$RESULTS" | grep 1765328378 >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:302\n"
        printf "\nISSUE: The machine is unable to talk to AD.\n\n"
        printf "REASON: VAS cannot find find its host/ object in AD\n\n"
        printf "FIX: Check Active Directory for the computer object for this installation.\n     It cannot be found using the information available on this machine.\n"
        printf "     Try running the following to re-create the computer object:\n\n"
        printf "$VAS -u <admin or joiner account> create computer host/\n\n"
        printf "     NOTE:  If a custom name was used, modify the above command to use the\n     desired name.\n\n"
        CRITICAL
    fi    

#   303 Machine timesync. ( TIME_SKEW ) 
    echo "$RESULTS" | grep 1765328347 >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:303\n"
        printf "\nISSUE: The machine is unable to talk to AD.\n\n"
        printf "REASON: VAS is not in time sync with the AD controller it is contacting.\n\n"
        printf "FIX: Try running:\n\n$VAS timesync\n\n"
        printf "INFO: Kerberos is time sensitive and the machine's time must be within 5\n      minutes of the AD DC's time. If running this OS in a virtual environment,\n      please set the vas.conf [vasd] timesync-interval setting to -1. This will\n      instruct vasd to timesync every 30 seconds.\n\n"
        printf "      Using NTP is preferred as it will gradually syncronize times.\n      'vastool timesync' is a hard set to the correct time, which can cause\n      issue with databases and other time-sensitive applications.\n"
        CRITICAL
    fi    

#   304 The host.keytab knows the right password. ( preauth failed ).
    echo "$RESULTS" | grep 1765328360 >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:304\n"
        printf "\nISSUE: The machine is unable to talk to AD.\n\n"
        printf "REASON: VAS does not know the correct password for the AD object.\n\n"
        printf "FIX: Try running:\n\n$VAS -u <admin or joining account> passwd -rk $KEYTAB host/\n\n"
        printf "INFO: This usually happens for three reasons:\n\n"
        printf "      1) Another machine is using the same computer object. Make sure this is\n         the only machine using this computer object, and if sharing the\n         computer object ( some cluster situations ), that it is setup correctly\n         to not cause this.\n\n"
        printf "      2) In AD, the computer object was accidentially re-set ( R-click on the\n         computer object, select Reset Account ).\n\n"
        printf "      3) In AD, the computer object was deleted/re-created.\n\n"
        printf "      If the reset of this object appears to break another machine, then it is\n      likely sharing the computer object. One machine should be re-joined using\n      the -n <new name> option to make unique computer objects in AD\n\n"
        CRITICAL
    fi    

#   305 The host/ account is usable ( CREDENTIALS REVOKED )
    echo "$RESULTS" | grep 1765328366 >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:305\n"
        printf "\nISSUE: The machine is unable to talk to AD.\n\n"
        printf "REASON: The AD object is not usable at this time.\n\n"
        printf "FIX: Try deleting and re-creating the computer object:\n\n"
        printf "$VAS -u <admin or joiner account> delete computer host/\n"
        printf "$VAS -u <admin or joiner account> create computer host/\n\n"
        printf "INFO: This does not tend to happen often. Some possible causes:\n\n"
        printf "      1) Account was set as disabled. Check the Account tab on the object.\n\n"
        printf "      2) Account had an expire time set. Use ADSI EDIT to check if the computer\n         object's accountExpires is set. ( Not 0 or 9223372036854775807 ).\n\n"
        CRITICAL
    fi    

#   306 DES is set, but no entries ( KRB5KDC_ERR_PREAUTH_REQUIRED )
    echo "$RESULTS" | grep 1765328359 >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:306\n"
        printf "\nISSUE: The machine is unable to talk to AD.\n\n"
        printf "REASON: The AD object is requireing DES keys, but locally there only exists\n        arc-four keys.\n\n"
        printf "FIX: An administrator needs to re-set the password and enable DES keys:\n\n"
        printf "$VAS -u <admin or joiner account> passwd -erk /etc/opt/quest/vas/host.keytab host/\n\n     OR\n\n     In AD, un-set the Use DES keys only flag on the computer object.\n\n"
        printf "INFO: By default VAS only generates the stronger arc-four keys.\n\n"
        CRITICAL
    fi    

#   307 ( Misc kinit error, not one of the above known ones ).
    $NUM && printf "NUMBER:307\n"
    printf "\nUnrecognised vastool kinit error obtained, please contact Quest Support. Error:\n\n`$VAS -u host/ kinit 2>&1`\n\n"
    CRITICAL
fi 


# * Computer account checks.
#   401 Computer can read AD object. 
OUT="`$VAS -u host/ attrs host/ cn | 2>&1`"
echo "$OUT" | grep -i "could not resolve Service"
if [ $? -eq 0 ] ; then
    $NUM && printf "NUMBER:401\n"
    printf "\nISSUE: The machine is unable to read its own object.\n\n"
    printf "REASON: The AD permissions do not allow the host to read its own attributes.\n\n"
    printf "FIX: Give the host/ object rights to read itself.\n\n"
    printf "INFO: VAS requires this to allow it to compute certin requests, and resolve\n"
    printf "      certain information.\n"
    CRITICAL
fi    



#   402 Computer account has valid UPN.
UPN="`$VAS -u host/ attrs host/ userPrincipalName 2>/dev/null | cut -d: -f2 | sed 's/^[ \t]*//'`"
EXPECTEDUPN="host/$COMPUTERFQDN@`echo $DEFAULTREALM | tr  '[:lower:]' '[:upper:]'`"
U1="`echo $UPN | tr '[:upper:]' '[:lower:]'`"
U2="`echo $EXPECTEDUPN | tr '[:upper:]' '[:lower:]'`"
if [ ! "$U1" = "$U2" ] ; then
    $NUM && printf "NUMBER:402\n"
    printf "\nWARNING: The computer object does not have the expected userPrincipalName.\n\n"
    printf "REASON: AD Object userPrincipalName is set to\n        <$UPN>\n        it is supposed to be\n        <$EXPECTEDUPN>\n\n"
    printf "INFO: This can usually be ignored. If using other Kerberos applications or VSJ\n      with this computer object it could cause difficulty in the application\n      finding the computer object.\n\n"
fi

#   403 Computer account userAccountControl has VAS_UF_PASSWD_NOTREQD
UAC="`$VAS -u host/ attrs host/ userAccountControl 2>/dev/null | cut -d: -f2 | sed 's/^[ \t]*//'`"
# The 'No password change required' flag is kept in the 17th bit.
NOPASSWD=17
#echo $UAC
BINUAC="`echo \"obase=2; $UAC\" | bc`"
NOPWSET=0
LEN="`printf \"$BINUAC\" | wc -m | awk '{print $1}'`"
if [ $LEN -eq $NOPASSWD ] ; then 
    NOPWSET=1
elif [ $LEN -gt $NOPASSWD ] ; then
    LEN2=`expr $LEN - $NOPASSWD + 1`
    if [ `echo $BINUAC | cut -c$LEN2` -eq 1 ] ; then 
        NOPWSET=1
    fi    
fi
   
if [ $NOPWSET -eq 0 ] ; then
    $NUM && printf "NUMBER:403\n"
    printf "\nWARNING: The computer object userAccountControl does not have the\n         DONT_EXPIRE_PASSWORD flag set.\n\n"
    printf "INFO: This means the computer account's password can expire. If the vas.conf\n      option [vasd] password-update-interval is not set to shorter then the AD\n      password expire policy, this machine could fail to communicate with AD.\n\n"
fi

#   404 Computer account userAccountControl has VAS_UF_WORKSTATION_TRUST_ACCOUNT
WSTRUSTBIN=13
WSTRUST=0
LEN="`printf \"$BINUAC\" | wc -m | awk '{print $1}'`"
if [ $LEN -eq $WSTRUSTBIN ] ; then 
    WSTRUST=1
elif [ $LEN -gt $WSTRUSTBIN ] ; then
    LEN2=`expr $LEN - $WSTRUSTBIN + 1`
    if [ `echo $BINUAC | cut -c$LEN2` -eq 1 ] ; then 
        WSTRUST=1
    fi    
fi
   
if [ $WSTRUST -eq 0 ] ; then
    $NUM && printf "NUMBER:404\n"
    printf "\nWARNING: The computer object userAccountControl does not have the\n         ADS_UF_WORKSTATION_TRUST_ACCOUNT flag set.\n\n"
    printf "INFO: The computer object in AD will not be properly recognised as a computer\n      object.\n\n"
fi

#   405 Computer account userAccountControl doesn't have ADS_UF_USE_DES_KEY_ONLY
DESBIN=22
USEDES=0
LEN="`printf \"$BINUAC\" | wc -m | awk '{print $1}'`"
if [ $LEN -eq $DESBIN ] ; then 
    USEDES=1
elif [ $LEN -gt $DESBIN ] ; then
    LEN2=`expr $LEN - $DESBIN + 1`
    if [ `echo $BINUAC | cut -c$LEN2` -eq 1 ] ; then 
        USEDES=1
    fi    
fi
   
if [ $USEDES -eq 1 ] ; then
    $NUM && printf "NUMBER:405\n"
    printf "\nWARNING: The computer object userAccountControl has USE_DES_KEY_ONLY set.\n\n"
    printf "INFO: This machine is using the less secure DES keys instead of the stronger\n      arc-four keys. It is recommended that this be un-set, unless there is a\n      specific need for it, like compatability with applications using an older\n      Kerberos implimentation.\n\n"
fi

#   406 Computer account userAccountControl doesn't have ADS_UF_DONT_REQUIRE_PREAUTH
NOPREAUTHBIN=23
NOPREAUTH=0
LEN="`printf \"$BINUAC\" | wc -m | awk '{print $1}'`"
if [ $LEN -eq $NOPREAUTHBIN ] ; then 
    NOPREAUTH=1
elif [ $LEN -gt $NOPREAUTHBIN ] ; then
    LEN2=`expr $LEN - $NOPREAUTHBIN + 1`
    if [ `echo $BINUAC | cut -c$LEN2` -eq 1 ] ; then 
        NOPREAUTH=1
    fi    
fi
   
if [ $NOPREAUTH -eq 1 ] ; then
    $NUM && printf "NUMBER:406\n"
    printf "\nWARNING: The computer object userAccountControl has\n         ADS_UF_DONT_REQUIRE_PREAUTH set.\n\n"
    printf "INFO: This setting is usually insecure, unless there is a compelling reason to\n      use this setting, it is recommended that it is turned off.\n\n"
fi

#   501 Computer account has valid SPNs.
ADSPNs="`$VAS -u host/ attrs host/ servicePrincipalName 2>/dev/null | cut -d: -f2 | sed 's/^[ \t]*//' | tr '\n' ' '`"
EXPECTEDSPNS="host/$COMPUTERFQDN host/$COMPUTERNAME"
MISSINGSPN=0
for spn in $EXPECTEDSPNS ; do
    echo $ADSPNs | grep -i "$spn" >/dev/null
    if [ $? -ne 0 ] ; then
        printf "Did not find SPN <$spn> in list of SPNs <$ADSPNs>\n"
        MISSINGSPN=1
    fi
done

if [ $MISSINGSPN -eq 1 ] ; then
    $NUM && printf "NUMBER:501\n"
    printf "\nWARNING: The computer object is missing servicePrincipalName entry(s).\n\n"
    printf "FIX: Try running:\n\n$VAS -u <admin or joining account> setattrs -m host/ servicePrincipalName $EXPECTEDSPNS\n\n"
    printf "INFO: When VAS is authenticating users, it does so against its SPN entry.\n      Lacking this entry could trigger KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN failures\n      during authentication.\n\n      Later VAS versions should still allow logins in this situation.\n\n"
fi

#   502 host.keytab contains matching entries for each SPN in AD.
KEYTABSPNS="`$VAS ktutil list | grep -v '^Vno' | awk '{print $3}' | sort | uniq | cut -d@ -f1 | grep -v \"^$COMPUTERNAME\" | grep -v \"cifs/$COMPUTERFQDN\" | grep -v '^[ \t]*$' | tr '\n' ' '`"
MISSINGSPN=0
for spn in $ADSPNs; do
    echo $KEYTABSPNS | grep -i "$spn" >/dev/null
    if [ $? -ne 0 ] ; then
        printf "Did not find AD SPN <$spn> in list of keytab SPNs <$KEYTABSPNS>\n"
        MISSINGSPN=1
    fi
done

if [ $MISSINGSPN -eq 1 ] ; then
    $NUM && printf "NUMBER:502\n"
    printf "\nWARNING: host.keytab is missing entries for AD SPN entries.\n\n"
    printf "FIX: Add the missing SPN to host.keytab using the vastool ktutil alias command.\n\n"
    printf "INFO: VAS could fail to authenticate some requests if it cannot find a matching\n      keytab entry for the service used.\n\n"
fi

#   503 SPNs in AD match each entry in host.keytab.
MISSINGSPN=0
for spn in $KEYTABSPNS; do
    echo $ADSPNs | grep -i "$spn" >/dev/null
    if [ $? -ne 0 ] ; then
        printf "Did not find keytab SPN <$spn> in list of AD SPNs <$ADSPNs>\n"
        MISSINGSPN=1
    fi
done

if [ $MISSINGSPN -eq 1 ] ; then
    $NUM && printf "NUMBER:503\n"
    printf "\nWARNING: host.keytab has extra SPN entries.\n\n"
    printf "FIX: Add the missing SPN to the computer object's servicePrincipalName in AD.\n\n"
    printf "INFO: VAS could attempt to contact AD for authentication using a SPN that is\n      known to not exist in AD.\n\n"
fi

OUT="`$VAS -u host/ auth -S host/$COMPUTERFQDN 2>&1`"
if [ $? -ne 0 ] ; then
#   504 auth -S host/$FQDN returns KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN, duplicate.
    echo "$OUT" | grep 1765328377 >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:504\n"
        printf "\nISSUE: VAS is unable to properly authenticate against AD.\n\n"
        printf "REASON: AD is not able to find the specified SPN <host/$COMPUTERFQDN>.\n\n"
        printf "FIX: Two possible situations:\n\n     1) No SPN set. There would be a WARNING message above about a missing SPN.\n\n"
        printf "     2) Duplicate SPN. Another object in AD has the same SPN, so AD does not\n        know which one to respond with. Locate by running:\n\n$VAS -u host/ search \"(servicePrincipalName=host/$COMPUTERFQDN)\" dn\n\n        If that only shows this computer's object, get an AD admin to do the\n        search ( it is very possible to conflict with an object host/ does not\n        have access to find ).\n\n"
        CRITICAL
    fi    

#   505 auth -S host/$FQDN fails.
    $NUM && printf "NUMBER:505\n"
    printf "\nUnrecognised 'vastool -u host/ auth -S host/$COMPUTERFQDN' error obtained, please contact Quest Support. Error:\n\n`$VAS -u host/ auth -S host/$COMPUTERFQDN 2>&1`\n\n"
    CRITICAL
fi

BADPERMS1=0
BADPERMS2=0
DP="drwxr-xr-x(755)"
for D in /etc/opt/quest /etc/opt/quest/vas /var/opt/quest /var/opt/quest/vas /var/opt/quest/vas/vasd ; do
    ls -lad $D | cut -c2,3,4,5,7,8,10 | grep "\-" >/dev/null    
    if [ $? -eq 0 ] ; then
        BADPERMS1=1
        printf "Missing needed permissions on <$D>, currently:\n<`ls -lad $D | awk '{print $1}'`>\nshould be:\n<$DP>\n\n"
    fi
    ls -lad $D | cut -c9 | grep "\-" >/dev/null    
    if [ $? -ne 0 ] ; then
        BADPERMS2=1
        printf "'other' has write permissions on:\n<$D><`ls -lad $D | awk '{print $1}'`>\nThat permission should be removed.\n\n"
    fi
done

FP="-rw-r--r--(644)"
for F in /etc/opt/quest/vas/vas.conf /var/opt/quest/vas/vasd/vas_ident.vdb /var/opt/quest/vas/vasd/vas_misc.vdb ; do
    ls -la $F | cut -c2,3,5,8 | grep "\-" >/dev/null    
    if [ $? -eq 0 ] ; then
        BADPERMS1=1
        printf "Missing needed permissions on <$F>, currently:\n<`ls -la $F | awk '{print $1}'`>\nshould be:\n<$FP>\n\n"
    fi
    ls -la $F | cut -c9 | grep "\-" >/dev/null    
    if [ $? -ne 0 ] ; then
        BADPERMS2=1
        printf "'other' has write permissions on:\n<$F>(`ls -la $F | awk '{print $1}'`)\nThat permission should be removed.\n\n"
    fi
done

#   601 File/directory permissions. Not enough.
if [ $BADPERMS1 -eq 1 ] ; then
    $NUM && printf "NUMBER:601\n"
    printf "\nISSUE: The local files/directories do not have needed permissions.\n\n"
    printf "FIX: Change the permissions on the mentioned file(s)/directory(s).\n\n"
    CRITICAL
fi

#   602 File/directory permissions. Too much.
if [ $BADPERMS2 -eq 1 ] ; then
    $NUM && printf "NUMBER:602\n"
    printf "\nISSUE: The local files/directories allow more access than they should.\n\n"
    printf "FIX: Change the permissions on the mentioned file(s)/directory(s).\n\n"
    CRITICAL
fi

#   603 /etc/nsswitch.conf contains vas3. ( check for map entries ). 
if [ -f /etc/nsswitch.conf ] ; then
    grep vas3 /etc/nsswitch.conf >/dev/null
    if [ $? -ne 0 ] ; then
        $NUM && printf "NUMBER:603\n"
        printf "\nISSUE: /etc/nsswitch.conf does not appear to be configured to use VAS.\n\n"
        printf "INFO: If only using mapped users this is ok, otherwise it could cause VAS\n      information to not be seen on the system.\n\n"
        printf "FIX: If not exclusively using mapped user mode, try running:\n\n$VAS configure nss\n\n"
        MF="`$SQL3 $MISCDB \"select value from misc where key='userMappings'\"`"
        if [ -z "$MF" ] ; then
            CRITICAL
        fi
    fi
fi

#   604 Relevent PAM configuraiton file contains at least one pam_vas3 line.
for file in /etc/pam.conf /etc/pam.d/system-auth /etc/pam.d/common-auth ; do
    if [ -f $file ] ; then
        grep pam_vas3 $file >/dev/null
        if [ $? -ne 0 ] ; then
            $NUM && printf "NUMBER:604\n"
            printf "\nISSUE: $file does not appear to be configured to use VAS.\n\n"
            printf "INFO: VAS users cannot be authenticated using PAM.\n\n"
            printf "FIX: Try running:\n\n$VAS configure pam\n\n"
            CRITICAL
        fi
    fi
done

#   605 AIX: /usr/lib/security/methods.cfg contains VAS
for file in /etc/security/user /usr/lib/security/methods.cfg ; do
    if [ -f $file ] ; then
        grep "VAS" $file >/dev/null
        if [ $? -ne 0 ] ; then
            $NUM && printf "NUMBER:605\n"
            printf "\nISSUE: $file does not appear to be configured to use VAS.\n\n"
            printf "INFO: VAS users cannot be authenticated using LAM.\n\n"
            printf "FIX: Try running:\n\n$VAS configure irs\n\n"
            CRITICAL
        fi
    fi
done

BADPERMS1=0
BADPERMS2=0
FP="-r-r--r--(444)"
for F in /etc/netsvc.cfg /etc/nsswitch.conf /etc/resolv.conf /etc/passwd /etc/group /usr/lib/security/methods.cfg /etc/pam.conf /etc/pam.d/system-auth /etc/pam.d/common-auth; do
    if [ -f $F ] ; then 
        ls -la $F | cut -c2,5,8 | grep "\-" >/dev/null    
        if [ $? -eq 0 ] ; then
            BADPERMS1=1
            printf "Missing needed permissions on <$F>, currently:\n<`ls -la $F | awk '{print $1}'`>\nshould be at least:\n<$FP>\n\n"
        fi
        ls -la $F | cut -c9 | grep "\-" >/dev/null    
        if [ $? -ne 0 ] ; then
            BADPERMS2=1
            printf "'other' has write permissions on:\n<$F>(`ls -la $F | awk '{print $1}'`)\nThat permission should be removed.\n\n"
        fi
    fi
done

#   606 System file permissions. Not enough.
if [ $BADPERMS1 -eq 1 ] ; then
    $NUM && printf "NUMBER:606\n"
    printf "\nISSUE: The system files/directories do not have needed permissions.\n\n"
    printf "FIX: Change the permissions on the mentioned file(s)/directory(s).\n\n"
    CRITICAL
fi

#   607 System file permissions. Too much.
if [ $BADPERMS2 -eq 1 ] ; then
    $NUM && printf "NUMBER:607\n"
    printf "\nISSUE: The system files/directories allow more access then they should.\n\n"
    printf "FIX: Change the permissions on the mentioned file(s)/directory(s).\n\n"
    CRITICAL
fi


#   701 Valid license 1 ( expired )
$VAS license -s 2>&1 | grep "is a site" >/dev/null
if [ $? -ne 0 -o "$DO_LICENSING" = "1" ] ; then
    OUT="`$VAS -d5 license -q 2>&1 | grep \"is expired\" >/dev/null`"
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:701\n"
        printf "\nWARNING: VAS has an expired license.\n\n"
        printf "Error: <$OUT>:\n\n"
        printf "INFO: VAS requires a valid license in order to load users from AD.\n\n"
        printf "      If there are no other errors, then a valid license exists, and this\n"
        printf "      is only house-keeping information.\n\n"
    fi
fi

#   702 Valid license 2
$VAS license -s 2>&1 | grep "is a site" >/dev/null
if [ $? -ne 0 -o "$DO_LICENSING" = "1" ] ; then
    $VAS license -q 2>&1 | awk '/Smartcard/ {exit}; {print $0}' | grep "No.*licenses are installed" >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:702\n"
        printf "\nISSUE: VAS does not have any valid licenses available.\n\n"
        printf "INFO: VAS requires a valid license in order to load users/groups into the cache.\n\n"
        printf "FIX: Place a valid license file in </etc/opt/quest/vas/.licenses>.\n\n"
        printf "Error: <$VAS license -q>:\n`$VAS license -q 2>&1`\n\n"
        CRITICAL
    fi
fi

#   703 Valid license 3
$VAS license -s 2>&1 | grep "is a site" >/dev/null
if [ $? -ne 0 -o "$DO_LICENSING" = "1" ] ; then
    $VAS license -q 2>&1 | grep "Loading licenses failed" >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:703\n"
        printf "\nISSUE: VAS does not have a valid license directory.\n\n"
        printf "INFO: VAS requires a valid license in a valid directory in order to\n"
        printf "      load users/groups into the cache.\n\n"
        printf "FIX: Create the license directory: </etc/opt/quest/vas/.licenses>\n"
        printf "     and place a valid license in that directory.\n\n"
        printf "Error: <$VAS license -q>:\n`$VAS license -q 2>&1`\n\n"
        CRITICAL
    fi
fi


#   704 vasd running
VASDC="`ps -ef | grep [v]asd | wc -l | awk '{print $1}'`"
if [ $VASDC -lt 2 ] ; then
    $NUM && printf "NUMBER:704\n"
    printf "\nISSUE: vasd does not appear to be running ( or only one process ).\n\n"
    printf "FIX: Restart the process by running the vasd init script:\n"
    printf "     /etc/init.d/vasd restart (Solaris/Linux)\n"
    printf "     /sbin/init.d/vasd restart (HP)\n"
    printf "     /etc/rc.d/init.d/vasd restart (AIX)\n\n"
    printf "INFO: vasd handles querying for AD updates and storing them in the local cache.\n"
    printf "      If it is not running then all authentications will be against the\n"
    printf "      disconnected cache, and the machine will get no updates from AD.\n\n"
    printf "      If after a restart and a short wait this script still complains,\n"
    printf "      run the init script with 'stop' then run:\n\n"
    printf "/opt/quest/sbin/vasd -d2 -g5 2>&1 | tee /tmp/vasd.out\n\n"
    printf "      And after it stops, or run for about a minute, ctrl-c it, and\n"
    printf "      send the /tmp/vasd.out file in to Support.\n\n"
    CRITICAL
fi

#   705 vasd responding
RESP="`$VAS list users 2>&1 1>/dev/null`"
echo "$RESP" | grep "Could not communicate with vasd" >/dev/null
if [ $? -eq 0 ] ; then
    $NUM && printf "NUMBER:705\n"
    printf "\nISSUE: vasd does not appear to be responding to requests.\n\n"
    printf "FIX: Wait 5 minutes and try this script again. If after that this script still\n"
    printf "     complains, restart the process by running the vasd init script:\n"
    printf "     /etc/init.d/vasd restart (Solaris/Linux)\n"
    printf "     /sbin/init.d/vasd restart (HP)\n"
    printf "     /etc/rc.d/init.d/vasd restart (AIX)\n\n"
    printf "     /sbin/init.d/vasd on HP, and /etc/rc.d/init.d/vasd on AIX ).\n\n"
    printf "INFO: vasd handles querying for AD updates and storing them in the local cache.\n"
    printf "      If it is not accepting requests then the machine will get no updates.\n\n"
    printf "      If after a restart and a short wait this script still complains,\n"
    printf "      run the init script with 'stop' then run:\n\n"
    printf "/opt/quest/sbin/vasd -d2 -g5 2>&1 | tee /tmp/vasd.out\n\n"
    printf "      And after it stops, or run for about a minute, ctrl-c it, and\n"
    printf "      send the /tmp/vasd.out file in to Support.\n\n"
    CRITICAL
fi

$NUM && printf "NUMBER:001\n"
printf "\nAll tests passed.\nNo serious issue with this machine's connection to AD has been found.\n"
printf "\nIf this is incorrect, please contact Quest Support for additional help.\n\n"
