#!/bin/sh
################################################################################
# Copyright (c) 2009 Quest Software, Inc.  All Rights Reserved.
#
# bundle.sh
#
# Purpose:      Create a .tar.gz of normal system files and system information to 
#               aid in the resolving of issues.
#
# Author(s):    Seth Ellsworth (seth.ellsworth@quest.com)
#
# Version:      0.3
#
# This script relies on internal knowledge of vasd's internal cache schema, 
# which is undocumented and subject to change between releases.
#
################################################################################

################################################################################
# 
# The vas_snapshot.sh script gathers the following information:
# 
# Basic:
# Hostname
# Full hostname
# Platform
# uname -a
# Date
# 
# VAS Basics if installed:
# Vastool version
# Site version
# Domain
# Date
# Date in AD (Doesn't modify local time)
# If a Kinit works.
# vastool info id
# vastool info site
# Number of users/groups cached.                 
#                
# System commands, patch information: 
# ipcs
# df -k
# Solaris:
#     showrev -p
# HP-UX:
#     swlist -l product *,c=patch
# AIX:
#     oslevel -s
#     oslevel -r
#     instfix -i
# vm_stat, w, free, vmstat.
# env
# ifconfig -a ( and HP equiv )
# package dump
# 
# Extended VAS information:
# license info
# host object attrs
# A dump of the misc and srvinfo cache.
# A dump of the access_control cache.
# A dump of the override and mapped_user tables.
# File listings of the VAS directories
# vastool ktutil list
# preflight output
# status output
# access control output
# AD security info                
# 
# System files:
# nsswitch.conf
# hosts, only localhost and host lines, others are not gathered.
# /etc/resolv.conf
# /etc/syslog.conf
# /etc/pam.conf
# All files in /etc/pam.d/
# /etc/security/user
# /etc/security/limits
# /etc/security/login.cfg
# /usr/lib/security/methods.cfg
# /etc/irs.conf
# /etc/passwd and /etc/group, +/- entries only ( to follow compat settings )                
# 
# VAS files:
# vas.conf
# users.allow|deny
# user-override
# groups-override
# All files in .licenses/
# All files in access.d/
# lastjoin
# 
# VGP info:
# listgpc
# listgpc -l
# listgpt
# listlinks
# rsop
# It does an apply with full debug, capturing the output. 
# register
# 
# VASYPD:
# A dump of the db file, just the table names.
# A dump of the xlate table.
# A dump of the vasypinfo table.
# ypwhich -m
#
# OTHER:
#   /opt/quest/sbin/sshd -t
#   /opt/quest/bin/testparm -s
#
# If 'nss' or 'NSS' is passed as a parameter to the script it will also gather:
# vastool nss getpwent
# vastool nss getgrent
# vastool list users
# vastool list groups
# 
# This set of information was chosen to best assist Support with any issue, and
# familiarize them with the system so they can give better support when asking
# for additional information. 
# 
# The resulting file will be in /tmp/ with a name of
# vas_snapshot.<hostname>.<date>.tar.gz.
# 
# If you don't wish Support to have specific gathered information, either
# modify the script to not gather it, or open the archive after and remove
# the file(s) before sending.
# 
################################################################################
ARG=$1

################################################################################
# Functions. 
################################################################################
Prep()
{
    echo "*** This script will be gathering System and VAS information ***"

    # First thing, check for root.
    if [ "`id | sed 's/uid=\([0-9]*\).*/\1/'`" -ne 0 ] ; then
        echo "Must be run as root."
        exit
    fi
    
    # Set up the variables and prep the directories. 
    DATE=`date "+%Y-%m-%d_%H-%M-%S"`
    HOSTNAME=`hostname | cut -d. -f 1`
    DOMAINNAME=`domainname`
    VAS=/opt/quest/bin/vastool
    PREFLIGHT=/opt/quest/bin/preflight
    UPTOOL=/opt/quest/bin/uptool
    VGP=/opt/quest/bin/vgptool
    VGPM=/opt/quest/bin/vgpmod
    SQL3=/opt/quest/libexec/vas/sqlite3
    MISC_DB=/var/opt/quest/vas/vasd/vas_misc.vdb
    VAS_IDENT=/var/opt/quest/vas/vasd/vas_ident.vdb
    VASYPD_DB=/var/opt/quest/vas/vasypd/nismaps/rfc2307_nismaps.vdb
    if [ -x "${VAS}" ] ; then
        HAVE_VAS=1
    else
        HAVE_VAS=0
    fi

    OUT_DIR=/tmp/${DATE}
    OUT_MAIN=${OUT_DIR}/main.txt
    OUT_DIR_CMDS=${OUT_DIR}/cmds_out
    OUT_DIR_FILES=${OUT_DIR}/files
    OUT_BUNDLE=/tmp/vas_snapshot.${HOSTNAME}.${DATE}
    mkdir ${OUT_DIR}
    mkdir ${OUT_DIR_CMDS}
    mkdir ${OUT_DIR_FILES}
}


DeterminePlatform()
{
    unamea=`uname -a`
    name=`echo $unamea | awk '{ print $1 }'`
    case $name in
        "Linux")
            if [ -f /etc/redhat-release ] ; then
                dist=`cat /etc/redhat-release | grep Enterprise`
                if [ $? -eq 0 ]; then
                    level=`cat /etc/redhat-release | sed 's/.*release\ //' | sed 's/\ .*//'`
                    platform="RedHat$level"
                else
                    platform="RedHat"
                fi
            elif [ -f /etc/SuSE-release ] ; then
                level=`cat /etc/SuSE-release | tr "\n" ' ' | sed 's/.*=\ //' | sed 's/\([0-9]*\).*/\1/'`
                platform="SuSE$level"
            elif [ -f /etc/fedora-release ] ; then
                level=`cat /etc/fedora-release | tr "\n" ' ' | sed 's/.*=\ //' | sed 's/\([0-9]*\).*/\1/'`
                platform="Fedora$level"
            else
                platform="Linux - `uname -r`"
            fi
        ;;
        "SunOS")
            level=`echo $unamea | awk '{ print $3 }' | sed 's/[0-9]*\.\([0-9]*\)/\1/'`
            platform="Solaris$level"
            ;;
        "AIX")
            level=`oslevel | sed 's/\([0-9]*\.[0-9]*\).*/\1/'`
            platform="AIX$level"
            ;;
        "HP-UX")
            level=`echo $unamea | awk '{print $3}' | sed 's/[a-zA-Z]*\.\(.*\)/\1/'`
            platform="HP-UX${level}"
            ;;
        "OSF1")
            level=`echo $unamea | awk '{printf "%s %s", $3, $4}'`
            platform="Tru64 ${level}"
            ;;
        *)
            platform="UNKNOWN"
            ;;
    esac
}

GetCommandOutput()
{
    # Runs command $1, puts the output into file ${OUT_DIR_CMDS}/$2
    LogMainBasic "Saving output of command <${1}>"    
    eval "$1" >${OUT_DIR_CMDS}/$2 2>&1
}

LogMain()
{
    echo "Saving <${1}>"
    echo "${1}" >> ${OUT_MAIN}
}

LogMainBasic()
{
    echo "${1}"
    echo "${1}" >> ${OUT_MAIN}
}

GrabFile()
{
    # If it exists, grab file $1.
    if [ -f "${1}" ] ; then
        cp ${1} ${OUT_DIR_FILES}/ 2>/dev/null
        LogMainBasic "Storing file ${1}"
    fi
}

GetSystemBasics()
{
    LogMainBasic "*** Gathering the system basics ***"
    DeterminePlatform
    LogMain "Hostname:          ${HOSTNAME}"    
    LogMain "Full Hostname:     `hostname`"    
    LogMain "Platform:          ${platform}"
    LogMain "Uname -a:          ${unamea}"
    LogMain "Domainname:        ${DOMAINNAME}"
    LogMain "Date:              ${DATE}"
}

GetVASBasics()
{
    if [ "${HAVE_VAS}" -eq 0 ] ; then
        LogMain "VAS does not appear to be installed, skipping VAS Basics."
        return
    fi
    LogMainBasic "*** Gathering VAS basics ***"   
    LogMain "Vastool version:      `${VAS} -v 2>&1 | tr '\n' ' '`"
    if [ -f /opt/quest/sbin/vasypd ] ; then
        LogMain "Vasypd version:       `/opt/quest/sbin/vasypd -v 2>&1 | tr '\n' ' '`"
    fi        
    if [ -f /opt/quest/bin/vgptool ] ; then
        LogMain "Vgptool version:      `/opt/quest/bin/vgptool -v 2>&1 | tr '\n' ' '`"
    fi        
    if [ -f /opt/quest/sbin/vasproxyd ] ; then
        LogMain "VasProxy version:     `/opt/quest/sbin/vasproxyd -v 2>&1 | tr '\n' ' '`"
    fi        
    if [ -f /opt/quest/sbin/smbd ] ; then
        LogMain "Quest Samba version:  `/opt/quest/sbin/smbd --version 2>&1 | tr '\n' ' '`"
    fi        
    LogMain "Site:              `${VAS} license -s`"
    DOMAIN=`${VAS} info domain 2>&1`
    LogMain "Domain:            ${DOMAIN}"
    LogMain "Date:              `date`"
    LogMain "AD Date:           `${VAS} timesync -q -d ${DOMAIN}`"
    RESULT="`${VAS} -u host/ kinit 2>&1`"
    if [ "$?" -eq 0 ] ; then
        LogMain "Kinit:             Kinit worked."
    else
        LogMain "Kinit:             Kinit of host/ failed, text: ${RESULT}"
    fi
    LogMain "info id:           `${VAS} -u host/ info id 2>&1 | tr '\n' ' '`"
    LogMain "info site:         `${VAS} -u host/ info site 2>&1`"
    LogMain "Users cached:      `${VAS} list -c users 2>&1 | wc -l`"
    LogMain "Groups cached:     `${VAS} list -c groups 2>&1 | wc -l`"
    for file in /opt/quest/sbin/.vasd-site /opt/quest/sbin/.vasd-lic /opt/quest/sbin/vasd ; do
        if [ -f $file ] ; then
            LogMain "vasd file type:    `file $file`"
            break
        fi
    done    
}

GetSystemCommands()
{
    LogMainBasic "*** Gathering output of OS Specific commands ***"
    GetCommandOutput "ipcs -a" "ipcs.a.out"
    GetCommandOutput "df -k" "df.k.out"
    GetCommandOutput "w" "w.out"
    GetCommandOutput "env" "env.out"
    for file in free vm_stat vmstat; do
        if [ -f /usr/bin/$file ] ; then
            GetCommandOutput "/usr/bin/$file" "$file.out"
        fi
    done    
    case ${platform} in
        Solaris*)
            GetCommandOutput "showrev -p" "showrev.p.out"
            GetCommandOutput "pkglist" "pkglist.out"
            GetCommandOutput "ifconfig -a" "ifconfig.out"
            ;;
        HP-UX*)
            GetCommandOutput "/usr/sbin/swlist -l product *,c=patch" "swlist.l.out"
            GetCommandOutput "/usr/sbin/swlist" "swlist.out"
            GetCommandOutput "ls -la /tcb/files/auth" "tcb.ls.out"
            GetCommandOutput "for lan in `lanscan | grep lan | awk '{print $5}' | tr '\n' ' '` ; do echo \$lan; ifconfig \$lan; done" "ifconfig.out"
            ;;
        AIX*)
            GetCommandOutput "oslevel -r" "oslevel.r.out"
            GetCommandOutput "oslevel -s" "oslevel.s.out"
            GetCommandOutput "instfix -i" "instfix.i.out"
            GetCommandOutput "ifconfig -a" "ifconfig.out"
            GetCommandOutput "lslpp -L" "lslpp.L.out"
            ;;
        *)
            GetCommandOutput "ifconfig -a" "ifconfig.out"
            GetCommandOutput "rpm -qa" "rpm.qa.out"
            ;;
    esac
    GetCommandOutput "ls -la /etc /etc/security /usr/lib/security /etc/pam.d" "system.ls.out"     
}

GetVASCommands()
{
    if [ "${HAVE_VAS}" -eq 0 ] ; then
        LogMain "VAS does not appear to be installed, skipping VAS Commands."
        return
    fi
    LogMainBasic "*** Gathering output of VAS commands ***"
    GetCommandOutput "${VAS} license -q" "vas.license.q.out"
    GetCommandOutput "${VAS} license -s" "vas.license.s.out"
    GetCommandOutput "${UPTOOL} -u host/ info" "uptool.info.out"
    GetCommandOutput "${VAS} -d5 -e4 -u host/ attrs host/" "vas.host.attrs.out"
    if [ ! -z "$ARG" ] && [ "$ARG" = "nss" -o "$ARG" = "NSS" ] ; then
        GetCommandOutput "${VAS} list users" "vas.list.users.out"
        GetCommandOutput "${VAS} list groups" "vas.list.groups.out"
        GetCommandOutput "${VAS} nss getpwent" "vas.nss.getpwent.out"
        GetCommandOutput "${VAS} nss getgrent" "vas.nss.getgrnam.out"
        GetCommandOutput "${SQL3} ${VAS_IDENT} \"select grName,groupKey,LENGTH(members) from grent;\"" "vas.grent.out"
    fi    
    GetCommandOutput "${SQL3} ${MISC_DB} \"SELECT * FROM misc\"" "vas.misc.dump.out"
    GetCommandOutput "${SQL3} ${VAS_IDENT} \"SELECT * FROM access_control\"" "vas.access_control.dump.out"
    GetCommandOutput "${SQL3} ${VAS_IDENT} \"SELECT * FROM table_ctrl\"" "vas.ident.table_crtl.dump.out"
    GetCommandOutput "${SQL3} ${VAS_IDENT} \"SELECT * FROM user_ovrd\"" "vas.ident.user_ovrd.dump.out"
    GetCommandOutput "${SQL3} ${VAS_IDENT} \"SELECT * FROM user_ovrd_bygroup\"" "vas.ident.user_ovrd_bygroup.dump.out"
    GetCommandOutput "${SQL3} ${VAS_IDENT} \"SELECT * FROM group_ovrd\"" "vas.ident.group_ovrd.dump.out"
    GetCommandOutput "${SQL3} ${VAS_IDENT} \"SELECT * FROM mapped_user\"" "vas.ident.mapped_user.dump.out"
    GetCommandOutput "${SQL3} ${MISC_DB} \"SELECT * FROM table_ctrl\"" "vas.misc.table_crtl.dump.out"
    GetCommandOutput "${SQL3} ${MISC_DB} \"SELECT * FROM srvinfo\"" "vas.srvinfo.dump.out"
    GetCommandOutput "${VAS} info servers" "vas.info.servers.out"
    GetCommandOutput "${VAS} ktutil list --timestamp" "vas.ktutil.list.out"
    GetCommandOutput "${VAS} -u host/ -d5 auth -S host/" "vas.auth.out"
    FQDN=`${SQL3} ${MISC_DB} "SELECT value from misc where key='computerFQDN'"`
    if [ ! -z "$FQDN" ] ; then
        GetCommandOutput "${VAS} -u host/ -d5 auth -S host/${FQDN}" "vas.auth.fqdn.out"
    fi        
    GetCommandOutput "ls -laR /var/opt/quest" "ls.var.laR.out"
    GetCommandOutput "ls -laR /etc/opt/quest" "ls.etc.laR.out"
    GetCommandOutput "ls -laR /opt/quest" "ls.opt.laR.out"
    GetCommandOutput "iptables -L || /usr/sbin/iptables -L" "iptables.L.out"
    GetCommandOutput "fuser /var/opt/quest/vas/vasd/* || /usr/sbin/fuser /var/opt/quest/vas/vasd/* || /bin/fuser /var/opt/quest/vas/vasd/*" "fuser.vasd.out"
    GetCommandOutput "ps -ef" "ps.ef.out"
    GetCommandOutput "${PREFLIGHT} -u host/ `${VAS} info domain 2>&1`" "preflight.out"
    if [ -f /etc/opt/quest/vas/host.keytab ] ; then
        GetCommandOutput "${VAS} -u host/ status" "vas.status.out"
        GetCommandOutput "${VAS} -u host/ status dc -p -v" "vas.status.dc.p.v.out"
        GetCommandOutput "${VAS} -u host/ info acl" "vas.info.acl.out"
        GetCommandOutput "${VAS} -u host/ info adsecurity" "vas.info.adsecurity.out"
    fi
    if [ -f /opt/quest/libexec/vas/scripts/vas_check_host.sh ] ; then
        GetCommandOutput "/opt/quest/libexec/vas/scripts/vas_check_host.sh" "vas_check_host.out"
    fi
    if [ -f /opt/quest/libexec/vas/scripts/vas_status.sh ] ; then
        GetCommandOutput "/opt/quest/libexec/vas/scripts/vas_status.sh" "vas_status.out"
    fi
    for file in .vasd-site .vasd-lic vasd vasypd vasproxyd vasgpd; do
        if [ -f /opt/quest/sbin/$file ] ; then
            GetCommandOutput "file /opt/quest/sbin/$file" "file.$file.out"
            break
        fi
    done    
    for file in .vastool-site .vastool-lic vastool vgptool uptool; do
        if [ -f /opt/quest/bin/$file ] ; then
            GetCommandOutput "file /opt/quest/bin/$file" "file.$file.out"
            break
        fi
    done    
    GetCommandOutput "ls -laZ /var/opt/quest/vasd" "ls.var.laZ.out"
    if [ -f /opt/quest/bin/testparm ] ; then
        GetCommandOutput "/opt/quest/bin/testparm -s" "samba.testparm.out"
    fi
    if [ -f /opt/quest/sbin/sshd ] ; then
        GetCommandOutput "/opt/quest/sbin/sshd -t" "sshd.t.out"
    fi
}

GetSystemFiles()
{
    LogMainBasic "*** Gathering system files ***"
    GrabFile /etc/nsswitch.conf
    LogMainBasic "Storing only localhost and ${HOSTNAME} information from /etc/hosts"
    cat /etc/hosts | grep localhost >> ${OUT_DIR_FILES}/hosts 2>/dev/null
    cat /etc/hosts | grep ${HOSTNAME} >> ${OUT_DIR_FILES}/hosts 2>/dev/null
    cat /etc/inet/ipnodes 2>/dev/null | grep localhost >> ${OUT_DIR_FILES}/ipnodes 2>/dev/null
    cat /etc/inet/ipnodes 2>/dev/null | grep ${HOSTNAME} >> ${OUT_DIR_FILES}/ipnodes 2>/dev/null
    if [ -f $SQL3 ] ; then
        SERVERS=`${SQL3} ${MISC_DB} "SELECT fqdn FROM srvinfo UNION SELECT srvfqdn FROM usn_cache;" | awk -F. '{print $1}'`
        if [ ! -z "$SERVERS" ] ; then
            for server in $SERVERS ; do
                cat /etc/hosts | grep $server >> ${OUT_DIR_FILES}/hosts 2>/dev/null
                cat /etc/inet/ipnodes 2>/dev/null | grep $server >> ${OUT_DIR_FILES}/ipnodes 2>/dev/null
            done
        fi    
    fi    
    GrabFile /etc/resolv.conf
    GrabFile /etc/syslog.conf
    GrabFile /etc/pam.conf
    if [ -d /etc/pam.d ] ; then
        mkdir ${OUT_DIR_FILES}/pam.d
        cp -r /etc/pam.d/* ${OUT_DIR_FILES}/pam.d
        LogMainBasic "Storing /etc/pam.d/*"                   
    fi
    for file in /etc/security/user /etc/security/login.cfg /etc/security/limits /usr/lib/security/methods.cfg /etc/irs.conf /etc/sia/matrix.conf /etc/netsvc.conf /etc/svc.conf; do
        GrabFile $file
    done        
    for d in /etc /etc/ssh /etc/ssh2 /usr/local/etc /usr/local/etc/ssh /usr/local/etc/ssh2 /opt/ssh/etc /etc/opt/quest/ssh /etc/openssh ; do
        GrabFile $d/sshd_config
        GrabFile $d/ssh_config
    done        
    grep "^+" < /etc/passwd > ${OUT_DIR_FILES}/passwd
    grep "^-" < /etc/passwd >> ${OUT_DIR_FILES}/passwd
    grep "^+" < /etc/group > ${OUT_DIR_FILES}/group
    grep "^-" < /etc/group >> ${OUT_DIR_FILES}/group
}

GetVASFiles()
{
    if [ "${HAVE_VAS}" -eq 0 ] ; then
        LogMain "VAS does not appear to be installed, skipping VAS files."
        return
    fi
    LogMainBasic "*** Gathering VAS files ***"
    GrabFile /etc/opt/quest/vas/vas.conf
    GrabFile /etc/opt/quest/vas/users.allow
    GrabFile /etc/opt/quest/vas/users.deny
    GrabFile /etc/opt/quest/vas/user-override
    GrabFile /etc/opt/quest/vas/group-override
    GrabFile /etc/opt/quest/vas/lastjoin
    GrabFile /etc/opt/quest/samba/smb.conf
    LogMainBasic "Storing any license and access.d files."
    mkdir ${OUT_DIR_FILES}/.licenses 2>/dev/null
    mkdir ${OUT_DIR_FILES}/access.d 2>/dev/null
    cp -r /etc/opt/quest/vas/.licenses/* ${OUT_DIR_FILES}/.licenses 2>/dev/null
    cp -r /etc/opt/quest/vas/access.d/* ${OUT_DIR_FILES}/.licenses 2>/dev/null
    USER_MAP_FILES=`cat /etc/opt/quest/vas/vas.conf | grep user-map-files | grep -v "^#" | awk -F= '{print $2}' | sed 's/;/ /g'`
    if [ ! -z "${USER_MAP_FILES}" ] ; then
        for file in $USER_MAP_FILES ; do
            GrabFile $file
        done
    fi    
}

GetVGPInfo()
{
    if [ ! -x /opt/quest/bin/vgptool ] ; then
        return
    fi
    LogMainBasic "*** Gathering VGP information ***"
    cp /etc/opt/quest/vgp/vgp.conf ${OUT_DIR_FILES}/ 2>/dev/null
    GetCommandOutput "${VGP} listgpc" "vgp.listgpc.out"
    GetCommandOutput "${VGP} listgpc -l" "vgp.listgpc.l.out"
    GetCommandOutput "${VGP} listgpt" "vgp.listgpt.out"
    GetCommandOutput "${VGP} listgpt -l" "vgp.listgpt.l.out"
# This grabs ALL AD GP's, can take a while, and only applies to a few set of issues.    
#    GetCommandOutput "${VGPM} listgpos" "vgp.listgpos.out"
    GetCommandOutput "${VGP} rsop" "vgp.rsop.out"
    GetCommandOutput "${VGP} -d 6 -g 9 apply" "vgp.apply.debug.out"
    GetCommandOutput "${VGP} register" "vgp.register.out"
}

GetVASYPDInfo()
{
    ps -ef | grep vasyp | grep -v grep | grep vasyp 1>/dev/null 2>&1
    if [ $? -ne 0 ] ; then
        return
    fi
    LogMainBasic "*** Gathering VASYPD information ***"
    GetCommandOutput "${SQL3} ${VASYPD_DB} \".tables\"" "vasypd.tables.dump.out"
    GetCommandOutput "${SQL3} ${VASYPD_DB} \"select * from vas2307_xlate\"" "vasypd.vas2307_xlate.dump.out"
    GetCommandOutput "${SQL3} ${VASYPD_DB} \"select * from vasypinfo\"" "vasypd.vasypinfo.dump.out"
    GetCommandOutput "ypwhich -m" "ypwhich.m.out"
}

Finish()
{
    LogMainBasic "Script is finished, information is saved in the file ${OUT_BUNDLE}.tar.gz" 
    tar -cf ${OUT_BUNDLE}.tar -C /tmp ${DATE} 2>/dev/null
    gzip ${OUT_BUNDLE}.tar 2>/dev/null
    rm -r ${OUT_DIR} 2>/dev/null
}

################################################################################
# Main
################################################################################
Prep
GetSystemBasics
GetVASBasics
GetSystemCommands
GetVASCommands
GetSystemFiles
GetVASFiles
GetVGPInfo
GetVASYPDInfo
Finish
