#!/bin/sh
################################################################################
# Copyright (c) 2008 Quest Software, Inc.  All Rights Reserved.
#
# vas_status.sh
#
# Purpose:      Test the VAS join against AD and local configuration for various
#               issues. Report style format and output, useful for status
#               checks on machines.
#
# Author(s):    Seth Ellsworth (seth.ellsworth@quest.com)
#
VSVERSION=0.3.6
#
################################################################################
#   History: 
#       0.3.6
#           * The mapped user check was failing if the file was DOS formatted.
#
#       0.3.5
#           * The mapped user check was reporting on blank lined in the mapped 
#             user file. This has been fixed ( they are now properly ignored ).
#
#       0.3.4
#           * New tests:
#               612 AIX: /etc/security/user doesn't have default registry entry
#               613 AIX: methods.cfg is ok
#           * Removed test 214, it is covered by 209/212 now.
#           * Support Redhat PAM module substack.
#
#       0.3.3
#           * PRAGMA results were not parsed correctly, fixed.
#
#       0.3.2
#           * Added a ./vas_status.sh -v option to show version.
#           * If a critical error occured, skip the VAS * enabled tests. 
#
#       0.3.1
#           * Got false positives on the new PRAGMA integrity_check because I 
#             didn't put a timeout ( locked cache errors ). Fixed. 
#           * New tests:
#               109 If installed, vgptool -v runs without segfault/errors. 
#               110 If installed, vasgpd -v runs without segfault/erros. 
#               715 If vgp is installed, vasgpd is running.
#
#       0.3.0
#           * On the dup tests, the temp file wasn't cleaned up. Fixed.
#
#       0.2.9
#           * If on VAS 3.0.3, don't run 713/714. Those rely on newer cache.
#           * Better handle test 711 when using VAS 3.0.3. 
#             ( NOTE: VAS 3.0.3 is unsupported. )
#
#       0.2.8
#           * Better path detection for test 609 on HP.
#           * When running test 611, need to call GetHostInfo first.
#
#       0.2.7
#           * Use ls -L better follow links when checking perms of files/did. 
#           * Moved mapped user and dup user/group tests to 700, not 700b. So if
#             there is a critical failure, they don't still try to run. 
#           * Moved the /tmp and /var full tests to critical so they stop the
#             other tests that might need room in /tmp to report correctly.
#           * Improved tests 209/212 with a PRAGMA command, this will catch
#             ALL vdb file issues as far as sqlite3 is concerned. 
#
#       0.2.6
#           * New tests:
#           *     712 Socket file permissions
#           *     713 duplicate users
#           *     714 duplicate groups
#
#       0.2.5 
#           * Handle test 703 on both 3.3.2 and 3.5 versions of VAS.
#           * Up the memory threshold for test 710 to 60MB across all platforms.
#
#       0.2.4 
#           * Ignore vmware-guestd pam.d errors.
#
#       0.2.3 
#           * Added test 711, where all installed binary versions are checked.
#
#       0.2.2
#           * Added help dialog when bad arguments passed in, and ignore
#             nvuauth.
#
#       0.2.1 
#           * A little speed increase, store a ccache for host/ instead of 
#             getting a TGT for each query.
#
#       0.2.0 
#           * Better memory handling for OSX. Increase size for some OSes.
#               
#       0.1.9
#           * Make SPN matching case insensitive ( since samAccountName of the
#             joined machine can be lowercase ).
#
#       0.1.8
#           * Modify critical failures to say CRITICAL instead of FAILURE.
#
#       0.1.7
#           * Include error number in output for easier automated parsing.
#
#       0.1.6
#           * Slight re-format for easier customization by individual companies.
#
#       0.1.5
#           * check for running vasd processes now faster and more precise. 
#
#       0.1.4
#           * New option: SKIP_PAM_SERVICES, if that variable is set to anything
#             then the pam service specific tests will be skipped. i.e.:
#             SKIP_PAM_SERVICES=1 ./vas_status.sh                
#
#       0.1.3
#           * Added /etc/hosts to file permission checks. 
#           * Handle links during file permission checks.
#           * Skip nss test failures if using mapped users. 
#
#       0.1.2
#           * Added 611, tmp/var space free checks
#           * Added 710, process memory size checks.
#
#       0.1.1
#           * Added 609, NSS Link tests. 
#           * Added 610, NSS Link tests. 
#
#       0.1.0
#           * Skip tests if bc is not found ( ESX servers ).
#           * New option: REPORT, print out each test as run.
#
#       0.0.9
#           * Use timeout when reading misc settings (stopping false failures).
#           * Ignore any pam.f file with a '.' in its name to ignore
#             saved/backup files. (Assuming any real service will not have a '.'
#             in its name.)
#           * Better support of Suse/Redhat pam_vas service setups. 
#
#       0.0.8
#           * Validate each pam service has pam_vas links.
#
#       0.0.7
#           * Little re-works for easier automated testing.
#
#       0.0.6
#           * Re-work a couple of back-tick/re-direction usages to eliminate 
#             AIX shell incompatability. ( 'OUT="`$VAS <<_EOF...' style usage )
#
#       0.0.5
#           * Better OS report for host ( instead of just running uname -a )
#           * Return code added: 5:NO VAS
#           * More portable use of tr.
#
#       0.0.4
#           * Cleanup reporting messages more, for easier parsing.
#           * New return code method: 0:SUCCESS 1:WARNINGS 2:FAILURES 3:CRITICAL
# 
#       0.0.3
#           * Better response when VAS is not installed.
#           * Better response when VAS is installed but not joined.
#           * Add VAS version to output.
#           * Add joined domain to output.
#           * Check users.allow/deny for permissions.
#           * Add check for the vasd ipc socket file.
#           * Cleaner response for mapped user issues.
#           * Works now on 8.11 (10.4) OSX.
#           * Tru64 testing is working now.
#           * report messages cleaned up.
#           * Reporting style more consistent.
#
#       0.0.2
#           * Re-work the file permission checks to report cleaner.
#           * /etc/nsswitch.conf missing vas, print FAILURE if not mapped.
#           * Change final output.
#           * Skip SPN/UPN tests if misc cache was missing needed elements.
#           * Still run system tests if critical VAS tests fail.
#
#       0.0.1 (2008_05_11): 
#           * Initial offering.    
#

################################################################################
#
# The vas_status.sh script tests the following situations:
# 
#    001 Nothing serious found ( can still have warnings ).
#
# * vas.conf/vastool/keytab
#    101 vastool exists.
#    102 vastool runs ( no segfault/library issues )
#    103 VAS version ( < 3.0.3.17 or so and fail to run, as we won't have vastool auth )
#    104 vas.conf exists
#    105 vas.conf is valid ( ERROR: Could not allocate VAS context, err = 2 )
#    106 Check for missing default_realm setting.
#    107 Keytab file exists
#    108 keytab is valid ( not an empty/corrupted file ( krb5_kt_start_seq_get ) )
#    109 vgptool runs ( if installed, no segfault/library issues )
#    110 vasgpd runs ( if installed, no segfault/library issues )
#
# * Local DB
#    201 misc cache exists.
#    202 ident cache exists.
#    203 misc cache is not locked.
#    204 misc cache is not soft locked.
#    205 ident cache is not locked.
#    206 ident cache is not soft locked.
#    207 Misc cache is a DB file.
#    208 Misc cache is not corrupt.
#    209 Misc cache is not otherwise broken.
#    210 Ident cache is a DB file.
#    211 Ident cache is not corrupt.
#    212 Ident cache is not otherwise broken.
#    213 All needed tables exist. ( No missing tables, careful here, given version differences ).
#    (REMOVED)214 No tables are corrupt. 
#    215 machine has found a site, or has an override site set.
#    216 Misc table has defaultRealm 
#    217 Misc table has computerfqdn
#    218 Misc table has computername
#    219 Misc table has forestRoot
#
# * kinit results 
#    301 Machine can talk to AD ( KDC_UNREACH )
#    302 Object exists in AD. ( client unknown )
#    303 Machine timesync. ( TIME_SKEW ) 
#    304 The host.keytab knows the right password. ( preauth failed ).
#    305 The host/ account is usable ( CREDENTIALS REVOKED )
#    306 DES is set, but no entries ( KRB5KDC_ERR_PREAUTH_REQUIRED )
#    307 ( Misc kinit error, not one of the above known ones ).
# 
# * Computer account checks.
#    401 Computer can read AD object. 
#    402 Computer account has valid UPN.
#    403 Computer account userAccountControl has VAS_UF_PASSWD_NOTREQD
#    404 Computer account userAccountControl has VAS_UF_WORKSTATION_TRUST_ACCOUNT
#    405 Computer account userAccountControl doesn't have ADS_UF_USE_DES_KEY_ONLY
#    406 Computer account userAccountControl doesn't have ADS_UF_DONT_REQUIRE_PREAUTH
#
# * vastool auth/spn.
#    501 Computer account has valid SPNs.
#    502 host.keytab containes matching entries for each SPN in AD.
#    503 SPNs in AD match each entry in host.keytab.
#    504 auth -S host/$FQDN returns KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN, duplicate.
#    505 auth -S host/$FQDN fails.
#
# * Final local stuff.
#    601 File/directory permissions. Not enough.
#    602 File/directory permissions. Too much.
#    603 /etc/nsswitch.conf contains vas3. ( check for map entries ). 
#    604 Relevent PAM configuraiton file contains at least one pam_vas3 line.
#    605 AIX: IRS files have VAS configured.
#    606 System file permissions. Not enough.
#    607 System file permissions. Too much.
#    608 Each pam.conf section has pam_vas enabled.
#    609 nss links exist. 
#    610 pam links exist.
#    611 var and tmp directories are not full
#    612 AIX: /etc/security/user doesn't have default registry entry
#    613 AIX: methods.cfg is ok
#
# * And whatever else.
#    701 Valid license 1 ( expired )
#    702 Valid license 2 ( no license )
#    703 Valid license 3 ( no directory )
#    (not run)704 Valid license 4 ( invalid file )
#    705 vasd running at all
#    706 vasd running one process
#    707 vasd responding
#    708 Mapped user consistency
#    709 vasd ipc socket file missing
#    710 process memory size
#    711 All binaries are same version
#    712 Socket file permissions
#    713 duplicate users
#    714 duplicate groups
#    715 If vgp is installed, vasgpd is running.
# 
################################################################################

if [ -z "$DEBUG" ] ; then
    DEBUG=false
else
    DEBUG=true
fi

$DEBUG && set -x

# Shhh... look away. This is a testing nomenclature, when called
# with the right option in addition to normal printout, print out
# an arbitraty but unique number, so a testing script can make sure
# the right response was given for a particular situation.

# Now the number is in the normal output too, but testing is already written
# around this usage, so intentionally keeping it.
if [ "x$1" = "xNUMBER" -o "x$2" = "xNUMBER" ] ; then
    NUM=true
else
    NUM=false
fi
                
if [ "x$1" = "xREPORT" -o "x$2" = "xREPORT" ] ; then
    REPORT=true
else
    REPORT=false
fi

if [ ! -z "$1" ] ; then
    case "$1" in 
        REPORT)
            ;;
        NUMBER)
            ;;
        -v)
            echo "vas_status: Version $VSVERSION"
            exit 0
            ;;
        *)
            echo "usage: ./vas_status.sh [REPORT | -v]"
            echo ""
            echo "REPORT:    Print success as each test passes"
            echo "-v:        Print version and exit"
            exit 1
            ;;
    esac
fi

VAS=/opt/quest/bin/vastool
VGP=/opt/quest/bin/vgptool
VGPD=/opt/quest/sbin/vasgpd
SQL=/opt/quest/libexec/vas/sqlite3
SQL3="$SQL -noheader -list -separator '|'"
VASIPC=/var/opt/quest/vas/vasd/.vasd_ipc_sock
IDENTDB=/var/opt/quest/vas/vasd/vas_ident.vdb
MISCDB=/var/opt/quest/vas/vasd/vas_misc.vdb
VASCONF=/etc/opt/quest/vas/vas.conf
KEYTAB=/etc/opt/quest/vas/host.keytab
UALLOW=/etc/opt/quest/vas/users.allow
UDENY=/etc/opt/quest/vas/users.deny
FAILURE=false
WARNING=false
CRITICAL_FAILURE=false
ADFAILURE=false
STARTTIME=
MappedFile=
fOUT=/tmp/_tmp.vs.$$
DBCMD=/tmp/_tmp.db.$$
CLEANUP_FILES="$fOUT $DBCMD"
HAVE_MISC=1
HAVE_IDENT=1
PLATFORM=UNKNOWN
PAM_MASK="\\.|system-auth-ac|common-*-ac|nvuauth|vmware-guestd"
EXIT_VALUE=3
VASVERMAJOR=
VASVERMINOR=
VASVERREVISION=
VASVERBUILD=

UPPER=ABCDEFGHIJKLMNOPQRSTUVWXYZ
LOWER=abcdefghijklmnopqrstuvwxyz

pam_conf ()
{
# Use this for pam.conf version:
    if [ -f /etc/pam.conf ] ; then
        vTMPFILE1=/tmp/_vas_pc_test.1.$$
        vTMPFILE2=/tmp/_vas_pc_test.2.$$
        cat /etc/pam.conf | sed -e 's/#.*//' -e '/^[ \t]*$/d' | awk '{print $1 "><" $2}' | sort | uniq > $vTMPFILE1
        cat /etc/pam.conf | sed -e 's/#.*//' -e '/^[ \t]*$/d' | grep pam_vas | awk '{print $1 "><" $2}' | sort | uniq > $vTMPFILE2
        PCFAILURES="`comm -23 $vTMPFILE1 $vTMPFILE2 | grep -v nvuauth`"
        rm -f $vTMPFILE1 $vTMPFILE2
        if [ ! -z "$PCFAILURES" ] ; then
            for service in $PCFAILURES ; do
                printf "FAILURE: 608 Pam <$service> not configured for VAS.\n"
            done        
            FAILUTE=true
            return 1
        fi
    fi
}

pam_redhat_modules ()
{
# $1 file
    F="~`sed 's/#.*//' < /etc/pam.d/$1 | tr '\n' '~'`~"
    for m in account auth password session ; do
        # Has service=system-auth, pam_vas, pam_permit, or pam_rhosts.
        # Any service that could return success and is ok with VAS.
        echo "$F" | egrep "~$m[^~]*(service=system-auth|include|substack|pam_vas|pam_permit|pam_rhosts)" >/dev/null
#        sed 's/#.*//' < /etc/pam.d/$1 | egrep "^$m.*service=system-auth|^$m.*pam_vas|^$m.*pam_permit|^$m.*pam_rhosts" >/dev/null
        if [ $? -ne 0 ] ; then
            # Silly exception cases. Not certain why this module needs pam_unix for session.
            if [ "$1" = "runuser" -a "$m" = "session" ] ; then
                return
            fi        
            # Any service that could return failure explicitly or does nothing, regardless of VAS.
            echo "$F" | sed -e "s/~$m[^~]*pam_warn[^~]*~/~/" -e "s/~$m[^~]*pam_deny[^~]*~/~/" -e "s/~$m[^~]*pam_limits[^~]*~/~/" -e "s/~$m[^~]*pam_console[^~]*~/~/" -e "s/~$m[^~]*pam_loginuid[^~]*~/~/" -e "s/~$m[^~]*pam_rootok[^~]*~/~/" -e "s/~$m[^~]*pam_keyinit[^~]*~/~/" -e "s/~$m[^~]*pam_namespace[^~]*~/~/" -e "s/~$m[^~]*pam_xauth[^~]*~/~/" -e "s/~$m[^~]*pam_timestamp[^~]*~/~/"| grep "~$m[^~]*~" >/dev/null  
            if [ $? -eq 0 ] ; then
                printf "FAILURE: 608 Pam <$1><$m> not configured for VAS.\n"
                PAM_FAILED=1
            fi    
        fi    
    done    
}

pam_redhat()
{
# Use this for /etc/pam.d/system-auth setup.
    PAM_FAILED=0
    CWD="`pwd`"
    cd /etc/pam.d    
    for file in `ls | egrep -v "$PAM_MASK"` ; do
        pam_redhat_modules $file
    done
    cd "$CWD"
    return $PAM_FAILED
}

pam_suse_modules ()
{
# $1 file
    for m in account auth password session ; do
        # Has include common-<module>
        sed 's/#.*//' < /etc/pam.d/$1 | egrep "^($m|@).*include.*common-$m.*" >/dev/null
        if [ $? -ne 0 ] ; then
            sed 's/#.*//' < /etc/pam.d/$1 | egrep "^$m.*pam_(permit|rootok|vas).*" >/dev/null  
            if [ $? -ne 0 ] ; then
                # The idea is to catch any module that has a line other then warn/deny, meaning
                # it is still possible to get in without VAS.
                sed 's/#.*//' < /etc/pam.d/$1 | grep "^$m" | grep -v pam_warn | grep -v pam_deny | grep "[a-zA-Z]" >/dev/null  
                if [ $? -eq 0 ] ; then
                    printf "FAILURE: 608 Pam <$1><$m> not configured for VAS.\n"
                    PAM_FAILED=1
                fi    
            fi    
        fi    
    done    
}

pam_suse()
{
# Use this for /etc/pam.d/common-* setup.
    PAM_FAILED=0
    CWD="`pwd`"
    cd /etc/pam.d    
    for file in `ls | egrep -v "$PAM_MASK"` ; do
        pam_suse_modules $file
    done
    cd "$CWD"
    return $PAM_FAILED
}

# don't use this for now, but keeping in case we go this route. 
# Run time 1.2s went to 1.8 with pam_suse, this was an attempt 
# to run faster. As-is it runs in 1.6, and doesn't handle the 'other'
# case. Its not worth it to me to continue down this path.
pam_suse2()
{
# Use this for /etc/pam.d/common-* setup.
    CWD="`pwd`"
    vTMPFILE1=/tmp/_vas_pc_test.1.$$
    vTMPFILE2=/tmp/_vas_pc_test.2.$$
    rm -rf $vTMPFILE1
    cd /etc/pam.d    
    for file in * ; do
        sed -e 's/#.*//' -e 's/^[ \t]*$//' -e '/^$/d' < $file | awk "{printf \"$file~\" \$1 \"\\n\"}" | sort | uniq >> $vTMPFILE1
        sed -e 's/#.*//' -e 's/^[ \t]*$//' -e '/^$/d' < $file | egrep "pam_vas|include|pam_permit" | awk "{printf \"$file~\" \$1 \"\\n\"}" | sort | uniq >> $vTMPFILE2
    done
    PCFAILURES="`comm -23 $vTMPFILE1 $vTMPFILE2 | grep -v nvauth`"
    for service in $PCFAILURES ; do
        printf "FAILURE: Pam 608 <$service> not configured for VAS.\n"
        FAILUTE=true
    done        
#    vTMPFILE2=/tmp/_vas_pc_test.2.$$
#    sed -e 's/#.*//' -e '/^[ \t]*$/d' < $vTMPFILE | awk '{print $1 "><" $2}' | sort | uniq > $vTMPFILE1
#    cat /etc/pam.conf | sed -e 's/#.*//' -e '/^[ \t]*$/d' | grep pam_vas | awk '{print $1 "><" $2}' | sort | uniq > $vTMPFILE2
#    rm -rf $vTMPFILE1 $vTMPFILE2 $vTMPFILE3 $vTMPFILE4
    echo $vTMPFILE1 $vTMPFILE2 $vTMPFILE3 $vTMPFILE4
    cd "$CWD"
}

nss_link_tests ()
{
    NSS_PATHS=
    MISSING_NSS_PATHS=0
    case $PLATFORM in 
        LINUX_S390X)
            NSS_PATHS="/lib64/libnss_vas3.so.2"
            ;;
        LINUX*)
            if [ -d /lib ] ; then
                NSS_PATHS="/lib/libnss_vas3.so.2"
            fi        
            if [ -d /lib32 ] ; then
                NSS_PATHS="$NSS_PATHS /lib32/libnss_vas3.so.2"
            fi        
            if [ -d /lib64 ] ; then
                NSS_PATHS="$NSS_PATHS /lib64/libnss_vas3.so.2"
            fi
            ;;
        SOLARIS*)
            if [ -d /lib/amd64 ] ; then
                NSS_PATHS="$NSS_PATHS /lib/amd64/nss_vas3.so.1 /lib/nss_vas3.so.1"
            elif [ -d /usr/lib ] ; then
                NSS_PATHS="$NSS_PATHS /usr/lib/nss_vas3.so.1"
            fi
            if [ -d /usr/lib/sparcv9 ] ; then
                NSS_PATHS="$NSS_PATHS /usr/lib/sparcv9/nss_vas3.so.1"
            fi
            ;;
        HPUX_IA64)
            NSS_PATHS="/usr/lib/libnss_vas3.1 /usr/lib/hpux32/libnss_vas3.so.1 /usr/lib/hpux64/libnss_vas3.so.1"
            ;;
        HPUX_9000)
            NSS_PATHS="/usr/lib/libnss_vas3.1 /usr/lib/pa20_64/libnss_vas3.1"
            ;;
        AIX_4_3)
            grep "netgroup.*vas" /etc/irs.conf >/dev/null 2>&1
            if [ $? -eq 0 ] ; then
                NSS_PATHS="/usr/lib/netsvc/dynload/vas.so"
            fi
            ;;
        AIX*)
            grep "netgroup.*vas" /etc/irs.conf >/dev/null 2>&1
            if [ $? -eq 0 ] ; then
                NSS_PATHS="/usr/lib/netsvc/dynload/vas.so /usr/lib/netsvc/dynload/vas_64.so"
            fi
            ;;
        IRIX*)
            NSS_PATHS="/var/ns/lib/libns_vas3.so"
            ;;
    esac

    if [ -z "$MappedFile" ] ; then
        MSG="FAILURE: 609"
    else
        MSG="INFO: 609"
    fi

    if [ ! -z "$NSS_PATHS" ] ; then
        for p in $NSS_PATHS ; do
            ls $p >/dev/null 2>&1
            if [ $? -ne 0 ] ; then
                echo "$MSG Missing expected NSS path <$p>"
                MISSING_NSS_PATHS=1
            fi        
        done
    fi

    if [ $MISSING_NSS_PATHS -eq 0 ] ; then
        return 0
    else
        return 1
    fi        
}

pam_link_tests ()
{
    PAM_PATHS=
    MISSING_PAM_PATHS=0
    case $PLATFORM in 
        LINUX_S390X)
            PAM_PATHS="/lib64/security/pam_vas3.so"
            ;;
        LINUX*)
            if [ -d /lib ] ; then
                PAM_PATHS="/lib/security/pam_vas3.so"
            fi        
            if [ -d /lib32 ] ; then
                PAM_PATHS="/lib32/security/pam_vas3.so"
            fi        
            if [ -d /lib64 ] ; then
                PAM_PATHS="/lib64/security/pam_vas3.so"
            fi        
            ;;
        SOLARIS*)
            if [ -d /lib/security/amd64 ] ; then
                PAM_PATHS="$PAM_PATHS /lib/security/amd64/pam_vas3.so /lib/security/pam_vas3.so"
            elif [ -d /usr/lib/security ] ; then
                PAM_PATHS="$PAM_PATHS /usr/lib/security/pam_vas3.so"
            fi
            if [ -d /usr/lib/security/sparcv9 ] ; then
                PAM_PATHS="$PAM_PATHS /usr/lib/security/sparcv9/pam_vas3.so"
            fi
            ;;
        HPUX*)
            if [ -d /usr/lib/security ] ; then
                PAM_PATHS="/usr/lib/security/libpam_vas3.1"
            fi        
            if [ -d /usr/lib/security/hpux32 ] ; then
                PAM_PATHS="$PAM_PATHS /usr/lib/security/hpux32/libpam_vas3.so.1"
            fi        
            if [ -d /usr/lib/security/hpux64 ] ; then
                PAM_PATHS="$PAM_PATHS /usr/lib/security/hpux64/libpam_vas3.so.1"
            elif [ -d /usr/lib/security/pa20_64 ] ; then
                if [ `uname -r | cut -d. -f3` -gt 11 ] ; then
                    # Just info, because this depends on package used. 
                    ls /usr/lib/security/pa20_64/libpam_vas3.1 >/dev/null 2>&1
                    if [ $? -ne 0 ] ; then
                        echo "FAILURE: Missing PAM path <$p>"
                    fi
                fi        
            fi        
            ;;
        AIX*)
            if [ -f /etc/pam.conf ] ; then 
                PAM_PATHS="/usr/lib/security/pam_vas3.so /usr/lib/security/pam_vas364.so"
                if [  ${VASVERMAJOR}${VASVERMINOR} -ge 35 ] ; then
                    PAM_PATHS="/usr/lib/security/pam_vas3.so /usr/lib/security/64/pam_vas3.so"
                fi
                
            fi        
            ;;
    esac

    if [ ! -z "$PAM_PATHS" ] ; then
        for p in $PAM_PATHS ; do
            ls $p >/dev/null 2>&1
            if [ $? -ne 0 ] ; then
                echo "FAILURE: 610 Missing PAM path <$p>"
                MISSING_PAM_PATHS=1
            fi        
        done
    fi

    if [ $MISSING_PAM_PATHS -eq 0 ] ; then
        return 0
    else
        return 1
    fi        
}

dir_full_tests ()
{
    case $PLATFORM in
        LINUX*)
            TMPUSED="`df -k /tmp | grep '[0-9]%' | sed 's/.* \([0-9]*\)% .*/\1/' `"
            VARUSED="`df -k /var/opt/quest/vas | grep '[0-9]%' | sed 's/.* \([0-9]*\)% .*/\1/' `"
        ;;
        SOLARIS*)
            TMPUSED="`df -k /tmp | grep '[0-9]%' | sed 's/.* \([0-9]*\)% .*/\1/' `"
            VARUSED="`df -k /var/opt/quest/vas | grep '[0-9]%' | sed 's/.* \([0-9]*\)% .*/\1/' `"
        ;;
        HPUX*)
            TMPUSED="`df -k /tmp | grep '%' | awk '{print $1}'`"
            VARUSED="`df -k /var/opt/quest/vas | grep '%' | awk '{print $1}'`"
        ;;
        AIX*)
            TMPUSED="`df -k /tmp | grep '^/' | awk '{print $4}' | sed 's/%$//' `"
            VARUSED="`df -k /var/opt/quest/vas | grep '^/' | awk '{print $4}' | sed 's/%$//' `"
        ;;
        OSX*)
            TMPUSED="`df -k /tmp | grep '^/' | awk '{print $5}' | sed 's/%$//' `"
            VARUSED="`df -k /var/opt/quest/vas | grep '^/' | awk '{print $5}' | sed 's/%$//' `"
        ;;
        IRIX*)
            TMPUSED="`df -k /tmp | grep '^/' | awk '{print $6}'`"
            VARUSED="`df -k /var/opt/quest/vas | grep '^/' | awk '{print $6}'`"
        ;;
        TRU64*)
            TMPUSED="`df -k /tmp | grep '^/' | awk '{print $5}' | sed 's/%$//' `"
            VARUSED="`df -k /var/opt/quest/vas | grep '^/' | awk '{print $5}' | sed 's/%$//' `"
        ;;
    esac

    RVAL=0

    if [ -d /tmp -a ! -z "$TMPUSED" ] ; then
        if [ "x$TMPUSED" = "x100" ] ; then
            echo "CRITICAL: 611 /tmp is full"
            CRITICAL_FAILURE=true
            FAILURE=true
            RVAL=2
        elif [ "x$TMPUSED" = "x98" -o "x$TMPUSED" = "x99" ] ; then
            echo "WARNING: 611 /tmp is almost full"
            WARNING=true
            RVAL=1
        fi
    fi    
                
    if [ -d /var/opt/quest/vas -a ! -z "$VARUSED" ] ; then
        if [ "x$VARUSED" = "x100" ] ; then
            echo "CRITICAL: 611 /var/opt/quest/vas is full"
            CRITICAL_FAILURE=true
            FAILURE=true
            RVAL=2
        elif [ "x$VARUSED" = "x98" -o "x$VARUSED" = "x99" ] ; then
            echo "WARNING: 611 /var/opt/quest/vas is almost full"
            WARNING=true
            RVAL=1
        fi
    fi    
#    printf "TMP<%2s> VAR<%2s> Platform <$PLATFORM>\n" "$TMPUSED" "$VARUSED"


    return $RVAL
}

mem_tests ()
{
    SZL=65000
    PROCESSES=""
    case $PLATFORM in
        AIX*)
            PROCESSES="`ps -eo vsz,pid,comm | grep \"[ /][v]as[gproxy]*d\" | awk '{print $1 \"~\" $2 \"~\" $3}'`"
        ;;
        LINUX_X64)
            PROCESSES="`ps -eo vsz,pid,comm | grep \"[ /][v]as[gproxy]*d\" | awk '{print $1 \"~\" $2 \"~\" $3}'`"
        ;;
        LINUX_IA64)
            PROCESSES="`ps -eo vsz,pid,comm | grep \"[ /][v]as[gproxy]*d\" | awk '{print $1 \"~\" $2 \"~\" $3}'`"
        ;;
        IRIX*)
            PROCESSES="`ps -eo vsz,pid,comm | grep \"[ /][v]as[gproxy]*d\" | awk '{print $1 \"~\" $2 \"~\" $3}'`"
        ;;
        HPUX_*)
            F=/tmp/_vs_$$
            COUNT="`UNIX95= ps -eo pid | wc -l`"
            top -d1 -s1 -n`expr $COUNT + 10` -q -u -f $F
            grep "^CPU" $F >/dev/null
            if [ $? -eq 0 ] ; then
                PROCESSES="`cat $F | grep \"[ /][v]as[gproxy]*d\" | tr -d 'KM' | awk '{print $7 \"~\" $3 \"~\" $13 }'`"
            else
                PROCESSES="`cat $F | grep \"[ /][v]as[gproxy]*d\" | tr -d 'KM' | awk '{print $6 \"~\" $2 \"~\" $12 }'`"
            fi
            rm -f $F
        ;;
        OSX*)
            PROCESSES="`ps -eAo vsz,pid,ucomm | grep \"[ /.][v]as[gproxy]*d\" | awk '{print $1 \"~\" $2 \"~\" $3}'`"
        ;;
        TRU*)
            PROCESSES="`ps -eo vsz,pid,comm | grep \"[ /][v]as[gproxy]*d\" | sed -e 's/^\([0-9]*\)\./\1/' -e 's/\([0-9]*\)M/\10/' -e 's/^\([0-9]*\)G/\1000000/' -e 's/^\([0-9]*\)K/\1/' | awk '{print $1 \"~\" $2 \"~\" $3}'`"
        ;;
        *)
            PROCESSES="`ps -eo vsz,pid,comm | grep \"[ /][v]as[gproxy]*d\" | awk '{print $1 \"~\" $2 \"~\" $3}'`"
        ;;
    esac

    RVAL=0
    for proc in $PROCESSES ; do
        SZ="`echo $proc | cut -d~ -f1`"
        PD="`echo $proc | cut -d~ -f2`"
        PN="`echo $proc | cut -d~ -f3`"
        if [ $SZ -ge $SZL ] ; then
            echo "WARNING: 710 Process <$PD><$PN> is too large <$SZ>"
            WARNING=true
            RVAL=1
        fi    
    done

    return $RVAL
}

get_time ()
{
    date +"%Y_%m_%d %H:%M:%S"
}

get_vasconf_setting ()
{
# $1 section
# $2 setting
# Returns: value set, nothing if nothing is set

    $DEBUG && set -x

    SECTION="$1"
    SETTING="$2"

    if [ -z "$1" -o -z "$2" ] ; then
        return
    fi        

    SETTING="`sed -e '/^[ \t]*#/d' -e 's/#.*//' -e 's/^[ \t]*//' < $VASCONF | grep \"^$2 \" | head -1`"
    RESULT="`echo $SETTING | sed 's/.* = \(.*\)[ \t]*/\1/'`"
    if [ ! -z "$RESULT" ] ; then echo "$RESULT" ; fi
}

VerifyRoot ()
{
    $DEBUG && set -x
# Test for root access.
    if [ "`id | sed 's/uid=\([0-9]*\).*/\1/'`" -ne 0 ] ; then
        echo "Must be root to run."
        exit 1
    fi
}

# Portable way of computing a second count for elapsed time calculations.
# Very basic, doesn't handle day change well, but for whts needed this will do.
GetTime ()
{
    $DEBUG && set -x
    DATE="`date +%H:%M:%S`"

    HOUR="`echo $DATE | cut -d: -f1`"
    if [ "$HOUR" = "00" ] ; then
         HOUR=24
    fi
    MIN="`echo $DATE | cut -d: -f2`"
    SEC="`echo $DATE | cut -d: -f3`"

    HOUR="`expr $HOUR \* 3600`"
    MIN="`expr $MIN \* 60`"
    SEC="`expr $MIN + $SEC`"
    SEC="`expr $HOUR + $SEC`"
    echo $SEC
}

GetTimeDiff ()
{
    $DEBUG && set -x
    LOCAL_TIME_NOW="`GetTime`"
    SEC="`expr $LOCAL_TIME_NOW - $1`"
    if [ $SEC -le 9 ] ; then
        echo 0$SEC
    else
        echo $SEC
    fi        
}

check_dups ()
{
    $DEBUG && set -x
    vDBFILE=/tmp/_vas_dup_test.DB.$$

    if [ ! "$1" = "user" -a ! "$1" = "group" ] ; then
        printf "Called with $1, not user/group"
        return 1
    fi    

    rm -rf $vDBFILE
    echo ".timeout 5000" >> $vDBFILE
    echo "select 1 from $1_ad where guid is not null group by guid having count(*) > 1;" >> $vDBFILE
    echo ".q" >> $vDBFILE
    COUNT="`echo | $SQL -init $vDBFILE $IDENTDB | wc -l`"
    rm -rf $vDBFILE

    if [ $COUNT -gt 0 ] ; then 
        return 1
    fi
    return 0
}

validate_db ()
{
    $DEBUG && set -x
    vDBFILE=/tmp/_vas_mu_test.DB.$$
    vTMPFILE1=/tmp/_vas_mu_test.1.$$
    vTMPFILE2=/tmp/_vas_mu_test.2.$$
    vMAPPEDFILE=/tmp/_vas_mu_test.map.$$

    rm -rf $vTMPFILE1 $vTMPFILE2 $vDBFILE $vMAPPEDFILE

    echo ".timeout 5000" >> $vDBFILE
    echo "BEGIN EXCLUSIVE TRANSACTION;" >> $vDBFILE
    echo "select value from misc where key='userMappings';" >> $vDBFILE
    echo "END TRANSACTION;" >> $vDBFILE
    echo ".q" >> $vDBFILE
    FILES="`echo | $SQL3 -init $vDBFILE $MISCDB`"


    FILES="`echo $FILES | tr ';' ' '`"
    LOCALCOUNT=0
    TMPCOUNT=0
    DID_ONE=0
    for file in $FILES ; do
        if [ -f $file ] ; then
            cat $file 2>/dev/null | tr -d '\r' | grep -v "^[ \t]*$" > $vTMPFILE1
            vTMPCOUNT=`wc -l < $file`
            LOCALCOUNT=`expr $LOCALCOUNT + $vTMPCOUNT`
            DID_ONE=1
        fi
    done

    if [ "$DID_ONE" = "0" ] ; then
        printf "FAILURE: 708 No mapped files existed: <$FILES>.\n"
        FAILURE=true
        rm -rf $vTMPFILE1 $vTMPFILE2 $vDBFILE $vMAPPEDFILE
        return 1
    fi

    sort < $vTMPFILE1 > $vTMPFILE2
    uniq < $vTMPFILE2 > $vMAPPEDFILE

    echo | $SQL3 $IDENTDB >$vTMPFILE1 2>&1 <<_EOF
.timeout 5000
.separator :
select loginname, userprincipalname from mapped_user;
.q
_EOF
    DBCOUNT=`wc -l < $vTMPFILE1`
    sort < $vTMPFILE1 > $vTMPFILE2
    uniq < $vTMPFILE2 > $vDBFILE

    comm -23 $vMAPPEDFILE $vDBFILE | awk '{print $1}' > $vTMPFILE1
    UNIQTOFILECOUNT="`wc -l < $vTMPFILE1`"

    comm -13 $vMAPPEDFILE $vDBFILE | awk '{print $1}' > $vTMPFILE2
    UNIQTODBCOUNT="`wc -l < $vTMPFILE2`"

    rm -rf $vTMPFILE1 $vTMPFILE2 $vDBFILE $vMAPPEDFILE

    RVAL=`expr $UNIQTOFILECOUNT + $UNIQTODBCOUNT`

    if [  $LOCALCOUNT -eq 0 ] ; then
        printf "WARNING: 708 Local files <$FILES> empty of valid entries.\n"
        WARNING=true
        return 1
    fi

    if [ $RVAL -ne 0 ] ; then
        printf "FAILURE: 708 In-consistent mapped user cache.\n"
        FAILURE=true
    fi
    return $RVAL
}

get_setting ()
{
    echo ".timeout 5000" > $DBCMD 2>/dev/null
    echo "select value from misc where key='$1';">> $DBCMD 2>/dev/null
    echo ".q" >> $DBCMD 2>/dev/null
    echo | $SQL3 -init $DBCMD $MISCDB 2>&1
}

R ()
{
    $REPORT && printf "SUCCESS: $1\n"
}

GetHostInfo ()
{
    case `uname -s` in
        Linux)
            uname -s -m
            case `uname -m` in
                ia64)
                    PLATFORM="LINUX_IA64"
                    ;;
                x86_64)
                    PLATFORM="LINUX_X64"
                    ;;
                s390)
                    PLATFORM="LINUX_S390"
                    ;;
                s390x)
                    PLATFORM="LINUX_S390X"
                    ;;
                *)
                    PLATFORM="LINUX"
                    ;;
            esac
            ;;
        SunOS)
            uname -s -r -p
            PLATFORM="SOLARIS_`uname -r|sed 's/[0-9]*\.\([0-9]*\)/\1/'`"
            uname -p | grep sparc >/dev/null 2>&1
            if [ "$?" -eq 0 ] ; then
                PLATFORM="${PLATFORM}_SPARC"
            else
                if [ -d "/usr/lib/64" ] ; then
                    PLATFORM="${PLATFORM}_X64"
                else
                    PLATFORM="${PLATFORM}_X86"
                fi
            fi
            ;;
        AIX)
            printf "AIX " 
            oslevel -r
            PLATFORM="AIX_`uname -v`_`uname -r`"
            ;;
        HP-UX)
            uname -m | grep "ia64" >/dev/null 2>&1
            if [ "$?" -eq 0 ] ; then
                PLATFORM="HPUX_IA64"
            else
                PLATFORM="HPUX_9000"
            fi
            uname -s -r -m
            ;;
        Darwin)
            if [ `uname -r|cut -d. -f1` -eq 8 ] ; then
                printf "OSX 14_`uname -r | cut -d. -f2`"  
                PLATFORM="OSX_14_`uname -r | cut -d. -f2`"  
            else
                printf "OSX 15_`uname -r | cut -d. -f2`"  
                PLATFORM="OSX_15_`uname -r | cut -d. -f2`"  
            fi
            ;;
        IRIX*)
            uname -s -r
            PLATFORM="`uname -s`_`uname -r | tr '.' '_'`"
            ;;
        OSF1)
            printf "Tru64 " 
            uname -r
            PLATFORM="TRU64_`uname -r | tr '.' '_'`"
            ;;
        *)
            uname -a
            ;;
    esac
}

t000 ()
{
    $DEBUG && set -x
    printf "Host:   <`hostname`, `GetHostInfo`>\n"
    printf "Date:   <`date`>\n"
    D="N/A"
    NOVAS=false
    if [ -f $VAS ] ; then
        TVASVERSION="`$VAS -v | grep -i version | head -1 | sed 's/vastool: .AS Version \([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)/\1.\2.\3.\4/'`"
        printf "VAS:    <$TVASVERSION>\n"
        if [ ! -f $MISCDB ] ; then
            HAVE_MISC=0
        fi
        if [ ! -f $IDENTDB ] ; then
            HAVE_IDENT=0
        fi
        if [ "$NUM" = "false" ] ; then
            D="`$VAS info domain 2>/dev/null`"
        fi        
        if [ -z "$D" ] ; then 
            D="N/A"
        fi
    else
#   101 vastool exists.
        printf "VAS:    <101 No Binary>\n"
        $NUM && printf "NUMBER:101\n"
        if [ ! -f $SQL ] ; then
            NOVAS=true
            if [ -f $VASCONF -o -f $IDENTDB ] ; then
                printf "INFO: 101 VAS related files on machine\n"
            fi        
        fi        
    fi
    printf "Domain: <$D>\n"
    STARTTIME="`GetTime`"
    $NOVAS && return 1
    R "101 $VAS exists"
    return 0
}

t100 ()
{
    $DEBUG && set -x

#   102 vastool runs ( no segfault/library issues )
    OUT="`$VAS -v 2>&1`"
    echo "$OUT" | grep -i copyright >/dev/null
    if [ $? -ne 0 ] ; then
        $NUM && printf "NUMBER:102\n"
        printf "CRITICAL: 102 $VAS -v failed <$OUT>.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    fi
    R "102 $VAS runs"

#    103 VAS version ( < 3.0.3.17 or so and fail to run, as we won't have vastool auth )
    VASVERSION="`$VAS -v | grep -i version | head -1 | sed 's/vastool: .AS Version \([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)/\1 \2 \3 \4/'`"
    VASVERMAJOR="`echo $VASVERSION | awk '{print $1}'`"
    VASVERMINOR="`echo $VASVERSION | awk '{print $2}'`"
    VASVERREVISION="`echo $VASVERSION | awk '{print $3}'`"
    VASVERBUILD="`echo $VASVERSION | awk '{print $4}'`"
    if [ "$VASVERMAJOR" -eq "3" -a "$VASVERMINOR" = "0" -a -n "$VASVERREVISION" -a $VASVERREVISION -lt 3 ] ; then
        $NUM && printf "NUMBER:103\n"
        printf "FAILURE: 103 VAS version <$VASVERSION> too low\n"
        FAILURE=true
    else
        R "103 usable VAS Version"
    fi

#    104 vas.conf exists
    if [ ! -f $VASCONF ] ; then
        $NUM && printf "NUMBER:104\n"
        printf "CRITICAL: 104 No $VASCONF, is VAS joined?\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "104 $VASCONF exists"
    fi

#    105 vas.conf is valid ( ERROR: Could not allocate VAS context, err = 2 )
    $VAS ktutil list | grep krb5_init_context >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:105\n"
        printf "CRITICAL: 105 $VASCONF is an invalid file or misconfigured.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "105 $VASCONF valid format"
    fi

#    106 Check for missing default_realm setting.
    REALM="`get_vasconf_setting libdefaults default_realm`"
    if [ -z "$REALM" ] ; then
        $NUM && printf "NUMBER:106\n"
        printf "CRITICAL: 106 $VASCONF missing default_realm setting\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "106 $VASCONF default_realm"
    fi

#    107 Keytab file exists
    if [ ! -f $KEYTAB ] ; then
        $NUM && printf "NUMBER:107\n"
        printf "CRITICAL: 107 $KEYTAB does not exist.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "107 $KEYTAB exists"
    fi

#    108 keytab is valid ( not an empty/corrupted file ( krb5_kt_start_seq_get ) )
    $VAS ktutil list | grep krb5_kt_start_seq_get >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:108\n"
        printf "CRITICAL: 108 $KEYTAB is invalid\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "108 $KEYTAB not empty/corrupt"
    fi
#    109 vgptool runs ( if installed, no segfault/library issues )
    if [ -f $VGP ] ; then
        OUT="`$VGP -v 2>&1 | tr '\n' ' '`"
        echo "$OUT" | grep -i copyright >/dev/null
        if [ $? -ne 0 ] ; then
            $NUM && printf "NUMBER:109\n"
            printf "CRITICAL: 109 $VGP -v failed, OUTPUT: <$OUT>\n"
            FAILURE=true
            CRITICAL_FAILURE=true
            return 1
        fi
    fi
    R "109 $VGP runs"
    
#    110 vasgpd runs ( if installed, no segfault/library issues )
    if [ -f $VGPD ] ; then
        OUT="`$VGPD -v 2>&1 | tr '\n' ' '`"
        echo "$OUT" | grep -i copyright >/dev/null
        if [ $? -ne 0 ] ; then
            $NUM && printf "NUMBER:110\n"
            printf "CRITICAL: 110 $VGPD -v failed, OUTPUT: <$OUT>\n"
            FAILURE=true
            CRITICAL_FAILURE=true
            return 1
        fi
    fi
    R "110 $VGPD runs"

#   611 var and tmp directories are not full
    GetHostInfo >/dev/null
    dir_full_tests
    RVAL=$?
    if [ $RVAL -ne 0 ] ; then
        $NUM && printf "NUMBER:611\n"
        if [ $RVAL -eq 2 ] ; then
            CRITICAL_FAILURE=true
            return 1
        fi
    else
        R "611 tmp/var directories ok"
    fi


return 0
}

t200 ()
{
    $DEBUG && set -x
# * Local DB
#    201 misc cache exists.
    if [ ! -f $MISCDB -o "$HAVE_MISC" = "0" ] ; then
        $NUM && printf "NUMBER:201\n"
        printf "CRITICAL: 201 $MISCDB does not exist.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "201 $MISCDB exists"
    fi

#    202 ident cache exists.
    if [ ! -f $IDENTDB -o "$HAVE_IDENT" = "0" ] ; then
        $NUM && printf "NUMBER:202\n"
        printf "CRITICAL: 202 $IDENTDB does not exist.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "202 $IDENTDB exists"
    fi

#    203 misc cache is not hard locked.
    echo | $SQL3 $MISCDB >$fOUT 2>&1 <<_EOF
.timeout 10500
.tables
.q
_EOF
    cat "$fOUT" | grep "database is locked" >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:203\n"
        printf "CRITICAL: 203 The $MISCDB file is hard locked.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "203 $MISCDB not hard locked"    
    fi

#    204 misc cache is not locked.
    echo | $SQL3 $MISCDB >$fOUT 2>&1 <<_EOF
.timeout 10000
begin exclusive transaction;
end transaction;
.q
_EOF
    cat "$fOUT" | grep "database is locked" >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:204\n"
        printf "WARNING: 204 The $MISCDB file is soft locked.\n"
        WARNING=true
    else
        R "204 $MISCDB not soft locked"    
    fi

#    205  ident cache is not hard locked.
    echo | $SQL3 $IDENTDB >$fOUT 2>&1 <<_EOF 
.timeout 10000
.tables
.q
_EOF
    cat "$fOUT" | grep "database is locked" >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:205\n"
        printf "CRITICAL: 205 The $IDENTDB file is hard locked.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "205 $IDENTDB not hard locked"    
    fi

#    206 ident cache is not locked.
    echo | $SQL3 $IDENTDB >$fOUT 2>&1 <<_EOF
.timeout 10500
begin exclusive transaction;
end transaction;
.q
_EOF
    cat "$fOUT" | grep "database is locked" >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:206\n"
        printf "WARNING: 206 The $IDENTDB file is soft locked.\n"
        WARNING=true
    else
        R "206 $IDENTDB not soft locked"    
    fi

#    207 Misc cache is a DB file.
    echo | $SQL3 $MISCDB >$fOUT 2>&1 <<_EOF
.timeout 10500
.tables
.q
_EOF
    RVAL=$?
    cat "$fOUT" | grep "is not a database" >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:207\n"
        printf "CRITICAL: 207 The $MISCDB file is not a valid database file.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "207 $MISCDB is a valid file"
    fi

#    208 Misc cache is not corrupt.
    cat "$fOUT" | grep "malformed" >/dev/null
    if [ $?  -eq 0 ] ; then
        $NUM && printf "NUMBER:208\n"
        printf "CRITICAL: 208 The $MISCDB cache is corrupt.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "208 $MISCDB is not corrupt"
    fi

#   209 Misc cache is not otherwise broken...     
    echo | $SQL3 $MISCDB >$fOUT 2>&1 <<_EOF
.timeout 10500
PRAGMA integrity_check;
.q
_EOF
    RVAL=$?
    cat "$fOUT" | grep -i "error" >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:209\n"
        printf "CRITICAL: 209 The $MISCDB cache is invalid <`cat $fOUT`>\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "208 $MISCDB is not otherwise broken"
    fi


#   210 ident cache is a DB file.
    echo | $SQL3 $IDENTDB >$fOUT 2>&1 <<_EOF
.timeout 1000
.tables
.q
_EOF
    RVAL=$?
    cat "$fOUT" | grep "is not a database" >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:210\n"
        printf "CRITICAL: 210 The $IDENTDB file is not a valid database file.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "210 $IDENTDB is a valid file"
    fi

#   211 Ident cache is not corrupt.
    cat "$fOUT" | grep "malformed" >/dev/null
    if [ $?  -eq 0 ] ; then
        $NUM && printf "NUMBER:211\n"
        printf "CRITICAL: 211 The $IDENTDB cache is corrupt.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "211 $IDENTDB is not corrupt"
    fi

#   212 Ident cache is not otherwise broken...     
    echo | $SQL3 $IDENTDB >$fOUT 2>&1 <<_EOF
.timeout 10500
PRAGMA integrity_check;
.q
_EOF
    RVAL=$?
    cat "$fOUT" | grep -i "error" >/dev/null
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:212\n"
        printf "CRITICAL: 212 The $IDENTDB cache is invalid <`cat $fOUT`>\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "212 $IDENTDB is not otherwise broken"
    fi

#    213 All needed tables exist. ( No missing tables, careful here, given version differences ).
    MISCTABLES="misc srvinfo usn_cache table_ctrl"
    IDENTTABLES="user_ad user_posix user_ovrd user_ovrd_bygroup group_ad group_posix group_ovrd group_member domain_sids nudn table_ctrl"
    MISSINGTABLE=0
    echo | $SQL3 $MISCDB >$fOUT 2>&1 <<_EOF
.timeout 1000    
.tables
.q
_EOF
    CURRENTMTABLES="`cat $fOUT`"
    for table in $MISCTABLES ; do
        echo $CURRENTMTABLES | grep $table >/dev/null
        if [ $? -ne 0 ] ; then 
            printf "FAILURE: Unable to locate table $table in $MISCDB\n"
            MISSINGTABLE=1
        fi
    done    
        
    echo | $SQL3 $IDENTDB >$fOUT 2>&1 <<_EOF 
.timeout 1000    
.tables
.q
_EOF
    CURRENTITABLES="`cat $fOUT`"
    for table in $IDENTTABLES ; do
        echo $CURRENTITABLES | grep $table >/dev/null
        if [ $? -ne 0 ] ; then 
            printf "FAILURE: Unable to locate table $table in $IDENTDB\n"
            MISSINGTABLE=1
        fi
    done    
        
    if [ "$MISSINGTABLE" = "1" ] ; then
        $NUM && printf "NUMBER:213\n"
        printf "CRITICAL: 213 The local DB cache is missing one or more needed tables.\n"
        FAILURE=true
        CRITICAL_FAILURE=true
        return 1
    else
        R "213 all required tables found"
    fi

#    210 No tables are corrupt. 
#    CORRUPTTABLE=0
#    for table in $CURRENTMTABLES ; do
#        echo ".timeout 5000" > $DBCMD
#        echo "select count(*) from $table;">> $DBCMD
#        echo ".q" >> $DBCMD
#        echo | $SQL3 -init $DBCMD $MISCDB 2>&1 | grep malformed >/dev/null
#        if [ $? -eq 0 ] ; then 
#            printf "FAILURE: Table $table in $MISCDB is corrupt.\n"
#            CORRUPTTABLE=1
#        fi
#    done    
#        
#    for table in $CURRENTITABLES ; do
#        echo ".timeout 5000" > $DBCMD
#        echo "select count(*) from $table;">> $DBCMD
#        echo ".q" >> $DBCMD
#        echo | $SQL3 -init $DBCMD $IDENTDB 2>&1 | grep malformed >/dev/null
#        if [ $? -eq 0 ] ; then 
#            printf "FAILURE: Table $table in $IDENTDB is corrupt.\n"
#            CORRUPTTABLE=1
#        fi
#    done    
#        
#    if [ "$CORRUPTTABLE" = "1" ] ; then
#        $NUM && printf "NUMBER:214\n"
#        printf "CRITICAL: 214 The one or more local DB cache tables are corrupt.\n"
#        FAILURE=true
#        CRITICAL_FAILURE=true
#        return 1
#    else
#        R "214 no tables corrupt"
#    fi

    FORESTROOT="`get_setting forestRoot`"
    COMPUTERFQDN="`get_setting computerFQDN`"
    COMPUTERNAME="`get_setting computerName`"
    LOCALSITE="`get_setting localSite`"
    DEFAULTREALM="`get_setting defaultRealm`"

#    215 machine has found a site, or has an override site set.
    if [ -z "$LOCALSITE" ] ; then
        $NUM && printf "NUMBER:215\n"
        printf "WARNING: 215 The computer object is not in a site.\n"
        WARNING=true
    else
        R "215 computer object in site"
    fi

#    216 Misc table has defaultRealm 
    if [ -z "$DEFAULTREALM" ] ; then
        $NUM && printf "NUMBER:216\n"
        printf "FAILURE: 216 Missing defaultRealm|<DOMAIN> setting in the misc DB.\n"
        FAILURE=true
    else
        R "216 misc cache has defaultRealm"
    fi


#    217 Misc table has computerfqdn
    if [ -z "$COMPUTERFQDN" ] ; then
        $NUM && printf "NUMBER:217\n"
        printf "FAILURE: 217 The misc cache is missing the computerFQDN setting for this machine.\n"
        FAILURE=true
    else
        R "217 misc cache has computerFQDN"
    fi

#    218 Misc table has computername
    if [ -z "$COMPUTERNAME" ] ; then
        $NUM && printf "NUMBER:218\n"
        printf "FAILURE: 218 The misc cache is missing the computerName setting for this machine.\n"
        FAILURE=true
    else
        R "218 misc cache has computerName"
    fi

#    219 Misc table has forestRoot
    if [ -z "$FORESTROOT" ] ; then
        $NUM && printf "NUMBER:219\n"
        printf "FAILURE: 219 The misc cache is missing the forestRoot setting for this machine.\n"
        FAILURE=true
    else
        R "219 misc cache has forestRoot"
    fi
    return 0
}

t300 ()
{
    $DEBUG && set -x
# * kinit results 
# First, kdestroy just in case, so anything left in the credentials cache isn't 
# interfearing with the testing, which occasionally might fix things, but not
# sure how to check/report that, so will jsut deal with it if it happens. 
    $VAS kdestroy >/dev/null 2>&1
    KRB5CCNAME=/tmp/_vas_status_ccache_$$
    export KRB5CCNAME
    RESULTS="`$VAS kinit host/ 2>&1`"
    if [ $? -ne 0 ] ; then
        ADFAILURE=true
        FAILURE=true
#   310 Machine can talk to AD ( KDC_UNREACH )
        echo "$RESULTS" | grep 1765328228 >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:301\n"
            printf "FAILURE: 301 VAS cannot find a Domain Controller (DC) that it can communicate with.\n"
            return 0
        fi    

#   302 Object exists in AD. ( client unknown )
        echo "$RESULTS" | grep 1765328378 >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:302\n"
            printf "FAILURE: 302 VAS cannot find its host/ object in AD\n"
            return 0
        fi    

#   303 Machine timesync. ( TIME_SKEW ) 
        echo "$RESULTS" | grep 1765328347 >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:303\n"
            printf "FAILURE: 303 VAS is not in time sync with the AD controller it is contacting.\n"
            return 0
        fi    

#   304 The host.keytab knows the right password. ( preauth failed ).
        echo "$RESULTS" | grep 1765328360 >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:304\n"
            printf "FAILURE: 304 VAS does not know the correct password for the AD object.\n"
            return 0
        fi    

#   305 The host/ account is usable ( CREDENTIALS REVOKED )
        echo "$RESULTS" | grep 1765328366 >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:305\n"
            printf "FAILURE: 305 The AD object is not usable at this time(revoked credentials).\n"
            return 0
        fi    

#   306 DES is set, but no entries ( KRB5KDC_ERR_PREAUTH_REQUIRED )
        echo "$RESULTS" | grep 1765328359 >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:306\n"
            printf "FAILURE: 306 The AD object is requiring unkown key type.\n"
            return 0
        fi    

#   307 ( Misc kinit error, not one of the above known ones ).
        $NUM && printf "NUMBER:307\n"
        printf "FAILURE: 307 Unknown vastool kinit error: <$RESULTS>.\n"
    else
        R "301 kdc reachable"
        R "302 computer object in AD"
        R "303 timesync ok"
        R "304 can access computer object"
        R "305 valid credentials"
        R "306 correct key type in kinit"
        R "307 no unknown errors talking to AD"
    fi 
    return 0
}

t400 ()
{
    $DEBUG && set -x
    $ADFAILURE && return 0
# * Computer account checks.
#   401 Computer can read AD object. 
    OUT="`$VAS attrs host/ cn 2>&1`"
    echo "$OUT" | grep -i "could not resolve Service"
    if [ $? -eq 0 ] ; then
        $NUM && printf "NUMBER:401\n"
        printf "WARNING: 401 The AD permissions do not allow the host to read its own attributes.\n"
        WARNING=true
        return 0
    else
        R "401 host/ can read self"
    fi    

#   402 Computer account has valid UPN.
    if [ ! -z "$COMPUTERFQDN" ] ; then
        UPN="`$VAS attrs host/ userPrincipalName 2>/dev/null | cut -d: -f2 | sed 's/^[ \t]*//'`"
        EXPECTEDUPN="host/$COMPUTERFQDN@`echo $DEFAULTREALM | tr  $LOWER $UPPER`"
        U1="`echo $UPN | tr $UPPER $LOWER`"
        U2="`echo $EXPECTEDUPN | tr $UPPER $LOWER`"
        if [ ! "$U1" = "$U2" ] ; then
            $NUM && printf "NUMBER:402\n"
            printf "WARNING: 402 Computer object has UPN of: <$UPN> (expected <$EXPECTEDUPN>).\n"
            WARNING=true
        else
            R "402 host/ has valid upn"
        fi
    fi    

    echo | bc >/dev/null 2>&1
    if [ $? -eq 0 ] ; then
#   403 Computer account userAccountControl has VAS_UF_PASSWD_NOTREQD
        UAC="`$VAS attrs host/ userAccountControl 2>/dev/null | cut -d: -f2 | sed 's/^[ \t]*//'`"
# The 'No password change required' flag is kept in the 17th bit.
        NOPASSWD=17
#echo $UAC
        BINUAC="`echo \"obase=2; $UAC\" | bc`"
        NOPWSET=0
        LEN="`printf \"$BINUAC\" | wc -m | awk '{print $1}'`"
        if [ $LEN -eq $NOPASSWD ] ; then 
            NOPWSET=1
        elif [ $LEN -gt $NOPASSWD ] ; then
            LEN2=`expr $LEN - $NOPASSWD + 1`
            if [ `echo $BINUAC | cut -c$LEN2` -eq 1 ] ; then 
                NOPWSET=1
            fi    
        fi
           
        if [ $NOPWSET -eq 0 ] ; then
            $NUM && printf "NUMBER:403\n"
            printf "WARNING: 403 Computer object userAccountControl does not have DONT_EXPIRE_PASSWORD set.\n"
            WARNING=true
        else
            R "403 host/ has DONT_EXPIRE_PASSWORD set"
        fi

#   404 Computer account userAccountControl has VAS_UF_WORKSTATION_TRUST_ACCOUNT
        WSTRUSTBIN=13
        WSTRUST=0
        LEN="`printf \"$BINUAC\" | wc -m | awk '{print $1}'`"
        if [ $LEN -eq $WSTRUSTBIN ] ; then 
            WSTRUST=1
        elif [ $LEN -gt $WSTRUSTBIN ] ; then
            LEN2=`expr $LEN - $WSTRUSTBIN + 1`
            if [ `echo $BINUAC | cut -c$LEN2` -eq 1 ] ; then 
                WSTRUST=1
            fi    
        fi
           
        if [ $WSTRUST -eq 0 ] ; then
            $NUM && printf "NUMBER:404\n"
            printf "WARNING: 404 Computer object userAccountControl does not have ADS_UF_WORKSTATION_TRUST_ACCOUNT set.\n"
            WARNING=true
        else
            R "404 host/ has ADS_UF_WORKSTATION_TRUST_ACCOUNT set"
        fi

#   405 Computer account userAccountControl doesn't have ADS_UF_USE_DES_KEY_ONLY
        DESBIN=22
        USEDES=0
        LEN="`printf \"$BINUAC\" | wc -m | awk '{print $1}'`"
        if [ $LEN -eq $DESBIN ] ; then 
            USEDES=1
        elif [ $LEN -gt $DESBIN ] ; then
            LEN2=`expr $LEN - $DESBIN + 1`
            if [ `echo $BINUAC | cut -c$LEN2` -eq 1 ] ; then 
                USEDES=1
            fi    
        fi
           
        if [ $USEDES -eq 1 ] ; then
            $NUM && printf "NUMBER:405\n"
            printf "WARNING: 405 The computer object userAccountControl has USE_DES_KEY_ONLY set.\n"
            WARNING=true
        else
            R "405 host/ doesn't have USE_DES_KEY_ONLY set"
        fi

#   406 Computer account userAccountControl doesn't have ADS_UF_DONT_REQUIRE_PREAUTH
        NOPREAUTHBIN=23
        NOPREAUTH=0
        LEN="`printf \"$BINUAC\" | wc -m | awk '{print $1}'`"
        if [ $LEN -eq $NOPREAUTHBIN ] ; then 
            NOPREAUTH=1
        elif [ $LEN -gt $NOPREAUTHBIN ] ; then
            LEN2=`expr $LEN - $NOPREAUTHBIN + 1`
            if [ `echo $BINUAC | cut -c$LEN2` -eq 1 ] ; then 
                NOPREAUTH=1
            fi    
        fi
           
        if [ $NOPREAUTH -eq 1 ] ; then
            $NUM && printf "NUMBER:406\n"
            printf "WARNING: 406 Computer object userAccountControl has ADS_UF_DONT_REQUIRE_PREAUTH set.\n"
            WARNING=true
        else
            R "406 host/ doesn't have ADS_UF_DONT_REQUIRE_PREAUTH set"
        fi
    fi
}

t500 ()
{
    $DEBUG && set -x
    $ADFAILURE && return 0
    if [ ! -z "$COMPUTERFQDN" -a ! -z "$COMPUTERNAME" ] ; then
#   501 Computer account has valid SPNs.
        ADSPNs="`$VAS attrs host/ servicePrincipalName 2>/dev/null | cut -d: -f2 | sed 's/^[ \t]*//' | tr '\n' ' '`"
        EXPECTEDSPNS="host/$COMPUTERFQDN host/$COMPUTERNAME"
        MISSINGSPN=0
        for spn in $EXPECTEDSPNS ; do
            echo $ADSPNs | grep -i "$spn" >/dev/null
            if [ $? -ne 0 ] ; then
                printf "WARNING: Did not find SPN <$spn> in list of SPNs <$ADSPNs>\n"
                MISSINGSPN=1
            fi
        done

        if [ $MISSINGSPN -eq 1 ] ; then
            $NUM && printf "NUMBER:501\n"
            printf "WARNING: 501 The computer object is missing servicePrincipalName entry(s).\n"
            WARNING=true
        else
            R "501 host/ has servicePrincipalName"
        fi

#   502 host.keytab contains matching entries for each SPN in AD.
        KEYTABSPNS="`$VAS ktutil list | grep -v '^Vno' | awk '{print $3}' | sort | uniq | cut -d@ -f1 | grep -iv \"^$COMPUTERNAME\" | grep -iv \"cifs/$COMPUTERFQDN\" | grep -v '^[ \t]*$' | tr '\n' ' '`"
        MISSINGSPN=0
        for spn in $ADSPNs; do
            echo $KEYTABSPNS | grep -i "$spn" >/dev/null
            if [ $? -ne 0 ] ; then
                printf "WARNING: Did not find AD SPN <$spn> in list of keytab SPNs <$KEYTABSPNS>\n"
                MISSINGSPN=1
            fi
        done

        if [ $MISSINGSPN -eq 1 ] ; then
            $NUM && printf "NUMBER:502\n"
            printf "WARNING: 502 host.keytab is missing entries for AD SPN entries.\n"
            WARNING=true
        else
            R "502 host.keytab has entries for each servicePrincipalName"
        fi

#   503 SPNs in AD match each entry in host.keytab.
        MISSINGSPN=0
        for spn in $KEYTABSPNS; do
            echo $ADSPNs | grep -i "$spn" >/dev/null
            if [ $? -ne 0 ] ; then
                printf "WARNING: Did not find keytab SPN <$spn> in list of AD SPNs <$ADSPNs>\n"
                MISSINGSPN=1
            fi
        done

        if [ $MISSINGSPN -eq 1 ] ; then
            $NUM && printf "NUMBER:503\n"
            printf "WARNING: 503 host.keytab has SPN entries that are not in AD.\n"
            WARNING=true
        else
            R "503 host.keytab has no extra entries"
        fi

        OUT="`$VAS -u host/ auth -S host/$COMPUTERFQDN 2>&1`"
        if [ $? -ne 0 ] ; then
#   504 auth -S host/$FQDN returns KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN, duplicate.
            echo "$OUT" | grep 1765328377 >/dev/null
            if [ $? -eq 0 ] ; then
                $NUM && printf "NUMBER:504\n"
                printf "WARNING: 504 AD is not able to find the specified SPN <host/$COMPUTERFQDN>.\n"
                WARNING=true
            else
#   505 auth -S host/$FQDN fails.
                $NUM && printf "NUMBER:505\n"
                printf "FAILURE: 505 Unrecognized auth error <$OUT>.\n"
                FAILURE=true
            fi
        else
            R "504 vastool auth can find fqdn"
            R "505 no failure with auth"
        fi
    fi    
}

t600 ()
{
    $DEBUG && set -x
    BADPERMS1=0
    BADPERMS2=0
    DP="drwxr-xr-x(755)"
    for D in /etc/opt/quest /etc/opt/quest/vas /var/opt/quest /var/opt/quest/vas /var/opt/quest/vas/vasd ; do
        if [ -d $D ] ; then
            ls -Llad $D | cut -c2,3,4,5,7,8,10 | grep "\-" >/dev/null
            if [ $? -eq 0 ] ; then
                BADPERMS1=1
                printf "FAILURE: 601 Missing needed permissions on <$D>, currently: <`ls -Llad $D | awk '{print $1}'`> should be:<$DP>.\n"
            fi
            ls -Llad $D | cut -c9 | grep "\-" >/dev/null    
            if [ $? -ne 0 ] ; then
                BADPERMS2=1
                printf "FAILURE: 602 'other' has write permissions on: <$D><`ls -Llad $D | awk '{print $1}'`> That permission should be removed.\n"
            fi
       fi     
    done

    FP="-rw-r--r--(644)"
    for F in $VASCONF $MISCDB $IDENTDB $UALLOW $UDENY ; do
        if [ -f $F ] ; then
            ls -Lla $F | cut -c2,3,5,8 | grep "\-" >/dev/null    
            if [ $? -eq 0 ] ; then
                BADPERMS1=1
                printf "FAILURE: 601 Missing needed permissions on <$F>, currently: <`ls -Lla $F | awk '{print $1}'`> should be: <$FP>.\n"
            fi
            ls -Lla $F | cut -c9 | grep "\-" >/dev/null    
            if [ $? -ne 0 ] ; then
                BADPERMS2=1
                printf "FAILURE: 602 'other' has write permissions on: <$F>(`ls -Lla $F | awk '{print $1}'`) That permission should be removed.\n"
            fi
        fi
    done

    if [ -f $MISCDB ] ; then
        MappedFile="`get_setting userMappings`"
    fi

#   601 File/directory permissions. Not enough.
    if [ $BADPERMS1 -eq 1 ] ; then
        $NUM && printf "NUMBER:601\n"
        FAILURE=true
    else
        R "601 file permissions ok (not enough)"
    fi

#   602 File/directory permissions. Too much.
    if [ $BADPERMS2 -eq 1 ] ; then
        $NUM && printf "NUMBER:602\n"
        FAILURE=true
    else
        R "602 file permissions ok (too much)"
    fi

    BADPERMS1=0
    BADPERMS2=0
    FP="-r-r--r--(444)"
    for F in /etc/netsvc.cfg /etc/nsswitch.conf /etc/resolv.conf /etc/hosts /etc/passwd /etc/group /usr/lib/security/methods.cfg /etc/pam.conf /etc/pam.d/system-auth /etc/pam.d/common-auth; do
        file=
#        if [ -h $F -a FALSE ] ; then
#            f="`ls -Lla $F | cut -d\> -f2 | tr -d ' '`"
#            if [ "`echo $f | cut -c1`" = "." ] ; then
#                file="`dirname $F`/$f"
#            else
#                file=$f
#            fi        
#        elif [ -f $F ] ; then
            file=$F
#        fi    
        
        if [ -f "$file" ] ; then 
            ls -laL $file | cut -c2,5,8 | grep "\-" >/dev/null    
            if [ $? -eq 0 ] ; then
                BADPERMS1=1
                printf "FAILURE: 605 Missing needed permissions on <$file>, currently: <`ls -Lla $file | awk '{print $1}'`> should be at least: <$FP>.\n"
            fi
            ls -laL $file | cut -c9 | grep "\-" >/dev/null    
            if [ $? -ne 0 ] ; then
                BADPERMS2=1
                printf "FAILURE: 606 'other' has write permissions on: <$file>(`ls -Lla $file | awk '{print $1}'`) That permission should be removed.\n"
            fi
        fi
    done


#   606 System file permissions. Not enough.
    if [ $BADPERMS1 -eq 1 ] ; then
        $NUM && printf "NUMBER:606\n"
        FAILURE=true
    else
        R "606 system file permissions ok (not enough)"
    fi

#   607 System file permissions. Too much.
    if [ $BADPERMS2 -eq 1 ] ; then
        $NUM && printf "NUMBER:607\n"
        FAILURE=true
    else
        R "607 system file permissions ok (too much)"
    fi

    $CRITICAL_FAILURE && return
    
#   603 /etc/nsswitch.conf contains vas3. ( check for map entries ). 
    if [ -f /etc/sia/matrix.conf ] ; then
        grep vas3 /etc/sia/matrix.conf >/dev/null
        if [ $? -ne 0 ] ; then
            $NUM && printf "NUMBER:603\n"
            printf "FAILURE: 603 /etc/sia/matrix.conf does not appear to be configured to use VAS.\n"
            FAILURE=true
        else
            R "603 /etc/sia/matrix.conf configured to use VAS"
        fi
    elif [ -f /etc/nsswitch.conf ] ; then
        grep vas3 /etc/nsswitch.conf >/dev/null
        if [ $? -ne 0 ] ; then
            $NUM && printf "NUMBER:603\n"
            if [ -z "$MappedFile" ] ; then
                printf "FAILURE: 603 /etc/nsswitch.conf does not appear to be configured to use VAS.\n"
                FAILURE=true
            else    
                printf "INFO: 603 /etc/nsswitch.conf does not appear to be configured to use VAS\n"
            fi
        else
            R "603 /etc/nsswitch.conf configured to use VAS"
        fi
    fi

#   604 Relevent PAM configuraiton file contains at least one pam_vas3 line.
    PAM_GOOD=true
    for file in /etc/pam.d/system-auth /etc/pam.d/common-auth /etc/pam.conf ; do
        if [ -f $file ] ; then
            grep pam_vas3 $file >/dev/null
            if [ $? -ne 0 ] ; then
                $NUM && printf "NUMBER:604\n"
                printf "FAILURE: 604 $file does not appear to be configured to use VAS.\n"
                FAILURE=true
                PAM_GOOD=false
            fi
            break
        fi
    done
    $PAM_GOOD && R "604 pam configured to use VAS"

#   605 AIX: /usr/lib/security/methods.cfg contains VAS
    AIX_GOOD=false
    for file in /etc/security/user /usr/lib/security/methods.cfg ; do
        if [ -f $file ] ; then
            grep "VAS" $file >/dev/null
            if [ $? -ne 0 ] ; then
                $NUM && printf "NUMBER:605\n"
                printf "FAILURE: 605 $file does not appear to be configured to use VAS.\n"
                FAILURE=true
                AIX_GOOD=false
            else
                AIX_GOOD=true
            fi
        fi
    done
    $AIX_GOOD && R "605 AIX files configured to use VAS"

    if $PAM_GOOD ; then 
        if [ -z "$SKIP_PAM_SERVICES" ] ; then
        #   608 pam_vas for each pam service. 
            if [ -f /etc/pam.d/common-auth ] ; then 
                pam_suse
            elif [ -f /etc/pam.d/system-auth ] ; then
                pam_redhat
            elif [ -d /etc/pam.d ] ; then 
        # Only old Suse machines it seems don't have the other methods.
                pam_suse
            else
                pam_conf
            fi
            if [ $? -ne 0 ] ; then
                $NUM && printf "NUMBER:608\n"
                FAILURE=true
            else
                R "608 each pam service configured to use VAS"
            fi
        fi    
    fi

#    609 nss links exist. 
    nss_link_tests 
    if [ $? -ne 0 -a -z "$MappedFile" ] ; then
        $NUM && printf "NUMBER:609\n"
        FAILURE=true
    else
        R "609 nss links ok"
    fi
    
    
#    610 pam links exist.
    pam_link_tests 
    if [ $? -ne 0 ] ; then
        $NUM && printf "NUMBER:610\n"
        FAILURE=true
    else
        R "610 pam links ok"
    fi
    
#    612 AIX: /etc/security/user doesn't have default registry entry
    if [ -f /etc/security/user ] ; then
        P="`lssec -f /etc/security/user -s default -a registry | cut -d= -f2`"
        if [ -n "$P" ] ; then
            $NUM && printf "NUMBER:612\n"
            echo "FAILURE: 612 /etc/security/user has default: registry setting <$P>"
            FAILURE=true
        else
            R "612 AIX: /etc/security/user doesn't have default registry entry"
        fi
    fi

#    613 AIX: methods.cfg is ok
    if [ -f /usr/lib/security/methods.cfg ] ; then
        lsuser -R VAS "no such user" 2>&1 1>/dev/null | grep VAS >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:613\n"
            echo "CRITICAL: 613 VAS method cannot be loaded, /usr/lib/security/methods.cfg invalid"
            CRITICAL_FAILURE=true
            FAILURE=true
        else
            R "613 AIX: methods.cfg is ok, VAS can load"
        fi
    fi

}

t700 ()
{
    $DEBUG && set -x
#   701 Valid license 1 ( expired )
    $VAS license -s 2>&1 | grep "is a site" >/dev/null
    if [ $? -ne 0 -o "$DO_LICENSING" = "1" ] ; then
        OUT="`$VAS -d5 license -q 2>&1`"
        echo "$OUT" | grep "is expired" >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:701\n"
            printf "INFO: 701 VAS has an expired license.\n"
            printf "INFO: <`$VAS -d5 license -q 2>&1 | grep \"is expired\"`>\n"
        else
            R "701 no expired licenses"
        fi
    fi

# Hmm.... This is triggering on smarcard licenses... So remove for now.
#   708 Valid license 4 ( invalid file )
#    $VAS license -s 2>&1 | grep "is a site" >/dev/null
#    if [ $? -ne 0 -o "$DO_LICENSING" = "1" ] ; then
#        OUT="`$VAS -d5 license -q 2>&1 | awk '/Smartcard/ {exit}; {print $0}'`"
#        echo "$OUT" | grep "required fields are missing" >/dev/null
#        if [ $? -eq 0 ] ; then
#            $NUM && printf "NUMBER:701\n"
#            printf "INFO: VAS has an invalid license.\n"
#            printf "INFO: <`$VAS -d5 license -q 2>&1 | awk '/Smartcard/ {exit}; {print $0}'| grep \"required fields are missing\" | sed 's/.*missing from//' | tr -d '\n'`>\n"
#        fi
#    fi

#   703 Valid license 3
    $VAS license -s 2>&1 | grep "is a site" >/dev/null
    if [ $? -ne 0 -o "$DO_LICENSING" = "1" ] ; then
        $VAS -d 5 license -q 2>&1 | grep "could not open license dir (/etc/opt/quest/vas/.licenses)" >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:703\n"
            printf "FAILURE: 703 VAS does not have a valid license directory.\n"
            FAILURE=true
        else
            R "703 license directory found"
        fi
    fi

#   702 Valid license 2
    $VAS license -s 2>&1 | grep "is a site" >/dev/null
    if [ $? -ne 0 -o "$DO_LICENSING" = "1" ] ; then
        $VAS license -q 2>&1 | awk '{if ($0 ~ /Smartcard/) {NOPRINT=1}; if(NOPRINT==0) {print $0};}' | awk '{if ($0 ~ /Siebel/) {NOPRINT=1}; if(NOPRINT==0) {print $0};}' | grep "No.*licenses are installed" >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:702\n"
            printf "FAILURE: 702 VAS does not have any valid licenses available.\n"
            FAILURE=true
        else
            R "702 valid license found"
        fi
    fi


#    <test>
#    if [ good ] ; then
#        R "<NUM> <msg>"
#    else
#        $NUM && printf "NUMBER:<NUM>\n"
#        FAILURE=true    
#    fi
}

t700b ()
{
    $DEBUG && set -x
    $ADFAILURE && return 0
#   705/706 vasd running
    case $PLATFORM in
        OSX*)
            VASDC="`ps -eAo command | grep \"^[^ ]*/vasd \" | wc -l | awk '{print $1}'`"
        ;;
        SOLARIS*)
            VASDC="`ps -eo comm | grep \"^/opt/quest/sbin/vasd\" | wc -l | awk '{print $1}'`"
        ;;
        *)
            VASDC="`UNIX95= ps -eo comm | grep \"^[.]*vasd\" | wc -l | awk '{print $1}'`"
        ;;
    esac

    if [ $VASDC -lt 1 ] ; then
        $NUM && printf "NUMBER:705\n"
        printf "FAILURE: 705 vasd does not appear to be running.\n"
        FAILURE=true
        return
    elif [ $VASDC -eq 1 ] ; then
        $NUM && printf "NUMBER:706\n"
        printf "FAILURE: 706 vasd does not appear to be running completely ( only one process ).\n"
        FAILURE=true
        return
    else
        R "705 vasd running"
        R "706 vasd running properly (two+ processes)"
    fi

#    715 If vgp is installed, vasgpd is running.
    if [ -f $VGPD ] ; then
        case $PLATFORM in
            OSX*)
                VGPDC="`ps -eAo command | grep \"^[^ ]*/vasgpd \" | wc -l | awk '{print $1}'`"
            ;;
            SOLARIS*)
                VGPDC="`ps -eo comm | grep \"^/opt/quest/sbin/vasgpd\" | wc -l | awk '{print $1}'`"
            ;;
            *)
                VGPDC="`UNIX95= ps -eo comm | grep \"^[.]*vasgpd\" | wc -l | awk '{print $1}'`"
            ;;
        esac

        if [ $VGPDC -lt 1 ] ; then
            $NUM && printf "NUMBER:715\n"
            printf "FAILURE: 715 vasgpd does not appear to be running.\n"
            FAILURE=true
            return
        else
            R "715 vasgpd running"
        fi
    fi

#    708 Mapped user consistency.
    if [ ! -z "$MappedFile" ] ; then
        validate_db
        if [ $? -ne 0 ] ; then
            $NUM && printf "NUMBER:708\n"
            FAILURE=true    
        else
            R "708 mapped user consistent"
        fi
    else
        R "708 mapped user consistent"
    fi


#   713 duplicate users
    if [ "$VASVERMAJOR.$VASVERMINOR.$VASVERREVISION" != "3.0.3" ]; then
        check_dups user
        if [ $? -eq 0 ] ; then
            R "713 duplicate users"
        else
            $NUM && printf "NUMBER:713\n"
            printf "FAILURE: 713 duplicate users in the cache\n"
            FAILURE=true    
        fi

#   714 no duplicate groups
        check_dups group
        if [ $? -eq 0 ] ; then
            R "714 duplicate groups"
        else
            $NUM && printf "NUMBER:714\n"
            printf "FAILURE: 714 duplicate groups in the cache\n"
            FAILURE=true    
        fi
    fi


#   708 vasd responding
# depending on os -f on the ipc file doesn't work.
# and -e doesn't seem universal ( thats what I get for writing these notes 
# long after I tested, I don't know what OS fails, jstu that this seemed to be
# the best way to universally test this )
ls $VASIPC >/dev/null 2>&1
    if [ $? -ne 0 ] ; then
        $NUM && printf "NUMBER:709\n"
        printf "FAILURE: 709 Missing <$VASIPC>, vasd cannot handle IPC requests.\n"
        FAILURE=true
        return
    else
#   707 vasd responding
        RESP="`$VAS list users 2>&1 1>/dev/null`"
        echo "$RESP" | grep -i "Could not communicate with vasd" >/dev/null
        if [ $? -eq 0 ] ; then
            $NUM && printf "NUMBER:707\n"
            printf "WARNING: 707 vasd did not respond to an IPC ping, could be busy.\n"
            WARNING=true
        else
            R "707 vasd responding to IPC ping"
            R "709 $VASIPC exists"
        fi
    fi

#   710 process memory size
    mem_tests
    if [ $? -ne 0 ] ; then
        $NUM && printf "NUMBER:710\n"
    else
        R "710 process memory size"
    fi

#    711 all binaries are same version
    TVASVERSION="`$VAS -v | grep -i version | head -1 | sed 's/vastool: .AS Version \([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)\.\([0-9]*\)/\1.\2.\3.\4/'`"
    BAD_VERSION=0
    for file in bin/vastool sbin/vasd sbin/vasypd sbin/vasgpd bin/vgptool bin/uptool sbin/vasproxyd; do
        if [ -f /opt/quest/$file ] ; then
            VER="`/opt/quest/$file -v | grep -i version | head -1 | sed 's/^\([^:]*\):.* \([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*\)/\2/'`"
            if [ ! "$TVASVERSION" = "$VER" ] ; then
                printf "WARNING: 711 /opt/quest/$file version <$VER> does not match VAS version\n"
                WARNING=true
                BAD_VERSION=1    
            fi
        fi
    done

    if [ $BAD_VERSION -ne 0 ] ; then
        $NUM && printf "NUMBER:711\n"
    else
        R "711 all binaries same version"
    fi

# 712, socket file permissions. Should be srwxrwxrwx. 
    SPERMS=`ls -la $VASIPC 2>/dev/null| awk '{print $1}' | tr $UPPER $LOWER`
    if [ "$SPERMS" = "srwxrwxrwx" ] ; then
        R "712 socket file permissions"
    else
        $NUM && printf "NUMBER:712\n"
        printf "FAILURE: 712 file permissions on <$VASIPC> are <$SPERMS>, must be <srwxrwxrwx>\n"
        FAILURE=true
    fi
}

CleanUp ()
{
    rm -f $CLEANUP_FILES
}

EndSuccess ()
{
    $DEBUG && set -x
    $NUM && printf "NUMBER:001\n"    
    printf "Result: <No tests failed> (`GetTimeDiff $STARTTIME` seconds)\n"
    CleanUp
    EXIT_VALUE=0
}

ExitCritical ()
{
    $DEBUG && set -x
    printf "Result: <Critical test(s) failed> (`GetTimeDiff $STARTTIME` seconds)\n"
    CleanUp
    EXIT_VALUE=3
}

ExitNoVAS ()
{
    $DEBUG && set -x
    printf "Result: <VAS does not appear to be installed> (`GetTimeDiff $STARTTIME` seconds)\n"
    CleanUp
    EXIT_VALUE=5
}

ExitFailure ()
{
    $DEBUG && set -x
    printf "Result: <Test(s) failed> (`GetTimeDiff $STARTTIME` seconds)\n"
    CleanUp
    EXIT_VALUE=2    
}

ExitWarning()
{
    $DEBUG && set -x
    printf "Result: <Test(s) reported warnings> (`GetTimeDiff $STARTTIME` seconds)\n"
    CleanUp
    EXIT_VALUE=1    
}

RunEverything ()
{
    t000 ; if [ $? -ne 0 ] ; then ExitNoVAS; return ; fi
    t100 ; if [ $? -ne 0 ] ; then t600; t700; ExitCritical ; return ; fi
    t200 ; if [ $? -ne 0 ] ; then t600; t700; ExitCritical ; return ; fi
    t300
    t400
    t500
    t600
    t700
    t700b
    if $CRITICAL_FAILURE ; then ExitCritical ; return ; fi
    if $FAILURE ; then ExitFailure ; return ; fi
    if $WARNING ; then ExitWarning ; return ; fi
    EndSuccess
}

VerifyRoot

RunEverything
$VAS kdestroy 2>/dev/null 1>&2
exit $EXIT_VALUE
