#!/bin/sh
#==============================================================================
# Copyright (c) 2009 Quest Software, Inc.  All Rights Reserved.
#
# vasjoin.sh        A utility for helping create a valid 'vastool join' command.
#                   It also displays the final command before executing it; in
#                   doing so, it is pedagogical.
#
# Version:          3.5.1.9
#
#==============================================================================
VASJOIN_SH_NAME=$0

set -u                  # this catches uninitialized variables as soon as they're encountered
umask 000 > /dev/null   # So all tmp files are private

    # Constants
           TRUE=0
          FALSE=1
        SUCCESS=0
        FAILURE=1

    # Binaries
    VASTOOL=/opt/quest/bin/vastool
    VGPTOOL=/opt/quest/bin/vgptool

    # global options --------------------------------------------------------------
    #
    # This is basically our interface. We work in this script to set up all of the
    # necessary options for the 'vastool join' command to work. Our consumer can
    # pass in options already set as he/she learns of them (this tool is pedagogi-
    # cal in addition to being helpful).
    #
    # Everything starts out unset; if option passed on the command line, it's set
    # before we get started and we don't ask about it.
    #
    # The -u username and -u user-search-string options are disambiguated.
    #
    # Below, "naked" means "not a switch option."

    # variables representing vastool options --------------------------------------
    QUIET=$FALSE   # -q:  less verbose: lose explanations, keep questions
    INTERACTIVE=$FALSE  # -i:  verbose: numerous, well documented options
    outputfile=         # file to save generated join command to

    DEBUG_LEVEL=        # -d n (vastool) debug level during command exection
    username=           # -u username (of privileged user performing the join)
    keytab=             # -k keytab
    password=           # password

    # variables representing join options -----------------------------------------
    force=              # -f (vastool)
    computer=           # -n computer-name
    container=          # -c container
    no_vgp=             # -l
    upm_search=         # -p personality container search path
    group_search=       # -g group-search-string
    user_search=        # -u user-search-string
    workstation_mode=   # -w workstation mode
    domain_list=        # -r cross-forest domain list
    site_name=          # -s site-name if not taking existing default

    domain_name=        # mandatory argument
    controllers=        # optional second (optional) argument

    # global variables for ValidateTaggedOuAsPersonalityContainer, etc. -----------
    tagged_ou=
    tagged_cn=

    # common global variables -----------------------------------------------------
    kinit_called=$FALSE # indicates whether we've got a temporary keytab file
    command_line=       # everything past "vastool" once we're done

    curDomain=          # Currently joined domain


# Functions
#------------------------------------------------------------------------------
# print proper usage for the script and exit.
#------------------------------------------------------------------------------
PrintUsage()
{
    printf "
Usage: vasjoin.sh [script options] [vastool options] [join options] domain-name [domain-controller..]

This utility walks through creation of a 'vastool join' command. After the
command has been created, the user had the option of executing it. Most options
of the standard 'vastool join' command can be passed on the command line
by-passing interactive treatment by this script (they will show up in the final
command).

vasjoin.sh script options:
    -h,--help               Display this message
    -v,--version            Display script version
    -q                      Quiet: Don't display option descriptions
    -o <file>               If specified, the script will save generated join command to file 
                            so that user may perform join at a later time.

    <none>                  Normal mode: Prompt for common options
    -i                      Interactive mode: Prompt for all options

vastool options:
    -d level                Debug verbosity level 0-6 (Default is 0)
    -k keytab               Keytab used to authenticate
    -u,--user=<username>    User to perform join

join options:
    -f                      Overwrite existing computer object in the domain
    -l                      Don't apply Group Policy Settings (if VGP is installed)
    -w                      Enable workstation mode - users will not be cached until they login
    -n computer-name        Specify name of computer object
    -c container            LDAP DN of the container where the computer will be created
    -p upm-path             Specify the path of the Primary Personality Container
    -u user-path            Specify an alternate search path from which to populate the users cache
    -g group-path           Specify an alternate search path from which to populate the groups cache
    -r domain-list          Comma separated list of alternate authentication domains
    -s site-name            Specify the site name for this machine

Arguments:
    domain-name             Target domain for join
    domain-controller       List of domain controllers for join

"
}

#------------------------------------------------------------------------------
# Echo prompt without kicking a new line. Different bashes and especially
# Bourne do this differently (and incompatibly).
#------------------------------------------------------------------------------
Prompt()
{
    __prompt=$1
    __indent=$2
    
    eval printf "'%${__indent}s${__prompt}'"
}

#------------------------------------------------------------------------------
# Asks a provided yes/no question and returns the answer.
#------------------------------------------------------------------------------
askyesno=           # global variable set by AskYesNo with return
AskYesNo()
{
    yn_prompt="$1 (yes|no)?"            # the basic prompt
    yn_default=$2                       # the default (if any)
    yn_answer=
    yn_echoopt=

    if [ -n "$yn_default" ]; then       # add the default to prompt
        yn_prompt="$yn_prompt [$yn_default]: "
    else
        yn_prompt="$yn_prompt "
    fi

    until [ -z "$yn_prompt" ]; do
        printf "$yn_prompt"; read yn_answer

        case $yn_answer in
            # we'll accept all of these as valid responses...
            "yes") askyesno="yes" ; yn_prompt= ;;
                "y") askyesno="yes" ; yn_prompt= ;;
            "no") askyesno="no"  ; yn_prompt= ;;
                "n") askyesno="no"  ; yn_prompt= ;;

        *) # if default, don't require answer...
            if [ -z "$yn_answer" ]; then
                if [ -n "$yn_default" ]; then
                    yn_prompt=
                    askyesno=$yn_default
                fi
            fi
            # bogus answers don't count as...
            # ...taking the default!
            ;;
        esac
    done
}

EchoDetail()
{
    detail=${1:-}

    if [ $QUIET -eq $TRUE ]; then
        return
    fi
    printf "$detail\n"
}

vastoolAuth=
GetVastoolAuthString()
{
    if [ -n "$keytab" ];then
        vastoolAuth="$VASTOOL -u '$username' -k '$keytab'"
    else
        vastoolAuth="echo '$password' | $VASTOOL -u '$username' -s"
    fi
}

#------------------------------------------------------------------------------
# Code to validate an OU for being a personality container.
#------------------------------------------------------------------------------
ValidateTaggedOuAsPersonalityContainer()
{
    check_dn=$1     # known to be a valid OU, but we still do error-checking
    displayName=

    result=$FAILURE

    # Establish both $tagged_ou and $tagged_cn here for later use in
    # TagOuAsPersonalityContainer(). The CN must be identical to the OU...
    tagged_ou=`eval $vastoolAuth attrs -d "$check_dn" ou | awk '/ou:/{ print $2 }' 2>/dev/null`
    if [ $? -ne $SUCCESS ]; then
        result=$FAILURE
        return
    fi

    tagged_cn=`eval $vastoolAuth attrs -d "$check_dn" cn | awk '/cn:/{ print $2 }' 2>/dev/null`
    if [ $? -ne $SUCCESS ]; then
        result=$FAILURE
        return
    fi

    if [ "x$tagged_cn" != "x$tagged_ou" ]; then
        result=$FAILURE
        return
    fi

    # The displayName must be set and contain "VASUPMPRI"...
    displayName=`eval $vastoolAuth attrs -d "$check_dn" displayName | awk '/displayName:/{ print $2 }' 2>/dev/null`
    if [ $? -ne $SUCCESS ]; then
        result=$FAILURE
        return
    fi

    if [ "x$displayName" != "xVASUPMPRI" ]; then
        result=$FAILURE
        return
    fi

    # if we get here, $check_dn is an OU tagged as a personality container...
    result=$SUCCESS
}

#------------------------------------------------------------------------------
# Code to tag an OU as a personality container.
#------------------------------------------------------------------------------
TagOuAsPersonalityContainer()
{
    __ou=$1   # known to be a valid OU, but we still do error-checking
    save_cn=  # the old CN attribute, if any present, for restoring
    err=
    result=$FAILURE

    # Here's how to tag a personality container so that it gets recognized as
    # such by vastool and other VAS utilities. We assume in here that $__ou is
    # good since it was validated by ValidateOu() before getting in here. Also,
    # we called ValidateTaggedOuAsPersonalityContainer() and got both the OU and
    # the CN while we were at it. $__ou was not found to be a tagged OU; hence,
    # we're in here.
    #
    # Notes: vastool attrs returns either the answer we want (a simple attribute
    # in the form attr: value or, upon failure, a goopy error message. However,
    # if we get the answer we want, $err will be $SUCCESS. Upon error, $err has a
    # numeric value.
    #
    # We should not need to specify -U to vastool attrs or setattrs. We already
    # ensured that $username is set up when we checked to see if this was already
    # tagged. We would not be here if that failed/was rejected.
    #
    # Isolate to "unprompted" values...
    save_cn=$tagged_cn

    # Set the ou attribute on the cn for they are both to contain the same thing.
    # What, however, are we to do if there's already something there? Hmmmm...
    # for now we'll just ignore this case.
    tmp=`eval $vastoolAuth setattrs -d "$__ou" cn "$tagged_ou" 2>&1`
    if [ $? -ne $SUCCESS ]; then
        echo "ERROR: Unable to mark personality container as such."
        result=$FAILURE
        return
    fi

    # Then, set displayName to this magic string of characters...
    tmp=`eval $vastoolAuth setattrs -d "$__ou" displayName VASUPMPRI 2>&1`
    if [ $? -ne $SUCCESS ]; then
        # Back ou what we changed on cn; this was probably blank so don't modify
        # the attribute to a null string, but wipe it out with the -r option.
        if [ -z "$saved_cn" ]; then
            tmp=`eval $vastoolAuth setattrs -r -d "$__ou" cn 2>&1`
        else
            tmp=`eval $vastoolAuth setattrs -d "$__ou" cn "$saved_cn" 2>&1`
        fi
        result=$FAILURE
        return
    fi

    result=$SUCCESS
}

#------------------------------------------------------------------------------
# Take a DN and escape all funny characters so that it can be used with
#  'vastool search' 
#
# NOTE: comma and equals are not allowed since they are special characters in the DN
#       and cannot easily be distinguished from the OU name.
#------------------------------------------------------------------------------
escapeDN=
EscapeDN()
{
    dn=${1:-}
    tmpFile=/tmp/.escapeDN
# valid test: OU=`~!@#$%^&*()_-+{}[]|\\:;'"<>.? ,DC=l,DC=automation

    echo $dn > $tmpFile 2>&1
    escapeDN=`sed -e '
        s/\\\/\\\\\\\\\\\\\\\\/g;'"\
        s/'/'\\\\\''/g;"'\
        s/^/'\''distinguishedName=/;\
        s/$/'\''/;\
        s/*/\\\\*/g;\
        s/(/\\\\(/g;\
        s/)/\\\\)/g;\
        s/+/\\\\\\\\+/g;\
        s/;/\\\\\\\\;/g;\
        s/</\\\\\\\\</g;\
        s/>/\\\\\\\\>/g;\
        s/"/\\\\\\\\"/g;' $tmpFile`
}

#------------------------------------------------------------------------------
# If the OU doesn't exist, an error is printed and result set to 1. We make use
# of 'vastool search' to determine this. We do it quietly because the consumer
# of this script is here to use (and maybe learn) 'vastool join' and not the
# search command.
#------------------------------------------------------------------------------
ValidateOu()
{
    __type="${1:-}"   # container, personality, user or group container
    __dn="${2:-}"     # this distinguished name is the OU in question
    __upm="${3:-}"    # set only when this is a proposed personality container

    # We create a keytab for doing all of our vastool search, setattrs, etc. once
    # and use it from now on.

    # This just ensures that $__dn is valid.
    echo "Validating $__type '$__dn'..."
    EscapeDN "$__dn"

    if [ -n "$keytab" ];then
        eval $VASTOOL -u "$username" -k "$keytab" search "$escapeDN" dn > /dev/null 2>&1
    else
        echo "$password" | eval $VASTOOL -u "$username" -s search "$escapeDN" dn > /dev/null 2>&1
    fi
    err=$?
    if [ $err -ne $SUCCESS ]; then
        echo ""
        echo "ERROR: '$__dn' does not exist"
        result=$FAILURE
        return
    else
        echo "Found AD Container!"
    fi

    # If this is about a Unix personality search path, vastool will fail starting
    # with version 3.1 if the OU is not "tagged." Unfortunately, no OUs are
    # tagged at this point. And there is no automatic way to do it. As it's
    # likely that this OU should be tagged (or it would not have been submitted
    # as the argument for the -p option), we'll offer to tag it for them.
    #
    # Of course, at this point, we certainly have a valid OU, so most of the
    # error handling from here on is a little over-kill.
    if [ -n "$__upm" ]; then
        echo "Verifying '$__dn' is a Unix personality container...
        "
        ValidateTaggedOuAsPersonalityContainer "$__dn"
        if [ $result -eq $SUCCESS ]; then
            result=$SUCCESS
            return
        fi

        EchoDetail "
  In order to use '$__dn'
  as a Unix personality container, it must be specially marked in Active
  Directory. Normally, this is done by the administrator, but if you have
  sufficient privileges, you may do it now. You must have privileges over
  '$__dn'
        "
        AskYesNo "  Do you wish to mark this OU as a personality container" "yes"
        if [ "$askyesno" = "yes" ]; then
            TagOuAsPersonalityContainer "$__dn" # (can set $result to $FAILURE...)
            result=$FAILURE
            return
        fi
    fi

    result=$SUCCESS
}

#------------------------------------------------------------------------------
# Determine if host is joined, etc
#------------------------------------------------------------------------------
DetermineState()
{
    joined=`$VASTOOL -u host/ info id 2> /dev/null`
    if [ -n "$joined" ];then
        curDomain=`$VASTOOL info domain`
    fi
}

#------------------------------------------------------------------------------
# Grab any vastool (not join) options from the command line.
#------------------------------------------------------------------------------
CollectVasToolOptions()
{
    if [ $INTERACTIVE -eq $TRUE ]; then
        while [ true ]; do
###############################################################################
# Domain Name
###############################################################################
            # Domain name is required
            while [ true ];do
                Prompt "Please specify AD domain name to join to [$domain_name]: " 1
                read tmp

                if [ -z "$tmp" ];then
                    tmp=$domain_name
                fi

                domain_name=$tmp
                if [ -n "$domain_name" ];then
                    break
                else
                    printf "
ERROR: You must specify a domain to join to
"
                fi
            done
            
###############################################################################
# Domain Controllers
###############################################################################
            # domain controllers is optional
            EchoDetail "
    [ Domain Controllers ]
    The servers for this domain will be automatically detected through DNS and LDAP lookups. Alternatively, you 
    may specify the servers to use. These servers will be preferred over the ones that vastool automatically detects.

            "
            Prompt "(optional) Specify a list of domain controllers to use during join [$controllers]: " 1
            read tmp

            if [ -z "$tmp" ];then
                tmp=$controllers
            fi
            controllers=$tmp

###############################################################################
# User name
###############################################################################
            # User name is required
            while [ true ];do
                Prompt "Please specify an AD user to join host to '$domain_name' [$username]: " 1
                read tmp

                if [ -z "$tmp" ];then
                    tmp=$username
                fi

                username=$tmp
                if [ -n "$username" ];then
                    break
                else
                    printf "
ERROR: You must specify a user to join with
"
                fi
            done

###############################################################################
# User Keytab
###############################################################################
            EchoDetail "
        [ User Keytab ]
        This option allows you to specify a Kerberos 5 keytab file to use for authentication instead of a password. This is useful for
        automation and provisioning, in particular with Active Directory service accounts.
            "
            Prompt "(optional) Please specify path to keytab file [$keytab]: " 1
            read tmp

            if [ -z "$tmp" ];then
                tmp=$keytab
            fi
            keytab=$tmp

###############################################################################
# Confirm Values
###############################################################################    
            printf -- "\n---------------------------------------------------------------------\n"
            printf "\t%-20s: %s\n" "Domain" "$domain_name"
            if [ -n "$controllers" ];then
                printf "\t%-20s: %s\n" "Domain Controllers" "$controllers"
            else
                printf "\t%-20s: %s\n" "Domain Controllers" "<none> -- use DNS"
            fi
            printf "\t%-20s: %s\n" "AD User" "$username"
            if [ -n "$keytab" ];then
                printf "\t%-20s: %s\n" "User Keytab" "$keytab"
            else
                printf "\t%-20s: %s\n" "User Keytab" "<none> -- will prompt for password"
            fi
            printf -- "---------------------------------------------------------------------\n\n"

            # Confirm this information before proceeding, otherwise, we'll go back and try it all again.
            AskYesNo "  Is this information correct" "yes"
            if [ "$askyesno" = "yes" ]; then
                break
            fi

            # go 'round because the information is incorrect...
            domain_name=
            username=
            keytab=
            controllers=
        done
    else
        # simple mode: just verify that there is a domain and user

        while [ -z "$domain_name" ];do
            Prompt "Please specify AD domain name to join to: " 1
            read domain_name

            if [ -n "$domain_name" ];then
                break
            else
                printf "
ERROR: You must specify a domain to join to
"
            fi
        done

        while [ -z "$username" ];do
            Prompt "Please specify an AD user to join host to '$domain_name': " 1
            read username

            if [ -n "$username" ];then
                break
            else
                printf "
ERROR: You must specify a user to join with
"
            fi
        done

        printf -- "\n---------------------------------------------------------------------\n"
        printf "\t%-20s: %s\n" "Domain" "$domain_name"
        if [ -n "$controllers" ];then
            printf "\t%-20s: %s\n" "Domain Controllers" "$controllers"
        else
            printf "\t%-20s: %s\n" "Domain Controllers" "<none> -- use DNS"
        fi
        printf "\t%-20s: %s\n" "AD User" "$username"
        if [ -n "$keytab" ];then
            printf "\t%-20s: %s\n" "User Keytab" "$keytab"
        else
            printf "\t%-20s: %s\n" "User Keytab" "<none> -- will prompt for password"
        fi
        printf -- "---------------------------------------------------------------------\n\n"
    fi
}

GetUserPassword()
{
    if [ -z "$keytab" ];then
        while [ -z "$password" ];do
            Prompt "\nPlease enter password for $username: " 1
            stty -echo
            read password
            stty echo
        done
    fi
    echo
}

#------------------------------------------------------------------------------
# Walk our consumer through the join process in lieu of having to compose a
# complicated 'vastool join' command with occasionally mutually exclusive
# options. Information is provided for each of the options that may be passed
# to 'vastool join' and validates as many options as it can prior to calling
# 'vastool join.' One particular piece of information that cannot be prev-
# iously validated is the validity of the keytab credential.
#------------------------------------------------------------------------------
CollectJoinOptions()
{
    string_answer=

    #----------------------------------------------------------------------------
    # If -f option already present, don't ask this...
    #----------------------------------------------------------------------------
    if [ -z "$force" ]; then
        EchoDetail "
[Force overwrite of existing computer object]
Note that this operation may be forced to overwrite an existing computer
object. The new computer object (for this host) will be placed in the same
container as the one previous one.
        "
        AskYesNo "  Would you like to overwrite any existing definition" "yes"
        if [ "$askyesno" = "yes" ]; then
            force="-f"
        fi
    fi

    #----------------------------------------------------------------------------
    # If -n option and its argument already present, don't ask this...
    #----------------------------------------------------------------------------
    if [ -z "$computer" ]; then
        thishost=`hostname`
        EchoDetail "
[Computer name]
Your computer's hostname is $thishost. You can specify the name of the computer
object that will appear in Active Directory. VAS will pick a default name for
you based on your computer's hostname if you answer 'no'.
        "
        AskYesNo "  Would you like to name your computer object something else" "no"
        if [ "$askyesno" = "yes" ]; then
            while [ 1 ]; do
                Prompt "Please enter a name for your computer object: " 1
                read string_answer
                if [ -z "$string_answer" ]; then
                    echo "  The computer object will remain $thishost."
                    break
                else
                    AskYesNo "  Is '$string_answer' correct" "yes"
                    if [ "$askyesno" = "yes" ]; then
                        break
                    fi
                fi
            done
            computer=$string_answer
        fi
    fi

    #----------------------------------------------------------------------------
    # If -l option already present or VGP isn't present, don't ask this...
    #----------------------------------------------------------------------------
    if [ -z "$no_vgp" ]; then
        if [ -x $VGPTOOL ]; then
            EchoDetail "
[Disable VAS group policy]
VGP is installed on your system. During a join operation, VAS will call vgptool
to enforce VAS-related policies immediately on this machine. By default, group
policies are applied, but this functionality may be disabled.
            "
            AskYesNo "  Would you like to disable group policy application" "no"
            if [ "$askyesno" = "yes" ]; then
                no_vgp="-l"
            fi
        fi
    fi

    EchoDetail "

-------------------------------------------------------------------------------
In the following questions and descriptions, DN refers to an LDAP distinguished
name. If you happen to enter an invalid or ill-formed DN, the search may take a
long time to fail. You will also have to enter the password each time unless
you specified a keytab file.
-------------------------------------------------------------------------------
    "

    #----------------------------------------------------------------------------
    # If -c option and its argument already present, don't ask this, but validate
    # the container nonetheless.
    #----------------------------------------------------------------------------
    if [ -z "$container" ]; then
        EchoDetail "
[Container name]
You can specify a container to join to in Active Directory. This is where your
computer object will be as a result of the join. If you just want VAS to put
the computer object into the usual Computers container, answer 'no'.
        "
        AskYesNo "  Would you like to join to a specific container" "no"
        if [ "$askyesno" = "yes" ]; then
            echo "The container must be specified as a valid, existing DN."
            while [ 1 ]; do
                Prompt "Enter the DN of the container: " 1
                read string_answer
                if [ -z "$string_answer" ]; then
                    echo "  No container will be specified."
                    break
                else
                    ValidateOu "container" "$string_answer"
                    if [ $result -eq $SUCCESS ]; then
                        break
                    fi
                fi
            done
            container=$string_answer
        fi
    else
        ValidateOu "container" "$container"
        if [ $result -eq $FAILURE ]; then
            container=
        fi
    fi

    #----------------------------------------------------------------------------
    # If -s option already present, don't ask this...
    #----------------------------------------------------------------------------
    if [ -z "$site_name" ]; then
        EchoDetail "
[Site name]
You can specify the name of the site in which to place your host after you
join. If instead you want VAS to determine a site for you automatically,
answer 'no'. (Note: This is an infrequent option.)
        "
        AskYesNo "  Would you like to specify the site name" "no"
        if [ "$askyesno" = "yes" ]; then
            while [ 1 ]; do
                Prompt "Please enter a name of the site: " 1
                read string_answer
                if [ -z "$string_answer" ]; then
                    echo "  No site will be specified."
                    break
                else
                    AskYesNo "  Is the information entered correct" "yes"
                    if [ "$askyesno" = "yes" ]; then
                        break
                    fi
                fi
            done
            site_name=$string_answer
        fi
    fi

    #----------------------------------------------------------------------------
    # This thing supercedes (is mutually exclusive to) both -u and -g options. If
    # these others are present, print an error.
    #
    # If -p option and its argument already present, don't ask this, but do
    # validate it.
    #----------------------------------------------------------------------------
    EchoDetail "
[Unix personality management--personality container search path]
VAS allows you to create user and group personalities for your Active
Directory users and groups. This allows the same security principal to have
different Unix account properties in different Unix personality organizational
units (called personality container OUs). If you wish to join the domain in
this mode, you will be required to specify an organizational unit in Active
Directory where VAS can look for such personalities.

If you answer 'yes' to the following question, you will be prompted for the
personality container search path. Later, you will not be prompted for user
and/or group(s) since these will be restricted to the path you are about to
indicate here.

For more information on UPM and the vastool join command, see the VAS
Administration Guide, Appendix C for 'join.'
    "
    if [ -z "$upm_search" ];then
        AskYesNo "  Would you like to join the domain in UPM mode" "no"
    else
        echo "Setting UPM Search path to: $upm_search"
        askyesno=yes
    fi
    if [ "$askyesno" = "yes" ]; then
        while [ 1 ]; do
            if [ -z "$upm_search" ];then
                Prompt "Please enter the UPM search path: " 1
                read string_answer
            else
                string_answer="$upm_search"
            fi
            if [ -z "$string_answer" ]; then
                echo "  No container will be specified."
                break
            else
                ValidateOu "UPM container" "$string_answer" "upm"
                if [ $result -eq $SUCCESS ]; then
                    break
                fi
                upm_search=
            fi
        done
        upm_search=$string_answer
    fi

    #----------------------------------------------------------------------------
    # Grumble about user_search and/or group_search being initialized if
    # $upm_search string was already specified/set. They were warned this
    # would happen, so ignore it and go on.
    #----------------------------------------------------------------------------
    if [ -n "$upm_search" ]; then
        if [ -n "$user_search" ]; then
            EchoDetail "
Existing user search paths, already specified on the command line, but
incompatible with UNIX personality mode, will be ignored.
            "
            user_search=
        fi
        if [ -n "$group_search" ]; then
            EchoDetail "
Existing group search paths, already specified on the command line, but
incompatible with UNIX personality mode, will be ignored.
            "
            group_search=
        fi
    else
#----------------------------------------------------------------------------
# No $upm_search, so handle $user_search and $group_search normally.
#
# If -u option already present, don't ask this, but do verify the path...
#----------------------------------------------------------------------------
        if [ -n "$user_search" ]; then
            ValidateOu "user container" "$user_search"
            if [ $result -eq $FAILURE ]; then
                echo "ERROR: The specified user search path option (-u) is invalid."
                user_search=
            fi
        fi
        if [ -z "$user_search" ]; then
            EchoDetail "
[User search path]
You may specify an alternate user search path anywhere in the Active Directory
forest where this machine will look for user information to put in its cache.
(A common option in complex environments.)
        "
        AskYesNo "  Would you like to specify an alternate user search path" "no"
        if [ "$askyesno" = "yes" ]; then
            echo "The container must be specified as a valid, existing DN."
            while [ 1 ]; do
                Prompt "Enter the DN of the container: " 1
                read string_answer
                if [ -z "$string_answer" ]; then
                    echo "  No container will be specified."
                    break
                else
                    ValidateOu "user container" "$string_answer"
                    if [ $result -eq $SUCCESS ]; then
                        break
                    fi
                fi
            done
            user_search=$string_answer
        fi
    fi

    #--------------------------------------------------------------------------
    # If -g option already present, don't ask this, but do validate it...
    #--------------------------------------------------------------------------
    if [ -n "$group_search" ]; then
        ValidateOu "group container" "$group_search"
        if [ $result -eq $FAILURE ]; then
            echo "ERROR: The specified group search path (-g) is invalid."
            group_search=
        fi
    fi

    if [ -z "$group_search" ]; then
        EchoDetail "
[Group search path]
You may specify an alternate group search path anywhere in the Active Directory
forest where this machine will look for group information to put in its cache.
(A common option in complex environments.)
        "
        AskYesNo "  Would you like to specify an alternate group search path" "no"
        if [ "$askyesno" = "yes" ]; then
            echo "The container must be specified as a valid, existing DN."
            while [ 1 ]; do
                Prompt "Enter the DN to the container: " 1
                read string_answer
                if [ -z "$string_answer" ]; then
                    echo "  No container will be specified."
                    break
                else
                    ValidateOu "group container" "$string_answer"
                    if [ $result -eq $SUCCESS ]; then
                        break
                    fi
                fi
            done
            group_search=$string_answer
        fi
    fi
fi

    #----------------------------------------------------------------------------
    # If -w option already present, don't ask this...
    #----------------------------------------------------------------------------
    if [ -z "$workstation_mode" ]; then
        EchoDetail "
[Workstation mode]
Workstation mode is an alternate caching method. Workstation mode allows you to
limit the user cache to those users who login to a particular workstation. This
behavior must be enabled on a per machine basis. By default workstation mode is
not enabled.
    "
AskYesNo "  Would you like to join in workstation mode" "no"
if [ "$askyesno" = "yes" ]; then
    workstation_mode="-w"
fi
fi

    #----------------------------------------------------------------------------
    # If -r option, don't ask this...
    #----------------------------------------------------------------------------
    if [ -z "$domain_list" ]; then
        EchoDetail "
[Domain list]
VAS allows you to specify alternate authentication domains in addition to the
one that you are joining. If you would like to enable cross-domain login with
simple name, you must specify all possible login domains in a comma-separated
list so VAS can record them. You will also need to set this up if you are going
to use Unix personality mode with personalities linked to Active Directory
users in another domain. This is not usually necessary except in a cross-
forest/cross-domain environment where you want to login using a simple name
or to restrict the intra-forest domains in which a user can use simple names
for authentication. (Note: This is an infrequent option.)
    "
    AskYesNo "  Would you like to specify alternate domains to search" "no"
    if [ "$askyesno" = "yes" ]; then
        echo "Include no spaces in the following comma separated list."
        while [ 1 ]; do
            Prompt "Enter a comma separated list of domains: " 1
            read string_answer
            if [ -z "$string_answer" ]; then
                echo "  No domains will be specified."
            break
            else
                AskYesNo "  Is the information entered correct" "yes"
                if [ "$askyesno" = "yes" ]; then
                    break
                fi
            fi
        done
        domain_list=$string_answer
    fi
fi

    #----------------------------------------------------------------------------
    # Self-enrollment...
    # Propose automatically enrolling /etc/passwd users with uid greater than a
    # certain threshold. This is done by adding a setting in configuration file
    # /etc/opt/quest/vas/vas.conf thus:
    #
    # [vas_auth]
    #   enable-automatic-enrollment = true
    #   automatic-enrollment-uid-ranges = { range }
    #----------------------------------------------------------------------------
    EchoDetail "
[Self-enrollment]
It is possible to cause all non Active Directory users whose uids fall within
specified ranges to be enrolled for Active Directory authentication.
This enrollment is a self-enrollment and will occur the first time a
non Active Directory user logs in after the initial VAS installation.
You may either specify yes to get all uids greater than 200 or specify a
different range including discontiguous ranges like 100-500,1000-3000 etc.
    "
    echo "  Would you like VAS to enroll all non Active Directory users"
    AskYesNo "    at login time" "no"
    if [ "$askyesno" = "yes" ]; then
        range="101-MAX"    # (for starters...)
        AskYesNo "  Would you like specify particular range or ranges" "no"
        if [ "$askyesno" = "yes" ]; then
            printf "  Enter range description: "
            read range    # override numeric range with range specified by user
        fi
        tmp=`$VASTOOL configure vas vas_auth self-enrollment-uid-ranges $range 2>&1`
        tmp=`$VASTOOL configure vas vas_auth enable-self-enrollment true 2>&1`
        err=$?

    if [ $err -ne $SUCCESS ]; then
        printf "
WARNING: vastool configure vas vas_auth enable-automatic-enrollment failed.
"
    fi
fi
}

#------------------------------------------------------------------------------
# Since Bourne doesn't support arrays, we have to build the command line slowly
# and painfully. This can be called any time to get whatever is known about the
# command at the moment called.
#------------------------------------------------------------------------------
BuildCommandLine()
{
    command=${1:-join}
    command_line=                         # zero out this variable

    # vastool options --------------------------------------
    if [ -n "$DEBUG_LEVEL" ]; then        # -d debug-level
        command_line="-d $DEBUG_LEVEL"
    fi

    if [ "x$command" = "xjoin" ];then
        # join options -----------------------------------------
        command_line="$command_line join"

        if [ -n "$force" ]; then              # -f
            command_line="$command_line $force"
        fi

        if [ -n "$no_vgp" ]; then             # -l
            command_line="$command_line $no_vgp"
        fi

        if [ -n "$workstation_mode" ]; then   # -w
            command_line="$command_line $workstation_mode"
        fi

        if [ -n "$computer" ]; then           # -n computer
            command_line="$command_line -n $computer"
        fi

        # We need to use quotes to keep any spaces from breaking up the single
        # argument...
        if [ -n "$container" ]; then          # -c container
            command_line="$command_line -c \"$container\""
        fi

        if [ -n "$site_name" ]; then          # -s site-name
            command_line="$command_line -s $site_name"
        fi

        if [ -n "$upm_search" ]; then         # -p upm-search-path
            command_line="$command_line -p \"$upm_search\""
        fi

        if [ -n "$group_search" ]; then       # -g group-search-string
            command_line="$command_line -g \"$group_search\""
        fi

        if [ -n "$user_search" ]; then        # -u user-search-string
            command_line="$command_line -u \"$user_search\""
        fi

        if [ -n "$domain_list" ]; then        # -r cross-forest domain list
            command_line="$command_line -r \"$domain_list\""
        fi

        # this is the only absolutely mandatory argument to a vastool join command...
        if [ -n "$domain_name" ]; then        # naked argument for domain-name
            command_line="$command_line $domain_name"
        fi

        if [ -n "$controllers" ]; then         # naked argument for domain-controller
            command_line="$command_line $controllers"
        fi
    else
        command_line="$command_line unjoin"
    fi
}

#------------------------------------------------------------------------------
# Display what the resulting vastool join command is. This makes it so that
# our consumer can learn to compose a 'vastool join' command directly if
# he/she cares to.
#------------------------------------------------------------------------------
ShowCommandLine()
{
    shell_script=${2:-}

    result=$SUCCESS

    BuildCommandLine

    if [ -z "$outputfile" ]; then
        echo "  $VASTOOL $command_line"
    else
        if [ -z "$extension" ]; then
            echo "$vastoolAuth $command_line" > $outputfile
        else
            echo "#!/bin/sh" > $outputfile
            echo "# vastool join command line from utility vasjoin.sh..." >> $outputfile
            echo "$VASTOOL $command_line" >> $outputfile
            chmod a+x $outputfile
            result=$?
        fi
    fi
}

#------------------------------------------------------------------------------
# Run 'vastool join' as the specified user with the correct credential. Any
# tailoring of command behavior such debug level or quiet operation is
# handled natively by the vastool command itself since we pass those options
# through.
#------------------------------------------------------------------------------
ExecuteCommand()
{
    BuildCommandLine ${1:-}

    echo "  Executing: $VASTOOL $command_line"
    echo ""

    # Okay, issue the command now...
    eval "$vastoolAuth $command_line"
    result=$?
}

# -----------------------------------------------------------------------------
# Execute (or write to a file) or not the assembled command and arguments.
# -----------------------------------------------------------------------------
PerformJoin()
{
    outputfile=
    extension=

    if [ -z "$outputfile" ];then
        printf "\nJoining the host to $domain_name:\n"
        ExecuteCommand

    else
        ShowCommandLine $outputfile
    fi
}

#############################################################
# Check whether executing user is root. root is required
#  for installation tasks on systems.
#############################################################
RequireRoot()
{
    # Check for root: can't do this stuff without it!
    id=${EUID:-`id | sed 's/(.*//;s/.*=//'`}

    if [ $id -ne 0 ]; then        
        printf "\nERROR: You must have root access to use this script!"
        exit 1
    fi
}

#############################################################
# Read command line arguments and update variables
#############################################################
ParseCommandLine()
{
    set +u  # disable -u

    # This allows the long arg names on platforms (e.g. hpux) with old getopt
    argv=
    for arg in $@;do
        argTmp=`echo $arg | grep -- "--user="`
        if [ -n "$argTmp" ];then
            username=`echo $argTmp | sed 's/--user=//'`
        elif [ "$arg" = "--version" ]; then
            argv="$argv -v"
        elif [ "$arg" = "--help" ]; then
            argv="$argv -h"
        else
            argv="$argv $arg"
        fi
    done
    set -- $argv

    argv=`getopt c:d:fg:hik:ln:o:p:qr:s:u:vw ${*-} 2> /dev/null`
    if [ $? -ne $SUCCESS ];then
        PrintUsage        
        printf "ERROR: Invalid option(s)\n\n";
        exit 1
    fi

    set -- $argv
    while [ $1 != -- ]; do
        case $1 in
            -c) container=$2                             ; shift ;;
            -d) DEBUG_LEVEL=$2                           ; shift ;;
            -f) force="-f"                                       ;;
            -g) group_search=$2                          ; shift ;;
            -h)                               PrintUsage; exit 0 ;;
            -i) INTERACTIVE=$TRUE                                ;;
            -k) keytab=$2                                ; shift ;;
            -l) no_vgp="-l"                                      ;;
            -n) computer=$2                              ; shift ;;
            -o) outputfile=$2                            ; shift ;;
            -p) upm_search=$2                            ; shift ;;
            -q) QUIET=$TRUE                                 ;;
            -r) domain_list=$2                           ; shift ;;
            -s) site_name=$2                             ; shift ;;
            -u) user_search=$2                           ; shift ;;
            -v) exit 0                                           ;;
            -w) workstation_mode="-w"                            ;;
             *) echo "ERROR: Invalid option\n\n"; PrintUsage; exit 1 ;;
        esac
        shift
    done
    shift
    set -u  # And turn it back on -- it fixes HPUX's strict -u interpretations

    if [ -n "${1:-}" ];then
        domain_name=${1:-}
        shift
    fi

    controllers=${*:-}

    # The -u option is totally ambiguous since both vastool can have one (and
    # usually does) and vastool join can have one. We're going to get clever
    # here to resolve the distinction between what might be passed in as:
    #   vastool -u russellb join ...
    # or:
    #   vastool ... join -u "OU=Unix Users,OU-Unix Container,DC=A,DC=DEV"
    if [ -z "$username" -a -n "$user_search" ]; then
        username=`echo "$user_search" | grep [\ =,]`
        if [ -z "$username" ]; then
            username="$user_search"
            user_search=
        fi
    fi
}

DoUnjoin()
{
    AskYesNo "You are currently joined to '$curDomain'. Do you wish to unjoin?" "yes"
    if [ "$askyesno" = "yes" ];then
        joinUser=$username

        # User name is required
        while [ true ];do
            Prompt "Please specify an AD user to unjoin host from '$curDomain' [$username]: " 1
            read tmp

            if [ -z "$tmp" ];then
                tmp=$username
            fi

            username=$tmp
            if [ -n "$username" ];then
                break
            else
                printf "
ERROR: You must specify a user to join with
"
            fi
        done

        # Get password
        while [ -z "$password" ];do
            Prompt "Please enter password for $username: " 1
            stty -echo
            read password
            stty echo
        done

        GetVastoolAuthString

        printf "\n\nUnjoining host '$curDomain'\n"
        $VASTOOL timesync -d "$curDomain" 2>&1 > /dev/null
        ExecuteCommand "unjoin"
        echo

        username=$joinUser
        password=

        if [ $result -ne $SUCCESS ];then
            printf "Failed to unjoin from domain. Exiting script"
            exit $result
        fi
    fi
}

#=============================================================================#
#                                                                             #
#                             Main script body                                #
#                                                                             #
#=============================================================================#
printf "
Vintela Authentication Services (VAS) installation script
Script build version: 3.5.1.9
Copyright (c) 2009 Quest Software, Inc.  All Rights Reserved.

"

result=$SUCCESS

ParseCommandLine "${@:-}"
RequireRoot

if [ ! -x $VASTOOL ]; then
    printf "ERROR: $VASTOOL must be installed for this script to be used!\n"
    exit 1
fi

# Unjoin if necessary
DetermineState

if [ -n "$curDomain" ];then
    DoUnjoin
fi

# Both minimal and interactive modes do this (or part of it)...
printf "Acquire basic information...\n"
CollectVasToolOptions

# Get User Credentials
GetUserPassword
GetVastoolAuthString

# Configure realm so that we can look stuff up later
tmp=`$VASTOOL configure vas libdefaults default_realm "$domain_name" 2>&1`
tmp=`$VASTOOL timesync -d "$domain_name" 2>&1`

if [ $INTERACTIVE -eq $TRUE ];then
        printf "Acquire join information...\n"

        # Only interactive mode goes down this lengthy road...
        CollectJoinOptions

else
        force="-f"
fi

PerformJoin

exit $result
