#!/bin/sh
set -e

CIMOM_URL="--url localhost:owipc"
PROVIDER_NS=vmx/provider
CCM_AGT_NS=vmx/ccm/invagt
STATUS_NS=vmx/status
PRODUCT_NAME=qmxsms
PRODUCT_USER=qmxsms
PRODUCT_TOOL_LOG=
DEBUG_ENABLED=0
ENABLE_DEBUGGING_NOW="set -x"

VMX_NAMESPACES="vmx vmx/policy vmx/localpolicy vmx/provider vmx/status vmx/ccm/invagt vmx/ccm vmx/SoftwareMeteringAgent vmx/ccm/LocationServices vmx/invagt"

Abort()
{
	echo "Error (aborting): $@" >&2
	exit 1
}

CompileMof()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	namespace="$1"
	mof_file="$2"
	echo "Running mof compiler as: /opt/quest/umi/bin/owmofc $CIMOM_URL -n ${namespace} \"${mof_file}\""  >&2
	/opt/quest/umi/bin/owmofc $CIMOM_URL -n ${namespace} "${mof_file}" || Abort "Failed to compile mof file: ${mof_file}"
}

RunProductTool()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	log_option="${PRODUCT_TOOL_LOG:+ --log-file ${PRODUCT_TOOL_LOG}}"
	echo "Running product tool as: /opt/quest/umi/bin/product_tool ${log_option:-} $@" >&2
	/opt/quest/umi/bin/product_tool ${log_option} "$@"
}

CheckRunningCimom()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if /opt/quest/umi/sbin/quest-owcimomd_init status; then
		:
	else
		echo "The CIMOM is not running.  Fondling the repository directly." >&2
		CIMOM_URL="-d /var/opt/quest/umi/openwbem/repository"
	fi
}

AlreadyRegistered()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if [ ${DEBUG_ENABLED:-0} = 1 ]; then
		echo "These products are registered:" >&2
		RunProductTool --list-registered-products >&2
	fi

	if RunProductTool --list-registered-products | grep "^${PRODUCT_NAME}\$" >&7; then
		# Already registered... This could be an upgrade...
		return 0
	fi
	return 1
}


RegisterProduct()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	RunProductTool --register --name ${PRODUCT_NAME} --username ${PRODUCT_USER}
}

RegisterNamespaces()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	REGISTRATION_LIST=
	for namespace in ${VMX_NAMESPACES}; do
		REGISTRATION_LIST="${REGISTRATION_LIST} --register-namespace ${namespace}"
	done
	RunProductTool --name ${PRODUCT_NAME} --username ${PRODUCT_USER} ${REGISTRATION_LIST}
}


UpdateRegisteredNamespaces()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	# Default to deleting all old namespaces and registering nothing.
	# Spaces within quotes on this next line are important.
	NAMESPACES_TO_DELETE=" `RunProductTool --name ${PRODUCT_NAME} --username ${PRODUCT_USER} --list-namespaces | tr '\n' ' '` "

	REGISTRATION_LIST=
	for namespace in ${VMX_NAMESPACES}; do
		# Spaces within quotes on this next line are important.
		if echo "${NAMESPACES_TO_DELETE}" | grep " ${namespace} " >&7; then
			# Present already.  No need to reregister or delete.
			NAMESPACES_TO_DELETE="`echo \"${NAMESPACES_TO_DELETE}\" | sed \"s? ${namespace} ? ?g\"`"
		else
			# A namespace not present in the old list...
			REGISTRATION_LIST="${REGISTRATION_LIST} --register-namespace ${namespace}"
		fi
	done
	# Append all of the deleted namespaces.
	for namespace in ${NAMESPACES_TO_DELETE} ""; do
		if [ x${namespace:+set} = xset ]; then
			REGISTRATION_LIST="${REGISTRATION_LIST} --deregister-namespace ${namespace}"
		fi
	done
	if [ x${REGISTRATION_LIST:+set} = xset ]; then
		RunProductTool --name ${PRODUCT_NAME} --username ${PRODUCT_USER} ${REGISTRATION_LIST}
	else
		echo "No namespaces need modification"
	fi
}

CompileProviderMof()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	CompileMof ${PROVIDER_NS} /opt/quest/umi/share/openwbem/Core/CIM_ManagedElement.mof
	CompileMof ${PROVIDER_NS} /opt/quest/umi/share/openwbem/Core/CIM_ManagedSystemElement.mof
	CompileMof ${PROVIDER_NS} /opt/quest/umi/share/openwbem/Core/CIM_LogicalElement.mof
	CompileMof ${PROVIDER_NS} /opt/quest/umi/share/openwbem/Event/CIM_ListenerDestination.mof
	CompileMof ${PROVIDER_NS} /opt/quest/umi/share/openwbem/Event/CIM_IndicationHandler.mof
	CompileMof ${PROVIDER_NS} /opt/quest/umi/share/openwbem/Event/CIM_IndicationFilter.mof
	CompileMof ${PROVIDER_NS} /opt/quest/umi/share/openwbem/Event/CIM_IndicationSubscription.mof

	CompileMof ${CCM_AGT_NS} /opt/quest/umi/share/openwbem/Core/CIM_ManagedElement.mof
	CompileMof ${CCM_AGT_NS} /opt/quest/umi/share/openwbem/Core/CIM_ManagedSystemElement.mof
	CompileMof ${CCM_AGT_NS} /opt/quest/umi/share/openwbem/Core/CIM_LogicalElement.mof
}

RegisterProviders()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	RunProductTool --name ${PRODUCT_NAME} --register-providers /opt/quest/qmxcm/share/qmxcm/reg/qmxcm.reg
}

CompileOperationalMof()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	# Install the client mof.  This includes instances needed to run a policy update.
	CompileMof ${PROVIDER_NS} /opt/quest/qmxcm/share/qmxcm/mof/Vintela_VMXClient.mof

	CompileMof ${STATUS_NS} /opt/quest/umi/share/openwbem/Core/CIM_ManagedElement.mof
	CompileMof ${STATUS_NS} /opt/quest/umi/share/openwbem/Event/CIM_ListenerDestination.mof
	CompileMof ${STATUS_NS} /opt/quest/umi/share/openwbem/Event/CIM_ListenerDestinationCIMXML.mof
	CompileMof ${STATUS_NS} /opt/quest/umi/share/openwbem/Event/CIM_IndicationHandler.mof
	CompileMof ${STATUS_NS} /opt/quest/umi/share/openwbem/Event/CIM_IndicationFilter.mof
	CompileMof ${STATUS_NS} /opt/quest/umi/share/openwbem/Event/CIM_IndicationSubscription.mof
	CompileMof ${STATUS_NS} /opt/quest/qmxcm/share/qmxcm/mof/Vintela_VMXStatus.mof
	CompileMof ${STATUS_NS} /opt/quest/qmxcm/share/qmxcm/mof/Vintela_VMXSoftwareMeteringEvents.mof
	CompileMof ${STATUS_NS} /opt/quest/qmxcm/share/qmxcm/mof/Vintela_VMXSoftwareMeteringAgent.mof
	CompileMof ${STATUS_NS} /opt/quest/qmxcm/share/qmxcm/mof/Vintela_VMXPollingDaemonListener.mof
	#import the schema into vmx/policy and vmx/status
}

InstallConfigFile()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	# The only config file that qmxcm has right now is for logging.
	RunProductTool --name ${PRODUCT_NAME} --set-owcimomd-config /etc/opt/quest/qmxcm/openwbem/openwbem.conf.d/qmx_logging.conf
}

Usage()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	echo "Usage: $0 [--debug] [--log-file <pt log file>]"
	exit 1
}

ParseCommandLine()
{
	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}
	while [ $# -gt 0 ]; do
		case $1 in
			"")
				# Empty args should just be eaten.
				;;
			"--log-file")
				[ -n "${2:-}" ] || Usage
				PRODUCT_TOOL_LOG="$2"
				shift
				;;
			"--debug")
				# This option isn't meant to be used by everyone.  It's here to
				# allow some way to propagate the debugging flags from one function
				# to another (some shells discard them).
				DEBUG_ENABLED=1
				${ENABLE_DEBUGGING_NOW}
				;;
			*)
				echo "Invalid option: $1" >&2
				Usage;;
		esac
		shift
	done
}

# Print an error message when the shell exits and clean up.
ShellExited()
{
	shell_return_value=$?

	[ ${DEBUG_ENABLED:-0} = 1 ] && ${ENABLE_DEBUGGING_NOW}

	if [ ${shell_return_value} != 0 ]; then
		echo "Error registering ${PRODUCT_NAME}" >&2
	else
		echo "Successfully registered ${PRODUCT_NAME}"
	fi

	return ${shell_return_value}
}

SetupDescriptors()
{
	exec 7>&-
	if [ ${DEBUG_ENABLED:-0} = 1 ]; then
		exec 7>&2
	else
		exec 7>/dev/null
	fi
}

Main()
{
	ParseCommandLine "$@"
	SetupDescriptors
	CheckRunningCimom
	if AlreadyRegistered; then
		# Register new namespaces, delete those that should no longer exist.
		UpdateRegisteredNamespaces
	else
		# Register product & namespaces
		RegisterProduct
		RegisterNamespaces
	fi

	CompileProviderMof
	RegisterProviders
	CompileOperationalMof
	InstallConfigFile
}

trap ShellExited EXIT

if [ $# -gt 0 ]; then
	Main "$@"
else
	Main ""
fi
