#!/bin/bash

# This script is used to install applications (.app), packages (.pkg), and
# metapackages (.mpkg) that are contained within a disk image file.  This can
# be a .dmg, .iso, or any other file type supported by the MacOS X hdiutil
# mount command.
#
# Full command-line usage is described in the Usage() function, which will be
# printed if this script is executed with the -h option (or no arguments).


INSTALL_IMAGE=
INSTALL_APPLICATION=
INSTALL_PACKAGE=
CUSTOM_COMMAND_LINE=
INSTALL_LANGUAGE=
INSTALL_TARGET=
INSTALL_VERBOSE=0
INSTALL_QUIETLY=0
INSTALL_OVERWRITE=0
INSTALLER_FLAGS=
VERBOSE_DESCRIPTOR=9
NORMAL_DESCRIPTOR=8
ERROR_DESCRIPTOR=7
declare -a EXTRA_PARAMETERS

IMAGE_HAS_LICENSE=0
ACCEPT_IMAGE_LICENSE=0
DEVICE_TO_EJECT=
INSTALL_PACKAGE_PREFIX=
PACKAGES_INSTALLED=0

VerifyRootUser()
{
	if [ -n "$EUID" -a "$EUID" != "0" ] || [ -n "$UID" -a "$UID" != "0" ]; then
	if  id | grep 'uid=0' >/dev/null 2>/dev/null; then
			:
		else
			echo "This script must be run as root."
			exit 1;
		fi
	fi
}

Usage()
{
	echo "Usage: $0 [-q|-v] [-i <image>] [-o] [-p <package directory>] [-a <app folder>] [-t <target volume>] [-l <language>] [-c <command>]"
	echo "  -h : Show this help text"
	echo "  -q : Be quiet"
	echo "  -v : Be verbose"
	echo "  -o : Allow older packages to overwrite newer ones (not for apps)"
	echo "  -i : Use <image> as the disk image (dmg) file"
	echo "  -a : Use <app folder> as a directory (or file) to copy to the Applications folder."
	echo "  -p : Use <package directory> as the package (pkg, mpkg, or app) to install"
	echo "  -l : Use <lang> as the language for install (not for apps)"
	echo "  -t : Install the specified package(s) into this target (default=\"/\")"
	echo "  -y : If the mounting of the disk image requires acceptance of a license"
	echo "       agreement, agree to the license."
	echo "  -c : Run <command> in the mount-point of the disk image.  Cannot be used"
	echo "       with the -a or -p options."
	echo " Note: One of -i or -p must be specified."
	echo "       If both -i and -p are specified, <package directory> is assumed to"
	echo "       exist within the disk image."
	echo ""
	if [ x${1:-EXIT} == xEXIT ]; then
		exit 1
	fi
}

ParseOptions()
{
	local args
	local number_to_eat=0

	EatArgs()
	{
		number_to_eat=$1
	}

	FlagNeedsOption()
	{
		if [ $# -gt 1 ]; then
			return 0
		else
			echo "Flag '$1' requires a value"
			return 1
		fi
	}

	for flag; do
		# If the last flag required some other parameters to be eaten, eat some.
		if [ $number_to_eat -gt 0 ]; then
			let number_to_eat="$number_to_eat-1"
			shift
		continue
		fi

		# Apply the flags.
		case $flag in
			-h)
				Usage NOEXIT
				exit 0
			;;
			-q)
				INSTALL_QUIETLY=1
				INSTALL_VERBOSE=0
			;;
			-v)
				INSTALL_QUIETLY=0
				INSTALL_VERBOSE=1
			;;
			-o)
				INSTALL_OVERWRITE=1
			;;
			-i)
				FlagNeedsOption "$@"
				INSTALL_IMAGE=`echo "$2" | sed -e 's/ /\\ /g'`
				EatArgs 1
			;;
			-a)
				FlagNeedsOption "$@"
				INSTALL_APPLICATION="$2"
				EatArgs 1
			;;
			-p)
				FlagNeedsOption "$@"
				INSTALL_PACKAGE="$2"
				EatArgs 1
			;;
			-t)
				FlagNeedsOption "$@"
				INSTALL_TARGET="$2"
				EatArgs 1
			;;
			-l)
				FlagNeedsOption "$@"
				INSTALL_LANGUAGE="$2"
				EatArgs 1
			;;
			-y)
				ACCEPT_IMAGE_LICENSE=1
			;;
			-c)
				FlagNeedsOption "$@"
				CUSTOM_COMMAND_LINE=$2
				EatArgs 1
			;;
			-*)
				echo "Invalid parameter '$flag'"
				return 1
			;;
			--)
				shift
				for remaining_flags; do
					EXTRA_PARAMETERS[${#EXTRA_PARAMETERS[@]}]="$1"
					shift
				done
				break
			;;
			*)
				EXTRA_PARAMETERS[${#EXTRA_PARAMETERS[@]}]="$1"
			;;
		esac
		shift
	done
	return 0
}

ChangeFileDescriptors()
{
	# Open up extra file descriptors so that later tests for each output are not required.
	if [ "$INSTALL_QUIETLY" = "1" ]; then
		eval "exec ${VERBOSE_DESCRIPTOR}>/dev/null"
		eval "exec ${NORMAL_DESCRIPTOR}>/dev/null"
		eval "exec ${ERROR_DESCRIPTOR}>/dev/null"
	else
		# Normal
		eval "exec ${NORMAL_DESCRIPTOR}>&1"
		# error
		eval "exec ${ERROR_DESCRIPTOR}>&2"
		# Open up the verbose descriptor.
		if [ "$INSTALL_VERBOSE" != "0" ]; then
			eval "exec ${VERBOSE_DESCRIPTOR}>&1"
		else
			eval "exec ${VERBOSE_DESCRIPTOR}>/dev/null"
		fi
	fi
	# Close STDIN so that any program expecting input on stdin will fail instead of hanging.
	exec 0</dev/null
}

CloseFileDescriptors()
{
	eval "exec ${VERBOSE_DESCRIPTOR}>&-"
	eval "exec ${NORMAL_DESCRIPTOR}>&-"
	eval "exec ${ERROR_DESCRIPTOR}>&-"
}

CheckImageName()
{
	if [ -z "$INSTALL_IMAGE" ] && [ -z "$INSTALL_PACKAGE" ] && [ -z "$INSTALL_APPLICATION" ]; then
		echo "One of '-i', '-a', or '-p' is required" >&2
		Usage >&2
	fi
}

CheckInstallTarget()
{
	if [ -z "$INSTALL_TARGET" ]; then
		# No install target specified.
		[ "$INSTALL_QUIETLY" != "1" ] && echo "Setting install target to default '/'"
		INSTALL_TARGET="/"
	fi
}

ConfigureInstallerFlags()
{
	[ "$INSTALL_VERBOSE" = "0" ]   || INSTALLER_FLAGS="$INSTALLER_FLAGS -verbose"
	[ "$INSTALL_OVERWRITE" = "0" ] || INSTALLER_FLAGS="$INSTALLER_FLAGS -allow"
	[ -z "$INSTALL_LANGUAGE" ]     || INSTALLER_FLAGS="$INSTALLER_FLAGS -lang $INSTALL_LANGUAGE"
	INSTALLER_FLAGS="$INSTALLER_FLAGS -target $INSTALL_TARGET"

	echo "The current install options are: $INSTALLER_FLAGS" >&$NORMAL_DESCRIPTOR
}

GetPlistKey()
{
	# $1=file
	# $2=key name
	#
	# This function assumes that the key you are searching for is unique among
	# the plist file.  No real XML parsing is done, as this would be a true pain
	# in shell script.  NOTE: When multiple keys have values, the last one found
	# is the one that is returned.

	plist_key_retval=
	plist_key_error=1
	temp_output_file=/tmp/dmg_installer.plist_key.$$.$RANDOM
	if grep -A 1 "<key>$2</key>" "$1" > ${temp_output_file} 2>/dev/null; then
		key_data_line="`tail -1 ${temp_output_file} | sed 's/^[ 	]*//g'`"
		# Possible values (ones that we expect) are:
		#   <true/>
		#   <false/>
		#   <string>.*</string> or <string/>
		#   <integer>.*</integer>
		#   <real>.*</real>
		#   <date>.*</date>
		# Everything else here is considered invalid (difficult to handle, and non-useful for us).
		#   <array>.*</array>
		#   <data>.*</data>
		#   <dict>.*</dict>
		GetPlistKeyInnerData()
		{
			echo "${key_data_line}" | sed "s?<$1>\(.*\)</$1>?\1?g"
		}
		case ${key_data_line} in
			'<true/>') plist_key_retval="true"; plist_key_error=0 ;;
			'<false/>') plist_key_retval="false"; plist_key_error=0 ;;
			'<string>'*'</string>') plist_key_retval="`GetPlistKeyInnerData string`"; plist_key_error=0 ;;
			'<integer>'*'</integer>') plist_key_retval="`GetPlistKeyInnerData integer`"; plist_key_error=0 ;;
			'<real>'*'</real>') plist_key_retval="`GetPlistKeyInnerData real`"; plist_key_error=0 ;;
			'<date>'*'</date>') plist_key_retval="`GetPlistKeyInnerData date`"; plist_key_error=0 ;;
			'<array>'* | '<dict>'* | '<data>'*) echo "ERROR: Array, dict, and data properties cannot be returned." >&2;;
			*) echo "Unexpected property line: \"${key_data_line}\"" ;;
		esac

		if [ ${plist_key_error} != 1 ]; then
			echo "${plist_key_retval}"
		fi
	else
		echo "Key not found." >&2
	fi
	rm -f ${temp_output_file}
	return ${plist_key_error}
}

MountDiskImage()
{
	if [ -n "$INSTALL_IMAGE" ]; then
		TEMP_FILE=/tmp/dmg_installer.mount.$$.$RANDOM

		hdiutil_input=""

		if [ ${IMAGE_HAS_LICENSE} = 1 ] && [ ${ACCEPT_IMAGE_LICENSE} = 1 ]; then
			hdiutil_input="Y"
		fi

		if echo -e "${hdiutil_input}" | hdiutil mount -plist "$INSTALL_IMAGE" -nobrowse -readonly > $TEMP_FILE 2>&1 ; then
			cat $TEMP_FILE >&$VERBOSE_DESCRIPTOR
		else
			cat $TEMP_FILE >&$VERBOSE_DESCRIPTOR
			rm -f $TEMP_FILE
			echo "Failed to mount $INSTALL_IMAGE" >&$ERROR_DESCRIPTOR
			exit 1
		fi
		# An image was given, mount it and find the location where it was mounted.
		DEVICE_TO_EJECT=`GetPlistKey ${TEMP_FILE} "dev-entry"`
		INSTALL_PACKAGE_PREFIX="`GetPlistKey ${TEMP_FILE} \"mount-point\"`"
		# Make sure there is only a single trailing /.
		INSTALL_PACKAGE_PREFIX="`echo \"${INSTALL_PACKAGE_PREFIX}/\" | sed 's|//|/|g'`"
		rm -f $TEMP_FILE
		unset TEMP_FILE
	fi
}

ContainsWildcards()
{
	if echo "$1" | grep '[*?]' >/dev/null; then
		return 0
	fi
	return 1
}

EscapeSpaces()
{
	echo "$1" | sed 's/\([ 	]\)/\\\1/g'
}

OneExists()
{
	while [ $# -gt 0 ]; do
		if [ -e "$1" ]; then
			return 0
		fi
		shift
	done
	return 1
}

EscapeForRegex()
{
	if [ $# -gt 0 ]; then
		echo "$@" | sed -e 's/\([+.]\)/\\\1/g'
	else
		sed -e 's/\([+.]\)/\\\1/g'
	fi
}

EndsWith()
{
	if echo "$1" | grep "`EscapeForRegex \"$2\"`\$" >/dev/null; then
		return 0
	fi
	return 1
}

InstallPackages()
{
	if [ $# -lt 1 ]; then
		echo "Found no package" >&$ERROR_DESCRIPTOR
		return 1
	fi
	ipkgs_retval=0

	# Pick a descriptor for chatty output.
	if [ "$INSTALL_VERBOSE" = "1" ]; then
		TARGET_DESCRIPTOR=$VERBOSE_DESCRIPTOR
	else
		TARGET_DESCRIPTOR=$NORMAL_DESCRIPTOR
	fi

	while [ $# -gt 0 ]; do
		PACKAGE="$1"

		# Skip failed expansions.
		if ContainsWildcards "${PACKAGE}"; then
			shift
			continue
		fi

		if OneExists "${PACKAGE}"; then
			:
		else
			echo "Unable to find package \"$1\"" >&$ERROR_DESCRIPTOR
			return 1
		fi

		if [ x${APPONLY:+set} != xset ] && { EndsWith "${PACKAGE}" ".pkg" || EndsWith "${PACKAGE}" ".mpkg"; }; then
			# It's a real package, not an app.
			echo "Installing package $PACKAGE" >&$NORMAL_DESCRIPTOR
			result=0
			echo -e "${INSTALLER_INPUT}" | /usr/sbin/installer $INSTALLER_FLAGS -pkg "$PACKAGE" >&$TARGET_DESCRIPTOR || result=$?
			if [ "$result" != "0" ]; then
				ipkgs_retval=$result
				echo "Failed to install package $PACKAGE" >&$ERROR_DESCRIPTOR
			else
				echo "Successfully installed $PACAKGE" >&$ERROR_DESCRIPTOR
			fi
		else
			# It's an app, not a package.
			if [ -e "$INSTALL_TARGET/Applications" ]; then
				echo "Installing application \"$PACKAGE\" into \"${INSTALL_TARGET}/Applications\"" >&$NORMAL_DESCRIPTOR
				ditto_options=-rsrc
				if [ ${INSTALL_VERBOSE} = 1 ]; then
					ditto_options="${ditto_options} -v"
				fi
				# The ditto program is smart enough to be able to handle
				# resource forks.  "cp -Rp" on earlier versions of OSX (10.3 and
				# earlier) could not copy resource forks.
				ditto ${ditto_options} "$PACKAGE" "$INSTALL_TARGET/Applications/`basename \"${PACKAGE}\"`" || ipkgs_retval=$?
			else
				echo "Applications directory does not exist on target	\"$INSTALL_TARGET\"" >&$ERROR_DESCRIPTOR
				ipkgs_retval=2
			fi
		fi

		shift
	done
	return ${ipkgs_retval}
}

# $1 = type (application or package)
# $2 = pattern
InstallPackagesOrApplicationsFromPrefix()
{
	package_type=$1
	package_pattern="${2:-}"

	escaped_prefix=`EscapeSpaces "${INSTALL_PACKAGE_PREFIX}"`
	packagewildcards=
	if [ "x${package_pattern:+set}" = xset ]; then
		if ContainsWildcards "${package_pattern}"; then
			packagewildcards=`EscapeSpaces "${package_pattern}"`
		fi
	else
		packagewildcards="*.{pkg,mpkg,app}"
	fi

	if [ x${packagewildcards:+set} = xset ]; then
		if eval OneExists "${escaped_prefix}"${packagewildcards}; then
			eval InstallPackages "${escaped_prefix}"${packagewildcards} || return $?
		else
			if [ -z "$INSTALL_IMAGE" ]; then
				echo "Unable to find any ${package_type} matching \"${packagewildcards}\"" >&$ERROR_DESCRIPTOR
			else
				echo "No ${package_type} matching \"${packagewildcards}\" found in image $INSTALL_IMAGE" >&$ERROR_DESCRIPTOR
			fi
			return 1
		fi
	else
		InstallPackages "${INSTALL_PACKAGE_PREFIX}${package_pattern}" || return $?
	fi
	return 0
}

RunCustomCommand()
{
	rcc_retval=0

	pushd "$INSTALL_PACKAGE_PREFIX"

	echo "Running command $CUSTOM_COMMAND_LINE" >&$NORMAL_DESCRIPTOR
	# This eval ensures that quotes are stripped (as a part of tokenization), variables are expanded, etc.
	eval $CUSTOM_COMMAND_LINE || rcc_retval=$?

	if [ "$rcc_retval" != "0" ]; then
		echo "Failed to run command: $CUSTOM_COMMAND_LINE" >&$ERROR_DESCRIPTOR
	else
		echo "Successfully ran custom command." >&$NORMAL_DESCRIPTOR
	fi

	popd > /dev/null
	return ${rcc_retval}
}
# -------



# Check the disk image to see if we can use it.
# Things we can't use:
#   encrypted images.
#
# Things that may require another level of work:
#   Software License Agreement
CheckImageOptions()
{
	if [ -z "$INSTALL_IMAGE" ]; then
		return 0
	fi

	cio_result=0
	TEMP_FILE=/tmp/dmg_installer.info.$$.$RANDOM

	if hdiutil imageinfo -plist "$INSTALL_IMAGE" > $TEMP_FILE 2>&1 ; then
		echo "Image info:" >&$VERBOSE_DESCRIPTOR
		cat $TEMP_FILE >&$VERBOSE_DESCRIPTOR
		if encryption_key="`GetPlistKey ${TEMP_FILE} Encrypted`"; then
			if [ "x${encryption_key}" != xfalse ]; then
				echo "Cannot use image: Image is encrypted." >&2
				cio_result=1
			fi
		fi

		if license_agreement_key="`GetPlistKey ${TEMP_FILE} \"Software License Agreement\"`"; then
			if [ "x${license_agreement_key}" != xfalse ]; then
				IMAGE_HAS_LICENSE=1
			fi
		fi
	else
		echo "Failed to obtain disk image information for $INSTALL_IMAGE :" >&$ERROR_DESCRIPTOR
		cat $TEMP_FILE >&$VERBOSE_DESCRIPTOR
		cio_result=1
	fi

	if [ ${IMAGE_HAS_LICENSE} -eq 1 ] && [ ${ACCEPT_IMAGE_LICENSE} -eq 0 ]; then
		echo "Disk image \"${INSTALL_IMAGE}\" has a license that we are not accepting.  Use the -y option to accept." >&2
		cio_result=2
	fi

	rm -f ${TEMP_FILE}

	if [ ${cio_result} != 0 ]; then
		exit ${cio_result}
	fi
}

Main()
{
	VerifyRootUser >&2
	ParseOptions "$@" || Usage >&2
	set -- "${EXTRA_PARAMETERS[@]}"

	# Set up debugging output descriptors.
	ChangeFileDescriptors

	# Set defaults for parameters which have not been set.
	CheckImageName
	CheckImageOptions

	CheckInstallTarget

	# Build up options for "installer"
	ConfigureInstallerFlags

	MountDiskImage

	installed_something=0
	RETURN_CODE=0
	PACKAGES_INSTALLED=0
	APPLICATIONS_INSTALLED=0

	# If a custom command was provided, do it and nothing else.
	if [ -n "$CUSTOM_COMMAND_LINE" ]; then
		RunCustomCommand || RETURN_CODE=$?
	else
		# If a specific application was selected, install it.
		if [ -n "$INSTALL_APPLICATION" ]; then
			installed_something=1
			# Prevent the installation function from installing a .pkg with the
			# installer.  The user may choose to copy instead of install a .pkg
			# file.
			APPONLY=1
			InstallPackagesOrApplicationsFromPrefix "application" "${INSTALL_APPLICATION}"|| RETURN_CODE=$?
			unset APPONLY
		fi

		# If a specific package was selected, install it.  Not exclusive with installing applications.
		if [ -n "$INSTALL_PACKAGE" ]; then
			installed_something=1
			InstallPackagesOrApplicationsFromPrefix "package" "${INSTALL_PACKAGE}"|| RETURN_CODE=$?
		fi

		# No applications or packages installed.  Default to installing all packages or apps.
		if [ $installed_something = 0 ] && [ ${RETURN_CODE} = 0 ]; then
			InstallPackagesOrApplicationsFromPrefix "package or application" || RETURN_CODE=$?
		fi
	fi

	if [ -n "$DEVICE_TO_EJECT" ]; then
		hdiutil detach $DEVICE_TO_EJECT >&$VERBOSE_DESCRIPTOR
	fi

	[ ${APPLICATIONS_INSTALLED} != 0 ] && [ "$RETURN_CODE" = "0" ] && echo "$APPLICATIONS_INSTALLED applications were successfully installed." >&$NORMAL_DESCRIPTOR
	[ ${PACKAGES_INSTALLED} != 0 ] && [ "$RETURN_CODE" = "0" ] && echo "$PACKAGES_INSTALLED packages were successfully installed." >&$NORMAL_DESCRIPTOR

	# Close the debugging output descriptors
	CloseFileDescriptors

	exit $RETURN_CODE
}

Main "$@"
